Package: seminrExtras
Type: Package
Title: Conduct Additional Modeling and Analysis for 'seminr'
Version: 1.0.2
Date: 2026-06-30
Authors@R: c(person("Soumya", "Ray", email = "soumya.ray@gmail.com", role = c("aut", "ths")),
             person("Nicholas Patrick", "Danks", email = "nicholasdanks@hotmail.com", role = c("aut","cre"))
            )
Description: Supplementary tools for evaluating and validating partial least
    squares structural equation models estimated with 'seminr'. Provides methods
    for predictive model assessment, importance-performance analysis with
    necessary condition testing, overfitting diagnostics, measurement model
    verification, mediator contribution analysis, unobserved heterogeneity
    detection via latent class and prediction-oriented segmentation, and
    congruence coefficient testing. All functions accept estimated 'seminr'
    model objects and return results with print, summary, and plot methods.
Imports: seminr (>= 2.4.0), stats, graphics, grDevices, utils, rpart
License: GPL-3
Encoding: UTF-8
Suggests: testthat (>= 3.0.0), knitr, rmarkdown, MASS, paran, psych,
        learnr
Config/testthat/edition: 3
URL: https://github.com/sem-in-r/seminrExtras
BugReports: https://github.com/sem-in-r/seminrExtras/issues
RoxygenNote: 7.3.2
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-06-30 12:26:39 UTC; nicholasdanks
Author: Soumya Ray [aut, ths],
  Nicholas Patrick Danks [aut, cre]
Maintainer: Nicholas Patrick Danks <nicholasdanks@hotmail.com>
Repository: CRAN
Date/Publication: 2026-06-30 13:30:02 UTC
Built: R 4.4.3; ; 2026-06-30 15:47:37 UTC; unix

Package: seminrExtras
Type: Package
Title: Conduct Additional Modeling and Analysis for 'seminr'
Version: 1.0.1
Date: 2026-05-21
Authors@R: c(person("Soumya", "Ray", email = "soumya.ray@gmail.com", role = c("aut", "ths")),
             person("Nicholas Patrick", "Danks", email = "nicholasdanks@hotmail.com", role = c("aut","cre"))
            )
Description: Supplementary tools for evaluating and validating partial least
    squares structural equation models estimated with 'seminr'. Provides methods
    for predictive model assessment, importance-performance analysis with
    necessary condition testing, overfitting diagnostics, measurement model
    verification, mediator contribution analysis, unobserved heterogeneity
    detection via latent class and prediction-oriented segmentation, and
    congruence coefficient testing. All functions accept estimated 'seminr'
    model objects and return results with print, summary, and plot methods.
Imports: seminr (>= 2.4.0), stats, graphics, grDevices, utils, rpart
License: GPL-3
Encoding: UTF-8
Suggests: testthat (>= 3.0.0), knitr, rmarkdown, MASS, paran, psych,
        learnr
Config/testthat/edition: 3
URL: https://github.com/sem-in-r/seminrExtras
BugReports: https://github.com/sem-in-r/seminrExtras/issues
RoxygenNote: 7.3.2
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-05-21 08:50:45 UTC; nicholasdanks
Author: Soumya Ray [aut, ths],
  Nicholas Patrick Danks [aut, cre]
Maintainer: Nicholas Patrick Danks <nicholasdanks@hotmail.com>
Repository: CRAN
Date/Publication: 2026-05-21 09:40:02 UTC
Built: R 4.6.0; ; 2026-05-22 00:33:59 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(plot,cipma_analysis)
S3method(plot,coa_analysis)
S3method(plot,cta_analysis)
S3method(plot,fimix_analysis)
S3method(plot,fimix_comparison)
S3method(plot,nca_analysis)
S3method(plot,nca_esse)
S3method(plot,pcm_analysis)
S3method(plot,pos_analysis)
S3method(plot,pos_comparison)
S3method(print,cipma_analysis)
S3method(print,coa_analysis)
S3method(print,cta_analysis)
S3method(print,fimix_analysis)
S3method(print,fimix_comparison)
S3method(print,nca_analysis)
S3method(print,nca_esse)
S3method(print,pcm_analysis)
S3method(print,pos_analysis)
S3method(print,pos_comparison)
S3method(print,summary.cipma_analysis)
S3method(print,summary.coa_analysis)
S3method(print,summary.cta_analysis)
S3method(print,summary.fimix_analysis)
S3method(print,summary.fimix_comparison)
S3method(print,summary.nca_analysis)
S3method(print,summary.nca_esse)
S3method(print,summary.pcm_analysis)
S3method(print,summary.pos_analysis)
S3method(summary,cipma_analysis)
S3method(summary,coa_analysis)
S3method(summary,cta_analysis)
S3method(summary,fimix_analysis)
S3method(summary,fimix_comparison)
S3method(summary,nca_analysis)
S3method(summary,nca_esse)
S3method(summary,pcm_analysis)
S3method(summary,pos_analysis)
S3method(summary,pos_comparison)
export(assess_cipma)
export(assess_coa)
export(assess_cta)
export(assess_cvpat)
export(assess_cvpat_compare)
export(assess_fimix)
export(assess_fimix_compare)
export(assess_ipma)
export(assess_nca)
export(assess_nca_esse)
export(assess_pcm)
export(assess_pos)
export(assess_pos_compare)
export(competes)
export(congruence_test)
export(deviance_tree)
export(group_rules)
export(pos_segments)
export(predictive_deviance)
export(unstable_params)
importFrom(grDevices,adjustcolor)
importFrom(grDevices,palette)
importFrom(grDevices,palette.colors)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,barplot)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,matlines)
importFrom(graphics,matplot)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,rect)
importFrom(graphics,text)
importFrom(rpart,rpart)
importFrom(seminr,estimate_pls)
importFrom(seminr,mean_replacement)
importFrom(seminr,predict_DA)
importFrom(seminr,predict_EA)
importFrom(seminr,predict_pls)
importFrom(stats,coef)
importFrom(stats,cov)
importFrom(stats,dnorm)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,p.adjust)
importFrom(stats,pt)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,t.test)
importFrom(stats,var)
importFrom(utils,combn)
importFrom(utils,head)

<!-- README.md is generated from README.Rmd. Please edit that file -->

<img src="vignettes/SEMinR_logo.jpg" width="25%" />

<!-- ![Build Status](https://travis-ci.org/sem-in-r/seminr.svg?branch=master) -->

[![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/seminrExtras)](https://cran.r-project.org/package=seminrExtras)
[![metacran
downloads](https://cranlogs.r-pkg.org/badges/grand-total/seminrExtras)](https://cran.r-project.org/package=seminrExtras)

SEMinRExtras adds functionality to the SEMinR package.

SEMinR (Ray, Danks, & Calero Valdez, 2026) is a domain specific language
for modeling and estimating structural equation models. This is a
supplementary package for SEMinR and not a standalone package. This
package serves to provide additional extra methods and functions that
can be used to analyze PLS-SEM models.

SEMinRExtras provides advanced SEM tools which are compatible with
SEMinR. It implements several methods for evaluating and validating
PLS-SEM models:

- **Cross-Validated Predictive Ability Test (CVPAT)** — Compare model
  predictive performance against benchmarks or alternative models
  (Liengaard et al., 2021; Sharma et al., 2022).
- **Predictive Contribution of the Mediator (PCM)** — Evaluate whether
  a mediating construct improves out-of-sample predictive accuracy by
  comparing Direct Antecedent (DA) and Earliest Antecedent (EA)
  prediction approaches (Danks, 2021).
- **Combined Importance-Performance Map Analysis (cIPMA)** — IPMA with
  NCA integration to identify constructs that are both important and
  necessary (Ringle & Sarstedt, 2016; Sarstedt et al., 2024; Hauff
  et al., 2024).
- **Composite Overfit Analysis (COA)** — Detect observation-level
  overfitting in PLS composite models via predictive deviance trees
  and parameter instability analysis.
- **Necessary Condition Analysis (NCA)** — Test whether predictors are
  necessary conditions for an outcome, complementing PLS-SEM's
  sufficiency logic (Dul, 2016; Richter et al., 2020).
- **NCA-ESSE** — Effect Size Sensitivity Extension that assesses how
  robust NCA results are to extreme response patterns (Becker et al.,
  2026).
- **Confirmatory Tetrad Analysis (CTA-PLS)** — Empirically test whether
  a construct's measurement model is reflective or formative, with
  indicator borrowing for constructs with fewer than 4 indicators
  (Gudergan et al., 2008).
- **Unobserved Heterogeneity** — Two complementary segmentation
  approaches for detecting latent classes in PLS-SEM:
  - **FIMIX-PLS** — EM-based probabilistic segmentation assuming
    normally distributed residuals (Hahn et al., 2002; Sarstedt et al.,
    2011).
  - **PLS-POS** — Deterministic hill-climbing segmentation that
    maximizes the sum of R-squared across segments, making no
    distributional assumptions (Becker et al., 2013).
- **Congruence Testing** — Bootstrapped congruence coefficient testing
  for construct validity (Franke, Sarstedt, & Danks, 2021).

SEMinRExtras also serves to host the example models used in the PLS-SEM
in R workbook (Hair et al., 2026).

### Functions

| Function | Description |
|---|---|
| `assess_cvpat()` | CVPAT against LM and IA benchmarks |
| `assess_cvpat_compare()` | Compare predictive loss of two PLS models |
| `assess_pcm()` | Predictive Contribution of the Mediator (Danks, 2021) |
| `assess_ipma()` | Importance-Performance Map Analysis (IPMA) |
| `assess_cipma()` | Combined IPMA with Necessary Condition Analysis (cIPMA) |
| `assess_coa()` | Composite Overfit Analysis (full pipeline) |
| `predictive_deviance()` | Compute predictive deviance scores |
| `deviance_tree()` | Identify deviant case groups via decision tree |
| `unstable_params()` | Parameter instability analysis |
| `group_rules()` | Extract decision rules for deviant groups |
| `competes()` | Show competing splits at tree nodes |
| `assess_nca()` | Necessary Condition Analysis for PLS-SEM |
| `assess_nca_esse()` | NCA with Effect Size Sensitivity Extension |
| `assess_cta()` | Confirmatory Tetrad Analysis (CTA-PLS) with indicator borrowing (Gudergan et al., 2008) |
| `assess_fimix()` | FIMIX-PLS latent class segmentation |
| `assess_fimix_compare()` | Compare FIMIX solutions across K values |
| `assess_pos()` | PLS-POS prediction-oriented segmentation (Becker et al., 2013) |
| `assess_pos_compare()` | Compare PLS-POS solutions across K values |
| `pos_segments()` | Extract segment-specific re-estimated PLS models |
| `congruence_test()` | Bootstrapped congruence coefficient testing |

## The demo files for Hair et al. (2026)

In order to access the demo files for the textbook, you can run the
`demo()` function after loading the SEMinRExtras package.

- seminr-help-debugging
- seminr-pls-cvpat
- seminr-pls-pcm
- seminr-pls-cipma
- seminr-pls-coa
- seminr-pls-nca
- seminr-pls-fimix
- seminr-pls-cta
- seminr-pls-pos
- seminr-pls-congruence
- seminr-primer-v2-chap2
- seminr-primer-v2-chap3
- seminr-primer-v2-chap4
- seminr-primer-v2-chap5
- seminr-primer-v2-chap6
- seminr-primer-v2-chap7
- seminr-primer-v2-chap8

E.g. `demo("seminr-help-debugging", package = "seminrExtras")`

## The Example model: Corporate Reputation

We are applying the CVPAT process to the corporate reputation example
bundled with SEMinR. Since we will be comparing two models, we will
first estimate and plot both models. Below you will find the competing
and established models which will be compared.

### Established Model

<img src="vignettes/est_mod.png" width="75%" />

### Competing Model

<img src="vignettes/comp_mod.png" width="75%" />

## Example

``` r

# Create measurement model ----
corp_rep_mm_ext <- constructs(
  composite("QUAL", multi_items("qual_", 1:8), weights = mode_B),
  composite("PERF", multi_items("perf_", 1:5), weights = mode_B),
  composite("CSOR", multi_items("csor_", 1:5), weights = mode_B),
  composite("ATTR", multi_items("attr_", 1:3), weights = mode_B),
  composite("COMP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  composite("CUSA", single_item("cusa")),
  composite("CUSL", multi_items("cusl_", 1:3))
)

alt_mm <- constructs(
  composite("QUAL", multi_items("qual_", 1:8), weights = mode_B),
  composite("PERF", multi_items("perf_", 1:5), weights = mode_B),
  composite("CSOR", multi_items("csor_", 1:5), weights = mode_B),
  composite("ATTR", multi_items("attr_", 1:3), weights = mode_B),
  composite("COMP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  composite("CUSA", single_item("cusa")),
  composite("CUSL", multi_items("cusl_", 1:3))
)

# Create structural model ----
corp_rep_sm_ext <- relationships(
  paths(from = c("QUAL", "PERF", "CSOR", "ATTR"), to = c("COMP", "LIKE")),
  paths(from = c("COMP", "LIKE"), to = c("CUSA", "CUSL")),
  paths(from = c("CUSA"),         to = c("CUSL"))
)
alt_sm <- relationships(
  paths(from = c("QUAL", "PERF", "CSOR", "ATTR"), to = c("COMP", "LIKE")),
  paths(from = c("COMP", "LIKE"), to = c("CUSA")),
  paths(from = c("CUSA"),         to = c("CUSL"))
)


# Estimate the models ----
established_model <- estimate_pls(
  data = corp_rep_data,
  measurement_model = corp_rep_mm_ext,
  structural_model  = corp_rep_sm_ext,
  missing = mean_replacement,
  missing_value = "-99")

competing_model <- estimate_pls(
  data = corp_rep_data,
  measurement_model = alt_mm,
  structural_model  = alt_sm,
  missing = mean_replacement,
  missing_value = "-99")

# Function to compare the Loss of two models
compare_results <- assess_cvpat_compare(established_model = established_model,
                                        alternative_model = competing_model,
                                        testtype = "two.sided",
                                        nboot = 2000,
                                        technique = predict_DA,
                                        seed = 123,
                                        noFolds = 10,
                                        reps = 10,
                                        cores = 1)


print(compare_results,
      digits = 3)

# Assess the base model ----
assess_results <- assess_cvpat(established_model,
                               seed = 123,
                               cores = 1)
print(assess_results$CVPAT_compare_LM,
      digits = 3)
print(assess_results$CVPAT_compare_IA,
      digits = 3)
```

## First, conduct CVPAT analysis of the established model.

``` r
# Assess the base model ----
assess_results <- assess_cvpat(established_model,
                               seed = 123,
                               cores = 1)
print(assess_results$CVPAT_compare_LM,
      digits = 3)
#>         PLS Loss LM Loss   Diff Boot T value Boot P Value
#> COMP       1.196   1.211 -0.015        0.542        0.588
#> LIKE       1.915   2.063 -0.148        3.761        0.000
#> CUSA       0.994   0.983  0.011       -0.489        0.625
#> CUSL       1.560   1.600 -0.040        2.914        0.004
#> Overall    1.416   1.464 -0.048        3.482        0.001
#>
#> CVPAT as per Sharma et al. (2023).
print(assess_results$CVPAT_compare_IA,
      digits = 3)
#>         PLS Loss IA Loss   Diff Boot T value Boot P Value
#> COMP       1.196   2.023 -0.827        8.580        0.000
#> LIKE       1.915   3.103 -1.187        8.293        0.000
#> CUSA       0.994   1.374 -0.379        5.004        0.000
#> CUSL       1.560   2.663 -1.102        7.572        0.000
#> Overall    1.416   2.290 -0.874       10.301        0.000
#>
#> CVPAT as per Sharma et al. (2023).
```

The established model has significantly lower predictive loss compared
to both the naive benchmark IA and the LM model. Thus, we can say that
the established model has predictive relevance.

Now we compare the results:

``` r
# Function to compare the Loss of two models
compare_results <- assess_cvpat_compare(established_model = established_model,
                                        alternative_model = competing_model,
                                        testtype = "two.sided",
                                        nboot = 2000,
                                        technique = predict_DA,
                                        seed = 123,
                                        noFolds = 10,
                                        reps = 10,
                                        cores = 1)

print(compare_results,
      digits = 3)
#>         Base Model Loss Alt Model Loss   Diff Boot T value Boot P Value
#> COMP              1.198          1.195  0.003       -0.460        0.645
#> LIKE              1.923          1.933 -0.010        0.883        0.378
#> CUSA              0.988          0.992 -0.004        0.809        0.419
#> CUSL              1.562          1.715 -0.152        3.286        0.001
#> Overall           1.418          1.459 -0.041        3.293        0.001
#>
#> CVPAT as per Sharma, Liengaard, Hair, Sarstedt, & Ringle, (2023).
#>   Both models under comparison have identical endogenous constructs with identical measurement models.
#>   Purely exogenous constructs can differ in regards to their relationships with both nomological
#>   partners and measurement indicators.
```

The established model has significantly lower predictive loss compared
to the competing model. Thus, we can say that the established model has
superior predictive performance compared to the competing model.

## Confirmatory Tetrad Analysis (CTA-PLS)

CTA-PLS (Gudergan et al., 2008) empirically tests whether a
construct's measurement model is consistent with a reflective (common
factor) specification. Under a reflective model, all model-implied
vanishing tetrads equal zero. If any tetrad is significantly non-zero,
the reflective specification is rejected in favour of a formative one.

CTA-PLS requires at least 4 indicators per construct. When `borrow =
TRUE` (the default), constructs with only 2 or 3 indicators can still
be tested by borrowing indicators from structurally connected
constructs. The borrowing rules follow Gudergan et al. (2008, Table 1):
a reflective construct with 3 own indicators borrows 1 from an adjacent
reflective construct (all tetrads vanish under H0), while a construct
with 2 own indicators borrows 2 from any adjacent construct (only the
cross-tetrad tau\_1342 vanishes).

``` r
library(seminr)
library(seminrExtras)

mobi_mm <- constructs(
  composite("Image",        multi_items("IMAG", 1:5)),
  composite("Expectation",  multi_items("CUEX", 1:3)),
  composite("Value",        multi_items("PERV", 1:2)),
  composite("Satisfaction", multi_items("CUSA", 1:3)),
  composite("Loyalty",      multi_items("CUSL", 1:3))
)

mobi_sm <- relationships(
  paths(from = "Image",       to = c("Expectation", "Satisfaction", "Loyalty")),
  paths(from = "Expectation", to = c("Value", "Satisfaction")),
  paths(from = "Value",       to = "Satisfaction"),
  paths(from = "Satisfaction", to = "Loyalty")
)

mobi_pls <- estimate_pls(data = mobi,
                          measurement_model = mobi_mm,
                          structural_model  = mobi_sm)

# CTA-PLS with borrowing (default) — constructs with 2-3 indicators
# borrow from adjacent constructs to form testable 4-tuples
cta_result <- assess_cta(mobi_pls, nboot = 5000, seed = 123)
print(cta_result)
summary(cta_result)

# Without borrowing — only constructs with >= 4 indicators are tested
cta_no_borrow <- assess_cta(mobi_pls, nboot = 5000, borrow = FALSE)
print(cta_no_borrow)
```

## Unobserved Heterogeneity (FIMIX-PLS and PLS-POS)

FIMIX-PLS and PLS-POS are complementary approaches for detecting
unobserved heterogeneity in PLS-SEM. FIMIX-PLS uses probabilistic
(EM-based) assignment and assumes normally distributed residuals,
while PLS-POS uses deterministic (hill-climbing) assignment with no
distributional assumptions. Both methods partition the sample into K
segments with segment-specific path coefficients.

### FIMIX-PLS (Finite Mixture PLS)

FIMIX-PLS (Hahn et al., 2002; Sarstedt et al., 2011) probabilistically
assigns observations to K segments, each with segment-specific structural
path coefficients.

`assess_fimix()` estimates a single K-segment solution, while
`assess_fimix_compare()` compares solutions across K values using
information criteria (AIC, BIC, CAIC, etc.) to help select the
optimal number of segments.

``` r
library(seminr)
library(seminrExtras)

# Estimate a PLS model
corp_pls <- estimate_pls(
  data = corp_rep_data,
  measurement_model = corp_rep_mm_ext,
  structural_model  = corp_rep_sm_ext,
  missing = mean_replacement,
  missing_value = "-99")

# FIMIX with K=2 segments
fimix_k2 <- assess_fimix(corp_pls, K = 2, nstart = 10, seed = 123)
print(fimix_k2)
summary(fimix_k2)
plot(fimix_k2)

# Compare across K=2..4 using information criteria
fimix_compare <- assess_fimix_compare(corp_pls,
                                       K_range = 2:4,
                                       nstart = 10,
                                       seed = 123)
print(fimix_compare)
plot(fimix_compare)
```

### PLS-POS (Prediction-Oriented Segmentation)

PLS-POS (Becker et al., 2013) maximizes the sum of R-squared across
all endogenous constructs across K segments using deterministic
hill-climbing. Unlike FIMIX-PLS, it makes no distributional
assumptions and can detect heterogeneity in both structural and
formative measurement models.

`assess_pos()` estimates a single K-segment solution, while
`assess_pos_compare()` compares solutions across multiple K values
using the objective criterion (sum of R-squared).

``` r
# PLS-POS with K=2 segments (using corp_pls from above)
pos_k2 <- assess_pos(corp_pls, K = 2, nstart = 10, seed = 123)
print(pos_k2)
summary(pos_k2)
plot(pos_k2, type = "rsquared")

# Compare across K=2..4
pos_compare <- assess_pos_compare(corp_pls,
                                   K_range = 2:4,
                                   nstart = 10,
                                   seed = 123)
print(pos_compare)
plot(pos_compare)
```

## Congruence Testing

Congruence testing (Franke, Sarstedt, & Danks, 2021) evaluates
whether PLS composite weights are stable across bootstrap samples by
computing congruence coefficients. A congruence coefficient close to
1 indicates that the composite weight pattern is robust.

``` r
cong_result <- congruence_test(mobi_pls,
                                nboot = 2000,
                                seed = 123)
print(cong_result)
summary(cong_result)
```

# References

- Becker, J.-M., Rai, A., Ringle, C. M., & Voelckner, F. (2013).
  Discovering Unobserved Heterogeneity in Structural Equation Models to
  Avert Validity Threats. MIS Quarterly, 37(3), 665-694.
- Becker, J.-M., Richter, N. F., Ringle, C. M., & Sarstedt, M. (2026).
  Must-have, or maybe not? A sensitivity-based extension to necessary
  condition analysis. Journal of Business Research, 206, 115920.
- Dul, J. (2016). Necessary Condition Analysis (NCA): Logic and
  methodology of "necessary but not sufficient" causality.
  Organizational Research Methods, 19(1), 10-52.
- Franke, G. R., Sarstedt, M., & Danks, N. P. (2021). An empirical
  comparison of factor score estimation methods. Journal of Business
  Research, 130, 318-334.
- Hahn, C., Johnson, M. D., Herrmann, A., & Huber, F. (2002). Capturing
  Customer Heterogeneity using a Finite Mixture PLS Approach.
  Schmalenbach Business Review, 54, 243-269.
- Gudergan, S. P., Ringle, C. M., Wende, S. & Will, A. (2008).
  Confirmatory Tetrad Analysis in PLS Path Modeling. Journal of Business
  Research, 61(12), 1238-1249.
- Hair, J.F. (Jr), Hult, T.M., Ringle, C.M., Sarstedt, M., Danks, N.P.,
  and Adler, S. (2026). Partial Least Squares Structural Equation
  Modeling (PLS-SEM) Using R (Second Edition) - A Workbook. Springer.
- Hauff, S., Richter, N. F., Sarstedt, M., & Ringle, C. M. (2024).
  Importance and Performance in PLS-SEM and NCA: Introducing the
  Combined Importance-Performance Map Analysis (cIPMA). Journal of
  Retailing and Consumer Services, 78, 103723.
- Liengaard, B. D., Sharma, P. N., Hult, G. T. M., Jensen, M. B.,
  Sarstedt, M., Hair, J. F., & Ringle, C. M. (2021). Prediction:
  coveted, yet forsaken? Introducing a cross-validated predictive ability
  test in partial least squares path modeling. Decision Sciences, 52(2),
  362-392.
- Ray, S., Danks, N.P., Calero Valdez, A. (2026) SEMinR: Domain-specific
  language for building, estimating, and visualizing structural equation
  models in R.
- Richter, N. F., Schubring, S., Hauff, S., Ringle, C. M., &
  Sarstedt, M. (2020). When predictors of outcomes are necessary:
  guidelines for the combined use of PLS-SEM and NCA. Industrial
  Management & Data Systems, 120(12), 2243-2267.
- Sarstedt, M., Becker, J.-M., Ringle, C. M. & Schwaiger, M. (2011).
  Uncovering and Treating Unobserved Heterogeneity with FIMIX-PLS.
  Schmalenbach Business Review, 63(1), 34-62.
- Ringle, C. M. & Sarstedt, M. (2016). Gain More Insight from Your
  PLS-SEM Results: The Importance-Performance Map Analysis. Industrial
  Management & Data Systems, 119(9), 1865-1886.
- Sarstedt, M., Richter, N. F., Hauff, S. & Ringle, C. M. (2024).
  Combined Importance-Performance Map Analysis (cIPMA): A SmartPLS 4
  Tutorial. Journal of Marketing Analytics, 12, 746-760.
- Sharma, P. N., Liengaard, B. D., Hair, J. F., Sarstedt, M., &
  Ringle, C. M. (2022). Predictive model assessment and selection in
  composite-based modeling using PLS-SEM: extensions and guidelines for
  using CVPAT. European Journal of Marketing, 57(6), 1662-1677.

### Accompanying Code for:
## Partial Least Squares Structural Equation Modeling (PLS-SEM) Using R - A Workbook (2021)
## Hair, J.F. (Jr), Hult, T.M., Ringle, C.M., Sarstedt, M., Danks, N.P., and Ray, S.

## Chapter 3: Introduction to SEMinR

# Download and install the SEMinR package
# You only need to do this once to equip RStudio on your computer with SEMinR
# and you will have to delete the '#' and then execute the below line of code:
# install.packages("seminr")

# Make the SEMinR library ready to use
# You must do this every time you restart RStudio and wish to use SEMinR
# and you will have to delete the '#' and then execute the below line of code:
# library(seminr)

# Load the corporate reputation data
corp_rep_data <- corp_rep_data

# Show the first several rows of the corporate reputation data
head(corp_rep_data)

# Create measurement model ----
simple_mm <- constructs(
  composite("COMP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  composite("CUSA", single_item("cusa")),
  composite("CUSL", multi_items("cusl_", 1:3)))

# Create structural model ----
simple_sm <- relationships(
  paths(from = c("COMP", "LIKE"), to = c("CUSA", "CUSL")),
  paths(from = c("CUSA"), to = c("CUSL")))

# Estimate the model
corp_rep_simple_model <- estimate_pls(data = corp_rep_data,
  measurement_model = simple_mm,
  structural_model  = simple_sm,
  inner_weights = path_weighting,
  missing = mean_replacement,
  missing_value = "-99")

# Estimate the model with default settings
corp_rep_simple_model <- estimate_pls(data = corp_rep_data,
  measurement_model = simple_mm,
  structural_model  = simple_sm,
  missing_value = "-99")

# Summarize the model results
summary_simple_corp_rep <- summary(corp_rep_simple_model)

# Inspect the structural paths
summary_simple_corp_rep$paths

# Inspect the construct reliability metrics
summary_simple_corp_rep$reliability

# Bootstrap the model
boot_simple_corp_rep <- bootstrap_model(seminr_model = corp_rep_simple_model,
  nboot = 1000,
  cores = NULL,
  seed = 123)

# Store the summary of the bootstrapped model
sum_boot_simple_corp_rep <- summary(boot_simple_corp_rep)

# Inspect the bootstrapped structural paths
sum_boot_simple_corp_rep$bootstrapped_paths

# Inspect the bootstrapped indicator loadings
sum_boot_simple_corp_rep$bootstrapped_loadings

# Write the bootstrapped paths object to csv file
# and you will have to delete the '#' and then execute the below line of code:
# write.csv(x = sum_boot_simple_corp_rep$bootstrapped_loadings, file = "boot_loadings.csv")

# Generate the plot for exporting
plot(summary_simple_corp_rep$reliability)

### Accompanying Code for:
## Partial Least Squares Structural Equation Modeling (PLS-SEM) Using R - A Workbook (2021)
## Hair, J.F. (Jr), Hult, T.M., Ringle, C.M., Sarstedt, M., Danks, N.P., and Ray, S.

## Chapter 8: Moderation analysis

# Load the SEMinR library
library(seminr)

# Load the data
corp_rep_data <- corp_rep_data

# Create measurement model ----
corp_rep_mm_mod <- constructs(
  composite("QUAL", multi_items("qual_", 1:8), weights = mode_B),
  composite("PERF", multi_items("perf_", 1:5), weights = mode_B),
  composite("CSOR", multi_items("csor_", 1:5), weights = mode_B),
  composite("ATTR", multi_items("attr_", 1:3), weights = mode_B),
  composite("COMP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  composite("CUSA", single_item("cusa")),
  composite("SC", multi_items("switch_", 1:4)),
  composite("CUSL", multi_items("cusl_", 1:3)),
  interaction_term(iv = "CUSA", moderator = "SC", method = two_stage, weights = mode_A))

# Create structural model ----
corp_rep_sm_mod <- relationships(
  paths(from = c("QUAL", "PERF", "CSOR", "ATTR"), to = c("COMP", "LIKE")),
  paths(from = c("COMP", "LIKE"), to = c("CUSA", "CUSL")),
  paths(from = c("CUSA", "SC", "CUSA*SC"), to = c("CUSL"))
)

# Estimate the model ----
corp_rep_pls_model_mod <- estimate_pls(
  data = corp_rep_data,
  measurement_model = corp_rep_mm_mod,
  structural_model  = corp_rep_sm_mod,
  missing = mean_replacement,
  missing_value = "-99")

# Extract the summary
sum_corp_rep_mod <- summary(corp_rep_pls_model_mod)

# Bootstrap the model ----
boot_corp_rep_mod <- bootstrap_model(seminr_model = corp_rep_pls_model_mod,
                                     nboot = 1000)

# Summarize the results of the bootstrap
sum_boot_corp_rep_mod <- summary(boot_corp_rep_mod, alpha = 0.05)

# Inspect the bootstrapped structural paths
sum_boot_corp_rep_mod$bootstrapped_paths

# Simple slope analysis plot
slope_analysis(
  moderated_model = corp_rep_pls_model_mod,
  dv = "CUSL",
  moderator = "SC",
  iv = "CUSA",
  leg_place = "bottomright")

# PLS-POS (Prediction-Oriented Segmentation) with SEMinR
library(seminr)
library(seminrExtras)

# Create measurement model ----
corp_rep_mm <- constructs(
  composite("QUAL", multi_items("qual_", 1:8), weights = mode_B),
  composite("PERF", multi_items("perf_", 1:5), weights = mode_B),
  composite("CSOR", multi_items("csor_", 1:5), weights = mode_B),
  composite("ATTR", multi_items("attr_", 1:3), weights = mode_B),
  composite("COMP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  composite("CUSA", single_item("cusa")),
  composite("CUSL", multi_items("cusl_", 1:3))
)

# Create structural model ----
corp_rep_sm <- relationships(
  paths(from = c("QUAL", "PERF", "CSOR", "ATTR"), to = c("COMP", "LIKE")),
  paths(from = c("COMP", "LIKE"), to = c("CUSA", "CUSL")),
  paths(from = c("CUSA"),         to = c("CUSL"))
)

# Estimate the model ----
corp_pls <- estimate_pls(
  data = corp_rep_data,
  measurement_model = corp_rep_mm,
  structural_model  = corp_rep_sm,
  missing = mean_replacement,
  missing_value = "-99")

# ============================================================================
# PLS-POS for a single K (assess_pos)
# ============================================================================
# PLS-POS uses a deterministic hill-climbing approach to maximize the sum of
# R-squared across all endogenous constructs across K segments. Unlike
# FIMIX-PLS, PLS-POS makes no distributional assumptions and can detect
# heterogeneity in both structural and (formative) measurement models.

pos_k2 <- assess_pos(corp_pls, K = 2, nstart = 10, max_iter = 100, seed = 123)

# Print: segment sizes, R-squared, path coefficients
print(pos_k2)

# Summary: detailed comparison with global model
summary(pos_k2)

# Visualize segment proportions
plot(pos_k2, type = "segments")

# Visualize R-squared across segments
plot(pos_k2, type = "rsquared")

# Visualize path coefficients across segments
plot(pos_k2, type = "paths")

# ============================================================================
# Extract segment-specific models (pos_segments)
# ============================================================================
# Retrieve the fully re-estimated PLS models for each segment
seg_models <- pos_segments(pos_k2)
summary(seg_models[[1]])

# ============================================================================
# Compare across K values (assess_pos_compare)
# ============================================================================
# Estimate PLS-POS for multiple K values and compare the objective criterion
# (sum of R-squared) to determine the optimal number of segments.

pos_compare <- assess_pos_compare(corp_pls,
                                   K_range = 2:4,
                                   nstart = 10,
                                   max_iter = 100,
                                   seed = 123)

# Print: objective comparison table across K values
print(pos_compare)

# Plot: objective criterion vs K
plot(pos_compare)

### Accompanying Code for:
## Partial Least Squares Structural Equation Modeling (PLS-SEM) Using R - A Workbook (2021)
## Hair, J.F. (Jr), Hult, T.M., Ringle, C.M., Sarstedt, M., Danks, N.P., and Ray, S.

## Chapter 4: Evaluation of reflective measurement models

# Load the SEMinR library
library(seminr)

# Load the data ----
corp_rep_data <- corp_rep_data

# Create measurement model ----
corp_rep_mm <- constructs(
  composite("COMP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  composite("CUSA", single_item("cusa")),
  composite("CUSL", multi_items("cusl_", 1:3)))

# Create structural model ----
corp_rep_sm <- relationships(
  paths(from = c("COMP", "LIKE"), to = c("CUSA", "CUSL")),
  paths(from = c("CUSA"), to = c("CUSL")))

# Estimate the model
corp_rep_pls_model <- estimate_pls(
  data = corp_rep_data,
  measurement_model = corp_rep_mm,
  structural_model  = corp_rep_sm,
  missing = mean_replacement,
  missing_value = "-99")

# Summarize the model results
summary_corp_rep <- summary(corp_rep_pls_model)

# Inspect iterations
summary_corp_rep$iterations

# Inspect the outer loadings
summary_corp_rep$loadings

# Inspect the indicator reliability
summary_corp_rep$loadings^2

# Inspect the internal consistency and reliability
summary_corp_rep$reliability

# Plot the reliabilities of constructs
plot(summary_corp_rep$reliability)

# Table of the FL criteria
summary_corp_rep$validity$fl_criteria

# HTMT Ratio
summary_corp_rep$validity$htmt

# Bootstrap the model
boot_corp_rep <- bootstrap_model(seminr_model = corp_rep_pls_model,
                                 nboot = 1000)

# Store the summary of the bootstrapped model
sum_boot_corp_rep <- summary(boot_corp_rep, alpha = 0.10)

# Extract the bootstrapped HTMT
sum_boot_corp_rep$bootstrapped_HTMT

### Accompanying Code for:
## Partial Least Squares Structural Equation Modeling (PLS-SEM) Using R - A Workbook (2025)
## Hair, J.F. (Jr), Hult, T.M., Ringle, C.M., Sarstedt, M., Danks, N.P., and Adler, S.

## Chapter 6: Evaluation of the structural model
# Load the SEMinR, SEMinRExtras libraries
library(seminr)
library(seminrExtras)

# Setup the plotting environment
oldpar <- par(mfrow = c(1,1))

# Load the data
corp_rep_data <- corp_rep_data

# Create measurement model ----
corp_rep_mm_ext <- constructs(
  composite("QUAL", multi_items("qual_", 1:8), weights = mode_B),
  composite("PERF", multi_items("perf_", 1:5), weights = mode_B),
  composite("CSOR", multi_items("csor_", 1:5), weights = mode_B),
  composite("ATTR", multi_items("attr_", 1:3), weights = mode_B),
  composite("COMP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  composite("CUSA", single_item("cusa")),
  composite("CUSL", multi_items("cusl_", 1:3))
)

# Create structural model ----
corp_rep_sm_ext <- relationships(
  paths(from = c("QUAL", "PERF", "CSOR", "ATTR"), to = c("COMP", "LIKE")),
  paths(from = c("COMP", "LIKE"), to = c("CUSA", "CUSL")),
  paths(from = c("CUSA"),         to = c("CUSL"))
)

# Estimate the model ----
corp_rep_pls_model_ext <- estimate_pls(
  data = corp_rep_data,
  measurement_model = corp_rep_mm_ext,
  structural_model  = corp_rep_sm_ext,
  missing = mean_replacement,
  missing_value = "-99")

# Extract the summary
summary_corp_rep_ext <- summary(corp_rep_pls_model_ext)

# Bootstrap the model ----
boot_corp_rep_ext <- bootstrap_model(
  seminr_model = corp_rep_pls_model_ext,
  nboot = 1000,
  cores = parallel::detectCores(),
  seed = 123)

# Summarize the results of the bootstrap
summary_boot_corp_rep_ext <- summary(boot_corp_rep_ext,
                                     alpha = 0.05)

# Inspect the structural model collinearity VIF
summary_corp_rep_ext$vif_antecedents

# Inspect the structural paths
summary_boot_corp_rep_ext$bootstrapped_paths

# Inspect the total effects
summary_boot_corp_rep_ext$bootstrapped_total_paths

# Inspect the model RSquares
summary_corp_rep_ext$paths

# Inspect the effect sizes
summary_corp_rep_ext$fSquare

# Conduct CVPAT assessment of the established model
assess_results <- assess_cvpat(seminr_model = corp_rep_pls_model_ext,
                               testtype = "greater",
                               nboot = 2000,
                               seed = 123,
                               technique = predict_DA,
                               noFolds = 10,
                               reps = 10)

print(assess_results$CVPAT_compare_LM,
      digits = 3)
print(assess_results$CVPAT_compare_IA,
      digits = 3)

# Conduct predictive model comparison
# Estimate alternative models
# Create measurement model ----
measurement_model <- constructs(
  composite("QUAL", multi_items("qual_", 1:8), weights = mode_B),
  composite("PERF", multi_items("perf_", 1:5), weights = mode_B),
  composite("CSOR", multi_items("csor_", 1:5), weights = mode_B),
  composite("ATTR", multi_items("attr_", 1:3), weights = mode_B),
  composite("COMP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  composite("CUSA", single_item("cusa")),
  composite("CUSL", multi_items("cusl_", 1:3))
)

# Create structural models
# Model 1
structural_model1 <- relationships(
  paths(from = c("QUAL","PERF","CSOR","ATTR"), to = c("COMP", "LIKE")),
  paths(from = c("COMP","LIKE"),  to = c("CUSA", "CUSL")),
  paths(from = "CUSA", to = c("CUSL"))
)
# Model 2
structural_model2 <- relationships(
  paths(from = c("QUAL","PERF","CSOR","ATTR"), to = c("COMP", "LIKE", "CUSA")),
  paths(from = c("COMP","LIKE"),  to = c("CUSA", "CUSL")),
  paths(from = "CUSA", to = c("CUSL"))
)
# Model 3
structural_model3 <- relationships(
  paths(from = c("QUAL","PERF","CSOR","ATTR"), to = c("COMP", "LIKE", "CUSA", "CUSL")),
  paths(from = c("COMP","LIKE"),  to = c("CUSA", "CUSL")),
  paths(from = "CUSA", to = c("CUSL"))
)

# Estimate and summarize the models ----
pls_model1 <- estimate_pls(data  = corp_rep_data,
                           measurement_model = measurement_model,
                           structural_model  = structural_model1,
                           missing_value = "-99")
sum_model1 <- summary(pls_model1)
pls_model2 <- estimate_pls(data  = corp_rep_data,
                           measurement_model = measurement_model,
                           structural_model  = structural_model2,
                           missing_value = "-99")
sum_model2 <- summary(pls_model2)
pls_model3 <- estimate_pls(data  = corp_rep_data,
                           measurement_model = measurement_model,
                           structural_model  = structural_model3,
                           missing_value = "-99")
sum_model3 <- summary(pls_model3)

# Inspect the IT Criteria matrix of Model1
sum_model1$it_criteria

# Subset the matrix to only return the BIC row and CUSA column
sum_model1$it_criteria["BIC", "CUSA"]

# Collect the vector of BIC values for CUSA
itcriteria_vector <- c(sum_model1$it_criteria["BIC","CUSA"],
                       sum_model2$it_criteria["BIC","CUSA"],
                       sum_model3$it_criteria["BIC","CUSA"])

# Assign the model names to IT Criteria vector
names(itcriteria_vector) <- c("Model1", "Model2", "Model3")

# Inspect the IT Criteria vector for competing models.
itcriteria_vector

# Calculate the model BIC Akaike Weights
compute_itcriteria_weights(itcriteria_vector)

# Conduct CVPAT analysis ----
# Select the models for analysis and comparison
established_model <- pls_model1
alternative_model <- pls_model3

# Conduct CVPAT model comparison
compare_results <- assess_cvpat_compare(established_model = established_model,
                                        alternative_model = alternative_model,
                                        testtype = "greater",
                                        nboot = 2000,
                                        technique = predict_DA,
                                        seed = 123,
                                        noFolds = 10,
                                        reps = 10)

print(compare_results,
      digits = 3)

# Calculate overfit ratio ----
predict_established <- summary(predict_pls(model = established_model))
predict_alternative <- summary(predict_pls(model = alternative_model))

# Overfit ratio for established model
predict_established$construct_error

# Overfit ratio for alternative model
predict_alternative$construct_error

### Accompanying Code for:
## Partial Least Squares Structural Equation Modeling (PLS-SEM) Using R - A Workbook (2021)
## Hair, J.F. (Jr), Hult, T.M., Ringle, C.M., Sarstedt, M., Danks, N.P., and Ray, S.

## Chapter 5: Evaluation of formative measurement models

# Load the SEMinR library
library(seminr)

# Load the corporate repuation data
corp_rep_data <- corp_rep_data

# Create measurement model ----
corp_rep_mm_ext <- constructs(
  composite("QUAL", multi_items("qual_", 1:8), weights = mode_B),
  composite("PERF", multi_items("perf_", 1:5), weights = mode_B),
  composite("CSOR", multi_items("csor_", 1:5), weights = mode_B),
  composite("ATTR", multi_items("attr_", 1:3), weights = mode_B),
  composite("COMP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  composite("CUSA", single_item("cusa")),
  composite("CUSL", multi_items("cusl_", 1:3))
)

# Create structural model ----
corp_rep_sm_ext <- relationships(
  paths(from = c("QUAL", "PERF", "CSOR", "ATTR"), to = c("COMP", "LIKE")),
  paths(from = c("COMP", "LIKE"), to = c("CUSA", "CUSL")),
  paths(from = c("CUSA"),         to = c("CUSL"))
)

# Estimate the model ----
corp_rep_pls_model_ext <- estimate_pls(
  data = corp_rep_data,
  measurement_model = corp_rep_mm_ext,
  structural_model  = corp_rep_sm_ext,
  missing = mean_replacement,
  missing_value = "-99")

# Summarize the model results
summary_corp_rep_ext <- summary(corp_rep_pls_model_ext)

# Iterations to converge
summary_corp_rep_ext$iterations

# Bootstrap the model
boot_corp_rep_ext <- bootstrap_model(seminr_model = corp_rep_pls_model_ext,
                                     nboot = 1000)

# Store the summary of the bootstrapped model
sum_boot_corp_rep_ext <- summary(boot_corp_rep_ext, alpha = 0.10)

# Inspect the indicator loadings
summary_corp_rep_ext$loadings

# Inspect the indicator reliability
summary_corp_rep_ext$loadings^2

# Inspect the internal consistency and reliability
summary_corp_rep_ext$reliability

# Table of the FL criteria
summary_corp_rep_ext$validity$fl_criteria

# HTMT Ratio
summary_corp_rep_ext$validity$htmt

# Extract the bootstrapped HTMT
sum_boot_corp_rep_ext$bootstrapped_HTMT

# Redundancy analysis ----
# ATTR ----
# Create measurement model
ATTR_redundancy_mm <- constructs(
  composite("ATTR_F", multi_items("attr_", 1:3), weights = mode_B),
  composite("ATTR_G", single_item("attr_global"))
)

# Create structural model
ATTR_redundancy_sm <- relationships(
  paths(from = c("ATTR_F"), to = c("ATTR_G"))
)

# Estimate the model
ATTR_redundancy_pls_model <- estimate_pls(data = corp_rep_data,
                                          measurement_model = ATTR_redundancy_mm,
                                          structural_model  = ATTR_redundancy_sm,
                                          missing = mean_replacement,
                                          missing_value = "-99")

# Summarize the model
sum_ATTR_red_model <- summary(ATTR_redundancy_pls_model)

# CSOR ----
# Create measurement model
CSOR_redundancy_mm <- constructs(
  composite("CSOR_F", multi_items("csor_", 1:5), weights = mode_B),
  composite("CSOR_G", single_item("csor_global"))
)

# Create structural model
CSOR_redundancy_sm <- relationships(
  paths(from = c("CSOR_F"), to = c("CSOR_G"))
)

# Estimate the model
CSOR_redundancy_pls_model <- estimate_pls(data = corp_rep_data,
                                          measurement_model = CSOR_redundancy_mm,
                                          structural_model  = CSOR_redundancy_sm,
                                          missing = mean_replacement,
                                          missing_value = "-99")

# Summarize the model
sum_CSOR_red_model <- summary(CSOR_redundancy_pls_model)

# PERF ----
# Create measurement model
PERF_redundancy_mm <- constructs(
  composite("PERF_F", multi_items("perf_", 1:5), weights = mode_B),
  composite("PERF_G", single_item("perf_global"))
)

# Create structural model
PERF_redundancy_sm <- relationships(
  paths(from = c("PERF_F"), to = c("PERF_G"))
)

# Estimate the model
PERF_redundancy_pls_model <- estimate_pls(data = corp_rep_data,
                                          measurement_model = PERF_redundancy_mm,
                                          structural_model  = PERF_redundancy_sm,
                                          missing = mean_replacement,
                                          missing_value = "-99")

# Summarize the model
sum_PERF_red_model <- summary(PERF_redundancy_pls_model)

# QUAL ----
# Create measurement model
QUAL_redundancy_mm <- constructs(
  composite("QUAL_F", multi_items("qual_", 1:8), weights = mode_B),
  composite("QUAL_G", single_item("qual_global"))
)

# Create structural model
QUAL_redundancy_sm <- relationships(
  paths(from = c("QUAL_F"), to = c("QUAL_G"))
)

# Estimate the model
QUAL_redundancy_pls_model <- estimate_pls(data = corp_rep_data,
                                          measurement_model = QUAL_redundancy_mm,
                                          structural_model  = QUAL_redundancy_sm,
                                          missing = mean_replacement,
                                          missing_value = "-99")

# Summarize the model
sum_QUAL_red_model <- summary(QUAL_redundancy_pls_model)

# Check the path coefficients for convergent validity
sum_ATTR_red_model$paths
sum_CSOR_red_model$paths
sum_PERF_red_model$paths
sum_QUAL_red_model$paths

# Collinearity analysis ----
summary_corp_rep_ext$validity$vif_items

# Bootstrap the model ----
# seminr_model is the SEMinR model to be bootstrapped
# nboot is the number of bootstrap iterations to run
# cores is the number of cpu cores to use in multicore bootstrapping
# parallel::detectCores() allows for using the maximum cores on your device
# seed is the seed to be used for making bootstrap replicable
boot_corp_rep_ext <- bootstrap_model(
  seminr_model = corp_rep_pls_model_ext,
  nboot = 1000,
  cores = parallel::detectCores(),
  seed = 123)

# Summarize the results of the bootstrap
# alpha sets the specified level for significance, i.e. 0.05
sum_boot_corp_rep_ext <- summary(boot_corp_rep_ext, alpha = 0.05)

# Inspect the bootstrapping results for outer weights
sum_boot_corp_rep_ext$bootstrapped_weights

# Inspect the bootstrapping results for the outer loadings
sum_boot_corp_rep_ext$bootstrapped_loadings

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(seminr)
library(seminrExtras)

# set the theme for plotting output
seminr_theme_set(seminr_theme_academic())

# Create measurement model ----
corp_rep_mm_ext <- constructs(
  composite("QUAL", multi_items("qual_", 1:8), weights = mode_B),
  composite("PERF", multi_items("perf_", 1:5), weights = mode_B),
  composite("CSOR", multi_items("csor_", 1:5), weights = mode_B),
  composite("ATTR", multi_items("attr_", 1:3), weights = mode_B),
  composite("COMP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  composite("CUSA", single_item("cusa")),
  composite("CUSL", multi_items("cusl_", 1:3))
)

alt_mm <- constructs(
  composite("QUAL", multi_items("qual_", 1:8), weights = mode_B),
  composite("PERF", multi_items("perf_", 1:5), weights = mode_B),
  composite("CSOR", multi_items("csor_", 1:5), weights = mode_B),
  composite("ATTR", multi_items("attr_", 1:3), weights = mode_B),
  composite("COMP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  composite("CUSA", single_item("cusa")),
  composite("CUSL", multi_items("cusl_", 1:3))
)

# Create structural model ----
corp_rep_sm_ext <- relationships(
  paths(from = c("QUAL", "PERF", "CSOR", "ATTR"), to = c("COMP", "LIKE")),
  paths(from = c("COMP", "LIKE"), to = c("CUSA", "CUSL")),
  paths(from = c("CUSA"),         to = c("CUSL"))
)
alt_sm <- relationships(
  paths(from = c("QUAL", "PERF", "CSOR", "ATTR"), to = c("COMP", "LIKE")),
  paths(from = c("COMP", "LIKE"), to = c("CUSA")),
  paths(from = c("CUSA"),         to = c("CUSL"))
)


# Estimate the models ----
established_model <- estimate_pls(
  data = corp_rep_data,
  measurement_model = corp_rep_mm_ext,
  structural_model  = corp_rep_sm_ext,
  missing = mean_replacement,
  missing_value = "-99")

competing_model <- estimate_pls(
  data = corp_rep_data,
  measurement_model = alt_mm,
  structural_model  = alt_sm,
  missing = mean_replacement,
  missing_value = "-99")



## ----fig.show='hold'----------------------------------------------------------
plot(established_model)
plot(competing_model)

## ----eval=FALSE, echo=TRUE----------------------------------------------------
# 
# # Create measurement model ----
# corp_rep_mm_ext <- constructs(
#   composite("QUAL", multi_items("qual_", 1:8), weights = mode_B),
#   composite("PERF", multi_items("perf_", 1:5), weights = mode_B),
#   composite("CSOR", multi_items("csor_", 1:5), weights = mode_B),
#   composite("ATTR", multi_items("attr_", 1:3), weights = mode_B),
#   composite("COMP", multi_items("comp_", 1:3)),
#   composite("LIKE", multi_items("like_", 1:3)),
#   composite("CUSA", single_item("cusa")),
#   composite("CUSL", multi_items("cusl_", 1:3))
# )
# 
# alt_mm <- constructs(
#   composite("QUAL", multi_items("qual_", 1:8), weights = mode_B),
#   composite("PERF", multi_items("perf_", 1:5), weights = mode_B),
#   composite("CSOR", multi_items("csor_", 1:5), weights = mode_B),
#   composite("ATTR", multi_items("attr_", 1:3), weights = mode_B),
#   composite("COMP", multi_items("comp_", 1:3)),
#   composite("LIKE", multi_items("like_", 1:3)),
#   composite("CUSA", single_item("cusa")),
#   composite("CUSL", multi_items("cusl_", 1:3))
# )
# 
# # Create structural model ----
# corp_rep_sm_ext <- relationships(
#   paths(from = c("QUAL", "PERF", "CSOR", "ATTR"), to = c("COMP", "LIKE")),
#   paths(from = c("COMP", "LIKE"), to = c("CUSA", "CUSL")),
#   paths(from = c("CUSA"),         to = c("CUSL"))
# )
# alt_sm <- relationships(
#   paths(from = c("QUAL", "PERF", "CSOR", "ATTR"), to = c("COMP", "LIKE")),
#   paths(from = c("COMP", "LIKE"), to = c("CUSA")),
#   paths(from = c("CUSA"),         to = c("CUSL"))
# )
# 
# 
# # Estimate the models ----
# established_model <- estimate_pls(
#   data = corp_rep_data,
#   measurement_model = corp_rep_mm_ext,
#   structural_model  = corp_rep_sm_ext,
#   missing = mean_replacement,
#   missing_value = "-99")
# 
# competing_model <- estimate_pls(
#   data = corp_rep_data,
#   measurement_model = alt_mm,
#   structural_model  = alt_sm,
#   missing = mean_replacement,
#   missing_value = "-99")
# 
# # Function to compare the Loss of two models
# compare_results <- assess_cvpat_compare(established_model = established_model,
#                                         alternative_model = competing_model,
#                                         testtype = "two.sided",
#                                         nboot = 2000,
#                                         technique = predict_DA,
#                                         seed = 123,
#                                         noFolds = 10,
#                                         reps = 10,
#                                         cores = 1)
# 
# 
# print(compare_results,
#       digits = 3)
# 
# # Assess the base model ----
# assess_results <- assess_cvpat(established_model,
#                                seed = 123,
#                                cores = 1)
# print(assess_results$CVPAT_compare_LM,
#       digits = 3)
# print(assess_results$CVPAT_compare_IA,
#       digits = 3)
# 

## ----echo=TRUE----------------------------------------------------------------
# Assess the base model ----
assess_results <- assess_cvpat(established_model,
                               seed = 123, 
                               cores = 1)
print(assess_results$CVPAT_compare_LM,
      digits = 3)
print(assess_results$CVPAT_compare_IA,
      digits = 3)

## ----echo=TRUE----------------------------------------------------------------
# Function to compare the Loss of two models
compare_results <- assess_cvpat_compare(established_model = established_model,
                                        alternative_model = competing_model,
                                        testtype = "two.sided",
                                        nboot = 2000,
                                        technique = predict_DA,
                                        seed = 123,
                                        noFolds = 10,
                                        reps = 10,
                                        cores = 1)

print(compare_results, 
      cores = 1,
      digits = 3)

# FIMIX-PLS (Finite Mixture PLS) with SEMinR
library(seminr)
library(seminrExtras)

# Create measurement model ----
corp_rep_mm <- constructs(
  composite("QUAL", multi_items("qual_", 1:8), weights = mode_B),
  composite("PERF", multi_items("perf_", 1:5), weights = mode_B),
  composite("CSOR", multi_items("csor_", 1:5), weights = mode_B),
  composite("ATTR", multi_items("attr_", 1:3), weights = mode_B),
  composite("COMP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  composite("CUSA", single_item("cusa")),
  composite("CUSL", multi_items("cusl_", 1:3))
)

# Create structural model ----
corp_rep_sm <- relationships(
  paths(from = c("QUAL", "PERF", "CSOR", "ATTR"), to = c("COMP", "LIKE")),
  paths(from = c("COMP", "LIKE"), to = c("CUSA", "CUSL")),
  paths(from = c("CUSA"),         to = c("CUSL"))
)

# Estimate the model ----
corp_pls <- estimate_pls(
  data = corp_rep_data,
  measurement_model = corp_rep_mm,
  structural_model  = corp_rep_sm,
  missing = mean_replacement,
  missing_value = "-99")

# ============================================================================
# FIMIX-PLS for a single K (assess_fimix)
# ============================================================================
# FIMIX-PLS uses EM-based latent class segmentation to uncover unobserved
# heterogeneity. Observations are probabilistically assigned to K segments,
# each with segment-specific structural path coefficients.

fimix_k2 <- assess_fimix(corp_pls, K = 2, nstart = 10, seed = 123)

# Print: segment proportions, path coefficients, fit indices
print(fimix_k2)

# Summary: detailed segment-level results
summary(fimix_k2)

# Visualize segment proportions and path coefficients
plot(fimix_k2)

# ============================================================================
# Compare across K values (assess_fimix_compare)
# ============================================================================
# Estimate FIMIX for multiple K values and compare information criteria
# (AIC, BIC, CAIC, etc.) to select the optimal number of segments.

fimix_compare <- assess_fimix_compare(corp_pls,
                                       K_range = 2:4,
                                       nstart = 10,
                                       seed = 123)

# Print: information criteria table across K values
print(fimix_compare)

# Plot: IC comparison across K
plot(fimix_compare)

### Accompanying Code for:
## Partial Least Squares Structural Equation Modeling (PLS-SEM) Using R - A Workbook (2026)
## Hair, J.F. (Jr), Hult, T.M., Ringle, C.M., Sarstedt, M., Danks, N.P., and Adler, S.

## Chapter 8: Mediation analysis

# Load the SEMinR and seminrExtras libraries
library(seminr)
library(seminrExtras)

# Load the data
corp_rep_data <- corp_rep_data

# Create measurement model ----
corp_rep_mm_ext <- constructs(
  composite("QUAL", multi_items("qual_", 1:8), weights = mode_B),
  composite("PERF", multi_items("perf_", 1:5), weights = mode_B),
  composite("CSOR", multi_items("csor_", 1:5), weights = mode_B),
  composite("ATTR", multi_items("attr_", 1:3), weights = mode_B),
  composite("COMP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  composite("CUSA", single_item("cusa")),
  composite("CUSL", multi_items("cusl_", 1:3))
)

# Create structural model ----
corp_rep_sm_ext <- relationships(
  paths(from = c("QUAL", "PERF", "CSOR", "ATTR"), to = c("COMP", "LIKE")),
  paths(from = c("COMP", "LIKE"), to = c("CUSA", "CUSL")),
  paths(from = c("CUSA"),         to = c("CUSL"))
)

# Estimate the model ----
corp_rep_pls_model_ext <- estimate_pls(
  data = corp_rep_data,
  measurement_model = corp_rep_mm_ext,
  structural_model  = corp_rep_sm_ext,
  missing = mean_replacement,
  missing_value = "-99")

# Extract the summary
summary_corp_rep_ext <- summary(corp_rep_pls_model_ext)

# Bootstrap the model ----
boot_corp_rep_ext <- bootstrap_model(
  seminr_model = corp_rep_pls_model_ext,
  nboot = 1000,
  cores = parallel::detectCores(),
  seed = 123)

# Summarize the results of the bootstrap
summary_boot_corp_rep_ext <- summary(boot_corp_rep_ext,
                                     alpha = 0.05)

# Inspect total indirect effects
summary_corp_rep_ext$total_indirect_effects

# Inspect indirect effects
specific_effect_significance(boot_corp_rep_ext,
                             from = "COMP",
                             through = "CUSA",
                             to = "CUSL",
                             alpha = 0.1)
specific_effect_significance(boot_corp_rep_ext,
                             from = "LIKE",
                             through = "CUSA",
                             to = "CUSL",
                             alpha = 0.05)

# Inspect the direct effects
summary_corp_rep_ext$paths

# Inspect the confidence intervals for direct effects
summary_boot_corp_rep_ext$bootstrapped_paths

# Calculate the sign of p1*p2*p3 for LIKE->CUSA->CUSL
summary_corp_rep_ext$paths["LIKE", "CUSL"] *
  summary_corp_rep_ext$paths["LIKE","CUSA"] *
  summary_corp_rep_ext$paths["CUSA","CUSL"]

# Calculate the sign of p1*p2*p3 for COMP->CUSA->CUSL
summary_corp_rep_ext$paths["COMP", "CUSL"] *
  summary_corp_rep_ext$paths["COMP","CUSA"] *
  summary_corp_rep_ext$paths["CUSA","CUSL"]

# Calculate the effect size v for LIKE->CUSA->CUSL
summary_corp_rep_ext$paths["LIKE","CUSA"]^2 *
  summary_corp_rep_ext$paths["CUSA","CUSL"]^2

# Calculate the effect size v for COMP->CUSA->CUSL
summary_corp_rep_ext$paths["COMP","CUSA"]^2 *
  summary_corp_rep_ext$paths["CUSA","CUSL"]^2

# Conduct a moderated mediation analysis ----
# Create the measurement model ----
corp_rep_mm_mod <- constructs(
  composite("QUAL", multi_items("qual_", 1:8), weights = mode_B),
  composite("PERF", multi_items("perf_", 1:5), weights = mode_B),
  composite("CSOR", multi_items("csor_", 1:5), weights = mode_B),
  composite("ATTR", multi_items("attr_", 1:3), weights = mode_B),
  composite("COMP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  composite("CUSA", single_item("cusa")),
  composite("SC", multi_items("switch_", 1:4)),
  composite("CUSL", multi_items("cusl_", 1:3)),
  interaction_term(iv = "CUSA", moderator = "SC", method = two_stage))

# Create the structural model ----
corp_rep_sm_mod <- relationships(
  paths(from = c("QUAL", "PERF", "CSOR", "ATTR"), to = c("COMP", "LIKE")),
  paths(from = c("COMP", "LIKE"), to = c("CUSA", "CUSL")),
  paths(from = c("CUSA", "SC", "CUSA*SC"), to = c("CUSL"))
)

# Estimate the new model with moderator ----
corp_rep_pls_model_mod <- estimate_pls(
  data = corp_rep_data,
  measurement_model = corp_rep_mm_mod,
  structural_model = corp_rep_sm_mod,
  missing = mean_replacement,
  missing_value = "-99"
)

# Extract the summary
sum_corp_rep_mod <- summary(corp_rep_pls_model_mod)

# Bootstrap the model ----
boot_corp_rep_mod <- bootstrap_model(
  seminr_model = corp_rep_pls_model_mod,
  seed = 12345,
  nboot = 1000)

# Index of moderated mediation ----
## Get the point estimate for the index of moderated mediation
# Compute p1*p5 for COMP
sum_corp_rep_mod$paths["COMP","CUSA"] *
  sum_corp_rep_mod$paths["CUSA*SC","CUSL"]

# Compute p1*p5 for LIKE
sum_corp_rep_mod$paths["LIKE","CUSA"] *
  sum_corp_rep_mod$paths["CUSA*SC","CUSL"]

## Get confidence intervals and p-values for the index of moderated mediation
# create a helper function to compute p-values
p_val <- function(x){
  2*min(mean(x<=0), mean(x>=0))
}

# Compute p1*p5 and 95% confidence interval for COMP
p1_p5_comp <- boot_corp_rep_mod$boot_path["COMP", "CUSA", ] *
  boot_corp_rep_mod$boot_path["CUSA*SC", "CUSL", ]
quantile(p1_p5_comp, probs = c(0.025, 0.975))

# Compute the p-value of path p1*p5 for COMP
p_val(p1_p5_comp)

# Compute p1*p5 and 95% confidence interval for LIKE
p1_p5_like <- boot_corp_rep_mod$boot_path["LIKE", "CUSA", ] *
  boot_corp_rep_mod$boot_path["CUSA*SC", "CUSL", ]
quantile(p1_p5_like, probs = c(0.025, 0.975))

# Compute the p-value of path p1*p5 for LIKE
p_val(p1_p5_like)

### Accompanying Code for:
## Partial Least Squares Structural Equation Modeling (PLS-SEM) Using R (Second Edition) - A Workbook (2026)
## Hair, J.F. (Jr), Hult, T.M., Ringle, C.M., Sarstedt, M., Danks, N.P., and Adler, S.

## Chapter 2: Introduction to R and R Studio

# Create a vector of integers
vector <- c(1, 2, 3, 4, 5)

# Searching for help using the ? operator
?read.csv

# Check all vignettes available in R
vignette()

# Load the SEMinR vignette
vignette("SEMinR")

# Check all demos available in R
demo()

# Load the SEMinR ECSI demo
demo("seminr-pls-ecsi")

# =============================================================================
# Demonstration of PCM (Predictive Contribution of the Mediator)
# Reference: Danks (2021), The DATA BASE for Advances in IS, 52(SI), 24-42.
# =============================================================================
# PCM evaluates the predictive contribution of a mediating construct by
# comparing predictions from the Direct Antecedents (DA) and Earliest
# Antecedents (EA) approaches on isolated mediation sub-models.
# =============================================================================

library(seminr)
library(seminrExtras)

# Load the MOBI customer satisfaction dataset bundled with seminr
data(mobi)

# --- Step 1: Specify and estimate a mediation model ---
# Image -> Satisfaction -> Loyalty (with direct path Image -> Loyalty)
mobi_mm <- constructs(
  composite("Image",        multi_items("IMAG", 1:5)),
  composite("Satisfaction", multi_items("CUSA", 1:3)),
  composite("Loyalty",      multi_items("CUSL", 1:3))
)
mobi_sm <- relationships(
  paths(from = "Image", to = "Satisfaction"),
  paths(from = "Satisfaction", to = "Loyalty"),
  paths(from = "Image", to = "Loyalty")
)
pls_model <- estimate_pls(mobi, mobi_mm, mobi_sm)
summary(pls_model)

# --- Step 2: Compute PCM ---
# PCM automatically detects the mediation path: Image -> Satisfaction -> Loyalty
pcm_result <- assess_pcm(pls_model,
                         target  = "Loyalty",
                         noFolds = 10,
                         reps    = 10)

# Print a concise overview with average PCM and classification
pcm_result

# Print a detailed summary with per-indicator PCM values
summary(pcm_result)

# Plot PCM barplot with threshold lines
plot(pcm_result)

# --- Step 3: Multi-mediator model ---
# Now use a more complex model with multiple mediation paths to Loyalty
mobi_mm_full <- constructs(
  composite("Image",        multi_items("IMAG", 1:5)),
  composite("Expectation",  multi_items("CUEX", 1:3)),
  composite("Value",        multi_items("PERV", 1:2)),
  composite("Satisfaction", multi_items("CUSA", 1:3)),
  composite("Loyalty",      multi_items("CUSL", 1:3))
)
mobi_sm_full <- relationships(
  paths(from = "Image",       to = c("Expectation", "Satisfaction", "Loyalty")),
  paths(from = "Expectation", to = c("Value", "Satisfaction")),
  paths(from = "Value",       to = "Satisfaction"),
  paths(from = "Satisfaction", to = "Loyalty")
)
pls_full <- estimate_pls(mobi, mobi_mm_full, mobi_sm_full)

# PCM automatically identifies all mediation paths:
# Image -> Satisfaction -> Loyalty
# Expectation -> Satisfaction -> Loyalty
# Value -> Satisfaction -> Loyalty
pcm_full <- assess_pcm(pls_full,
                        target  = "Loyalty",
                        noFolds = 10,
                        reps    = 10)

# Print results for all mediation paths
pcm_full

# Detailed indicator-level results
summary(pcm_full)

# Compare mediation paths visually
plot(pcm_full)

### Accompanying Code for:
## Partial Least Squares Structural Equation Modeling (PLS-SEM) Using R - A Workbook (2021)
## Hair, J.F. (Jr), Hult, T.M., Ringle, C.M., Sarstedt, M., Danks, N.P., and Ray, S.

## Chapter 7: Mediation analysis

# Load the SEMinR library
library(seminr)

# Load the data
corp_rep_data <- corp_rep_data

# Create measurement model ----
corp_rep_mm_ext <- constructs(
  composite("QUAL", multi_items("qual_", 1:8), weights = mode_B),
  composite("PERF", multi_items("perf_", 1:5), weights = mode_B),
  composite("CSOR", multi_items("csor_", 1:5), weights = mode_B),
  composite("ATTR", multi_items("attr_", 1:3), weights = mode_B),
  composite("COMP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  composite("CUSA", single_item("cusa")),
  composite("CUSL", multi_items("cusl_", 1:3))
)

# Create structural model ----
corp_rep_sm_ext <- relationships(
  paths(from = c("QUAL", "PERF", "CSOR", "ATTR"), to = c("COMP", "LIKE")),
  paths(from = c("COMP", "LIKE"), to = c("CUSA", "CUSL")),
  paths(from = c("CUSA"),         to = c("CUSL"))
)

# Estimate the model ----
corp_rep_pls_model_ext <- estimate_pls(
  data = corp_rep_data,
  measurement_model = corp_rep_mm_ext,
  structural_model  = corp_rep_sm_ext,
  missing = mean_replacement,
  missing_value = "-99")

# Extract the summary
summary_corp_rep_ext <- summary(corp_rep_pls_model_ext)

# Bootstrap the model ----
boot_corp_rep_ext <- bootstrap_model(
  seminr_model = corp_rep_pls_model_ext,
  nboot = 1000,
  cores = parallel::detectCores(),
  seed = 123)

# Summarize the results of the bootstrap
summary_boot_corp_rep_ext <- summary(boot_corp_rep_ext, alpha = 0.05)

# Inspect total indirect effects
summary_corp_rep_ext$total_indirect_effects

# Inspect indirect effects
specific_effect_significance(boot_corp_rep_ext, from = "COMP", through = "CUSA", to = "CUSL", alpha = 0.05)
specific_effect_significance(boot_corp_rep_ext, from = "LIKE", through = "CUSA", to = "CUSL", alpha = 0.05)

# Inspect the direct effects
summary_corp_rep_ext$paths

# Inspect the confidence intervals for direct effects
summary_boot_corp_rep_ext$bootstrapped_paths

# Calculate the sign of p1*p2*p3
summary_corp_rep_ext$paths["LIKE", "CUSL"] *
  summary_corp_rep_ext$paths["LIKE","CUSA"] *
  summary_corp_rep_ext$paths["CUSA","CUSL"]

# Congruence Coefficient Testing with SEMinR
library(seminr)
library(seminrExtras)

# Create measurement model ----
mobi_mm <- constructs(
  composite("Image",        multi_items("IMAG", 1:5)),
  composite("Expectation",  multi_items("CUEX", 1:3)),
  composite("Value",        multi_items("PERV", 1:2)),
  composite("Satisfaction", multi_items("CUSA", 1:3)),
  composite("Loyalty",      multi_items("CUSL", 1:3))
)

# Create structural model ----
mobi_sm <- relationships(
  paths(from = "Image",       to = c("Expectation", "Satisfaction", "Loyalty")),
  paths(from = "Expectation", to = c("Value", "Satisfaction")),
  paths(from = "Value",       to = "Satisfaction"),
  paths(from = "Satisfaction", to = "Loyalty")
)

# Estimate the model ----
mobi_pls <- estimate_pls(data = mobi,
                          measurement_model = mobi_mm,
                          structural_model  = mobi_sm)

# ============================================================================
# Congruence test (congruence_test)
# ============================================================================
# Tests whether the congruence coefficients between PLS composite weights
# and bootstrapped weights are significantly close to 1 (perfect congruence).

cong_result <- congruence_test(mobi_pls,
                                nboot = 2000,
                                seed = 123,
                                alpha = 0.05,
                                threshold = 1)

# Print results
print(cong_result)

# Summary with detailed bootstrap statistics
summary(cong_result)

# In this demo we seek to illustrate how users can check for potential bugs in their data and models.
# We consider each of potential mistakes and how they might be checked.
library(seminr)
library(seminrExtras)

## Using the `assess_syntax` argument in `estimate_pls()` ----
## Problem 1. Misspelled construct in MM ----
# intentionally misspell the construct `COMP` as `COP`
error_mm <- constructs(
  composite("COP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  composite("CUSA", single_item("cusa")),
  composite("CUSL", multi_items("cusl_", 1:3)))

simple_sm <- relationships(
  paths(from = c("COMP", "LIKE"), to = c("CUSA", "CUSL")),
  paths(from = c("CUSA"), to = c("CUSL")))

# Note that I have specified the argument `assess_syntax` as TRUE
# Now estimating the model should output an error
estimate_pls(data = corp_rep,
             measurement_model = error_mm,
             structural_model = simple_sm,
             assess_syntax = TRUE)

## Problem 2. Misspelled construct in SM ----
# correct measurement model
simple_mm <- constructs(
  composite("COMP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  composite("CUSA", single_item("cusa")),
  composite("CUSL", multi_items("cusl_", 1:3)))

# intentionally misspell the construct `COMP` as `COP`
error_sm <- relationships(
  paths(from = c("COP", "LIKE"), to = c("CUSA", "CUSL")),
  paths(from = c("CUSA"), to = c("CUSL")))

# Now estimating the model should output an error
estimate_pls(data = corp_rep,
             measurement_model = simple_mm,
             structural_model = error_sm,
             assess_syntax = TRUE)

## Problem 3. Missing construct in interaction ----
# Specify the measurement model with interaction
interaction_mm <- constructs(
  composite("COMP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  interaction_term("COMP", "LIKE", method = two_stage),
  composite("CUSL", multi_items("cusl_", 1:3)))

# Specify the structural model including the interaction,
# but omit the moderator (LIKE)
interaction_sm <- relationships(
  paths(from = c("COMP",  "COMP*LIKE"), to = "CUSL"))

# Now estimating the model should output an error
estimate_pls(data = corp_rep_data,
             measurement_model = interaction_mm,
             structural_model = interaction_sm,
             assess_syntax = TRUE)

## Problem 4. Incorrectly specified indicator names vs data
# intentionally misspell the indicators of COMP as co_1 through co_3
# rather than comp_1 through comp_3.
error_mm <- constructs(
  composite("COP", multi_items("co_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  composite("CUSA", single_item("cusa")),
  composite("CUSL", multi_items("cusl_", 1:3)))

estimate_pls(data = corp_rep_data,
             measurement_model = error_mm,
             structural_model = error_sm,
             assess_syntax = TRUE)

## But what do I do if I still have a bug?
# Plotting the measurement model and structural model pre-estimation can
# be a powerful tool for debugging your model.

# Create measurement model ----
simple_mm <- constructs(
  composite("COMP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  composite("CUSA", single_item("cusa")),
  composite("CUSL", multi_items("cusl_", 1:3)))

# Plot the conceptual measurement model
# eyeball the construct and indicator names and ensure they match the data
# and structural model.
plot(simple_mm, theme = seminr_theme_academic())

# Create structural model ----
simple_sm <- relationships(
  paths(from = c("COMP", "LIKE"), to = c("CUSA", "CUSL")),
  paths(from = c("CUSA"), to = c("CUSL")))

# Plot the conceptual structural model
# Eyeball the construct names and ensure they match the data and structural
# model, also check that the specified paths are correctly reflected.
plot(simple_sm, theme = seminr_theme_academic())
# Once you are satisfied that the models are correctly specified,
# you can proceed to estimate the model.

### Accompanying Code for:
## Partial Least Squares Structural Equation Modeling (PLS-SEM) Using R - A Workbook (2026)
## Hair, J.F. (Jr), Hult, T.M., Ringle, C.M., Sarstedt, M., Danks, N.P., and Adler, S.

## Chapter 7: Moderation analysis

# Load the SEMinR library
library(seminr)
library(seminrExtras)

# Load the data
corp_rep_data <- corp_rep_data

# Create measurement model ----
corp_rep_mm_mod <- constructs(
  composite("QUAL", multi_items("qual_", 1:8), weights = mode_B),
  composite("PERF", multi_items("perf_", 1:5), weights = mode_B),
  composite("CSOR", multi_items("csor_", 1:5), weights = mode_B),
  composite("ATTR", multi_items("attr_", 1:3), weights = mode_B),
  composite("COMP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  composite("CUSA", single_item("cusa")),
  composite("SC", multi_items("switch_", 1:4)),
  composite("CUSL", multi_items("cusl_", 1:3)),
  interaction_term(iv = "CUSA", moderator = "SC", method = two_stage, weights = mode_A))

# Create structural model ----
corp_rep_sm_mod <- relationships(
  paths(from = c("QUAL", "PERF", "CSOR", "ATTR"), to = c("COMP", "LIKE")),
  paths(from = c("COMP", "LIKE"), to = c("CUSA", "CUSL")),
  paths(from = c("CUSA", "SC", "CUSA*SC"), to = c("CUSL"))
)

# Estimate the model ----
corp_rep_pls_model_mod <- estimate_pls(
  data = corp_rep_data,
  measurement_model = corp_rep_mm_mod,
  structural_model  = corp_rep_sm_mod,
  missing = mean_replacement,
  missing_value = "-99")

# Extract the summary
sum_corp_rep_mod <- summary(corp_rep_pls_model_mod)

# Bootstrap the model ----
boot_corp_rep_mod <- bootstrap_model(
  seminr_model = corp_rep_pls_model_mod,
  nboot = 1000)

# Summarize the results of the bootstrap
sum_boot_corp_rep_mod <- summary(boot_corp_rep_mod, alpha = 0.05)

# Inspect the bootstrapped structural paths
sum_boot_corp_rep_mod$bootstrapped_paths

# Inspect the fsquare
sum_corp_rep_mod$fSquare

# Simple slope analysis plot
slope_analysis(
  moderated_model = corp_rep_pls_model_mod,
  dv = "CUSL",
  moderator = "SC",
  iv = "CUSA",
  leg_place = "bottomright")

#### Accompanying Code for:
## Partial Least Squares Structural Equation Modeling (PLS-SEM) Using R (Second Edition) - A Workbook (2026)
## Hair, J.F. (Jr), Hult, T.M., Ringle, C.M., Sarstedt, M., Danks, N.P., and Adler, S.

## Chapter 3: Introduction to SEMinR

# Download and install the SEMinR package
# You only need to do this once to equip RStudio with SEMinR and seminrExtras
#install.packages("seminr") # remove the # to execute this line of code
#install.packages("seminrExtras") # remove the # to execute this line of code

# Make the SEMinR library ready to use
# You must do this every time you restart RStudio and wish to use packages
library(seminr)
library(seminrExtras)

# Load the corporate reputation data
corp_rep_data <- corp_rep_data

# Show the first several rows of the corporate reputation data
head(corp_rep_data)

# Create measurement model ----
simple_mm <- constructs(
  composite("COMP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  composite("CUSA", single_item("cusa")),
  composite("CUSL", multi_items("cusl_", 1:3)))

# Create structural model ----
simple_sm <- relationships(
  paths(from = c("COMP", "LIKE"), to = c("CUSA", "CUSL")),
  paths(from = c("CUSA"), to = c("CUSL")))

# Estimate the model
corp_rep_simple_model <- estimate_pls(data = corp_rep_data,
  measurement_model = simple_mm,
  structural_model  = simple_sm,
  inner_weights = path_weighting,
  missing = mean_replacement,
  missing_value = "-99")

# Estimate the model with default settings
corp_rep_simple <- estimate_pls(data = corp_rep_data,
  measurement_model = simple_mm,
  structural_model  = simple_sm)

## Using the `assess_syntax` argument in `estimate_pls()`
# Specify the measurement model with a bug in the spelling of the construct
# `COMP`. Here it is `COP` and in the SM it is `COMP`.
error_mm <- constructs(
  composite("COP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  composite("CUSL", multi_items("cusl_", 1:3)),
  composite("CUSA", single_item("cusa")))

# Specify the structural model.
simple_sm <- relationships(
  paths(from = c("COMP", "LIKE"), to = c("CUSA", "CUSL")),
  paths(from = c("CUSA"), to = c("CUSL")))

# Note that I have specified the argument `assess_syntax` as TRUE
# Now estimating the model should output an error
estimate_pls(data = corp_rep_data,
             measurement_model = error_mm,
             structural_model = simple_sm,
             assess_syntax = TRUE)

## Plotting a model for debugging
# Plot the measurement model
plot(error_mm, theme = seminr_theme_academic())

# Plot the structural model
plot(simple_sm, theme = seminr_theme_academic())

# Summarize the model results
summary_simple_corp_rep <- summary(corp_rep_simple_model)

# Inspect the structural paths
summary_simple_corp_rep$paths

# Inspect the construct reliability metrics
summary_simple_corp_rep$reliability

# Bootstrap the model
boot_simple_corp_rep <- bootstrap_model(
  seminr_model = corp_rep_simple_model,
  nboot = 1000,
  cores = NULL,
  seed = 123)

# Store the summary of the bootstrapped model
sum_boot_simple_corp_rep <- summary(boot_simple_corp_rep)

# Inspect the bootstrapped structural paths
sum_boot_simple_corp_rep$bootstrapped_paths

# Inspect the bootstrapped indicator loadings
sum_boot_simple_corp_rep$bootstrapped_loadings

# Write the bootstrapped paths object to csv file
# and you will have to delete the '#' and then execute the below line of code:
# write.csv(x = sum_boot_simple_corp_rep$bootstrapped_loadings, file = "boot_loadings.csv")

# Generate the plot for exporting
plot(summary_simple_corp_rep$reliability)

### Accompanying Code for:
## Partial Least Squares Structural Equation Modeling (PLS-SEM) Using R - A Workbook 2nd Ed. (2026)
## Hair, J.F. (Jr), Hult, T.M., Ringle, C.M., Sarstedt, M., Danks, N.P., and Adler, S.

## Chapter 5: Evaluation of formative measurement models

# Load the SEMinR and seminrExtras libraries
library(seminr)
library(seminrExtras)

# Load the corporate repuation data
corp_rep_data <- corp_rep_data

# Create measurement model ----
corp_rep_mm_ext <- constructs(
  composite("QUAL", multi_items("qual_", 1:8), weights = mode_B),
  composite("PERF", multi_items("perf_", 1:5), weights = mode_B),
  composite("CSOR", multi_items("csor_", 1:5), weights = mode_B),
  composite("ATTR", multi_items("attr_", 1:3), weights = mode_B),
  composite("COMP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  composite("CUSA", single_item("cusa")),
  composite("CUSL", multi_items("cusl_", 1:3))
)

# Create structural model ----
corp_rep_sm_ext <- relationships(
  paths(from = c("QUAL", "PERF", "CSOR", "ATTR"), to = c("COMP", "LIKE")),
  paths(from = c("COMP", "LIKE"), to = c("CUSA", "CUSL")),
  paths(from = c("CUSA"),         to = c("CUSL"))
)

# Estimate the model ----
corp_rep_pls_model_ext <- estimate_pls(
  data = corp_rep_data,
  measurement_model = corp_rep_mm_ext,
  structural_model  = corp_rep_sm_ext,
  missing = mean_replacement,
  missing_value = "-99")

# Summarize the model results
summary_corp_rep_ext <- summary(corp_rep_pls_model_ext)

# Iterations to converge
summary_corp_rep_ext$iterations

# Bootstrap the model
boot_corp_rep_ext <- bootstrap_model(seminr_model = corp_rep_pls_model_ext,
                                     nboot = 1000,
                                     seed = 123)

# Store the summary of the bootstrapped model
sum_boot_corp_rep_ext <- summary(boot_corp_rep_ext, alpha = 0.10)

# Inspect the indicator loadings
summary_corp_rep_ext$loadings

# Inspect the indicator reliability
summary_corp_rep_ext$loadings^2

# Inspect the internal consistency and reliability
summary_corp_rep_ext$reliability

# HTMT Ratio
summary_corp_rep_ext$validity$htmt

# Extract the bootstrapped HTMT
sum_boot_corp_rep_ext$bootstrapped_HTMT

# Redundancy analysis ----
# ATTR ----
# Create measurement model
ATTR_redundancy_mm <- constructs(
  composite("ATTR_F", multi_items("attr_", 1:3), weights = mode_B),
  composite("ATTR_G", single_item("attr_global"))
)

# Create structural model
ATTR_redundancy_sm <- relationships(
  paths(from = c("ATTR_F"), to = c("ATTR_G"))
)

# Estimate the model
ATTR_redundancy_pls_model <- estimate_pls(data = corp_rep_data,
                                          measurement_model = ATTR_redundancy_mm,
                                          structural_model  = ATTR_redundancy_sm,
                                          missing = mean_replacement,
                                          missing_value = "-99")

# Summarize the model
sum_ATTR_red_model <- summary(ATTR_redundancy_pls_model)

# CSOR ----
# Create measurement model
CSOR_redundancy_mm <- constructs(
  composite("CSOR_F", multi_items("csor_", 1:5), weights = mode_B),
  composite("CSOR_G", single_item("csor_global"))
)

# Create structural model
CSOR_redundancy_sm <- relationships(
  paths(from = c("CSOR_F"), to = c("CSOR_G"))
)

# Estimate the model
CSOR_redundancy_pls_model <- estimate_pls(data = corp_rep_data,
                                          measurement_model = CSOR_redundancy_mm,
                                          structural_model  = CSOR_redundancy_sm,
                                          missing = mean_replacement,
                                          missing_value = "-99")

# Summarize the model
sum_CSOR_red_model <- summary(CSOR_redundancy_pls_model)

# PERF ----
# Create measurement model
PERF_redundancy_mm <- constructs(
  composite("PERF_F", multi_items("perf_", 1:5), weights = mode_B),
  composite("PERF_G", single_item("perf_global"))
)

# Create structural model
PERF_redundancy_sm <- relationships(
  paths(from = c("PERF_F"), to = c("PERF_G"))
)

# Estimate the model
PERF_redundancy_pls_model <- estimate_pls(data = corp_rep_data,
                                          measurement_model = PERF_redundancy_mm,
                                          structural_model  = PERF_redundancy_sm,
                                          missing = mean_replacement,
                                          missing_value = "-99")

# Summarize the model
sum_PERF_red_model <- summary(PERF_redundancy_pls_model)

# QUAL ----
# Create measurement model
QUAL_redundancy_mm <- constructs(
  composite("QUAL_F", multi_items("qual_", 1:8), weights = mode_B),
  composite("QUAL_G", single_item("qual_global"))
)

# Create structural model
QUAL_redundancy_sm <- relationships(
  paths(from = c("QUAL_F"), to = c("QUAL_G"))
)

# Estimate the model
QUAL_redundancy_pls_model <- estimate_pls(data = corp_rep_data,
                                          measurement_model = QUAL_redundancy_mm,
                                          structural_model  = QUAL_redundancy_sm,
                                          missing = mean_replacement,
                                          missing_value = "-99")

# Summarize the model
sum_QUAL_red_model <- summary(QUAL_redundancy_pls_model)

# Check the path coefficients for convergent validity
sum_ATTR_red_model$paths
sum_CSOR_red_model$paths
sum_PERF_red_model$paths
sum_QUAL_red_model$paths

# Collinearity analysis ----
summary_corp_rep_ext$validity$vif_items

# Bootstrap the model ----
# seminr_model is the SEMinR model to be bootstrapped
# nboot is the number of bootstrap iterations to run
# cores is the number of cpu cores to use in multicore bootstrapping
# parallel::detectCores() allows for using the maximum cores on your device
# seed is the seed to be used for making bootstrap replicable
boot_corp_rep_ext <- bootstrap_model(
  seminr_model = corp_rep_pls_model_ext,
  nboot = 1000,
  cores = parallel::detectCores(),
  seed = 123)

# Summarize the results of the bootstrap
# alpha sets the specified level for significance, i.e. 0.05
sum_boot_corp_rep_ext <- summary(boot_corp_rep_ext, alpha = 0.05)

# Inspect the bootstrapping results for outer weights
sum_boot_corp_rep_ext$bootstrapped_weights

# Inspect the bootstrapping results for the outer loadings
sum_boot_corp_rep_ext$bootstrapped_loadings

# Confirmatory Tetrad Analysis (CTA-PLS) with SEMinR
library(seminr)
library(seminrExtras)

# Create measurement model ----
mobi_mm <- constructs(
  composite("Image",        multi_items("IMAG", 1:5)),
  composite("Expectation",  multi_items("CUEX", 1:3)),
  composite("Value",        multi_items("PERV", 1:2)),
  composite("Satisfaction", multi_items("CUSA", 1:3)),
  composite("Loyalty",      multi_items("CUSL", 1:3))
)

# Create structural model ----
mobi_sm <- relationships(
  paths(from = "Image",       to = c("Expectation", "Satisfaction", "Loyalty")),
  paths(from = "Expectation", to = c("Value", "Satisfaction")),
  paths(from = "Value",       to = "Satisfaction"),
  paths(from = "Satisfaction", to = "Loyalty")
)

# Estimate the model ----
mobi_pls <- estimate_pls(data = mobi,
                          measurement_model = mobi_mm,
                          structural_model  = mobi_sm)

# ============================================================================
# CTA-PLS with borrowing (default)
# ============================================================================
# CTA-PLS tests whether a construct's measurement model is consistent with
# a reflective specification. Under the null, all vanishing tetrads = 0.
#
# With borrow = TRUE (default), constructs with 2-3 indicators borrow from
# structurally connected constructs to form testable 4-tuples.

cta_result <- assess_cta(mobi_pls, nboot = 5000, seed = 123)

# Per-construct summary: mode, indicators, tetrads tested, verdict
print(cta_result)

# Detailed results: individual tetrads, t-values, CIs, adjusted p-values
summary(cta_result)

# Visualize adjusted p-values per construct
plot(cta_result)

# ============================================================================
# CTA-PLS without borrowing
# ============================================================================
# Only constructs with >= 4 indicators are tested

cta_no_borrow <- assess_cta(mobi_pls, nboot = 5000, borrow = FALSE, seed = 123)
print(cta_no_borrow)

# CVPAT applied to the Corporate Reputation Model and dataset
library(seminr)
library(seminrExtras)

# Create measurement model ----
corp_rep_mm_ext <- constructs(
  composite("QUAL", multi_items("qual_", 1:8), weights = mode_B),
  composite("PERF", multi_items("perf_", 1:5), weights = mode_B),
  composite("CSOR", multi_items("csor_", 1:5), weights = mode_B),
  composite("ATTR", multi_items("attr_", 1:3), weights = mode_B),
  composite("COMP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  composite("CUSA", single_item("cusa")),
  composite("CUSL", multi_items("cusl_", 1:3))
)

alt_mm <- constructs(
  composite("QUAL", multi_items("qual_", 1:8), weights = mode_B),
  composite("PERF", multi_items("perf_", 1:5), weights = mode_B),
  composite("CSOR", multi_items("csor_", 1:5), weights = mode_B),
  composite("ATTR", multi_items("attr_", 1:3), weights = mode_B),
  composite("COMP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  composite("CUSA", single_item("cusa")),
  composite("CUSL", multi_items("cusl_", 1:3))
)

# Create structural model ----
corp_rep_sm_ext <- relationships(
  paths(from = c("QUAL", "PERF", "CSOR", "ATTR"), to = c("COMP", "LIKE")),
  paths(from = c("COMP", "LIKE"), to = c("CUSA", "CUSL")),
  paths(from = c("CUSA"),         to = c("CUSL"))
)
alt_sm <- relationships(
  paths(from = c("QUAL", "PERF", "CSOR", "ATTR"), to = c("COMP", "LIKE")),
  paths(from = c("COMP", "LIKE"), to = c("CUSA")),
  paths(from = c("CUSA"),         to = c("CUSL"))
)


# Estimate the models ----
established_model <- estimate_pls(
  data = corp_rep_data,
  measurement_model = corp_rep_mm_ext,
  structural_model  = corp_rep_sm_ext,
  missing = mean_replacement,
  missing_value = "-99")

competing_model <- estimate_pls(
  data = corp_rep_data,
  measurement_model = alt_mm,
  structural_model  = alt_sm,
  missing = mean_replacement,
  missing_value = "-99")

# Function to compare the Loss of two models
compare_results <- assess_cvpat_compare(established_model = established_model,
                                        alternative_model = competing_model,
                                        testtype = "two.sided",
                                        nboot = 2000,
                                        technique = predict_DA,
                                        seed = 123,
                                        noFolds = 10,
                                        reps = 10,
                                        cores = NULL)

print(compare_results,
      digits = 3)

# Assess the base model ----
assess_results <- assess_cvpat(established_model,
                               testtype = "two.sided",
                               nboot = 2000,
                               seed = 123,
                               technique = predict_DA,
                               noFolds = 10,
                               reps = 10)
print(assess_results$CVPAT_compare_LM,
      digits = 3)
print(assess_results$CVPAT_compare_IA,
      digits = 3)

# NCA (Necessary Condition Analysis) applied to a PLS-SEM model
library(seminr)
library(seminrExtras)

# Create measurement model ----
mobi_mm <- constructs(
  composite("Image",        multi_items("IMAG", 1:5)),
  composite("Value",        multi_items("PERV", 1:2)),
  composite("Satisfaction", multi_items("CUSA", 1:3)),
  composite("Loyalty",      multi_items("CUSL", 1:3))
)

# Create structural model ----
mobi_sm <- relationships(
  paths(from = c("Image", "Value"), to = "Satisfaction"),
  paths(from = "Satisfaction", to = "Loyalty")
)

# Estimate the model ----
mobi_pls <- estimate_pls(
  data = mobi,
  measurement_model = mobi_mm,
  structural_model  = mobi_sm
)

# Run NCA on Satisfaction ----
nca_sat <- assess_nca(mobi_pls,
                       target = "Satisfaction",
                       test.rep = 1000,
                       seed = 123)

# Print effect sizes and significance
print(nca_sat)

# Full summary with bottleneck tables
summary(nca_sat)

# Effect size bar plot
plot(nca_sat, type = "effects")

# Scatter plots with ceiling lines
plot(nca_sat, type = "scatter")

# Run NCA on Loyalty ----
nca_loy <- assess_nca(mobi_pls,
                       target = "Loyalty",
                       test.rep = 1000,
                       seed = 123)

print(nca_loy)
summary(nca_loy)

# ============================================================================
# NCA-ESSE: Effect Size Sensitivity Extension (Becker et al., 2026)
# ============================================================================
# NCA-ESSE assesses how robust NCA results are to extreme response patterns.
# It varies the ECDF threshold, removing upper-left observations, and
# compares empirical effect size changes against a uniform benchmark.

# Run NCA-ESSE on Satisfaction ----
esse_sat <- assess_nca_esse(mobi_pls,
                             target = "Satisfaction",
                             thresholds = seq(0, 0.05, by = 0.005),
                             seed = 123)

# Print: shows empirical effect sizes, benchmark, and sensitivity (delta)
print(esse_sat)

# Summary: Table A2-style output per predictor
summary(esse_sat)

# Sensitivity plot (Fig. 4 in Becker et al., 2026):
# Empirical vs benchmark effect sizes across ECDF thresholds
plot(esse_sat, type = "sensitivity")

# Difference plot (Fig. 6 in Becker et al., 2026):
# Incremental empirical-benchmark difference at each step
plot(esse_sat, type = "difference")

### Accompanying Code for:
## Partial Least Squares Structural Equation Modeling (PLS-SEM) Using R - A Workbook (2026)
## Hair, J.F. (Jr), Hult, T.M., Ringle, C.M., Sarstedt, M., Danks, N.P., and Adler, S.

## Chapter 4: Evaluation of reflective measurement models
# This analysis requires the psych and paran packages (refer Section 2.6)
# Load the SEMinR and seminrExtras libraries
library(seminr)
library(seminrExtras)

# Load the data ----
corp_rep_data <- corp_rep_data

# Create measurement model ----
corp_rep_mm <- constructs(
  composite("COMP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  composite("CUSA", single_item("cusa")),
  composite("CUSL", multi_items("cusl_", 1:3)))

# Create structural model ----
corp_rep_sm <- relationships(
  paths(from = c("COMP", "LIKE"), to = c("CUSA", "CUSL")),
  paths(from = c("CUSA"), to = c("CUSL")))

# Estimate the model
corp_rep_pls_model <- estimate_pls(
  data = corp_rep_data,
  measurement_model = corp_rep_mm,
  structural_model  = corp_rep_sm,
  missing = mean_replacement,
  missing_value = "-99")

# Summarize the model results
summary_corp_rep <- summary(corp_rep_pls_model)

# Inspect iterations
summary_corp_rep$iterations

# Evaluate the undimensionality of reflective constructs
## Set up the list of reflective constructs
con.list <- list(COMP = multi_items("comp_", 1:3),
                 LIKE = multi_items("like_", 1:3),
                 CUSL = multi_items("cusl_", 1:3))

## Collect the cleaned data from the seminr_model object
## (The missing values have already been treated/imputed)
dim_data <- corp_rep_pls_model$data

## Set up helper functions (using psych package)
est_dim <- function(construct, data) {
  ret <- psych::iclust(data[,construct],
                       nclusters = 1)$beta
}

## Test for unidimensionality
test_unidim <- function(con.list, sum.model, dim_data) {
  dims_results <- lapply(con.list, function(x) est_dim(x, dim_data))
  dims_results <- unlist(dims_results)
  names(dims_results) <- names(con.list)
  alphas <- sum.model$reliability[names(con.list),"alpha"]
  dims_results <- data.frame(
    "Cronbach Alpha" = alphas,
    "Revelle Beta" = dims_results)
  return(dims_results)
}

## Principal Components Analysis Test
PCA_results <- matrix(c(base::eigen(cor(dim_data[,con.list[["COMP"]]]))$values,
                        base::eigen(cor(dim_data[,con.list[["CUSL"]]]))$values,
                        base::eigen(cor(dim_data[,con.list[["LIKE"]]]))$values),
                        nrow = 3, byrow = TRUE,
                        dimnames = list(c("COMP", "CUSL", "LIKE"),
                                        c("PC1 EV", "PC2 EV", "PC3 EV")))

# Parallel analysis (using paran package)
paran_comp <- paran::paran(dim_data[,con.list[["COMP"]]],
                           iterations=1000,
                           centile=95)
paran_cusl <- paran::paran(dim_data[,con.list[["CUSL"]]],
                           iterations=1000,
                           centile=95)
paran_like <- paran::paran(dim_data[,con.list[["LIKE"]]],
                           iterations=1000,
                           centile=95)
para_results <- matrix(c(paran_comp$AdjEv,paran_cusl$AdjEv,paran_like$AdjEv),
                       nrow = 3, byrow = TRUE,
                       dimnames = list(c("COMP", "CUSL", "LIKE"),
                                      c("PC1 EV", "PC2 EV", "PC3 EV")))

## Display the results of the unidimensionality tests
round(test_unidim(con.list, summary_corp_rep, dim_data),2)
round(PCA_results,2)
round(para_results,2)

# Inspect the outer loadings
summary_corp_rep$loadings

# Inspect the indicator reliability
summary_corp_rep$loadings^2

# Inspect the internal consistency and reliability
summary_corp_rep$reliability

# Plot the reliabilities of constructs
plot(summary_corp_rep$reliability)

# HTMT criterion
summary_corp_rep$validity$htmt

# Bootstrap the model
boot_corp_rep <- bootstrap_model(seminr_model = corp_rep_pls_model,
                                 nboot = 1000)

# Store the summary of the bootstrapped model
sum_boot_corp_rep <- summary(boot_corp_rep, alpha = 0.10)

# Extract the bootstrapped HTMT
sum_boot_corp_rep$bootstrapped_HTMT

# Calculate the congruence coefficient rc
congruence_test(corp_rep_pls_model, alpha = 0.10)$results

### Accompanying Code for:
## Partial Least Squares Structural Equation Modeling (PLS-SEM) Using R (Second Edition) - A Workbook (2026)
## Hair, J.F. (Jr), Hult, T.M., Ringle, C.M., Sarstedt, M., Danks, N.P., and Adler, S.

## Chapter 2: Introduction to R and R Studio
# Create a vector of integers
vector <- c(1, 2, 3, 4, 5)

# Install the SEMinR package
install.packages(pkgs = "seminr")

# Load the SEMinR package into the environment
library(seminr)

# Install the textbook packages from CRAN
install.packages(pkgs = c("learnr", "paran", "psych", "seminrExtras"))

# Load the learnr package into the environment
library(learnr)

# Begin learning with learnr
run_tutorial()

# Searching for help using the ? operator
?read.csv

# Check all vignettes available in R
vignette()

# Load the vignette for the reshape() function from the stats package
vignette("reshape", "stats")

# Check all demos available in R
demo()

# Load the SEMinR ECSI demo
demo("seminr-pls-ecsi")

# Composite Overfit Analysis (COA) with SEMinR
library(seminr)
library(seminrExtras)

# Create measurement model ----
corp_mm <- constructs(
  composite("COMP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  composite("CUSA", single_item("cusa")),
  composite("CUSL", multi_items("cusl_", 1:3))
)

# Create structural model ----
corp_sm <- relationships(
  paths(from = c("COMP", "LIKE"), to = "CUSA"),
  paths(from = "CUSA", to = "CUSL")
)

# Estimate the model ----
corp_model <- estimate_pls(
  data = corp_rep_data,
  measurement_model = corp_mm,
  structural_model  = corp_sm,
  missing = mean_replacement,
  missing_value = "-99")

# ============================================================================
# Full COA pipeline (assess_coa)
# ============================================================================
# COA detects observation-level overfitting by computing predictive deviance,
# growing a decision tree to identify deviant subgroups, and analyzing
# parameter instability.

coa_result <- assess_coa(corp_model,
                          focal_construct = "CUSL",
                          noFolds = 10, reps = 1, cores = 1,
                          seed = 123)

# Print summary
print(coa_result)
summary(coa_result)

# Predictive deviance distribution
plot(coa_result, type = "pd")

# Decision tree
plot(coa_result, type = "tree")

# Deviant group highlights
plot(coa_result, type = "groups")

# ============================================================================
# Step-by-step COA (individual functions)
# ============================================================================

# Step 1: Compute predictive deviance
pd <- predictive_deviance(corp_model,
                           focal_construct = "CUSL",
                           noFolds = 10, reps = 1, cores = 1,
                           seed = 123)

# Step 2: Identify deviant groups via decision tree
tree <- deviance_tree(pd, deviance_bounds = c(0.025, 0.975))

# Step 3: Analyze parameter instability
# unstable_params() accepts the tree object directly (or tree$deviant_groups)
instab <- unstable_params(corp_model, tree, params = "path_coef")
print(instab)

# Extract decision rules for all deviant groups
group_rules(tree)

# Show competing splits at all group root nodes
competes(tree)

# You can also target a specific group:
# group_rules("A", coa_result)
# competes(tree$group_roots[["A"]], tree)

# Combined Importance-Performance Map Analysis (cIPMA) with SEMinR
library(seminr)
library(seminrExtras)

# Create measurement model ----
mobi_mm <- constructs(
  composite("Image",        multi_items("IMAG", 1:5)),
  composite("Expectation",  multi_items("CUEX", 1:3)),
  composite("Value",        multi_items("PERV", 1:2)),
  composite("Satisfaction", multi_items("CUSA", 1:3)),
  composite("Loyalty",      multi_items("CUSL", 1:3))
)

# Create structural model ----
mobi_sm <- relationships(
  paths(from = "Image",        to = c("Expectation", "Satisfaction", "Loyalty")),
  paths(from = "Expectation",  to = c("Value", "Satisfaction")),
  paths(from = "Value",        to = "Satisfaction"),
  paths(from = "Satisfaction", to = "Loyalty")
)

# Estimate the model ----
mobi_pls <- estimate_pls(data = mobi,
                          measurement_model = mobi_mm,
                          structural_model  = mobi_sm)

# ============================================================================
# IPMA-only analysis (assess_ipma)
# ============================================================================
# IPMA computes importance (unstandardized total effects) and performance
# (rescaled 0-100 construct scores) for each predictor of a target.

ipma_result <- assess_ipma(mobi_pls,
                            target = "Loyalty",
                            scale_min = 1,
                            scale_max = 10)

print(ipma_result)
summary(ipma_result)

# Standard IPMA map
plot(ipma_result, type = "ipma")

# ============================================================================
# cIPMA: Combined IPMA + NCA (assess_cipma)
# ============================================================================
# cIPMA extends IPMA by integrating NCA to identify which constructs
# are necessary conditions for the target outcome.

cipma_result <- assess_cipma(mobi_pls,
                              target = "Loyalty",
                              scale_min = 1,
                              scale_max = 10,
                              nca_test.rep = 1000,
                              seed = 123)

# Print: importance, performance, necessity classification
print(cipma_result)

# Summary: full IPMA + NCA details
summary(cipma_result)

# cIPMA map (importance vs. performance, with NCA overlay)
plot(cipma_result, type = "cipma")

# Standard IPMA map (without NCA distinction)
plot(cipma_result, type = "ipma")

# Use standardized total effects for importance axis
plot(cipma_result, importance_metric = "standardized")

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(seminr)
library(seminrExtras)

# set the theme for plotting output
seminr_theme_set(seminr_theme_academic())

# Create measurement model ----
corp_rep_mm_ext <- constructs(
  composite("QUAL", multi_items("qual_", 1:8), weights = mode_B),
  composite("PERF", multi_items("perf_", 1:5), weights = mode_B),
  composite("CSOR", multi_items("csor_", 1:5), weights = mode_B),
  composite("ATTR", multi_items("attr_", 1:3), weights = mode_B),
  composite("COMP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  composite("CUSA", single_item("cusa")),
  composite("CUSL", multi_items("cusl_", 1:3))
)

alt_mm <- constructs(
  composite("QUAL", multi_items("qual_", 1:8), weights = mode_B),
  composite("PERF", multi_items("perf_", 1:5), weights = mode_B),
  composite("CSOR", multi_items("csor_", 1:5), weights = mode_B),
  composite("ATTR", multi_items("attr_", 1:3), weights = mode_B),
  composite("COMP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  composite("CUSA", single_item("cusa")),
  composite("CUSL", multi_items("cusl_", 1:3))
)

# Create structural model ----
corp_rep_sm_ext <- relationships(
  paths(from = c("QUAL", "PERF", "CSOR", "ATTR"), to = c("COMP", "LIKE")),
  paths(from = c("COMP", "LIKE"), to = c("CUSA", "CUSL")),
  paths(from = c("CUSA"),         to = c("CUSL"))
)
alt_sm <- relationships(
  paths(from = c("QUAL", "PERF", "CSOR", "ATTR"), to = c("COMP", "LIKE")),
  paths(from = c("COMP", "LIKE"), to = c("CUSA")),
  paths(from = c("CUSA"),         to = c("CUSL"))
)


# Estimate the models ----
established_model <- estimate_pls(
  data = corp_rep_data,
  measurement_model = corp_rep_mm_ext,
  structural_model  = corp_rep_sm_ext,
  missing = mean_replacement,
  missing_value = "-99")

competing_model <- estimate_pls(
  data = corp_rep_data,
  measurement_model = alt_mm,
  structural_model  = alt_sm,
  missing = mean_replacement,
  missing_value = "-99")



## ----fig.show='hold'----------------------------------------------------------
plot(established_model)
plot(competing_model)

## ----eval=FALSE, echo=TRUE----------------------------------------------------
# 
# # Create measurement model ----
# corp_rep_mm_ext <- constructs(
#   composite("QUAL", multi_items("qual_", 1:8), weights = mode_B),
#   composite("PERF", multi_items("perf_", 1:5), weights = mode_B),
#   composite("CSOR", multi_items("csor_", 1:5), weights = mode_B),
#   composite("ATTR", multi_items("attr_", 1:3), weights = mode_B),
#   composite("COMP", multi_items("comp_", 1:3)),
#   composite("LIKE", multi_items("like_", 1:3)),
#   composite("CUSA", single_item("cusa")),
#   composite("CUSL", multi_items("cusl_", 1:3))
# )
# 
# alt_mm <- constructs(
#   composite("QUAL", multi_items("qual_", 1:8), weights = mode_B),
#   composite("PERF", multi_items("perf_", 1:5), weights = mode_B),
#   composite("CSOR", multi_items("csor_", 1:5), weights = mode_B),
#   composite("ATTR", multi_items("attr_", 1:3), weights = mode_B),
#   composite("COMP", multi_items("comp_", 1:3)),
#   composite("LIKE", multi_items("like_", 1:3)),
#   composite("CUSA", single_item("cusa")),
#   composite("CUSL", multi_items("cusl_", 1:3))
# )
# 
# # Create structural model ----
# corp_rep_sm_ext <- relationships(
#   paths(from = c("QUAL", "PERF", "CSOR", "ATTR"), to = c("COMP", "LIKE")),
#   paths(from = c("COMP", "LIKE"), to = c("CUSA", "CUSL")),
#   paths(from = c("CUSA"),         to = c("CUSL"))
# )
# alt_sm <- relationships(
#   paths(from = c("QUAL", "PERF", "CSOR", "ATTR"), to = c("COMP", "LIKE")),
#   paths(from = c("COMP", "LIKE"), to = c("CUSA")),
#   paths(from = c("CUSA"),         to = c("CUSL"))
# )
# 
# 
# # Estimate the models ----
# established_model <- estimate_pls(
#   data = corp_rep_data,
#   measurement_model = corp_rep_mm_ext,
#   structural_model  = corp_rep_sm_ext,
#   missing = mean_replacement,
#   missing_value = "-99")
# 
# competing_model <- estimate_pls(
#   data = corp_rep_data,
#   measurement_model = alt_mm,
#   structural_model  = alt_sm,
#   missing = mean_replacement,
#   missing_value = "-99")
# 
# # Function to compare the Loss of two models
# compare_results <- assess_cvpat_compare(established_model = established_model,
#                                         alternative_model = competing_model,
#                                         testtype = "two.sided",
#                                         nboot = 2000,
#                                         technique = predict_DA,
#                                         seed = 123,
#                                         noFolds = 10,
#                                         reps = 10,
#                                         cores = 1)
# 
# 
# print(compare_results,
#       digits = 3)
# 
# # Assess the base model ----
# assess_results <- assess_cvpat(established_model,
#                                seed = 123,
#                                cores = 1)
# print(assess_results$CVPAT_compare_LM,
#       digits = 3)
# print(assess_results$CVPAT_compare_IA,
#       digits = 3)
# 

## ----echo=TRUE----------------------------------------------------------------
# Assess the base model ----
assess_results <- assess_cvpat(established_model,
                               seed = 123, 
                               cores = 1)
print(assess_results$CVPAT_compare_LM,
      digits = 3)
print(assess_results$CVPAT_compare_IA,
      digits = 3)

## ----echo=TRUE----------------------------------------------------------------
# Function to compare the Loss of two models
compare_results <- assess_cvpat_compare(established_model = established_model,
                                        alternative_model = competing_model,
                                        testtype = "two.sided",
                                        nboot = 2000,
                                        technique = predict_DA,
                                        seed = 123,
                                        noFolds = 10,
                                        reps = 10,
                                        cores = 1)

print(compare_results,
      digits = 3)

## ----eval=FALSE, echo=TRUE----------------------------------------------------
# library(seminr)
# library(seminrExtras)
# 
# # Estimate a model
# mobi_mm <- constructs(
#   composite("Image",        multi_items("IMAG", 1:5)),
#   composite("Expectation",  multi_items("CUEX", 1:3)),
#   composite("Value",        multi_items("PERV", 1:2)),
#   composite("Satisfaction", multi_items("CUSA", 1:3)),
#   composite("Loyalty",      multi_items("CUSL", 1:3))
# )
# 
# mobi_sm <- relationships(
#   paths(from = "Image",        to = c("Expectation", "Satisfaction", "Loyalty")),
#   paths(from = "Expectation",  to = c("Value", "Satisfaction")),
#   paths(from = "Value",        to = "Satisfaction"),
#   paths(from = "Satisfaction", to = "Loyalty")
# )
# 
# mobi_pls <- estimate_pls(data = mobi,
#                           measurement_model = mobi_mm,
#                           structural_model  = mobi_sm)
# 
# # Run cIPMA (IPMA + NCA)
# cipma_result <- assess_cipma(mobi_pls,
#                               target = "Loyalty",
#                               scale_min = 1,
#                               scale_max = 10,
#                               nca_test.rep = 1000,
#                               seed = 123)
# 
# # View results
# print(cipma_result)
# summary(cipma_result)
# 
# # cIPMA map (importance vs. performance, with NCA overlay)
# plot(cipma_result, type = "cipma")
# 
# # Standard IPMA map (without NCA distinction)
# plot(cipma_result, type = "ipma")
# 
# # Use standardized total effects for importance axis
# plot(cipma_result, importance_metric = "standardized")

## ----eval=FALSE, echo=TRUE----------------------------------------------------
# library(seminr)
# library(seminrExtras)
# 
# # Estimate a simple model
# mobi_mm <- constructs(
#   composite("Image",        multi_items("IMAG", 1:5)),
#   composite("Value",        multi_items("PERV", 1:2)),
#   composite("Satisfaction", multi_items("CUSA", 1:3)),
#   composite("Loyalty",      multi_items("CUSL", 1:3))
# )
# 
# mobi_sm <- relationships(
#   paths(from = c("Image", "Value"), to = "Satisfaction"),
#   paths(from = "Satisfaction", to = "Loyalty")
# )
# 
# mobi_pls <- estimate_pls(data = mobi,
#                           measurement_model = mobi_mm,
#                           structural_model  = mobi_sm)
# 
# # Run NCA -- predictors auto-detected from structural model
# nca_result <- assess_nca(mobi_pls,
#                           target = "Satisfaction",
#                           test.rep = 1000,
#                           seed = 123)
# 
# # Effect sizes and significance
# print(nca_result)
# 
# # Full summary with bottleneck tables
# summary(nca_result)
# 
# # Visualize
# plot(nca_result, type = "effects")   # Effect size bar chart
# plot(nca_result, type = "scatter")   # Ceiling line scatter plots

## ----eval=FALSE, echo=TRUE----------------------------------------------------
# # Run NCA-ESSE on the same model
# esse_result <- assess_nca_esse(mobi_pls,
#                                 target = "Satisfaction",
#                                 thresholds = seq(0, 0.05, by = 0.005),
#                                 seed = 123)
# 
# # View results
# print(esse_result)
# 
# # Summary tables (Table A2 style)
# summary(esse_result)
# 
# # Sensitivity plot (Fig. 4 in Becker et al., 2026)
# plot(esse_result, type = "sensitivity")
# 
# # Difference plot (Fig. 6 in Becker et al., 2026)
# plot(esse_result, type = "difference")

## ----eval=FALSE, echo=TRUE----------------------------------------------------
# library(seminr)
# library(seminrExtras)
# 
# # Estimate a model
# corp_mm <- constructs(
#   composite("COMP", multi_items("comp_", 1:3)),
#   composite("LIKE", multi_items("like_", 1:3)),
#   composite("CUSA", single_item("cusa")),
#   composite("CUSL", multi_items("cusl_", 1:3))
# )
# 
# corp_sm <- relationships(
#   paths(from = c("COMP", "LIKE"), to = "CUSA"),
#   paths(from = "CUSA", to = "CUSL")
# )
# 
# corp_model <- estimate_pls(
#   data = corp_rep_data,
#   measurement_model = corp_mm,
#   structural_model  = corp_sm,
#   missing = mean_replacement,
#   missing_value = "-99")
# 
# # Run full COA pipeline
# coa_result <- assess_coa(corp_model,
#                           focal_construct = "CUSL",
#                           noFolds = 10, reps = 1, cores = 1,
#                           seed = 123)
# 
# # Print results
# print(coa_result)
# summary(coa_result)
# 
# # Visualize
# plot(coa_result, type = "pd")      # Predictive deviance distribution
# plot(coa_result, type = "tree")    # Decision tree
# plot(coa_result, type = "groups")  # Deviant group highlights

## ----eval=FALSE, echo=TRUE----------------------------------------------------
# library(seminr)
# library(seminrExtras)
# 
# mobi_mm <- constructs(
#   composite("Image",        multi_items("IMAG", 1:5)),
#   composite("Expectation",  multi_items("CUEX", 1:3)),
#   composite("Value",        multi_items("PERV", 1:2)),
#   composite("Satisfaction", multi_items("CUSA", 1:3)),
#   composite("Loyalty",      multi_items("CUSL", 1:3))
# )
# 
# mobi_sm <- relationships(
#   paths(from = "Image",       to = c("Expectation", "Satisfaction", "Loyalty")),
#   paths(from = "Expectation", to = c("Value", "Satisfaction")),
#   paths(from = "Value",       to = "Satisfaction"),
#   paths(from = "Satisfaction", to = "Loyalty")
# )
# 
# mobi_pls <- estimate_pls(data = mobi,
#                           measurement_model = mobi_mm,
#                           structural_model  = mobi_sm)
# 
# # CTA-PLS with borrowing (default)
# cta_result <- assess_cta(mobi_pls, nboot = 5000, seed = 123)
# print(cta_result)
# summary(cta_result)
# 
# # Without borrowing — only Image (5 indicators) is testable
# cta_no_borrow <- assess_cta(mobi_pls, nboot = 5000, borrow = FALSE)
# print(cta_no_borrow)
# 
# # Visualize adjusted p-values per construct
# plot(cta_result)

## ----eval=FALSE, echo=TRUE----------------------------------------------------
# # FIMIX with K=2 segments (using the corporate reputation model from the setup)
# fimix_k2 <- assess_fimix(established_model, K = 2, nstart = 10, seed = 123)
# print(fimix_k2)
# summary(fimix_k2)
# plot(fimix_k2)
# 
# # Compare across K=2..4 using information criteria
# fimix_compare <- assess_fimix_compare(established_model,
#                                        K_range = 2:4,
#                                        nstart = 10,
#                                        seed = 123)
# print(fimix_compare)
# plot(fimix_compare)

## ----eval=FALSE, echo=TRUE----------------------------------------------------
# # PLS-POS with K=2 segments
# pos_k2 <- assess_pos(established_model, K = 2, nstart = 10, seed = 123)
# print(pos_k2)
# summary(pos_k2)
# plot(pos_k2, type = "rsquared")
# plot(pos_k2, type = "paths")
# 
# # Compare across K=2..4 to find optimal number of segments
# pos_compare <- assess_pos_compare(established_model,
#                                    K_range = 2:4,
#                                    nstart = 10,
#                                    seed = 123)
# print(pos_compare)
# plot(pos_compare)
# 
# # Extract segment-specific models for further analysis
# seg_models <- pos_segments(pos_k2)
# summary(seg_models[[1]])

## ----eval=FALSE, echo=TRUE----------------------------------------------------
# # Specify a mediation model
# mobi_mm <- constructs(
#   composite("Image",        multi_items("IMAG", 1:5)),
#   composite("Expectation",  multi_items("CUEX", 1:3)),
#   composite("Value",        multi_items("PERV", 1:2)),
#   composite("Satisfaction", multi_items("CUSA", 1:3)),
#   composite("Loyalty",      multi_items("CUSL", 1:3))
# )
# mobi_sm <- relationships(
#   paths(from = "Image",       to = c("Expectation", "Satisfaction", "Loyalty")),
#   paths(from = "Expectation", to = c("Value", "Satisfaction")),
#   paths(from = "Value",       to = "Satisfaction"),
#   paths(from = "Satisfaction", to = "Loyalty")
# )
# pls_model <- estimate_pls(mobi, mobi_mm, mobi_sm)
# 
# # Compute PCM for all mediation paths to Loyalty
# pcm_result <- assess_pcm(pls_model,
#                          target  = "Loyalty",
#                          noFolds = 10,
#                          reps    = 10)
# 
# # Print concise overview
# pcm_result
# 
# # Detailed per-indicator results
# summary(pcm_result)
# 
# # Visual comparison of mediation paths
# plot(pcm_result)

## ----eval=FALSE, echo=TRUE----------------------------------------------------
# cong_result <- congruence_test(mobi_pls,
#                                 nboot = 2000,
#                                 seed = 123)
# print(cong_result)
# summary(cong_result)

# =============================================================================
# feature_cipma.R - Combined Importance-Performance Map Analysis (cIPMA)
# =============================================================================
# Implements IPMA and cIPMA (IPMA + NCA) for PLS-SEM models.
#
# References:
# - Ringle, C. M. & Sarstedt, M. (2016). Gain More Insight from Your PLS-SEM
#   Results: The Importance-Performance Map Analysis. Industrial Management &
#   Data Systems, 119(9), 1865-1886.
# - Sarstedt, M., Richter, N. F., Hauff, S. & Ringle, C. M. (2024). Combined
#   Importance-Performance Map Analysis (cIPMA): A SmartPLS 4 Tutorial.
#   Journal of Marketing Analytics, 12, 746-760.
# - Hauff, S. et al. (2024). Importance and Performance in PLS-SEM and NCA:
#   Introducing the cIPMA. Journal of Retailing and Consumer Services, 78,
#   103723.
# =============================================================================

# =============================================================================
# INTERNAL HELPERS
# =============================================================================

#' Detect interaction constructs (names containing "*").
#' @noRd
is_interaction_construct <- function(name) {
  grepl("*", name, fixed = TRUE)
}

#' Check that all outer weights are positive for IPMA constructs.
#' For HOC constructs, also checks LOC-to-indicator weights.
#' Returns character vector of constructs with negative weights.
#' @noRd
check_positive_weights <- function(model, constructs) {
  neg <- character(0)
  construct_cols <- colnames(model$outer_weights)

  for (construct in constructs) {
    items <- items_of_construct(construct, model)
    weights <- model$outer_weights[items, construct]
    if (any(weights < 0)) {
      neg <- c(neg, construct)
      next
    }

    # For HOC: also check LOC-to-indicator weights
    loc_names <- items[items %in% construct_cols]
    if (length(loc_names) > 0) {
      loc_neg <- check_positive_weights(model, loc_names)
      if (length(loc_neg) > 0) {
        neg <- c(neg, construct)
      }
    }
  }
  neg
}

#' Compute construct performance scores (0-100 rescaled).
#'
#' Performance is the weighted average of individual indicator performances,
#' using normalized outer weights. Indicator performance:
#' (mean_observed - scale_min) / (scale_max - scale_min) * 100.
#'
#' For higher-order constructs (HOC), performance is computed by chaining
#' through the lower-order constructs (LOCs): first compute each LOC's
#' performance from its actual indicators, then aggregate using HOC-to-LOC
#' weights. This applies to both two-stage and repeated-indicators HOCs.
#'
#' @return Named numeric vector of construct performances (0-100).
#' @noRd
compute_ipma_performance <- function(model, constructs, scale_min, scale_max) {
  scale_range <- scale_max - scale_min
  performance <- numeric(length(constructs))
  names(performance) <- constructs

  construct_cols <- colnames(model$outer_weights)

  for (construct in constructs) {
    items <- items_of_construct(construct, model)

    # Detect HOC: items that are themselves construct names in outer_weights
    loc_names <- items[items %in% construct_cols]

    if (length(loc_names) > 0) {
      # HOC: recursively compute LOC performances, then aggregate
      loc_perf <- compute_ipma_performance(model, loc_names, scale_min, scale_max)
      hoc_weights <- model$outer_weights[loc_names, construct]
      performance[construct] <- sum(hoc_weights * loc_perf) / sum(hoc_weights)
    } else {
      # Regular construct: compute from indicators
      available <- items %in% colnames(model$data)
      if (!all(available)) {
        warning("Construct '", construct, "': some items not found in data. ",
                "Performance set to NA.", call. = FALSE)
        performance[construct] <- NA_real_
        next
      }

      weights <- model$outer_weights[items, construct]
      indicator_means <- colMeans(model$data[, items, drop = FALSE])
      indicator_perf <- (indicator_means - scale_min) / scale_range * 100

      # Weighted average using normalized positive weights
      performance[construct] <- sum(weights * indicator_perf) / sum(weights)
    }
  }

  performance
}

#' Compute per-observation performance scores for each construct.
#'
#' Used to obtain SD of rescaled construct scores for unstandardized
#' total effect computation. Returns an N x length(constructs) matrix.
#'
#' For HOC constructs, chains through LOC observation performances
#' and aggregates using HOC-to-LOC weights.
#' @noRd
compute_observation_performance <- function(model, constructs, scale_min, scale_max) {
  scale_range <- scale_max - scale_min
  n <- nrow(model$data)
  perf_scores <- matrix(NA_real_, nrow = n, ncol = length(constructs),
                         dimnames = list(NULL, constructs))

  construct_cols <- colnames(model$outer_weights)

  for (construct in constructs) {
    items <- items_of_construct(construct, model)

    # Detect HOC: items that are themselves construct names
    loc_names <- items[items %in% construct_cols]

    if (length(loc_names) > 0) {
      # HOC: recursively compute LOC observation performances, then aggregate
      loc_obs_perf <- compute_observation_performance(model, loc_names, scale_min, scale_max)
      hoc_weights <- model$outer_weights[loc_names, construct]
      perf_scores[, construct] <- loc_obs_perf %*% hoc_weights / sum(hoc_weights)
    } else {
      # Regular construct: compute from indicators
      available <- items %in% colnames(model$data)
      if (!all(available)) next

      weights <- model$outer_weights[items, construct]
      indicator_perf <- (model$data[, items, drop = FALSE] - scale_min) / scale_range * 100
      perf_scores[, construct] <- as.matrix(indicator_perf) %*% weights / sum(weights)
    }
  }

  perf_scores
}

#' Compute total effects matrix from a path coefficient matrix.
#'
#' Total effects T = (I - B)^{-1} - I, where B[from, to] is the
#' direct path coefficient from construct 'from' to construct 'to'.
#' @noRd
compute_total_effects <- function(path_coef_matrix) {
  k <- nrow(path_coef_matrix)
  I_mat <- diag(k)
  dimnames(I_mat) <- dimnames(path_coef_matrix)
  solve(I_mat - path_coef_matrix) - I_mat
}

#' Compute unstandardized total effects for IPMA.
#'
#' Converts standardized path coefficients to unstandardized form using
#' the SDs of rescaled (0-100) construct scores, then computes total effects.
#'
#' Unstandardized path coefficient:
#'   beta_unstd(X -> Y) = beta_std(X -> Y) * SD(Y_rescaled) / SD(X_rescaled)
#'
#' @return Total effects matrix with constructs as row/column names.
#' @noRd
compute_unstd_total_effects <- function(model, constructs, scale_min, scale_max) {
  # Compute SD of rescaled performance scores
  obs_perf <- compute_observation_performance(model, constructs, scale_min, scale_max)
  sd_perf <- apply(obs_perf, 2, sd, na.rm = TRUE)

  # Build unstandardized path coefficient matrix
  B_std <- model$path_coef[constructs, constructs]
  B_unstd <- B_std

  for (i in seq_along(constructs)) {
    for (j in seq_along(constructs)) {
      if (B_std[i, j] != 0) {
        sd_from <- sd_perf[constructs[i]]
        sd_to <- sd_perf[constructs[j]]
        if (is.na(sd_from) || is.na(sd_to) ||
            sd_from < .Machine$double.eps) {
          B_unstd[i, j] <- NA_real_
        } else {
          B_unstd[i, j] <- B_std[i, j] * sd_to / sd_from
        }
      }
    }
  }

  compute_total_effects(B_unstd)
}

#' Classify constructs into cIPMA priority categories.
#' @noRd
classify_cipma_constructs <- function(importance, performance, nca_result) {
  constructs <- names(importance)

  imp_median <- median(importance, na.rm = TRUE)
  high_importance <- importance > imp_median

  necessary <- rep(FALSE, length(constructs))
  names(necessary) <- constructs

  if (!is.null(nca_result) && length(nca_result$necessary_predictors) > 0) {
    necessary[intersect(constructs, nca_result$necessary_predictors)] <- TRUE
  }

  priority <- ifelse(
    high_importance & necessary, "Top priority",
    ifelse(high_importance & !necessary, "Important driver",
      ifelse(!high_importance & necessary, "Bottleneck risk",
        "Low priority")))

  data.frame(
    Construct = constructs,
    Importance = round(importance, 4),
    Performance = round(performance, 2),
    High_Importance = high_importance,
    Necessary = necessary,
    Priority = priority,
    row.names = NULL,
    stringsAsFactors = FALSE
  )
}

# =============================================================================
# MAIN ENTRY POINTS
# =============================================================================

#' Importance-Performance Map Analysis (IPMA) for PLS-SEM
#'
#' \code{assess_ipma} conducts an Importance-Performance Map Analysis (IPMA)
#' for a given target construct. It computes each predecessor's importance
#' (unstandardized total effect on the target) and performance (rescaled
#' 0--100 mean construct score), identifying constructs with high importance
#' but low performance as priority areas for improvement.
#'
#' This is a convenience wrapper around \code{\link{assess_cipma}} with
#' \code{nca = FALSE}. Use \code{\link{assess_cipma}} instead if you also
#' want Necessary Condition Analysis (NCA) to produce a combined IPMA.
#'
#' @inheritParams assess_cipma
#'
#' @return An object of class \code{cipma_analysis} (see
#'   \code{\link{assess_cipma}} for details). The \code{nca} element will
#'   be \code{NULL}.
#'
#' @references
#' Ringle, C. M. & Sarstedt, M. (2016). Gain More Insight from Your PLS-SEM
#' Results: The Importance-Performance Map Analysis. Industrial Management &
#' Data Systems, 119(9), 1865-1886.
#'
#' @examples
#' library(seminr)
#' library(seminrExtras)
#'
#' mobi_mm <- constructs(
#'   composite("Image",        multi_items("IMAG", 1:5)),
#'   composite("Expectation",  multi_items("CUEX", 1:3)),
#'   composite("Value",        multi_items("PERV", 1:2)),
#'   composite("Satisfaction", multi_items("CUSA", 1:3)),
#'   composite("Loyalty",      multi_items("CUSL", 1:3))
#' )
#'
#' mobi_sm <- relationships(
#'   paths(from = "Image",       to = c("Expectation", "Satisfaction", "Loyalty")),
#'   paths(from = "Expectation", to = c("Value", "Satisfaction")),
#'   paths(from = "Value",       to = "Satisfaction"),
#'   paths(from = "Satisfaction",to = "Loyalty")
#' )
#'
#' mobi_pls <- estimate_pls(data = mobi,
#'                           measurement_model = mobi_mm,
#'                           structural_model  = mobi_sm)
#'
#' \donttest{
#' ipma_result <- assess_ipma(mobi_pls,
#'                             target = "Loyalty",
#'                             scale_min = 1,
#'                             scale_max = 10)
#' print(ipma_result)
#' plot(ipma_result)
#' }
#'
#' @seealso \code{\link{assess_cipma}} for cIPMA (IPMA + NCA)
#'
#' @export
assess_ipma <- function(seminr_model,
                         target,
                         scale_min = 1,
                         scale_max = 7,
                         seed = 123) {
  assess_cipma(seminr_model = seminr_model,
               target = target,
               scale_min = scale_min,
               scale_max = scale_max,
               nca = FALSE,
               seed = seed)
}

#' Combined Importance-Performance Map Analysis (cIPMA) for PLS-SEM
#'
#' \code{assess_cipma} conducts an Importance-Performance Map Analysis (IPMA)
#' and optionally combines it with Necessary Condition Analysis (NCA) to
#' produce a combined IPMA (cIPMA). The analysis identifies constructs that
#' are both important (high total effect) and necessary (without which the
#' outcome cannot reach high levels), following the 7-step cIPMA procedure
#' of Sarstedt et al. (2024).
#'
#' \strong{Importance} is measured by the unstandardized total effect of each
#' construct on the target. Unstandardized effects are obtained by scaling
#' standardized path coefficients using the standard deviations of rescaled
#' (0--100) construct scores.
#'
#' \strong{Performance} is the weighted average of rescaled indicator means,
#' where each indicator is rescaled from the original measurement scale
#' ([\code{scale_min}, \code{scale_max}]) to 0--100. Weights are the PLS
#' outer weights, which must all be positive for valid IPMA rescaling.
#'
#' Interaction constructs (e.g., \code{"X*W"}) are automatically excluded
#' from the IPMA because their performance is not meaningful on a 0--100
#' scale.
#'
#' When \code{nca = TRUE}, NCA is run on construct scores for each
#' predecessor--target pair, and constructs are classified into four
#' categories based on crossing importance (above/below median) with
#' necessity (NCA d >= 0.1 and p < 0.05).
#'
#' @param seminr_model An estimated SEMinR model from \code{estimate_pls()}.
#' @param target Name of the target (endogenous) construct for the IPMA.
#' @param scale_min Minimum of the measurement scale (default 1).
#' @param scale_max Maximum of the measurement scale (default 7).
#' @param nca Logical; if \code{TRUE} (default), run NCA to produce a cIPMA.
#' @param nca_ceilings Character vector of ceiling techniques for NCA
#'   (default \code{c("ce_fdh", "cr_fdh")}).
#' @param nca_test.rep Number of NCA permutation test repetitions (default 0).
#'   Set > 0 to obtain significance for the necessity classification.
#' @param nca_steps Number of bottleneck table steps (default 10).
#' @param seed Random seed for reproducibility (default 123).
#'
#' @return An object of class \code{cipma_analysis} containing:
#'   \item{importance_unstd}{Named numeric vector of unstandardized total effects}
#'   \item{importance_std}{Named numeric vector of standardized total effects}
#'   \item{performance}{Named numeric vector of construct performances (0--100)}
#'   \item{nca}{An \code{nca_analysis} object (NULL if \code{nca = FALSE})}
#'   \item{classification}{Data frame classifying each construct}
#'   \item{target}{Name of the target construct}
#'   \item{constructs}{Character vector of included constructs}
#'   \item{scale_range}{Numeric vector \code{c(scale_min, scale_max)}}
#'   \item{negative_weight_constructs}{Constructs with negative outer weights (if any)}
#'   \item{excluded_interactions}{Interaction constructs excluded from IPMA}
#'   \item{pls_model}{The original estimated seminr model}
#'
#' @references
#' Ringle, C. M. & Sarstedt, M. (2016). Gain More Insight from Your PLS-SEM
#' Results: The Importance-Performance Map Analysis. Industrial Management &
#' Data Systems, 119(9), 1865-1886.
#'
#' Sarstedt, M., Richter, N. F., Hauff, S. & Ringle, C. M. (2024). Combined
#' Importance-Performance Map Analysis (cIPMA): A SmartPLS 4 Tutorial.
#' Journal of Marketing Analytics, 12, 746-760.
#'
#' Hauff, S. et al. (2024). Importance and Performance in PLS-SEM and NCA:
#' Introducing the cIPMA. Journal of Retailing and Consumer Services, 78,
#' 103723.
#'
#' @examples
#' library(seminr)
#' library(seminrExtras)
#'
#' mobi_mm <- constructs(
#'   composite("Image",        multi_items("IMAG", 1:5)),
#'   composite("Expectation",  multi_items("CUEX", 1:3)),
#'   composite("Value",        multi_items("PERV", 1:2)),
#'   composite("Satisfaction", multi_items("CUSA", 1:3)),
#'   composite("Loyalty",      multi_items("CUSL", 1:3))
#' )
#'
#' mobi_sm <- relationships(
#'   paths(from = "Image",       to = c("Expectation", "Satisfaction", "Loyalty")),
#'   paths(from = "Expectation", to = c("Value", "Satisfaction")),
#'   paths(from = "Value",       to = "Satisfaction"),
#'   paths(from = "Satisfaction",to = "Loyalty")
#' )
#'
#' mobi_pls <- estimate_pls(data = mobi,
#'                           measurement_model = mobi_mm,
#'                           structural_model  = mobi_sm)
#'
#' \donttest{
#' cipma_result <- assess_cipma(mobi_pls,
#'                               target = "Loyalty",
#'                               scale_min = 1,
#'                               scale_max = 10)
#' print(cipma_result)
#' summary(cipma_result)
#' plot(cipma_result)
#' }
#'
#' @seealso \code{\link{assess_ipma}} for IPMA without NCA,
#'   \code{\link{assess_nca}} for standalone NCA analysis
#'
#' @export
assess_cipma <- function(seminr_model,
                          target,
                          scale_min = 1,
                          scale_max = 7,
                          nca = TRUE,
                          nca_ceilings = c("ce_fdh", "cr_fdh"),
                          nca_test.rep = 0,
                          nca_steps = 10,
                          seed = 123) {

  # ---------------------------------------------------------------------------
  # Step 1: Validate inputs
  # ---------------------------------------------------------------------------
  if (!validate_seminr_model(seminr_model, "assess_cipma")) return(NULL)

  construct_names <- colnames(seminr_model$construct_scores)

  if (!(target %in% construct_names)) {
    stop("target '", target, "' not found in model constructs: ",
         paste(construct_names, collapse = ", "), call. = FALSE)
  }

  if (!is.numeric(scale_min) || !is.numeric(scale_max) ||
      length(scale_min) != 1 || length(scale_max) != 1) {
    stop("scale_min and scale_max must be single numeric values.", call. = FALSE)
  }

  if (scale_min >= scale_max) {
    stop("scale_min must be less than scale_max.", call. = FALSE)
  }

  # ---------------------------------------------------------------------------
  # Step 2: Identify constructs for IPMA
  # ---------------------------------------------------------------------------
  is_interaction <- vapply(construct_names, is_interaction_construct, logical(1))
  excluded_interactions <- construct_names[is_interaction]
  if (length(excluded_interactions) > 0) {
    message("Excluding interaction term(s) from IPMA: ",
            paste(excluded_interactions, collapse = ", "))
  }

  # All non-interaction constructs that appear in path_coef
  pc_names <- colnames(seminr_model$path_coef)
  all_constructs <- intersect(construct_names[!is_interaction], pc_names)

  # IPMA constructs = all except target
  ipma_constructs <- setdiff(all_constructs, target)

  if (length(ipma_constructs) == 0) {
    stop("No constructs available for IPMA (all excluded or only target exists).",
         call. = FALSE)
  }

  # ---------------------------------------------------------------------------
  # Step 3: Check positive outer weights
  # ---------------------------------------------------------------------------
  neg_weight_constructs <- check_positive_weights(seminr_model, ipma_constructs)
  if (length(neg_weight_constructs) > 0) {
    warning("Negative outer weights detected for: ",
            paste(neg_weight_constructs, collapse = ", "),
            ". IPMA performance rescaling may be unreliable for these constructs. ",
            "Consider reverse-coding indicators or checking the model specification ",
            "(Ringle & Sarstedt, 2016).", call. = FALSE)
  }

  # ---------------------------------------------------------------------------
  # Step 4: Compute performance (0-100 rescaled construct scores)
  # ---------------------------------------------------------------------------
  performance <- compute_ipma_performance(seminr_model, ipma_constructs,
                                           scale_min, scale_max)

  # ---------------------------------------------------------------------------
  # Step 5: Compute importance (total effects on target)
  # ---------------------------------------------------------------------------
  # Standardized total effects
  T_std <- compute_total_effects(seminr_model$path_coef[all_constructs, all_constructs])
  importance_std <- T_std[ipma_constructs, target]

  # Unstandardized total effects
  T_unstd <- compute_unstd_total_effects(seminr_model, all_constructs,
                                          scale_min, scale_max)
  importance_unstd <- T_unstd[ipma_constructs, target]

  # Filter to constructs with non-zero total effect on target
  has_effect <- abs(importance_std) > .Machine$double.eps
  if (!any(has_effect)) {
    stop("No constructs have a non-zero total effect on '", target, "'.",
         call. = FALSE)
  }

  ipma_constructs <- ipma_constructs[has_effect]
  performance <- performance[has_effect]
  importance_std <- importance_std[has_effect]
  importance_unstd <- importance_unstd[has_effect]

  # ---------------------------------------------------------------------------
  # Step 6: Run NCA (for cIPMA)
  # ---------------------------------------------------------------------------
  nca_result <- NULL
  if (nca) {
    nca_result <- assess_nca(seminr_model,
                              target = target,
                              predictors = ipma_constructs,
                              ceilings = nca_ceilings,
                              test.rep = nca_test.rep,
                              steps = nca_steps,
                              seed = seed)
  }

  # ---------------------------------------------------------------------------
  # Step 7: Classify constructs
  # ---------------------------------------------------------------------------
  classification <- classify_cipma_constructs(importance_unstd, performance,
                                               nca_result)

  # ---------------------------------------------------------------------------
  # Step 8: Build and return result
  # ---------------------------------------------------------------------------
  result <- list(
    importance_unstd = importance_unstd,
    importance_std = importance_std,
    performance = performance,
    nca = nca_result,
    classification = classification,
    target = target,
    constructs = ipma_constructs,
    scale_range = c(scale_min, scale_max),
    negative_weight_constructs = neg_weight_constructs,
    excluded_interactions = excluded_interactions,
    pls_model = seminr_model
  )

  class(result) <- c("cipma_analysis", class(result))
  result
}

# =============================================================================
# S3 METHODS
# =============================================================================

#' @export
print.cipma_analysis <- function(x, ...) {
  has_nca <- !is.null(x$nca)

  if (has_nca) {
    cat("Combined Importance-Performance Map Analysis (cIPMA)\n")
    cat("=====================================================\n")
  } else {
    cat("Importance-Performance Map Analysis (IPMA)\n")
    cat("============================================\n")
  }

  cat("Target:", x$target, "\n")
  cat("Constructs:", paste(x$constructs, collapse = ", "), "\n")
  cat("Scale range:", x$scale_range[1], "-", x$scale_range[2], "\n")
  cat("Observations:", nrow(x$pls_model$construct_scores), "\n")

  if (length(x$excluded_interactions) > 0) {
    cat("Excluded (interaction):", paste(x$excluded_interactions, collapse = ", "), "\n")
  }
  if (length(x$negative_weight_constructs) > 0) {
    cat("Warning - negative weights:", paste(x$negative_weight_constructs, collapse = ", "), "\n")
  }

  cat("\nImportance-Performance Results:\n")
  tbl <- data.frame(
    Construct = x$constructs,
    `Unstd. Total Effect` = round(x$importance_unstd, 4),
    `Std. Total Effect` = round(x$importance_std, 4),
    Performance = round(x$performance, 2),
    check.names = FALSE,
    row.names = NULL
  )
  print(tbl, row.names = FALSE, ...)

  if (has_nca) {
    cat("\nNecessary Conditions (NCA):\n")
    nca_tbl <- data.frame(
      Construct = x$constructs,
      check.names = FALSE,
      row.names = NULL
    )
    for (ceil in colnames(x$nca$effect_sizes)) {
      nca_tbl[[paste0("d (", ceil, ")")]] <- round(x$nca$effect_sizes[x$constructs, ceil], 4)
    }
    necessary_flag <- ifelse(x$classification$Necessary, "Yes", "No")
    nca_tbl[["Necessary"]] <- necessary_flag
    print(nca_tbl, row.names = FALSE, ...)
  }

  if (has_nca) {
    cat("\ncIPMA Classification:\n")
  } else {
    cat("\nIPMA Classification:\n")
  }
  class_tbl <- x$classification[, c("Construct", "Priority")]
  print(class_tbl, row.names = FALSE, ...)

  invisible(x)
}

#' @export
summary.cipma_analysis <- function(object, ...) {
  result <- list(
    target = object$target,
    constructs = object$constructs,
    scale_range = object$scale_range,
    n_obs = nrow(object$pls_model$construct_scores),
    importance_unstd = object$importance_unstd,
    importance_std = object$importance_std,
    performance = object$performance,
    classification = object$classification,
    nca = object$nca,
    negative_weight_constructs = object$negative_weight_constructs,
    excluded_interactions = object$excluded_interactions
  )

  class(result) <- c("summary.cipma_analysis", class(result))
  result
}

#' @export
print.summary.cipma_analysis <- function(x, ...) {
  has_nca <- !is.null(x$nca)

  if (has_nca) {
    cat("Combined Importance-Performance Map Analysis (cIPMA) Summary\n")
    cat("=============================================================\n")
  } else {
    cat("Importance-Performance Map Analysis (IPMA) Summary\n")
    cat("====================================================\n")
  }

  cat("Target:", x$target, "\n")
  cat("Scale range:", x$scale_range[1], "-", x$scale_range[2], "\n")
  cat("Observations:", x$n_obs, "\n")

  if (length(x$excluded_interactions) > 0) {
    cat("Excluded interactions:", paste(x$excluded_interactions, collapse = ", "), "\n")
  }
  if (length(x$negative_weight_constructs) > 0) {
    cat("Negative weight constructs:", paste(x$negative_weight_constructs, collapse = ", "), "\n")
  }

  cat("\nImportance (Unstandardized Total Effects on", x$target, "):\n")
  print(round(x$importance_unstd, 4), ...)

  cat("\nImportance (Standardized Total Effects on", x$target, "):\n")
  print(round(x$importance_std, 4), ...)

  cat("\nPerformance (0-100 rescaled):\n")
  print(round(x$performance, 2), ...)

  if (has_nca) {
    cat("\nNCA Effect Sizes:\n")
    print(round(x$nca$effect_sizes[x$constructs, , drop = FALSE], 4), ...)

    if (!is.null(x$nca$significance)) {
      cat("\nNCA Permutation p-values:\n")
      print(round(x$nca$significance[x$constructs, , drop = FALSE], 4), ...)
    }

    if (length(x$nca$necessary_predictors) > 0) {
      cat("\nNecessary conditions:", paste(x$nca$necessary_predictors, collapse = ", "), "\n")
    } else {
      cat("\nNo necessary conditions identified.\n")
    }

    # Print bottleneck for necessary constructs
    if (length(x$nca$necessary_predictors) > 0) {
      for (ceil in names(x$nca$bottleneck)) {
        cat("\nBottleneck table (", ceil, "):\n", sep = "")
        print(x$nca$bottleneck[[ceil]], ...)
      }
    }
  }

  cat("\nConstruct Classification:\n")
  print(x$classification, row.names = FALSE, ...)

  invisible(x)
}

#' Plot cIPMA Results
#'
#' Produces the Importance-Performance Map with optional NCA overlay.
#' When \code{type = "cipma"} (default), necessary conditions are
#' highlighted with filled red circles; other constructs use open blue circles.
#' When \code{type = "ipma"}, all constructs use the same symbol.
#'
#' @param x A \code{cipma_analysis} object from \code{assess_cipma()}.
#' @param type One of \code{"cipma"} (default, with NCA overlay) or
#'   \code{"ipma"} (standard IPMA without NCA distinction).
#' @param importance_metric One of \code{"unstandardized"} (default) or
#'   \code{"standardized"} to choose which total effect is plotted on the
#'   x-axis.
#' @param ... Additional arguments passed to \code{plot()}.
#'
#' @export
plot.cipma_analysis <- function(x,
                                 type = c("cipma", "ipma"),
                                 importance_metric = c("unstandardized", "standardized"),
                                 ...) {
  type <- match.arg(type)
  importance_metric <- match.arg(importance_metric)

  if (type == "cipma" && is.null(x$nca)) {
    message("No NCA results available. Showing standard IPMA plot.")
    type <- "ipma"
  }

  imp <- if (importance_metric == "unstandardized") x$importance_unstd else x$importance_std
  perf <- x$performance
  constructs <- x$constructs

  # Point styles
  if (type == "cipma") {
    necessary <- x$classification$Necessary
    pch <- ifelse(necessary, 19, 1)
    col <- ifelse(necessary, "red", "steelblue")
  } else {
    pch <- rep(19, length(constructs))
    col <- rep("steelblue", length(constructs))
  }

  # Margins
  old_par <- par(no.readonly = TRUE)
  on.exit(par(old_par))
  par(mar = c(5, 5, 4, 2))

  # Axis limits
  imp_pad <- max(0.05, diff(range(imp, na.rm = TRUE)) * 0.15)
  xlim <- range(imp, na.rm = TRUE) + c(-imp_pad, imp_pad)
  ylim <- c(max(0, min(perf, na.rm = TRUE) - 5),
            min(100, max(perf, na.rm = TRUE) + 5))

  # X-axis label
  xlab <- if (importance_metric == "unstandardized") {
    "Importance (unstandardized total effect)"
  } else {
    "Importance (standardized total effect)"
  }

  # Title
  main_title <- if (type == "cipma") {
    paste("cIPMA:", x$target)
  } else {
    paste("IPMA:", x$target)
  }

  plot(imp, perf, pch = pch, col = col, cex = 1.8,
       xlim = xlim, ylim = ylim,
       xlab = xlab, ylab = "Performance (0-100)",
       main = main_title, bty = "n", ...)

  # Reference lines at means
  abline(v = mean(imp, na.rm = TRUE), col = "gray70", lty = 2)
  abline(h = mean(perf, na.rm = TRUE), col = "gray70", lty = 2)

  # Construct labels
  text(imp, perf, labels = constructs, pos = 3, cex = 0.8)

  # Legend
  if (type == "cipma") {
    legend("bottomleft",
           legend = c("Necessary + sufficient", "Sufficient only"),
           pch = c(19, 1), col = c("red", "steelblue"),
           pt.cex = 1.5, cex = 0.8, bty = "n")
  }
}

# =============================================================================
# Tests for assess_cipma() — Combined Importance-Performance Map Analysis
# =============================================================================

# --- Setup: pre-compute models and results -----------------------------------
library(seminr)

mobi_mm <- constructs(
  composite("Image",        multi_items("IMAG", 1:5)),
  composite("Expectation",  multi_items("CUEX", 1:3)),
  composite("Value",        multi_items("PERV", 1:2)),
  composite("Satisfaction", multi_items("CUSA", 1:3)),
  composite("Loyalty",      multi_items("CUSL", 1:3))
)
mobi_sm <- relationships(
  paths(from = "Image",       to = c("Expectation", "Satisfaction", "Loyalty")),
  paths(from = "Expectation", to = c("Value", "Satisfaction")),
  paths(from = "Value",       to = "Satisfaction"),
  paths(from = "Satisfaction", to = "Loyalty")
)
pls_model <- estimate_pls(data = mobi, measurement_model = mobi_mm,
                           structural_model = mobi_sm)

# Pre-compute results for reuse across tests
cipma_result <- assess_cipma(pls_model, target = "Loyalty",
                              scale_min = 1, scale_max = 10,
                              nca_test.rep = 0, seed = 123)

ipma_result <- assess_cipma(pls_model, target = "Loyalty",
                              scale_min = 1, scale_max = 10,
                              nca = FALSE, seed = 123)

# =============================================================================
# STRUCTURE TESTS
# =============================================================================

test_that("assess_cipma returns cipma_analysis object", {
  expect_s3_class(cipma_result, "cipma_analysis")
})

test_that("cipma result contains all expected components", {
  expected_names <- c("importance_unstd", "importance_std", "performance",
                      "nca", "classification", "target", "constructs",
                      "scale_range", "negative_weight_constructs",
                      "excluded_interactions", "pls_model")
  expect_true(all(expected_names %in% names(cipma_result)))
})

test_that("target is stored correctly", {
  expect_equal(cipma_result$target, "Loyalty")
})

test_that("constructs exclude the target", {
  expect_false("Loyalty" %in% cipma_result$constructs)
})

test_that("all included constructs are in the model", {
  model_constructs <- colnames(pls_model$construct_scores)
  expect_true(all(cipma_result$constructs %in% model_constructs))
})

test_that("scale_range is stored correctly", {
  expect_equal(cipma_result$scale_range, c(1, 10))
})

# =============================================================================
# IMPORTANCE TESTS
# =============================================================================

test_that("standardized total effects match seminr", {
  seminr_te <- summary(pls_model)$total_effects
  for (construct in cipma_result$constructs) {
    expect_equal(cipma_result$importance_std[[construct]],
                 seminr_te[construct, "Loyalty"],
                 tolerance = 1e-10)
  }
})

test_that("importance vectors have correct names", {
  expect_equal(names(cipma_result$importance_std), cipma_result$constructs)
  expect_equal(names(cipma_result$importance_unstd), cipma_result$constructs)
})

test_that("unstandardized total effects are finite and non-zero", {
  expect_true(all(is.finite(cipma_result$importance_unstd)))
  expect_true(all(abs(cipma_result$importance_unstd) > .Machine$double.eps))
})

test_that("unstandardized total effects differ from standardized", {
  # They should differ because rescaling changes SDs
  expect_false(all(abs(cipma_result$importance_unstd - cipma_result$importance_std) <
                     .Machine$double.eps))
})

# =============================================================================
# PERFORMANCE TESTS
# =============================================================================

test_that("performance values are within 0-100 range", {
  expect_true(all(cipma_result$performance >= 0))
  expect_true(all(cipma_result$performance <= 100))
})

test_that("performance has correct names", {
  expect_equal(names(cipma_result$performance), cipma_result$constructs)
})

test_that("performance changes with scale range", {
  result_5pt <- assess_cipma(pls_model, target = "Loyalty",
                               scale_min = 1, scale_max = 5,
                               nca = FALSE, seed = 123)
  result_10pt <- assess_cipma(pls_model, target = "Loyalty",
                                scale_min = 1, scale_max = 10,
                                nca = FALSE, seed = 123)

  # Performance differs with different scales
  expect_false(all(abs(result_5pt$performance - result_10pt$performance) <
                     .Machine$double.eps))
})

test_that("performance is correct for known calculation", {
  # Manual calculation for one construct
  items <- seminr:::items_of_construct("Image", pls_model)
  weights <- pls_model$outer_weights[items, "Image"]
  indicator_means <- colMeans(pls_model$data[, items])
  indicator_perf <- (indicator_means - 1) / (10 - 1) * 100
  expected_perf <- sum(weights * indicator_perf) / sum(weights)

  expect_equal(cipma_result$performance[["Image"]], expected_perf, tolerance = 1e-10)
})

# =============================================================================
# NCA INTEGRATION TESTS
# =============================================================================

test_that("cIPMA includes NCA results", {
  expect_s3_class(cipma_result$nca, "nca_analysis")
})

test_that("IPMA-only mode has NULL NCA", {
  expect_null(ipma_result$nca)
})

test_that("NCA target matches cIPMA target", {
  expect_equal(cipma_result$nca$target, cipma_result$target)
})

test_that("NCA predictors match cIPMA constructs", {
  expect_equal(cipma_result$nca$predictors, cipma_result$constructs)
})

# =============================================================================
# CLASSIFICATION TESTS
# =============================================================================

test_that("classification is a data frame with expected columns", {
  expect_s3_class(cipma_result$classification, "data.frame")
  expected_cols <- c("Construct", "Importance", "Performance",
                     "High_Importance", "Necessary", "Priority")
  expect_true(all(expected_cols %in% colnames(cipma_result$classification)))
})

test_that("classification has one row per construct", {
  expect_equal(nrow(cipma_result$classification), length(cipma_result$constructs))
})

test_that("classification priority is a valid category", {
  valid_priorities <- c("Top priority", "Important driver",
                        "Bottleneck risk", "Low priority")
  expect_true(all(cipma_result$classification$Priority %in% valid_priorities))
})

test_that("High_Importance splits at median", {
  imp <- cipma_result$importance_unstd
  high <- cipma_result$classification$High_Importance
  expect_true(all(imp[high] > median(imp)))
  expect_true(all(imp[!high] <= median(imp)))
})

test_that("classification without NCA has all Necessary = FALSE", {
  expect_true(all(!ipma_result$classification$Necessary))
})

test_that("classification without NCA uses only driver/low priority", {
  valid <- c("Important driver", "Low priority")
  expect_true(all(ipma_result$classification$Priority %in% valid))
})

# =============================================================================
# TOTAL EFFECTS COMPUTATION TESTS
# =============================================================================

test_that("total effects include indirect paths", {
  # Image has both direct and indirect effects on Loyalty
  # Direct: Image -> Loyalty

  # Indirect: Image -> Satisfaction -> Loyalty, Image -> Expectation -> ... -> Loyalty
  direct_coef <- pls_model$path_coef["Image", "Loyalty"]
  total_std <- cipma_result$importance_std[["Image"]]

  # Total effect should exceed direct effect when indirect paths exist
  expect_true(total_std > direct_coef)
})

test_that("only constructs with non-zero total effect on target are included", {
  # All included constructs must have a non-zero total effect on the target
  seminr_te <- summary(pls_model)$total_effects
  for (construct in cipma_result$constructs) {
    expect_true(abs(seminr_te[construct, "Loyalty"]) > .Machine$double.eps)
  }
})

# =============================================================================
# INTERACTION TERM EXCLUSION TESTS
# =============================================================================

test_that("interaction constructs are excluded from IPMA", {
  mm_mod <- constructs(
    composite("Image",        multi_items("IMAG", 1:5)),
    composite("Value",        multi_items("PERV", 1:2)),
    composite("Satisfaction", multi_items("CUSA", 1:3)),
    interaction_term(iv = "Image", moderator = "Value", method = two_stage)
  )
  sm_mod <- relationships(
    paths(from = c("Image", "Value", "Image*Value"), to = "Satisfaction")
  )
  pls_mod <- estimate_pls(data = mobi, measurement_model = mm_mod,
                            structural_model = sm_mod)

  expect_message(
    result_mod <- assess_cipma(pls_mod, target = "Satisfaction",
                                 nca = FALSE, scale_min = 1, scale_max = 10),
    "Excluding interaction"
  )

  expect_false("Image*Value" %in% result_mod$constructs)
  expect_true("Image" %in% result_mod$constructs)
  expect_true("Value" %in% result_mod$constructs)
  expect_true("Image*Value" %in% result_mod$excluded_interactions)
})

# =============================================================================
# INPUT VALIDATION TESTS
# =============================================================================

test_that("non-seminr model input returns NULL with warning", {
  expect_warning(result <- assess_cipma("not a model", target = "X"),
                 "SEMinR models")
  expect_null(result)
})

test_that("invalid target construct raises error", {
  expect_error(assess_cipma(pls_model, target = "NonExistent"),
               "not found in model constructs")
})

test_that("invalid scale_min >= scale_max raises error", {
  expect_error(assess_cipma(pls_model, target = "Loyalty",
                              scale_min = 10, scale_max = 1),
               "scale_min must be less than scale_max")
})

test_that("equal scale_min and scale_max raises error", {
  expect_error(assess_cipma(pls_model, target = "Loyalty",
                              scale_min = 5, scale_max = 5),
               "scale_min must be less than scale_max")
})

test_that("non-numeric scale values raise error", {
  expect_error(assess_cipma(pls_model, target = "Loyalty",
                              scale_min = "a", scale_max = 10),
               "single numeric values")
})

# =============================================================================
# S3 METHOD TESTS
# =============================================================================

test_that("print.cipma_analysis returns invisibly", {
  expect_invisible(print(cipma_result))
})

test_that("print output contains key information", {
  out <- capture.output(print(cipma_result))
  out_text <- paste(out, collapse = "\n")

  expect_true(grepl("cIPMA", out_text))
  expect_true(grepl("Loyalty", out_text))
  expect_true(grepl("Image", out_text))
  expect_true(grepl("Top priority", out_text))
})

test_that("print without NCA shows IPMA header", {
  out <- capture.output(print(ipma_result))
  out_text <- paste(out, collapse = "\n")
  expect_true(grepl("Importance-Performance Map Analysis \\(IPMA\\)", out_text))
})

test_that("summary returns summary.cipma_analysis object", {
  s <- summary(cipma_result)
  expect_s3_class(s, "summary.cipma_analysis")
})

test_that("summary print includes bottleneck table", {
  out <- capture.output(print(summary(cipma_result)))
  out_text <- paste(out, collapse = "\n")
  expect_true(grepl("Bottleneck", out_text))
})

test_that("plot.cipma_analysis runs without error", {
  expect_no_error(plot(cipma_result, type = "cipma"))
  expect_no_error(plot(cipma_result, type = "ipma"))
  expect_no_error(plot(cipma_result, importance_metric = "standardized"))
})

test_that("plot with nca=FALSE defaults to ipma type", {
  expect_message(plot(ipma_result, type = "cipma"),
                 "No NCA results available")
})

# =============================================================================
# REPRODUCIBILITY TESTS
# =============================================================================

test_that("results are deterministic with same seed", {
  r1 <- assess_cipma(pls_model, target = "Loyalty",
                       scale_min = 1, scale_max = 10,
                       nca_test.rep = 0, seed = 42)
  r2 <- assess_cipma(pls_model, target = "Loyalty",
                       scale_min = 1, scale_max = 10,
                       nca_test.rep = 0, seed = 42)
  expect_equal(r1$importance_unstd, r2$importance_unstd)
  expect_equal(r1$performance, r2$performance)
})

# =============================================================================
# DIFFERENT TARGET CONSTRUCT TESTS
# =============================================================================

test_that("IPMA works with a different target (Satisfaction)", {
  result_sat <- assess_cipma(pls_model, target = "Satisfaction",
                               scale_min = 1, scale_max = 10,
                               nca = FALSE, seed = 123)
  expect_equal(result_sat$target, "Satisfaction")
  expect_false("Satisfaction" %in% result_sat$constructs)

  # Only constructs with non-zero total effect on Satisfaction should be included
  seminr_te <- summary(pls_model)$total_effects
  constructs_with_effect <- names(which(abs(seminr_te[, "Satisfaction"]) > .Machine$double.eps))
  constructs_with_effect <- setdiff(constructs_with_effect, "Satisfaction")
  expect_true(all(result_sat$constructs %in% constructs_with_effect))
})

# =============================================================================
# assess_ipma() WRAPPER TESTS
# =============================================================================

test_that("assess_ipma returns cipma_analysis object without NCA", {
  result <- assess_ipma(pls_model, target = "Loyalty",
                          scale_min = 1, scale_max = 10)
  expect_s3_class(result, "cipma_analysis")
  expect_null(result$nca)
})

test_that("assess_ipma gives identical results to assess_cipma with nca=FALSE", {
  ipma <- assess_ipma(pls_model, target = "Loyalty",
                        scale_min = 1, scale_max = 10, seed = 42)
  cipma_no_nca <- assess_cipma(pls_model, target = "Loyalty",
                                 scale_min = 1, scale_max = 10,
                                 nca = FALSE, seed = 42)
  expect_equal(ipma$importance_unstd, cipma_no_nca$importance_unstd)
  expect_equal(ipma$importance_std, cipma_no_nca$importance_std)
  expect_equal(ipma$performance, cipma_no_nca$performance)
  expect_equal(ipma$classification, cipma_no_nca$classification)
})

test_that("assess_ipma print shows IPMA header, not cIPMA", {
  result <- assess_ipma(pls_model, target = "Loyalty",
                          scale_min = 1, scale_max = 10)
  out <- capture.output(print(result))
  out_text <- paste(out, collapse = "\n")
  expect_true(grepl("Importance-Performance Map Analysis \\(IPMA\\)", out_text))
  expect_false(grepl("cIPMA", out_text))
})

# =============================================================================
# MEDIATION MODEL TESTS
# =============================================================================

test_that("IPMA works with mediation model", {
  mm_med <- constructs(
    composite("Image",        multi_items("IMAG", 1:5)),
    composite("Expectation",  multi_items("CUEX", 1:3)),
    composite("Satisfaction", multi_items("CUSA", 1:3)),
    composite("Loyalty",      multi_items("CUSL", 1:3))
  )
  sm_med <- relationships(
    paths(from = "Image", to = c("Expectation", "Loyalty")),
    paths(from = "Expectation", to = "Satisfaction"),
    paths(from = "Satisfaction", to = "Loyalty")
  )
  pls_med <- estimate_pls(data = mobi, measurement_model = mm_med,
                            structural_model = sm_med)

  result <- assess_ipma(pls_med, target = "Loyalty",
                          scale_min = 1, scale_max = 10)

  expect_s3_class(result, "cipma_analysis")
  # Image has direct + indirect (via Expectation -> Satisfaction) effect
  expect_true("Image" %in% result$constructs)
  expect_true("Satisfaction" %in% result$constructs)
  # Expectation has indirect effect (Expectation -> Satisfaction -> Loyalty)
  expect_true("Expectation" %in% result$constructs)

  # Total effect of Image > direct path coefficient (due to indirect paths)
  direct_coef <- pls_med$path_coef["Image", "Loyalty"]
  expect_true(result$importance_std[["Image"]] > direct_coef)

  # Performance in valid range
  expect_true(all(result$performance >= 0 & result$performance <= 100))
})

test_that("cIPMA works with mediation model", {
  mm_med <- constructs(
    composite("Image",        multi_items("IMAG", 1:5)),
    composite("Expectation",  multi_items("CUEX", 1:3)),
    composite("Satisfaction", multi_items("CUSA", 1:3)),
    composite("Loyalty",      multi_items("CUSL", 1:3))
  )
  sm_med <- relationships(
    paths(from = "Image", to = c("Expectation", "Loyalty")),
    paths(from = "Expectation", to = "Satisfaction"),
    paths(from = "Satisfaction", to = "Loyalty")
  )
  pls_med <- estimate_pls(data = mobi, measurement_model = mm_med,
                            structural_model = sm_med)

  result <- assess_cipma(pls_med, target = "Loyalty",
                           scale_min = 1, scale_max = 10,
                           nca_test.rep = 0, seed = 123)

  expect_s3_class(result, "cipma_analysis")
  expect_s3_class(result$nca, "nca_analysis")
  expect_true(all(c("Image", "Satisfaction") %in% result$nca$predictors))
})

# =============================================================================
# MODERATION MODEL TESTS
# =============================================================================

test_that("IPMA works with moderation model (two-stage)", {
  mm_mod <- constructs(
    composite("Image",        multi_items("IMAG", 1:5)),
    composite("Value",        multi_items("PERV", 1:2)),
    composite("Satisfaction", multi_items("CUSA", 1:3)),
    composite("Loyalty",      multi_items("CUSL", 1:3)),
    interaction_term(iv = "Image", moderator = "Value", method = two_stage)
  )
  sm_mod <- relationships(
    paths(from = c("Image", "Value", "Image*Value"), to = "Satisfaction"),
    paths(from = "Satisfaction", to = "Loyalty")
  )
  pls_mod <- estimate_pls(data = mobi, measurement_model = mm_mod,
                            structural_model = sm_mod)

  expect_message(
    result <- assess_ipma(pls_mod, target = "Loyalty",
                            scale_min = 1, scale_max = 10),
    "Excluding interaction"
  )

  # Interaction excluded, constituents included
  expect_false("Image*Value" %in% result$constructs)
  expect_true("Image" %in% result$constructs)
  expect_true("Value" %in% result$constructs)
  expect_true("Image*Value" %in% result$excluded_interactions)

  # Performance of non-interaction constructs in valid range
  expect_true(all(result$performance >= 0 & result$performance <= 100))
})

test_that("cIPMA works with moderation model", {
  mm_mod <- constructs(
    composite("Image",        multi_items("IMAG", 1:5)),
    composite("Value",        multi_items("PERV", 1:2)),
    composite("Satisfaction", multi_items("CUSA", 1:3)),
    composite("Loyalty",      multi_items("CUSL", 1:3)),
    interaction_term(iv = "Image", moderator = "Value", method = two_stage)
  )
  sm_mod <- relationships(
    paths(from = c("Image", "Value", "Image*Value"), to = "Satisfaction"),
    paths(from = "Satisfaction", to = "Loyalty")
  )
  pls_mod <- estimate_pls(data = mobi, measurement_model = mm_mod,
                            structural_model = sm_mod)

  expect_message(
    result <- assess_cipma(pls_mod, target = "Loyalty",
                             scale_min = 1, scale_max = 10,
                             nca_test.rep = 0, seed = 123),
    "Excluding interaction"
  )

  expect_s3_class(result$nca, "nca_analysis")
  # NCA predictors should not include the interaction term
  expect_false("Image*Value" %in% result$nca$predictors)
})

# =============================================================================
# HIGHER-ORDER CONSTRUCT (HOC) TESTS — TWO-STAGE
# =============================================================================

test_that("IPMA works with HOC two-stage model", {
  mm_hoc <- constructs(
    composite("Image",        multi_items("IMAG", 1:5)),
    composite("Expectation",  multi_items("CUEX", 1:3)),
    higher_composite("Quality", c("Image", "Expectation"), method = two_stage),
    composite("Satisfaction", multi_items("CUSA", 1:3)),
    composite("Loyalty",      multi_items("CUSL", 1:3))
  )
  sm_hoc <- relationships(
    paths(from = "Quality", to = c("Satisfaction", "Loyalty")),
    paths(from = "Satisfaction", to = "Loyalty")
  )
  pls_hoc <- estimate_pls(data = mobi, measurement_model = mm_hoc,
                            structural_model = sm_hoc)

  result <- assess_ipma(pls_hoc, target = "Loyalty",
                          scale_min = 1, scale_max = 10)

  expect_s3_class(result, "cipma_analysis")
  expect_true("Quality" %in% result$constructs)
  expect_true("Satisfaction" %in% result$constructs)
  # LOCs should NOT be in constructs (they're inside the HOC)
  expect_false("Image" %in% result$constructs)
  expect_false("Expectation" %in% result$constructs)

  # Performance in valid range (HOC chained through LOCs)
  expect_true(all(result$performance >= 0 & result$performance <= 100))
})

test_that("HOC two-stage performance chains through LOC indicators", {
  mm_hoc <- constructs(
    composite("Image",        multi_items("IMAG", 1:5)),
    composite("Expectation",  multi_items("CUEX", 1:3)),
    higher_composite("Quality", c("Image", "Expectation"), method = two_stage),
    composite("Satisfaction", multi_items("CUSA", 1:3)),
    composite("Loyalty",      multi_items("CUSL", 1:3))
  )
  sm_hoc <- relationships(
    paths(from = "Quality", to = c("Satisfaction", "Loyalty")),
    paths(from = "Satisfaction", to = "Loyalty")
  )
  pls_hoc <- estimate_pls(data = mobi, measurement_model = mm_hoc,
                            structural_model = sm_hoc)

  result <- assess_ipma(pls_hoc, target = "Loyalty",
                          scale_min = 1, scale_max = 10)

  # Manual computation of HOC Quality performance via LOC chain
  img_items <- seminr:::items_of_construct("Image", pls_hoc)
  img_w <- pls_hoc$outer_weights[img_items, "Image"]
  img_means <- colMeans(pls_hoc$data[, img_items])
  img_perf <- sum(img_w * (img_means - 1) / 9 * 100) / sum(img_w)

  exp_items <- seminr:::items_of_construct("Expectation", pls_hoc)
  exp_w <- pls_hoc$outer_weights[exp_items, "Expectation"]
  exp_means <- colMeans(pls_hoc$data[, exp_items])
  exp_perf <- sum(exp_w * (exp_means - 1) / 9 * 100) / sum(exp_w)

  hoc_w <- pls_hoc$outer_weights[c("Image", "Expectation"), "Quality"]
  expected_quality_perf <- sum(hoc_w * c(img_perf, exp_perf)) / sum(hoc_w)

  expect_equal(result$performance[["Quality"]], expected_quality_perf,
               tolerance = 1e-10)
})

test_that("cIPMA works with HOC two-stage model", {
  mm_hoc <- constructs(
    composite("Image",        multi_items("IMAG", 1:5)),
    composite("Expectation",  multi_items("CUEX", 1:3)),
    higher_composite("Quality", c("Image", "Expectation"), method = two_stage),
    composite("Satisfaction", multi_items("CUSA", 1:3)),
    composite("Loyalty",      multi_items("CUSL", 1:3))
  )
  sm_hoc <- relationships(
    paths(from = "Quality", to = c("Satisfaction", "Loyalty")),
    paths(from = "Satisfaction", to = "Loyalty")
  )
  pls_hoc <- estimate_pls(data = mobi, measurement_model = mm_hoc,
                            structural_model = sm_hoc)

  result <- assess_cipma(pls_hoc, target = "Loyalty",
                           scale_min = 1, scale_max = 10,
                           nca_test.rep = 0, seed = 123)

  expect_s3_class(result, "cipma_analysis")
  expect_s3_class(result$nca, "nca_analysis")
  expect_true("Quality" %in% result$nca$predictors)
  expect_true("Satisfaction" %in% result$nca$predictors)
})

# =============================================================================
# HIGHER-ORDER CONSTRUCT (HOC) TESTS — REPEATED INDICATORS
# =============================================================================

test_that("IPMA works with HOC repeated indicators model", {
  mm_ri <- constructs(
    composite("Image",        multi_items("IMAG", 1:5)),
    composite("Expectation",  multi_items("CUEX", 1:3)),
    higher_composite("Quality", c("Image", "Expectation"),
                      method = repeated_indicators),
    composite("Satisfaction", multi_items("CUSA", 1:3)),
    composite("Loyalty",      multi_items("CUSL", 1:3))
  )
  sm_ri <- relationships(
    paths(from = "Quality", to = c("Satisfaction", "Loyalty")),
    paths(from = "Satisfaction", to = "Loyalty")
  )
  pls_ri <- estimate_pls(data = mobi, measurement_model = mm_ri,
                           structural_model = sm_ri)

  result <- assess_ipma(pls_ri, target = "Loyalty",
                          scale_min = 1, scale_max = 10)

  expect_s3_class(result, "cipma_analysis")
  expect_true("Quality" %in% result$constructs)
  expect_true(all(result$performance >= 0 & result$performance <= 100))
})

test_that("HOC two-stage and repeated indicators give same performance", {
  mm_ts <- constructs(
    composite("Image",        multi_items("IMAG", 1:5)),
    composite("Expectation",  multi_items("CUEX", 1:3)),
    higher_composite("Quality", c("Image", "Expectation"), method = two_stage),
    composite("Satisfaction", multi_items("CUSA", 1:3)),
    composite("Loyalty",      multi_items("CUSL", 1:3))
  )
  mm_ri <- constructs(
    composite("Image",        multi_items("IMAG", 1:5)),
    composite("Expectation",  multi_items("CUEX", 1:3)),
    higher_composite("Quality", c("Image", "Expectation"),
                      method = repeated_indicators),
    composite("Satisfaction", multi_items("CUSA", 1:3)),
    composite("Loyalty",      multi_items("CUSL", 1:3))
  )
  sm <- relationships(
    paths(from = "Quality", to = c("Satisfaction", "Loyalty")),
    paths(from = "Satisfaction", to = "Loyalty")
  )
  pls_ts <- estimate_pls(data = mobi, measurement_model = mm_ts,
                           structural_model = sm)
  pls_ri <- estimate_pls(data = mobi, measurement_model = mm_ri,
                           structural_model = sm)

  r_ts <- assess_ipma(pls_ts, target = "Loyalty",
                        scale_min = 1, scale_max = 10)
  r_ri <- assess_ipma(pls_ri, target = "Loyalty",
                        scale_min = 1, scale_max = 10)

  # Both approaches should produce the same HOC performance
  # (chaining LOC → indicator performances via LOC weights)
  expect_equal(r_ts$performance[["Quality"]], r_ri$performance[["Quality"]],
               tolerance = 1e-10)
})

# =============================================================================
# NCA ON DIFFERENT MODEL TYPES
# =============================================================================

test_that("NCA works with mediation model", {
  mm_med <- constructs(
    composite("Image",        multi_items("IMAG", 1:5)),
    composite("Expectation",  multi_items("CUEX", 1:3)),
    composite("Satisfaction", multi_items("CUSA", 1:3)),
    composite("Loyalty",      multi_items("CUSL", 1:3))
  )
  sm_med <- relationships(
    paths(from = "Image", to = c("Expectation", "Loyalty")),
    paths(from = "Expectation", to = "Satisfaction"),
    paths(from = "Satisfaction", to = "Loyalty")
  )
  pls_med <- estimate_pls(data = mobi, measurement_model = mm_med,
                            structural_model = sm_med)

  result <- assess_nca(pls_med, target = "Loyalty", test.rep = 0, seed = 123)

  expect_s3_class(result, "nca_analysis")
  # Direct predictors of Loyalty: Image, Satisfaction
  expect_true(all(c("Image", "Satisfaction") %in% result$predictors))
  # Expectation has no direct path to Loyalty
  expect_false("Expectation" %in% result$predictors)
})

test_that("NCA works with moderation model", {
  mm_mod <- constructs(
    composite("Image",        multi_items("IMAG", 1:5)),
    composite("Value",        multi_items("PERV", 1:2)),
    composite("Satisfaction", multi_items("CUSA", 1:3)),
    composite("Loyalty",      multi_items("CUSL", 1:3)),
    interaction_term(iv = "Image", moderator = "Value", method = two_stage)
  )
  sm_mod <- relationships(
    paths(from = c("Image", "Value", "Image*Value"), to = "Satisfaction"),
    paths(from = "Satisfaction", to = "Loyalty")
  )
  pls_mod <- estimate_pls(data = mobi, measurement_model = mm_mod,
                            structural_model = sm_mod)

  # NCA on Satisfaction (includes interaction as predictor)
  result <- assess_nca(pls_mod, target = "Satisfaction",
                         test.rep = 0, seed = 123)

  expect_s3_class(result, "nca_analysis")
  expect_true("Image*Value" %in% result$predictors)
  expect_true(all(c("Image", "Value") %in% result$predictors))
})

test_that("NCA works with HOC two-stage model", {
  mm_hoc <- constructs(
    composite("Image",        multi_items("IMAG", 1:5)),
    composite("Expectation",  multi_items("CUEX", 1:3)),
    higher_composite("Quality", c("Image", "Expectation"), method = two_stage),
    composite("Satisfaction", multi_items("CUSA", 1:3)),
    composite("Loyalty",      multi_items("CUSL", 1:3))
  )
  sm_hoc <- relationships(
    paths(from = "Quality", to = c("Satisfaction", "Loyalty")),
    paths(from = "Satisfaction", to = "Loyalty")
  )
  pls_hoc <- estimate_pls(data = mobi, measurement_model = mm_hoc,
                            structural_model = sm_hoc)

  result <- assess_nca(pls_hoc, target = "Loyalty",
                         test.rep = 0, seed = 123)

  expect_s3_class(result, "nca_analysis")
  expect_true("Quality" %in% result$predictors)
  expect_true("Satisfaction" %in% result$predictors)
})

# =============================================================================
# feature_cta.R - Confirmatory Tetrad Analysis for PLS-SEM (CTA-PLS)
# =============================================================================
# Implements CTA-PLS for empirically testing whether a construct's measurement
# model is reflective or formative.
#
# Under a reflective (common factor) model, all model-implied vanishing tetrads
# equal zero. If any tetrad is significantly non-zero, the reflective
# specification is rejected in favour of a formative specification.
#
# References:
# - Gudergan, S. P., Ringle, C. M., Wende, S. & Will, A. (2008). Confirmatory
#   Tetrad Analysis in PLS Path Modeling. Journal of Business Research, 61(12),
#   1238-1249.
# - Cefis, M., Angelelli, M., Carpita, M. & Ciavolino, E. (2025). Confirmatory
#   Tetrad Analysis in PLS-SEM: A Multiple Testing Correction Perspective.
#   Social Indicators Research (advance online).
# - Bollen, K. A. & Ting, K.-F. (2000). A Tetrad Test for Causal Indicators.
#   Psychological Methods, 5(1), 3-22.
# =============================================================================

# =============================================================================
# INTERNAL HELPERS
# =============================================================================

#' Enumerate non-redundant vanishing tetrads for a set of indicators.
#'
#' For each combination of 4 indicators (i,j,k,l), there are 3 possible tetrads
#' but only 2 are algebraically independent (tau1 - tau2 + tau3 = 0 always).
#' We return 2 per 4-tuple.
#'
#' @param indicators Character vector of indicator names (length >= 4).
#' @return A data.frame with columns: i, j, k, l (indicator names) and
#'   tetrad_id (1 or 2) identifying which of the two non-redundant tetrads.
#' @noRd
enumerate_tetrads <- function(indicators) {
  combos <- utils::combn(indicators, 4)
  n_combos <- ncol(combos)

  # Pre-allocate: 2 tetrads per 4-tuple
  result <- data.frame(
    i = character(2 * n_combos),
    j = character(2 * n_combos),
    k = character(2 * n_combos),
    l = character(2 * n_combos),
    tetrad_id = integer(2 * n_combos),
    stringsAsFactors = FALSE
  )

  for (idx in seq_len(n_combos)) {
    vars <- combos[, idx]
    row_base <- (idx - 1) * 2

    # Tetrad 1: sigma_ij * sigma_kl - sigma_ik * sigma_jl
    result[row_base + 1, ] <- list(vars[1], vars[2], vars[3], vars[4], 1L)
    # Tetrad 2: sigma_ij * sigma_kl - sigma_il * sigma_jk
    result[row_base + 2, ] <- list(vars[1], vars[2], vars[3], vars[4], 2L)
  }

  result
}

#' Compute tetrad values from a covariance matrix.
#'
#' @param cov_mat Covariance matrix with named rows/columns.
#' @param tetrad_df Data.frame from enumerate_tetrads().
#' @return Numeric vector of tetrad values.
#' @noRd
compute_tetrads <- function(cov_mat, tetrad_df) {
  n <- nrow(tetrad_df)
  values <- numeric(n)

  for (r in seq_len(n)) {
    i <- tetrad_df$i[r]
    j <- tetrad_df$j[r]
    k <- tetrad_df$k[r]
    l <- tetrad_df$l[r]

    # Both tetrads share the first product: sigma_ij * sigma_kl
    prod_common <- cov_mat[i, j] * cov_mat[k, l]

    if (tetrad_df$tetrad_id[r] == 1L) {
      # tau1 = sigma_ij * sigma_kl - sigma_ik * sigma_jl
      values[r] <- prod_common - cov_mat[i, k] * cov_mat[j, l]
    } else if (tetrad_df$tetrad_id[r] == 2L) {
      # tau2 = sigma_ij * sigma_kl - sigma_il * sigma_jk
      values[r] <- prod_common - cov_mat[i, l] * cov_mat[j, k]
    } else {
      # tau3 (tau_1342) = sigma_ik * sigma_jl - sigma_il * sigma_jk
      values[r] <- cov_mat[i, k] * cov_mat[j, l] - cov_mat[i, l] * cov_mat[j, k]
    }
  }

  values
}

#' Format a tetrad label for display.
#' @noRd
format_tetrad_label <- function(i, j, k, l, tetrad_id) {
  if (tetrad_id == 1L) {
    paste0("s(", i, ",", j, ")s(", k, ",", l, ") - s(",
           i, ",", k, ")s(", j, ",", l, ")")
  } else if (tetrad_id == 2L) {
    paste0("s(", i, ",", j, ")s(", k, ",", l, ") - s(",
           i, ",", l, ")s(", j, ",", k, ")")
  } else {
    # tau_1342: i,j own; k,l borrowed
    paste0("s(", i, ",", k, ")s(", j, ",", l, ") - s(",
           i, ",", l, ")s(", j, ",", k, ")")
  }
}

#' Resolve indicators for a construct, handling HOC chaining.
#'
#' For standard constructs, returns raw indicator names and the data source.
#' For HOC constructs, returns LOC names as "indicators" and uses construct
#' scores as the data source (testing whether the HOC is reflective over LOCs).
#'
#' @param construct Construct name.
#' @param model seminr model object.
#' @return List with: indicators (character vector), data (matrix),
#'   is_hoc (logical), needs_reestimation (logical).
#' @noRd
resolve_indicators <- function(construct, model) {
  items <- items_of_construct(construct, model)
  construct_cols <- colnames(model$outer_weights)

  # Detect HOC: items that are themselves construct names in outer_weights
  loc_names <- items[items %in% construct_cols]

  if (length(loc_names) > 0 && length(loc_names) == length(items)) {
    # This is a HOC -- use LOC construct scores as "indicators"
    # LOC scores are in model$data (NOT model$construct_scores)
    list(
      indicators = loc_names,
      data = model$data[, loc_names, drop = FALSE],
      is_hoc = TRUE,
      needs_reestimation = TRUE
    )
  } else {
    # Standard construct -- use raw indicator data
    list(
      indicators = items,
      data = model$data[, items, drop = FALSE],
      is_hoc = FALSE,
      needs_reestimation = FALSE
    )
  }
}


# =============================================================================
# BORROWING HELPERS (Gudergan et al., 2008, Table 1)
# =============================================================================

#' Get measurement mode ("A" or "B") for a construct.
#' @noRd
get_construct_mode <- function(construct, model) {
  mm_sub <- model$mmMatrix[model$mmMatrix[, "construct"] == construct, , drop = FALSE]
  unique(mm_sub[, "type"])[1]
}

#' Get constructs structurally connected (as source or target) to a construct.
#' @noRd
get_structurally_connected <- function(construct, model) {
  sm <- model$smMatrix
  targets <- unname(sm[sm[, "source"] == construct, "target"])
  sources <- unname(sm[sm[, "target"] == construct, "source"])
  unique(c(targets, sources))
}

#' Find the best donor construct for borrowing indicators.
#'
#' Selects the adjacent construct that maximises testable vanishing tetrads.
#' Rules from Gudergan et al. (2008), Table 1:
#' - Formative focal: no vanishing tetrads regardless -> NULL
#' - 3 own (refl.) + 1 borrowed from reflective: ALL tetrads vanish (score 2)
#' - 3 own (refl.) + 1 borrowed from formative: NONE vanish (score 0)
#' - 2 own (refl.) + 2 borrowed: only tau_1342 vanishes (score 1, any donor mode)
#'
#' @return List with donor info, or NULL if no suitable donor found.
#' @noRd
find_donor <- function(focal, focal_info, model) {
  focal_mode <- get_construct_mode(focal, model)
  if (focal_mode != "A") return(NULL)

  n_own <- length(focal_info$indicators)
  if (n_own < 2 || n_own >= 4) return(NULL)

  n_borrow <- 4L - n_own

  neighbors <- get_structurally_connected(focal, model)
  neighbors <- neighbors[!grepl("*", neighbors, fixed = TRUE)]
  if (length(neighbors) == 0) return(NULL)

  best <- NULL
  best_score <- 0L


  for (donor_name in neighbors) {
    donor_info <- resolve_indicators(donor_name, model)

    # Don't mix HOC (LOC scores) with standard (raw indicators)
    if (focal_info$is_hoc != donor_info$is_hoc) next
    if (length(donor_info$indicators) < n_borrow) next

    donor_mode <- get_construct_mode(donor_name, model)

    if (n_own == 3L) {
      if (donor_mode == "A") {
        score <- 2L; pattern <- "all"
      } else {
        score <- 0L; pattern <- "none"
      }
    } else {
      # n_own == 2: tau_1342 vanishes regardless of donor mode
      score <- 1L; pattern <- "tau_1342"
    }

    if (score > best_score ||
        (score == best_score && !is.null(best) &&
         length(donor_info$indicators) > best$n_donor_indicators)) {
      best_score <- score
      best <- list(
        construct = donor_name,
        mode = donor_mode,
        donor_indicators = donor_info$indicators,
        donor_data = donor_info$data,
        is_hoc = donor_info$is_hoc,
        needs_reestimation = donor_info$needs_reestimation,
        borrowed = donor_info$indicators[seq_len(n_borrow)],
        vanishing_pattern = pattern,
        n_vanishing = score,
        n_donor_indicators = length(donor_info$indicators)
      )
    }
  }

  if (is.null(best) || best$n_vanishing == 0L) return(NULL)
  best
}

#' Enumerate vanishing tetrads for the borrowing case.
#'
#' @param own_indicators Character vector of focal construct's own indicators.
#' @param borrowed_indicators Character vector of borrowed indicator names.
#' @param vanishing_pattern "all" (standard enumeration) or "tau_1342".
#' @return Data frame with columns i, j, k, l, tetrad_id.
#' @noRd
enumerate_borrowed_tetrads <- function(own_indicators, borrowed_indicators,
                                       vanishing_pattern) {
  if (vanishing_pattern == "all") {
    return(enumerate_tetrads(c(own_indicators, borrowed_indicators)))
  }

  # tau_1342: sigma_{ik}*sigma_{jl} - sigma_{il}*sigma_{jk}
  # i, j from own; k, l from borrowed
  own_combos <- utils::combn(own_indicators, 2, simplify = FALSE)
  bor_combos <- utils::combn(borrowed_indicators, 2, simplify = FALSE)

  results <- vector("list", length(own_combos) * length(bor_combos))
  idx <- 1L
  for (oc in own_combos) {
    for (bc in bor_combos) {
      results[[idx]] <- data.frame(
        i = oc[1], j = oc[2], k = bc[1], l = bc[2],
        tetrad_id = 3L, stringsAsFactors = FALSE
      )
      idx <- idx + 1L
    }
  }

  do.call(rbind, results)
}

# =============================================================================
# MAIN EXPORTED FUNCTION
# =============================================================================

#' Confirmatory Tetrad Analysis for PLS-SEM (CTA-PLS)
#'
#' `assess_cta` tests whether each construct's measurement model is consistent
#' with a reflective (common factor) specification. Under a reflective model,
#' all vanishing tetrads equal zero. If any tetrad is significantly non-zero
#' (after multiple testing correction), the reflective specification is rejected.
#'
#' @param seminr_model A PLS-SEM model estimated by [seminr::estimate_pls()].
#' @param constructs Character vector of construct names to test. If `NULL`
#'   (default), all constructs in the model are tested.
#' @param nboot Number of bootstrap subsamples (default 5000).
#' @param seed Seed for reproducibility (default 123).
#' @param alpha Significance level (default 0.05).
#' @param correction Multiple testing correction method: `"BH"`
#'   (Benjamini-Hochberg, default), `"bonferroni"`, or `"none"`.
#' @param borrow Logical. If `TRUE` (default), constructs with 2--3
#'   indicators borrow indicators from structurally connected constructs
#'   to reach the minimum of 4 needed for tetrad testing (Gudergan et al.,
#'   2008, Table 1). If `FALSE`, constructs with fewer than 4 indicators
#'   are skipped.
#'
#' @return An S3 object of class `cta_analysis` containing:
#' \describe{
#'   \item{construct_results}{Data frame summarising each construct: mode,
#'     number of indicators, tetrads tested, significant tetrads, and verdict.}
#'   \item{tetrad_details}{Named list of per-construct data frames with
#'     individual tetrad estimates, bootstrap CIs, and adjusted p-values.}
#'   \item{nboot}{Number of bootstrap subsamples used.}
#'   \item{alpha}{Significance level used.}
#'   \item{correction}{Multiple testing correction method used.}
#'   \item{skipped}{Character vector of constructs that were skipped.}
#'   \item{borrowing}{Named list of borrowing details for constructs that
#'     borrowed indicators (donor name, mode, and vanishing pattern).}
#' }
#'
#' @details
#' **Minimum indicator requirement:** Without borrowing, constructs with fewer
#' than 4 indicators are skipped. With `borrow = TRUE` (default), constructs
#' with 2--3 indicators borrow from structurally adjacent constructs.
#' A minimum of 2 own indicators is always required.
#'
#' **Borrowing rules** (Gudergan et al., 2008, Table 1): For a reflective
#' focal construct with 3 own indicators, 1 indicator is borrowed from an
#' adjacent reflective construct; all tetrads vanish under H0. For 2 own
#' indicators, 2 are borrowed from any adjacent construct; only the
#' tau_1342 tetrad vanishes. Formative focal constructs cannot be tested
#' via borrowing.
#'
#' For HOC constructs, the lower-order constructs
#' (LOCs) serve as "indicators"; at least 4 LOCs are required.
#'
#' **Interaction constructs** (moderation terms with `*` in the name) are
#' automatically excluded because their measurement specification is determined
#' by the interaction method, not the data.
#'
#' **Multiple testing correction:** The number of tetrads grows combinatorially
#' with indicators. Benjamini-Hochberg (BH) correction is recommended as the
#' default (Cefis et al., 2025). Bonferroni is more conservative. With only
#' 4 indicators (2 tetrads), correction has minimal impact.
#'
#' **Sample size:** CTA-PLS requires adequate sample sizes for reliable results.
#' A warning is issued if N < 200.
#'
#' @seealso [congruence_test()] for congruence coefficient testing
#'
#' @references
#' Gudergan, S. P., Ringle, C. M., Wende, S. & Will, A. (2008). Confirmatory
#' Tetrad Analysis in PLS Path Modeling. *Journal of Business Research*, 61(12),
#' 1238-1249.
#'
#' Cefis, M., Angelelli, M., Carpita, M. & Ciavolino, E. (2025). Confirmatory
#' Tetrad Analysis in PLS-SEM: A Multiple Testing Correction Perspective.
#' *Social Indicators Research* (advance online).
#'
#' @examples
#' library(seminr)
#' library(seminrExtras)
#'
#' # Specify measurement model
#' mobi_mm <- constructs(
#'   composite("Image",        multi_items("IMAG", 1:5)),
#'   composite("Expectation",  multi_items("CUEX", 1:3)),
#'   composite("Value",        multi_items("PERV", 1:2)),
#'   composite("Satisfaction", multi_items("CUSA", 1:3)),
#'   composite("Loyalty",      multi_items("CUSL", 1:3))
#' )
#'
#' # Specify structural model
#' mobi_sm <- relationships(
#'   paths(from = "Image",       to = c("Expectation", "Satisfaction", "Loyalty")),
#'   paths(from = "Expectation", to = c("Value", "Satisfaction")),
#'   paths(from = "Value",       to = "Satisfaction"),
#'   paths(from = "Satisfaction", to = "Loyalty")
#' )
#'
#' # Estimate the model
#' pls_model <- estimate_pls(data = mobi, measurement_model = mobi_mm,
#'                            structural_model = mobi_sm)
#'
#' # Run CTA-PLS with borrowing (low nboot for example speed)
#' cta <- assess_cta(pls_model, nboot = 50, seed = 123)
#' print(cta)
#' summary(cta)
#'
#' # Without borrowing -- only constructs with >= 4 indicators are tested
#' cta_no_borrow <- assess_cta(pls_model, nboot = 50, borrow = FALSE)
#'
#' @export
assess_cta <- function(seminr_model,
                       constructs = NULL,
                       nboot = 5000,
                       seed = 123,
                       alpha = 0.05,
                       correction = "BH",
                       borrow = TRUE) {

  # ---------------------------------------------------------------------------
  # Step 1: Validate inputs
  # ---------------------------------------------------------------------------
  if (!validate_seminr_model(seminr_model, "assess_cta")) {
    return(NULL)
  }

  correction <- match.arg(correction, c("BH", "bonferroni", "none"))

  n_obs <- nrow(seminr_model$data)
  if (n_obs < 200) {
    message("Note: CTA-PLS has limited power with N < 200 (current N = ", n_obs,
            "). Interpret results with caution.")
  }

  # ---------------------------------------------------------------------------
  # Step 2: Determine which constructs to test
  # ---------------------------------------------------------------------------
  all_constructs <- unique(seminr_model$mmMatrix[, "construct"])

  if (is.null(constructs)) {
    constructs <- all_constructs
  } else {
    invalid <- setdiff(constructs, all_constructs)
    if (length(invalid) > 0) {
      warning("Constructs not found in model: ",
              paste(invalid, collapse = ", "), call. = FALSE)
      constructs <- intersect(constructs, all_constructs)
    }
    if (length(constructs) == 0) {
      warning("No valid constructs to test.", call. = FALSE)
      return(NULL)
    }
  }

  # Exclude interaction constructs
  is_intxn <- grepl("*", constructs, fixed = TRUE)
  excluded_interactions <- constructs[is_intxn]
  if (length(excluded_interactions) > 0) {
    message("Excluding interaction constructs (measurement determined by method): ",
            paste(excluded_interactions, collapse = ", "))
    constructs <- constructs[!is_intxn]
  }

  # ---------------------------------------------------------------------------
  # Step 3: Resolve indicators per construct and check minimum count
  # ---------------------------------------------------------------------------
  skipped <- character(0)
  test_constructs <- character(0)
  construct_info <- list()
  borrowing_details <- list()

  for (construct in constructs) {
    info <- resolve_indicators(construct, seminr_model)

    if (length(info$indicators) >= 4) {
      # Standard case: enough indicators
      test_constructs <- c(test_constructs, construct)
      info$borrowing <- NULL
      construct_info[[construct]] <- info
    } else if (borrow && length(info$indicators) >= 2) {
      # Attempt borrowing
      donor <- find_donor(construct, info, seminr_model)
      if (!is.null(donor)) {
        test_constructs <- c(test_constructs, construct)
        borrowed_data <- donor$donor_data[, donor$borrowed, drop = FALSE]
        construct_info[[construct]] <- list(
          indicators = c(info$indicators, donor$borrowed),
          data = cbind(info$data, borrowed_data),
          is_hoc = info$is_hoc || donor$is_hoc,
          needs_reestimation = info$needs_reestimation || donor$needs_reestimation,
          own_indicators = info$indicators,
          borrowed_indicators = donor$borrowed,
          borrowing = list(
            donor = donor$construct,
            donor_mode = donor$mode,
            vanishing_pattern = donor$vanishing_pattern,
            n_vanishing = donor$n_vanishing
          )
        )
        borrowing_details[[construct]] <- construct_info[[construct]]$borrowing
        message("Borrowing ", length(donor$borrowed), " indicator(s) from '",
                donor$construct, "' for '", construct, "'.")
      } else {
        message("Skipping '", construct, "': ", length(info$indicators),
                " indicator(s), no suitable donor found for borrowing.")
        skipped <- c(skipped, construct)
      }
    } else {
      n_ind <- length(info$indicators)
      if (borrow) {
        message("Skipping '", construct, "': only ", n_ind,
                " indicator(s); CTA-PLS requires >= 2 for borrowing.")
      } else {
        message("Skipping '", construct, "': only ", n_ind,
                " indicator(s); CTA-PLS requires >= 4. Set borrow = TRUE ",
                "to test constructs with 2-3 indicators.")
      }
      skipped <- c(skipped, construct)
    }
  }

  if (length(test_constructs) == 0) {
    message("No constructs testable. ",
            if (!borrow) "Set borrow = TRUE to test constructs with 2-3 indicators."
            else "All constructs have too few indicators or no suitable donors.")
    result <- list(
      construct_results = data.frame(
        Construct = character(0), Mode = character(0),
        Indicators = integer(0), Tetrads = integer(0),
        Significant = integer(0), Verdict = character(0),
        stringsAsFactors = FALSE
      ),
      tetrad_details = list(),
      nboot = nboot, alpha = alpha, correction = correction,
      skipped = c(skipped, excluded_interactions),
      borrowing = borrowing_details
    )
    class(result) <- c("cta_analysis", class(result))
    return(result)
  }

  # ---------------------------------------------------------------------------
  # Step 4: Set seed and run CTA-PLS for each construct
  # ---------------------------------------------------------------------------
  set.seed(seed)

  # Pre-build summary data frame
  summary_df <- data.frame(
    Construct = character(0), Mode = character(0),
    Indicators = integer(0), Tetrads = integer(0),
    Significant = integer(0), Verdict = character(0),
    stringsAsFactors = FALSE
  )
  details_list <- list()

  # Determine if any construct needs model re-estimation in bootstrap
  any_hoc <- any(vapply(construct_info, function(x) x$needs_reestimation, logical(1)))

  # ---------------------------------------------------------------------------
  # Bootstrap: resample once, compute tetrads for all constructs
  # ---------------------------------------------------------------------------

  # Pre-compute: enumerate tetrads and compute original values per construct
  original_tetrads <- list()
  tetrad_indices <- list()

  for (construct in test_constructs) {
    info <- construct_info[[construct]]
    if (!is.null(info$borrowing)) {
      tetrad_indices[[construct]] <- enumerate_borrowed_tetrads(
        info$own_indicators, info$borrowed_indicators, info$borrowing$vanishing_pattern
      )
    } else {
      tetrad_indices[[construct]] <- enumerate_tetrads(info$indicators)
    }
    cov_mat <- stats::cov(info$data)
    original_tetrads[[construct]] <- compute_tetrads(cov_mat, tetrad_indices[[construct]])
  }

  # Bootstrap array: store tetrad values per bootstrap iteration
  boot_tetrads <- list()
  for (construct in test_constructs) {
    n_tetrads <- nrow(tetrad_indices[[construct]])
    boot_tetrads[[construct]] <- matrix(NA_real_, nrow = nboot, ncol = n_tetrads)
  }

  for (b in seq_len(nboot)) {
    # Resample rows
    idx <- sample(n_obs, n_obs, replace = TRUE)

    # If any HOC constructs need re-estimation, do it once per bootstrap
    if (any_hoc) {
      resampled_data <- seminr_model$rawdata[idx, ]
      boot_model <- tryCatch(
        suppressMessages(seminr::rerun(seminr_model, data = resampled_data)),
        error = function(e) NULL
      )
    }

    for (construct in test_constructs) {
      info <- construct_info[[construct]]

      if (info$needs_reestimation) {
        # HOC: need LOC construct scores from re-estimated model (in $data)
        if (is.null(boot_model)) {
          # Re-estimation failed; leave NAs
          next
        }
        boot_data <- boot_model$data[, info$indicators, drop = FALSE]
      } else {
        # Standard: just resample the indicator data
        boot_data <- info$data[idx, , drop = FALSE]
      }

      boot_cov <- stats::cov(boot_data)
      boot_tetrads[[construct]][b, ] <- compute_tetrads(boot_cov,
                                                         tetrad_indices[[construct]])
    }
  }

  # ---------------------------------------------------------------------------
  # Step 5: Compute p-values and CIs per construct
  # ---------------------------------------------------------------------------
  alpha_half <- alpha / 2

  for (construct in test_constructs) {
    info <- construct_info[[construct]]
    td <- tetrad_indices[[construct]]
    orig <- original_tetrads[[construct]]
    boot_mat <- boot_tetrads[[construct]]
    n_tetrads <- length(orig)

    # Get measurement mode from mmMatrix
    mm_sub <- seminr_model$mmMatrix[seminr_model$mmMatrix[, "construct"] == construct, ]
    if (is.matrix(mm_sub)) {
      mode_type <- unique(mm_sub[, "type"])[1]
    } else {
      mode_type <- mm_sub["type"]
    }
    mode_label <- if (mode_type == "A") "Mode A (reflective)" else "Mode B (formative)"

    # For HOC: mode is determined by the higher_composite specification
    if (info$is_hoc && is.null(info$borrowing)) {
      hoc_weights <- seminr_model$outer_weights[info$indicators, construct]
      mode_label <- paste0(mode_label, " [HOC]")
    } else if (info$is_hoc) {
      mode_label <- paste0(mode_label, " [HOC]")
    }

    # Add borrowing annotation
    if (!is.null(info$borrowing)) {
      mode_label <- paste0(mode_label, " [borrowed from ", info$borrowing$donor, "]")
    }

    # Compute per-tetrad statistics
    tetrad_labels <- character(n_tetrads)
    t_values <- numeric(n_tetrads)
    boot_means <- numeric(n_tetrads)
    boot_sds <- numeric(n_tetrads)
    ci_lower <- numeric(n_tetrads)
    ci_upper <- numeric(n_tetrads)
    p_values <- numeric(n_tetrads)

    for (t_idx in seq_len(n_tetrads)) {
      tetrad_labels[t_idx] <- format_tetrad_label(
        td$i[t_idx], td$j[t_idx], td$k[t_idx], td$l[t_idx], td$tetrad_id[t_idx]
      )

      boot_vals <- boot_mat[, t_idx]
      boot_vals <- boot_vals[!is.na(boot_vals)]

      if (length(boot_vals) < 10) {
        t_values[t_idx] <- NA_real_
        boot_means[t_idx] <- NA_real_
        boot_sds[t_idx] <- NA_real_
        ci_lower[t_idx] <- NA_real_
        ci_upper[t_idx] <- NA_real_
        p_values[t_idx] <- NA_real_
        next
      }

      boot_means[t_idx] <- mean(boot_vals)
      boot_sds[t_idx] <- sd(boot_vals)

      # Bootstrap t-value: tau / se (Gudergan et al., 2008, p. 1242)
      if (boot_sds[t_idx] < .Machine$double.eps) {
        t_values[t_idx] <- NA_real_
      } else {
        t_values[t_idx] <- orig[t_idx] / boot_sds[t_idx]
      }

      # Percentile confidence interval
      ci_lower[t_idx] <- stats::quantile(boot_vals, probs = alpha_half)
      ci_upper[t_idx] <- stats::quantile(boot_vals, probs = 1 - alpha_half)

      # Two-sided p-value: proportion of bootstrap distribution on opposite
      # side of zero from the observed value (or crossing zero)
      # H0: tau = 0. Reject if CI does not include 0.
      # P-value = 2 * min(proportion >= 0, proportion <= 0)
      p_above <- mean(boot_vals >= 0)
      p_below <- mean(boot_vals <= 0)
      p_values[t_idx] <- 2 * min(p_above, p_below)
    }

    # Apply multiple testing correction
    if (correction == "none" || all(is.na(p_values))) {
      adj_p <- p_values
    } else {
      adj_p <- stats::p.adjust(p_values, method = correction)
    }

    # Significance determination
    significant <- !is.na(adj_p) & adj_p <= alpha

    # Build detail data.frame
    ci_lower_name <- paste0(alpha_half * 100, "% CI")
    ci_upper_name <- paste0((1 - alpha_half) * 100, "% CI")

    detail_df <- data.frame(
      Tetrad = tetrad_labels,
      Estimate = orig,
      T_Value = t_values,
      Boot_Mean = boot_means,
      Boot_SD = boot_sds,
      CI_Lower = ci_lower,
      CI_Upper = ci_upper,
      P_Value = p_values,
      Adj_P = adj_p,
      Significant = significant,
      stringsAsFactors = FALSE,
      check.names = FALSE
    )
    colnames(detail_df)[6:7] <- c(ci_lower_name, ci_upper_name)

    details_list[[construct]] <- detail_df

    # Verdict
    n_sig <- sum(significant, na.rm = TRUE)
    if (n_sig == 0) {
      verdict <- "Reflective supported"
    } else {
      verdict <- "Reflective rejected"
    }

    summary_row <- data.frame(
      Construct = construct,
      Mode = mode_label,
      Indicators = length(info$indicators),
      Tetrads = n_tetrads,
      Significant = n_sig,
      Verdict = verdict,
      stringsAsFactors = FALSE
    )
    summary_df <- rbind(summary_df, summary_row)
  }

  # ---------------------------------------------------------------------------
  # Step 6: Compile result
  # ---------------------------------------------------------------------------
  result <- list(
    construct_results = summary_df,
    tetrad_details = details_list,
    nboot = nboot,
    alpha = alpha,
    correction = correction,
    skipped = c(skipped, excluded_interactions),
    borrowing = borrowing_details
  )
  class(result) <- c("cta_analysis", class(result))

  result
}


# =============================================================================
# S3 METHODS
# =============================================================================

#' @export
print.cta_analysis <- function(x, ...) {
  cat("Confirmatory Tetrad Analysis (CTA-PLS)\n")
  cat("=======================================\n")
  cat("Bootstrap samples:", x$nboot, "| Alpha:", x$alpha,
      "| Correction:", x$correction, "\n\n")

  if (nrow(x$construct_results) == 0) {
    cat("No constructs tested (all had < 4 indicators).\n")
  } else {
    # Print summary table
    df <- x$construct_results
    col_widths <- pmax(nchar(colnames(df)),
                       apply(df, 2, function(col) max(nchar(as.character(col)))))

    # Header
    header <- paste(mapply(function(name, w) formatC(name, width = w, flag = "-"),
                           colnames(df), col_widths), collapse = "  ")
    cat(header, "\n")
    cat(paste(rep("-", nchar(header)), collapse = ""), "\n")

    # Rows
    for (r in seq_len(nrow(df))) {
      row_str <- paste(mapply(function(val, w) formatC(as.character(val), width = w, flag = "-"),
                              df[r, ], col_widths), collapse = "  ")
      cat(row_str, "\n")
    }
  }

  if (length(x$borrowing) > 0) {
    cat("\nBorrowing:\n")
    for (cname in names(x$borrowing)) {
      b <- x$borrowing[[cname]]
      cat("  ", cname, ": borrowed from ", b$donor,
          " (", b$vanishing_pattern, " pattern, ",
          b$n_vanishing, " vanishing tetrad(s))\n", sep = "")
    }
  }

  if (length(x$skipped) > 0) {
    cat("\nSkipped:", paste(x$skipped, collapse = ", "), "\n")
  }

  invisible(x)
}

#' @export
summary.cta_analysis <- function(object, ...) {
  result <- list(
    construct_results = object$construct_results,
    tetrad_details = object$tetrad_details,
    nboot = object$nboot,
    alpha = object$alpha,
    correction = object$correction,
    skipped = object$skipped,
    borrowing = object$borrowing
  )
  class(result) <- c("summary.cta_analysis", class(result))
  result
}

#' @export
print.summary.cta_analysis <- function(x, digits = 4, ...) {
  cat("Confirmatory Tetrad Analysis (CTA-PLS) \u2014 Detailed Results\n")
  cat("==========================================================\n")
  cat("Bootstrap samples:", x$nboot, "| Alpha:", x$alpha,
      "| Correction:", x$correction, "\n\n")

  for (construct in names(x$tetrad_details)) {
    # Construct header with verdict
    row_idx <- which(x$construct_results$Construct == construct)
    verdict <- x$construct_results$Verdict[row_idx]
    mode <- x$construct_results$Mode[row_idx]

    cat("--- ", construct, " (", mode, ") ---\n", sep = "")
    if (construct %in% names(x$borrowing)) {
      b <- x$borrowing[[construct]]
      cat("Borrowed ", b$n_vanishing, " vanishing tetrad(s) from ", b$donor,
          " (", b$vanishing_pattern, ")\n", sep = "")
    }
    cat("Verdict: ", verdict, "\n\n", sep = "")

    detail <- x$tetrad_details[[construct]]

    # Format numeric columns
    numeric_cols <- c("Estimate", "T_Value", "Boot_Mean", "Boot_SD",
                      colnames(detail)[6], colnames(detail)[7],
                      "P_Value", "Adj_P")
    for (col in numeric_cols) {
      if (col %in% colnames(detail)) {
        detail[[col]] <- formatC(as.numeric(detail[[col]]),
                                 format = "f", digits = digits)
      }
    }
    detail$Significant <- ifelse(detail$Significant, "*", "")

    print(detail, row.names = FALSE, right = FALSE)
    cat("\n")
  }

  if (length(x$skipped) > 0) {
    cat("Skipped constructs: ", paste(x$skipped, collapse = ", "), "\n")
  }

  invisible(x)
}

#' @export
plot.cta_analysis <- function(x, ...) {
  if (nrow(x$construct_results) == 0) {
    message("No constructs were tested \u2014 nothing to plot.")
    return(invisible(x))
  }

  # Collect all adjusted p-values per construct
  constructs <- names(x$tetrad_details)
  n_constructs <- length(constructs)

  # Determine layout
  old_par <- par(no.readonly = TRUE)
  on.exit(par(old_par))

  if (n_constructs <= 4) {
    n_cols <- min(n_constructs, 2)
    n_rows <- ceiling(n_constructs / n_cols)
  } else {
    n_cols <- 3
    n_rows <- ceiling(n_constructs / n_cols)
  }
  par(mfrow = c(n_rows, n_cols), mar = c(4, 5, 3, 1))

  for (construct in constructs) {
    detail <- x$tetrad_details[[construct]]
    adj_p <- as.numeric(detail$Adj_P)
    n_tetrads <- length(adj_p)

    row_idx <- which(x$construct_results$Construct == construct)
    verdict <- x$construct_results$Verdict[row_idx]

    # Colour: green if supported, red if rejected
    cols <- ifelse(adj_p <= x$alpha, "firebrick", "steelblue")

    # Dot plot of adjusted p-values
    plot(seq_len(n_tetrads), adj_p,
         pch = 19, col = cols, cex = 1.2,
         xlab = "Tetrad index", ylab = "Adjusted p-value",
         main = construct,
         ylim = c(0, max(1, max(adj_p, na.rm = TRUE))),
         xaxt = "n", ...)
    axis(1, at = seq_len(n_tetrads))
    abline(h = x$alpha, lty = 2, col = "grey40")
    legend("topright", legend = verdict, bty = "n", cex = 0.8,
           text.col = if (grepl("supported", verdict)) "steelblue" else "firebrick")
  }

  invisible(x)
}

# =============================================================================
# Tests for assess_cta() — Confirmatory Tetrad Analysis for PLS-SEM
# =============================================================================

library(seminr)

# --- Setup: pre-compute models and results -----------------------------------

# Simple model with a 5-indicator construct (Image) and several 2-3 indicator ones
mobi_mm <- constructs(
  composite("Image",        multi_items("IMAG", 1:5)),
  composite("Expectation",  multi_items("CUEX", 1:3)),
  composite("Value",        multi_items("PERV", 1:2)),
  composite("Satisfaction", multi_items("CUSA", 1:3)),
  composite("Loyalty",      multi_items("CUSL", 1:3))
)
mobi_sm <- relationships(
  paths(from = "Image",       to = c("Expectation", "Satisfaction", "Loyalty")),
  paths(from = "Expectation", to = c("Value", "Satisfaction")),
  paths(from = "Value",       to = "Satisfaction"),
  paths(from = "Satisfaction", to = "Loyalty")
)
pls_model <- estimate_pls(data = mobi, measurement_model = mobi_mm,
                           structural_model = mobi_sm)

# Pre-compute CTA with borrowing (default) and without borrowing
cta_result <- assess_cta(pls_model, nboot = 50, seed = 123)
cta_no_borrow <- assess_cta(pls_model, nboot = 50, seed = 123, borrow = FALSE)

# =============================================================================
# STRUCTURE TESTS
# =============================================================================

test_that("assess_cta returns cta_analysis object", {
  expect_s3_class(cta_result, "cta_analysis")
})

test_that("cta result contains all expected components", {
  expected_names <- c("construct_results", "tetrad_details", "nboot",
                      "alpha", "correction", "skipped", "borrowing")
  expect_true(all(expected_names %in% names(cta_result)))
})

test_that("construct_results is a data frame with correct columns", {
  df <- cta_result$construct_results
  expect_s3_class(df, "data.frame")
  expected_cols <- c("Construct", "Mode", "Indicators", "Tetrads",
                     "Significant", "Verdict")
  expect_equal(colnames(df), expected_cols)
})

test_that("tetrad_details is a named list matching construct_results", {
  expect_type(cta_result$tetrad_details, "list")
  expect_equal(names(cta_result$tetrad_details),
               cta_result$construct_results$Construct)
})

# =============================================================================
# NO-BORROW TESTS (borrow = FALSE)
# =============================================================================

test_that("constructs with < 4 indicators are skipped when borrow = FALSE", {
  expect_true("Value" %in% cta_no_borrow$skipped)
  expect_true("Expectation" %in% cta_no_borrow$skipped)
  expect_true("Loyalty" %in% cta_no_borrow$skipped)
  expect_true("Satisfaction" %in% cta_no_borrow$skipped)
})

test_that("Image (5 indicators) is tested without borrowing", {
  expect_true("Image" %in% cta_no_borrow$construct_results$Construct)
})

test_that("correct number of tetrads for 5 indicators without borrowing", {
  # C(5,4) = 5 four-tuples, 2 tetrads each = 10 tetrads
  img_row <- cta_no_borrow$construct_results[
    cta_no_borrow$construct_results$Construct == "Image", ]
  expect_equal(img_row$Indicators, 5)
  expect_equal(img_row$Tetrads, 10)
})

# =============================================================================
# BORROWING TESTS
# =============================================================================

test_that("constructs with 3 indicators are tested via borrowing", {
  tested <- cta_result$construct_results$Construct
  expect_true("Expectation" %in% tested)
  expect_true("Satisfaction" %in% tested)
  expect_true("Loyalty" %in% tested)
})

test_that("construct with 2 indicators is tested via borrowing", {
  expect_true("Value" %in% cta_result$construct_results$Construct)
})

test_that("borrowing details recorded in result", {
  expect_type(cta_result$borrowing, "list")
  # All 4 constructs with < 4 indicators should have borrowing details
  expect_true("Expectation" %in% names(cta_result$borrowing))
  expect_true("Value" %in% names(cta_result$borrowing))
  expect_true("Satisfaction" %in% names(cta_result$borrowing))
  expect_true("Loyalty" %in% names(cta_result$borrowing))
})

test_that("3-indicator construct borrows with 'all' vanishing pattern", {
  # Expectation (3 own, Mode A) borrows from a Mode A neighbor
  b <- cta_result$borrowing[["Expectation"]]
  expect_equal(b$vanishing_pattern, "all")
  expect_equal(b$n_vanishing, 2L)
})

test_that("2-indicator construct borrows with 'tau_1342' vanishing pattern", {
  # Value (2 own, Mode A) borrows 2 indicators
  b <- cta_result$borrowing[["Value"]]
  expect_equal(b$vanishing_pattern, "tau_1342")
  expect_equal(b$n_vanishing, 1L)
})

test_that("borrowed construct has correct indicator count", {
  # 3 own + 1 borrowed = 4 total
  exp_row <- cta_result$construct_results[
    cta_result$construct_results$Construct == "Expectation", ]
  expect_equal(exp_row$Indicators, 4)

  # 2 own + 2 borrowed = 4 total
  val_row <- cta_result$construct_results[
    cta_result$construct_results$Construct == "Value", ]
  expect_equal(val_row$Indicators, 4)
})

test_that("borrowed 3+1 construct has 2 tetrads", {
  exp_row <- cta_result$construct_results[
    cta_result$construct_results$Construct == "Expectation", ]
  expect_equal(exp_row$Tetrads, 2)
})

test_that("borrowed 2+2 construct has 1 tetrad (tau_1342 only)", {
  val_row <- cta_result$construct_results[
    cta_result$construct_results$Construct == "Value", ]
  expect_equal(val_row$Tetrads, 1)
})

test_that("borrowed tetrad detail has tetrad_id 3 for tau_1342", {
  td <- cta_result$tetrad_details[["Value"]]
  # The label should show the tau_1342 pattern
  expect_equal(nrow(td), 1)
  expect_true(grepl("^s\\(", td$Tetrad))
})

test_that("mode label shows borrowing annotation", {
  exp_row <- cta_result$construct_results[
    cta_result$construct_results$Construct == "Expectation", ]
  expect_true(grepl("borrowed from", exp_row$Mode))
})

test_that("Image is not borrowed (has enough indicators)", {
  expect_false("Image" %in% names(cta_result$borrowing))
})

test_that("no constructs skipped with borrowing in mobi model", {
  expect_length(cta_result$skipped, 0)
})

# =============================================================================
# TETRAD DETAIL TESTS
# =============================================================================

test_that("tetrad detail data frame has correct structure", {
  detail <- cta_result$tetrad_details[["Image"]]
  expect_s3_class(detail, "data.frame")
  expect_true("Tetrad" %in% colnames(detail))
  expect_true("Estimate" %in% colnames(detail))
  expect_true("Boot_Mean" %in% colnames(detail))
  expect_true("Boot_SD" %in% colnames(detail))
  expect_true("P_Value" %in% colnames(detail))
  expect_true("Adj_P" %in% colnames(detail))
  expect_true("Significant" %in% colnames(detail))
})

test_that("tetrad estimates are finite", {
  detail <- cta_result$tetrad_details[["Image"]]
  expect_true(all(is.finite(detail$Estimate)))
})

test_that("bootstrap SDs are non-negative", {
  detail <- cta_result$tetrad_details[["Image"]]
  expect_true(all(detail$Boot_SD >= 0, na.rm = TRUE))
})

test_that("p-values are between 0 and 1", {
  detail <- cta_result$tetrad_details[["Image"]]
  expect_true(all(detail$P_Value >= 0 & detail$P_Value <= 1, na.rm = TRUE))
  expect_true(all(detail$Adj_P >= 0 & detail$Adj_P <= 1, na.rm = TRUE))
})

test_that("confidence intervals are ordered correctly", {
  detail <- cta_result$tetrad_details[["Image"]]
  ci_cols <- grep("% CI", colnames(detail), value = TRUE)
  expect_length(ci_cols, 2)
  lower <- as.numeric(detail[[ci_cols[1]]])
  upper <- as.numeric(detail[[ci_cols[2]]])
  expect_true(all(lower <= upper, na.rm = TRUE))
})

test_that("verdict is Reflective supported or rejected", {
  verdicts <- cta_result$construct_results$Verdict
  valid_verdicts <- c("Reflective supported", "Reflective rejected")
  expect_true(all(verdicts %in% valid_verdicts))
})

# =============================================================================
# REPRODUCIBILITY TESTS
# =============================================================================

test_that("assess_cta is reproducible with same seed", {
  r1 <- assess_cta(pls_model, nboot = 30, seed = 42)
  r2 <- assess_cta(pls_model, nboot = 30, seed = 42)
  d1 <- r1$tetrad_details[["Image"]]
  d2 <- r2$tetrad_details[["Image"]]
  expect_equal(d1$Estimate, d2$Estimate)
  expect_equal(d1$Boot_SD, d2$Boot_SD)
})

test_that("assess_cta differs with different seeds", {
  r1 <- assess_cta(pls_model, nboot = 30, seed = 42)
  r2 <- assess_cta(pls_model, nboot = 30, seed = 99)
  d1 <- r1$tetrad_details[["Image"]]
  d2 <- r2$tetrad_details[["Image"]]
  # Original estimates identical (same model)
  expect_equal(d1$Estimate, d2$Estimate)
  # Bootstrap SDs differ
  expect_false(identical(d1$Boot_SD, d2$Boot_SD))
})

# =============================================================================
# PARAMETER VARIATION TESTS
# =============================================================================

test_that("correction methods produce different adjusted p-values", {
  r_bh <- assess_cta(pls_model, constructs = "Image", nboot = 50,
                      seed = 123, correction = "BH")
  r_bon <- assess_cta(pls_model, constructs = "Image", nboot = 50,
                       seed = 123, correction = "bonferroni")
  r_none <- assess_cta(pls_model, constructs = "Image", nboot = 50,
                        seed = 123, correction = "none")

  d_bh <- r_bh$tetrad_details[["Image"]]
  d_bon <- r_bon$tetrad_details[["Image"]]
  d_none <- r_none$tetrad_details[["Image"]]

  # Raw p-values should be identical (same bootstrap)
  expect_equal(d_bh$P_Value, d_bon$P_Value)
  expect_equal(d_bh$P_Value, d_none$P_Value)

  # Adjusted p-values: none <= BH <= Bonferroni (generally)
  # At minimum, "none" adjusted = raw
  expect_equal(d_none$P_Value, d_none$Adj_P)
})

test_that("alpha parameter affects CI column names", {
  r_05 <- assess_cta(pls_model, constructs = "Image",
                      nboot = 30, seed = 123, alpha = 0.05)
  r_10 <- assess_cta(pls_model, constructs = "Image",
                      nboot = 30, seed = 123, alpha = 0.10)

  d_05 <- r_05$tetrad_details[["Image"]]
  d_10 <- r_10$tetrad_details[["Image"]]

  expect_true("2.5% CI" %in% colnames(d_05))
  expect_true("97.5% CI" %in% colnames(d_05))
  expect_true("5% CI" %in% colnames(d_10))
  expect_true("95% CI" %in% colnames(d_10))
})

test_that("selecting specific constructs works", {
  r <- assess_cta(pls_model, constructs = "Image", nboot = 30, seed = 123)
  expect_equal(nrow(r$construct_results), 1)
  expect_equal(r$construct_results$Construct, "Image")
  expect_length(r$skipped, 0)
})

# =============================================================================
# INPUT VALIDATION TESTS
# =============================================================================

test_that("assess_cta rejects non-seminr model objects", {
  expect_warning(
    result <- assess_cta(list(not = "a_model"), nboot = 30),
    "only works with SEMinR models"
  )
  expect_null(result)
})

test_that("assess_cta warns for invalid construct names", {
  expect_warning(
    r <- assess_cta(pls_model, constructs = c("Image", "Nonexistent"), nboot = 30),
    "Constructs not found"
  )
  expect_equal(nrow(r$construct_results), 1)
})

test_that("assess_cta returns empty result when all constructs invalid", {
  expect_warning(
    r <- assess_cta(pls_model, constructs = "Nonexistent", nboot = 30),
    "No valid constructs"
  )
  expect_null(r)
})

# =============================================================================
# MODERATION TESTS
# =============================================================================

test_that("interaction constructs are auto-excluded", {
  mm_mod <- constructs(
    composite("Image",        multi_items("IMAG", 1:5)),
    composite("Value",        multi_items("PERV", 1:2)),
    composite("Satisfaction", multi_items("CUSA", 1:3)),
    interaction_term(iv = "Image", moderator = "Value", method = two_stage)
  )
  sm_mod <- relationships(
    paths(from = c("Image", "Value", "Image*Value"), to = "Satisfaction")
  )
  pls_mod <- estimate_pls(data = mobi, measurement_model = mm_mod,
                           structural_model = sm_mod)

  r <- assess_cta(pls_mod, nboot = 30, seed = 123)

  # Interaction construct should be in skipped
  expect_true("Image*Value" %in% r$skipped)

  # Image should still be tested (5 indicators)
  expect_true("Image" %in% r$construct_results$Construct)
})

test_that("moderation model: non-interaction constructs can borrow", {
  mm_mod <- constructs(
    composite("Image",        multi_items("IMAG", 1:5)),
    composite("Value",        multi_items("PERV", 1:2)),
    composite("Satisfaction", multi_items("CUSA", 1:3)),
    interaction_term(iv = "Image", moderator = "Value", method = two_stage)
  )
  sm_mod <- relationships(
    paths(from = c("Image", "Value", "Image*Value"), to = "Satisfaction")
  )
  pls_mod <- estimate_pls(data = mobi, measurement_model = mm_mod,
                           structural_model = sm_mod)

  r <- assess_cta(pls_mod, nboot = 30, seed = 123)

  # Value (2 indicators) should borrow — its neighbors are Image and Satisfaction
  # (Image*Value is excluded as interaction). Both Image and Satisfaction are Mode A.
  # Value needs 2 borrowed, tau_1342 pattern
  expect_true("Value" %in% r$construct_results$Construct ||
              "Value" %in% names(r$borrowing))
})

# =============================================================================
# MEDIATION TESTS
# =============================================================================

test_that("mediation model works with borrowing", {
  mm_med <- constructs(
    composite("Image",        multi_items("IMAG", 1:5)),
    composite("Satisfaction", multi_items("CUSA", 1:3)),
    composite("Loyalty",      multi_items("CUSL", 1:3))
  )
  sm_med <- relationships(
    paths(from = "Image", to = c("Satisfaction", "Loyalty")),
    paths(from = "Satisfaction", to = "Loyalty")
  )
  pls_med <- estimate_pls(data = mobi, measurement_model = mm_med,
                           structural_model = sm_med)

  r <- assess_cta(pls_med, nboot = 30, seed = 123)

  # Image tested directly (5 indicators)
  expect_true("Image" %in% r$construct_results$Construct)

  # Satisfaction (3 indicators) borrows from Image (Mode A, 5 indicators)
  expect_true("Satisfaction" %in% r$construct_results$Construct)
  expect_true("Satisfaction" %in% names(r$borrowing))

  # Loyalty (3 indicators) borrows from Image or Satisfaction
  expect_true("Loyalty" %in% r$construct_results$Construct)
  expect_true("Loyalty" %in% names(r$borrowing))
})

test_that("mediation model: borrow = FALSE skips as before", {
  mm_med <- constructs(
    composite("Image",        multi_items("IMAG", 1:5)),
    composite("Satisfaction", multi_items("CUSA", 1:3)),
    composite("Loyalty",      multi_items("CUSL", 1:3))
  )
  sm_med <- relationships(
    paths(from = "Image", to = c("Satisfaction", "Loyalty")),
    paths(from = "Satisfaction", to = "Loyalty")
  )
  pls_med <- estimate_pls(data = mobi, measurement_model = mm_med,
                           structural_model = sm_med)

  r <- assess_cta(pls_med, nboot = 30, seed = 123, borrow = FALSE)
  expect_true("Satisfaction" %in% r$skipped)
  expect_true("Loyalty" %in% r$skipped)
  expect_true("Image" %in% r$construct_results$Construct)
})

# =============================================================================
# HOC TESTS
# =============================================================================

test_that("HOC with >= 4 LOCs is tested using LOC construct scores", {
  skip_on_cran()
  mm_hoc2 <- constructs(
    composite("IMAG1_2",  multi_items("IMAG", 1:2)),
    composite("IMAG3_4",  multi_items("IMAG", 3:4)),
    composite("CUEX_all", multi_items("CUEX", 1:3)),
    composite("CUSA_all", multi_items("CUSA", 1:3)),
    higher_composite("Super",
                     c("IMAG1_2", "IMAG3_4", "CUEX_all", "CUSA_all"),
                     method = two_stage),
    composite("Loyalty",  multi_items("CUSL", 1:3))
  )
  sm_hoc2 <- relationships(
    paths(from = "Super", to = "Loyalty")
  )
  pls_hoc <- estimate_pls(data = mobi, measurement_model = mm_hoc2,
                           structural_model = sm_hoc2)

  r <- assess_cta(pls_hoc, constructs = "Super", nboot = 30, seed = 123)

  expect_equal(nrow(r$construct_results), 1)
  expect_equal(r$construct_results$Construct, "Super")
  expect_equal(r$construct_results$Indicators, 4)
  # 4 LOCs → C(4,4) = 1 four-tuple → 2 tetrads
  expect_equal(r$construct_results$Tetrads, 2)
  expect_true(grepl("HOC", r$construct_results$Mode))
})

test_that("HOC with < 4 LOCs is skipped (no borrowing across HOC/standard)", {
  skip_on_cran()
  mm_hoc <- constructs(
    composite("Image",        multi_items("IMAG", 1:5)),
    composite("Expectation",  multi_items("CUEX", 1:3)),
    higher_composite("Quality", c("Image", "Expectation"), method = two_stage),
    composite("Satisfaction", multi_items("CUSA", 1:3)),
    composite("Loyalty",      multi_items("CUSL", 1:3))
  )
  sm_hoc <- relationships(
    paths(from = "Quality", to = c("Satisfaction", "Loyalty")),
    paths(from = "Satisfaction", to = "Loyalty")
  )
  pls_hoc <- estimate_pls(data = mobi, measurement_model = mm_hoc,
                           structural_model = sm_hoc)

  r <- assess_cta(pls_hoc, constructs = "Quality", nboot = 30, seed = 123)

  # Quality has 2 LOCs, neighbors Satisfaction/Loyalty are standard → can't borrow
  expect_true("Quality" %in% r$skipped)
  expect_equal(nrow(r$construct_results), 0)
})

test_that("LOCs under HOC are tested independently with their own indicators", {
  skip_on_cran()
  mm_hoc <- constructs(
    composite("Image",        multi_items("IMAG", 1:5)),
    composite("Expectation",  multi_items("CUEX", 1:3)),
    higher_composite("Quality", c("Image", "Expectation"), method = two_stage),
    composite("Satisfaction", multi_items("CUSA", 1:3)),
    composite("Loyalty",      multi_items("CUSL", 1:3))
  )
  sm_hoc <- relationships(
    paths(from = "Quality", to = c("Satisfaction", "Loyalty")),
    paths(from = "Satisfaction", to = "Loyalty")
  )
  pls_hoc <- estimate_pls(data = mobi, measurement_model = mm_hoc,
                           structural_model = sm_hoc)

  # Test Image specifically — it has 5 raw indicators, should work
  r <- assess_cta(pls_hoc, constructs = "Image", nboot = 30, seed = 123)
  expect_true("Image" %in% r$construct_results$Construct)
  expect_equal(r$construct_results$Indicators, 5)
})

# =============================================================================
# FORMATIVE CONSTRUCT BORROWING TESTS
# =============================================================================

test_that("formative focal construct cannot borrow", {
  mm_form <- constructs(
    composite("Image",  multi_items("IMAG", 1:5)),
    composite("Value",  multi_items("PERV", 1:2), weights = mode_B),
    composite("Satisfaction", multi_items("CUSA", 1:3))
  )
  sm_form <- relationships(
    paths(from = c("Image", "Value"), to = "Satisfaction")
  )
  pls_form <- estimate_pls(data = mobi, measurement_model = mm_form,
                            structural_model = sm_form)

  r <- assess_cta(pls_form, constructs = "Value", nboot = 30, seed = 123)

  # Value is Mode B with 2 indicators → find_donor returns NULL for formative focal
  expect_true("Value" %in% r$skipped)
})

test_that("3 own reflective + 1 borrowed from formative: no donor found", {
  # If all neighbors are formative, a 3-indicator reflective construct can't borrow
  mm_mixed <- constructs(
    composite("Image",        multi_items("IMAG", 1:5), weights = mode_B),
    composite("Expectation",  multi_items("CUEX", 1:3)),
    composite("Satisfaction", multi_items("CUSA", 1:3))
  )
  sm_mixed <- relationships(
    paths(from = "Image", to = c("Expectation", "Satisfaction"))
  )
  pls_mixed <- estimate_pls(data = mobi, measurement_model = mm_mixed,
                              structural_model = sm_mixed)

  r <- assess_cta(pls_mixed, constructs = "Expectation", nboot = 30, seed = 123)

  # Expectation's only neighbor (Image) is formative → can't borrow (3+1 from B = 0 vanishing)
  # Satisfaction isn't structurally connected to Expectation
  expect_true("Expectation" %in% r$skipped)
})

# =============================================================================
# S3 METHODS TESTS
# =============================================================================

test_that("print.cta_analysis runs without error", {
  expect_output(print(cta_result), "Confirmatory Tetrad Analysis")
})

test_that("print.cta_analysis shows borrowing info", {
  expect_output(print(cta_result), "Borrowing")
})

test_that("summary.cta_analysis returns summary.cta_analysis class", {
  s <- summary(cta_result)
  expect_s3_class(s, "summary.cta_analysis")
})

test_that("print.summary.cta_analysis runs without error", {
  s <- summary(cta_result)
  expect_output(print(s), "Detailed Results")
})

test_that("print.summary.cta_analysis shows borrowing details", {
  s <- summary(cta_result)
  expect_output(print(s), "Borrowed")
})

test_that("plot.cta_analysis runs without error", {
  expect_no_error(plot(cta_result))
})

# =============================================================================
# TETRAD COMPUTATION CORRECTNESS TESTS
# =============================================================================

test_that("tetrads are zero for perfectly correlated indicators (common factor)", {
  # Generate data from a single factor: x = lambda * f + epsilon
  set.seed(42)
  n <- 500
  f <- rnorm(n)
  lambdas <- c(0.8, 0.7, 0.9, 0.85)
  x <- sapply(lambdas, function(l) l * f + rnorm(n, sd = sqrt(1 - l^2)))
  colnames(x) <- paste0("x", 1:4)

  cov_mat <- cov(x)

  # Compute tetrads
  td <- seminrExtras:::enumerate_tetrads(colnames(x))
  vals <- seminrExtras:::compute_tetrads(cov_mat, td)

  # With n = 500 from a true factor model, tetrads should be close to zero
  expect_true(all(abs(vals) < 0.05))
})

test_that("tetrads are non-zero for composite indicators", {
  set.seed(42)
  n <- 500

  # Create data with a specific non-factor correlation structure
  Sigma <- matrix(c(1.0, 0.5, 0.3, 0.1,
                     0.5, 1.0, 0.6, 0.2,
                     0.3, 0.6, 1.0, 0.7,
                     0.1, 0.2, 0.7, 1.0), nrow = 4)
  x2 <- MASS::mvrnorm(n, mu = rep(0, 4), Sigma = Sigma)
  colnames(x2) <- paste0("x", 1:4)

  cov_mat2 <- cov(x2)
  td <- seminrExtras:::enumerate_tetrads(colnames(x2))
  vals2 <- seminrExtras:::compute_tetrads(cov_mat2, td)

  # At least some tetrads should be notably non-zero for non-factor structure
  expect_true(any(abs(vals2) > 0.01))
})

test_that("tau_1342 (tetrad_id 3) is computed correctly", {
  # Manual verification
  cov_mat <- matrix(c(1.0, 0.5, 0.3, 0.2,
                       0.5, 1.0, 0.4, 0.6,
                       0.3, 0.4, 1.0, 0.7,
                       0.2, 0.6, 0.7, 1.0), nrow = 4)
  rownames(cov_mat) <- colnames(cov_mat) <- c("a", "b", "c", "d")

  td <- data.frame(i = "a", j = "b", k = "c", l = "d",
                    tetrad_id = 3L, stringsAsFactors = FALSE)
  val <- seminrExtras:::compute_tetrads(cov_mat, td)

  # tau_1342 = sigma_ik * sigma_jl - sigma_il * sigma_jk
  # = sigma(a,c) * sigma(b,d) - sigma(a,d) * sigma(b,c)
  # = 0.3 * 0.6 - 0.2 * 0.4 = 0.18 - 0.08 = 0.10
  expect_equal(val, 0.10)
})

test_that("enumerate_borrowed_tetrads produces correct counts", {
  # 3 own + 1 borrowed, "all" pattern: standard enumeration
  td_all <- seminrExtras:::enumerate_borrowed_tetrads(
    c("x1", "x2", "x3"), "x4", "all"
  )
  expect_equal(nrow(td_all), 2)  # C(4,4)=1 combo, 2 tetrads

  # 2 own + 2 borrowed, "tau_1342" pattern: 1 tetrad
  td_tau <- seminrExtras:::enumerate_borrowed_tetrads(
    c("x1", "x2"), c("x3", "x4"), "tau_1342"
  )
  expect_equal(nrow(td_tau), 1)
  expect_equal(td_tau$tetrad_id, 3L)
  expect_equal(td_tau$i, "x1")
  expect_equal(td_tau$j, "x2")
  expect_equal(td_tau$k, "x3")
  expect_equal(td_tau$l, "x4")
})

# =============================================================================
# INTERNAL HELPER TESTS
# =============================================================================

test_that("get_construct_mode returns correct mode", {
  expect_equal(seminrExtras:::get_construct_mode("Image", pls_model), "A")
})

test_that("get_structurally_connected finds neighbors", {
  neighbors <- seminrExtras:::get_structurally_connected("Image", pls_model)
  expect_true("Expectation" %in% neighbors)
  expect_true("Satisfaction" %in% neighbors)
  expect_true("Loyalty" %in% neighbors)
})

test_that("find_donor returns NULL for formative focal", {
  mm_form <- constructs(
    composite("Image", multi_items("IMAG", 1:5)),
    composite("Value", multi_items("PERV", 1:2), weights = mode_B),
    composite("Satisfaction", multi_items("CUSA", 1:3))
  )
  sm_form <- relationships(
    paths(from = c("Image", "Value"), to = "Satisfaction")
  )
  pls_form <- estimate_pls(data = mobi, measurement_model = mm_form,
                            structural_model = sm_form)

  focal_info <- seminrExtras:::resolve_indicators("Value", pls_form)
  result <- seminrExtras:::find_donor("Value", focal_info, pls_form)
  expect_null(result)
})

# =============================================================================
# feature_coa.R - Composite Overfit Analysis (COA)
# =============================================================================
# This file implements the COA framework for PLS-SEM models, as described in:
#
# - Ray, S., Danks, N. P., & Valdez, A. C. (2022). "A framework for
#   Composite Overfit Analysis." Information Systems Research.
#
# COA identifies cases whose predictions are unstable (overfit) and groups
# them to reveal which model parameters are affected. The analysis proceeds
# in three steps:
# 1. Compute Predictive Deviance (PD) via cross-validation
# 2. Build a deviance tree to identify groups of deviant cases
# 3. Assess parameter instability by removing deviant groups
# =============================================================================

# =============================================================================
# MAIN ENTRY POINT
# =============================================================================

#' Composite Overfit Analysis (COA)
#'
#' `assess_coa` conducts a full Composite Overfit Analysis on a PLS-SEM model.
#' It computes predictive deviance for a focal construct, identifies groups of
#' deviant cases using a decision tree, and assesses parameter instability.
#'
#' @param seminr_model An estimated SEMinR model from `estimate_pls()`.
#' @param focal_construct Name of the endogenous construct to analyze (character).
#' @param deviance_bounds Two-element numeric vector of quantile bounds for
#'   identifying deviant cases (default `c(0.025, 0.975)`).
#' @param params Character vector of model parameters to assess for instability.
#'   Valid values include `"path_coef"`, `"outer_weights"`, `"outer_loadings"`,
#'   `"rSquared"` (default `"path_coef"`).
#' @param technique Prediction technique: `predict_DA` (default) or `predict_EA`.
#' @param noFolds Number of folds for k-fold cross-validation (default 10).
#' @param reps Number of CV repetitions (default 1).
#' @param cores Number of cores for parallel CV (default NULL = sequential).
#' @param seed Random seed for reproducibility (default 123).
#' @param predict_model Optional pre-computed `predict_pls_model` object. If
#'   provided, skips the cross-validation step (saves time when predictions
#'   have already been computed).
#'
#' @return An object of class `coa_analysis` containing:
#'   \item{pls_model}{The original estimated model}
#'   \item{focal_construct}{Name of the analyzed construct}
#'   \item{deviance_bounds}{Quantile bounds used}
#'   \item{predictive_deviance}{Predictive deviance results (class `coa_deviance`)}
#'   \item{deviance_tree}{Deviance tree and deviant groups (class `coa_dtree`)}
#'   \item{unstable}{Parameter instability results (class `coa_unstable`)}
#'
#' @seealso [predictive_deviance()], [deviance_tree()], [unstable_params()],
#'   [group_rules()], [competes()]
#'
#' @examples
#' \donttest{
#' library(seminr)
#' library(seminrExtras)
#'
#' mobi_mm <- constructs(
#'   composite("Image",  multi_items("IMAG", 1:5)),
#'   composite("Value",  multi_items("PERV", 1:2)),
#'   composite("Satisfaction", multi_items("CUSA", 1:3)),
#'   composite("Loyalty", multi_items("CUSL", 1:3))
#' )
#'
#' mobi_sm <- relationships(
#'   paths(from = c("Image", "Value"), to = "Satisfaction"),
#'   paths(from = "Satisfaction", to = "Loyalty")
#' )
#'
#' mobi_pls <- estimate_pls(
#'   data = mobi,
#'   measurement_model = mobi_mm,
#'   structural_model  = mobi_sm
#' )
#'
#' coa_result <- assess_coa(mobi_pls,
#'                           focal_construct = "Loyalty",
#'                           noFolds = 10,
#'                           cores = 1,
#'                           seed = 123)
#' print(coa_result)
#' summary(coa_result)
#' plot(coa_result, type = "pd")
#' }
#'
#' @export
assess_coa <- function(seminr_model,
                        focal_construct,
                        deviance_bounds = c(0.025, 0.975),
                        params = "path_coef",
                        technique = predict_DA,
                        noFolds = 10,
                        reps = 1,
                        cores = NULL,
                        seed = 123,
                        predict_model = NULL) {

  # ---------------------------------------------------------------------------
  # Step 1: Validate inputs
  # ---------------------------------------------------------------------------
  if (!validate_for_prediction(seminr_model, "assess_coa")) {
    return(NULL)
  }

  construct_names <- colnames(seminr_model$construct_scores)
  if (!(focal_construct %in% construct_names)) {
    stop("focal_construct '", focal_construct,
         "' not found in model constructs: ",
         paste(construct_names, collapse = ", "),
         call. = FALSE)
  }

  # Validate deviance_bounds: must be two probabilities with lower < upper
  if (length(deviance_bounds) != 2 ||
      any(deviance_bounds < 0) || any(deviance_bounds > 1) ||
      deviance_bounds[1] >= deviance_bounds[2]) {
    stop("deviance_bounds must be two values in [0, 1] with first < second, e.g. c(0.025, 0.975)",
         call. = FALSE)
  }

  # Validate params against known model slots
  valid_params <- c("path_coef", "outer_weights", "outer_loadings", "rSquared")
  invalid_params <- setdiff(params, valid_params)
  if (length(invalid_params) > 0) {
    stop("Invalid params: ", paste(invalid_params, collapse = ", "),
         ". Valid options: ", paste(valid_params, collapse = ", "),
         call. = FALSE)
  }

  # ---------------------------------------------------------------------------
  # Step 2: Compute predictive deviance via cross-validated predictions
  # ---------------------------------------------------------------------------
  pd <- predictive_deviance(seminr_model, focal_construct,
                            technique = technique,
                            noFolds = noFolds, reps = reps,
                            cores = cores, seed = seed,
                            predict_model = predict_model)

  # ---------------------------------------------------------------------------
  # Step 3: Build deviance tree and identify deviant case groups
  # ---------------------------------------------------------------------------
  dt <- deviance_tree(pd, deviance_bounds = deviance_bounds)

  # ---------------------------------------------------------------------------
  # Step 4: Assess parameter instability by removing deviant groups
  # ---------------------------------------------------------------------------
  unstable <- unstable_params(seminr_model,
                              deviant_groups = dt$deviant_groups,
                              params = params)

  analysis <- list(
    pls_model = seminr_model,
    focal_construct = focal_construct,
    deviance_bounds = deviance_bounds,
    predictive_deviance = pd,
    deviance_tree = dt,
    unstable = unstable
  )

  class(analysis) <- c("coa_analysis", class(analysis))
  analysis
}

# =============================================================================
# STEP 1: PREDICTIVE DEVIANCE
# =============================================================================

#' Compute Predictive Deviance
#'
#' Computes per-case predictive deviance (PD) for a focal construct using
#' k-fold cross-validated predictions from `predict_pls()`.
#'
#' PD = in-sample fitted score - out-of-sample predicted score.
#' High PD indicates a case that is well-fitted in-sample but poorly predicted
#' out-of-sample, suggesting overfit.
#'
#' @param seminr_model An estimated SEMinR model.
#' @param focal_construct Name of the construct to analyze.
#' @param technique Prediction technique (default `predict_DA`).
#' @param noFolds Number of CV folds (default 10).
#' @param reps Number of CV repetitions (default 1).
#' @param cores Number of cores for parallel CV (default NULL).
#' @param seed Random seed (default 123).
#' @param predict_model Optional pre-computed `predict_pls_model` object.
#'
#' @return An object of class `coa_deviance` containing PD values, metrics,
#'   and the data frame used for tree construction.
#'
#' @seealso [assess_coa()], [deviance_tree()]
#'
#' @export
predictive_deviance <- function(seminr_model,
                                focal_construct,
                                technique = predict_DA,
                                noFolds = 10,
                                reps = 1,
                                cores = NULL,
                                seed = 123,
                                predict_model = NULL) {

  if (is.null(predict_model)) {
    if (!validate_for_prediction(seminr_model, "predictive_deviance")) {
      return(NULL)
    }
    set.seed(seed)
    predict_model <- predict_pls(seminr_model,
                                 technique = technique,
                                 noFolds = noFolds,
                                 reps = reps,
                                 cores = cores)
  }

  fitted <- predict_model$composites$composite_in_sample[, focal_construct]
  predicted <- predict_model$composites$composite_out_of_sample[, focal_construct]
  actual_star <- predict_model$composites$actuals_star[, focal_construct]

  IS_MSE <- mean((actual_star - fitted)^2)
  OOS_MSE <- mean((actual_star - predicted)^2)
  overfit_ratio <- (OOS_MSE - IS_MSE) / IS_MSE

  PD <- fitted - predicted
  pd_data <- cbind(as.data.frame(seminr_model$construct_scores), PD = PD)

  result <- list(
    PD = PD,
    pd_data = pd_data,
    IS_MSE = IS_MSE,
    OOS_MSE = OOS_MSE,
    overfit_ratio = overfit_ratio,
    fitted_score = fitted,
    predicted_score = predicted
  )

  class(result) <- c("coa_deviance", class(result))
  result
}

# =============================================================================
# STEP 2: DEVIANCE TREE
# =============================================================================

#' Build Deviance Tree
#'
#' Grows a decision tree on predictive deviance scores to identify groups
#' of cases with extreme (deviant) PD values.
#'
#' @param pd_result A `coa_deviance` object from `predictive_deviance()`.
#' @param deviance_bounds Two-element numeric vector of quantile bounds
#'   (default `c(0.025, 0.975)`).
#'
#' @return An object of class `coa_dtree` containing the rpart tree,
#'   deviant groups, group roots, unique deviants, and sorted PD values.
#'
#' @seealso [assess_coa()], [predictive_deviance()], [group_rules()], [competes()]
#'
#' @export
deviance_tree <- function(pd_result, deviance_bounds = c(0.025, 0.975)) {

  # Grow a full (unpruned) regression tree on PD scores:
  # cp = 0 disables complexity pruning to capture all splits
  # minsplit = 2 prevents trivial single-case leaf nodes
  tree <- rpart(PD ~ ., data = pd_result$pd_data, minsplit = 2, cp = 0)

  dev_interval <- stats::quantile(pd_result$PD, probs = deviance_bounds)

  nodes <- extract_nodes(tree$frame, dev_interval)

  sorted_PD <- sort(nodes$leaves$yval, decreasing = TRUE)

  deviants <- cases(tree, nodes$is_deviant_leaf)

  deviant_groups <- lapply(nodes$dev_parent_leaves,
                           function(group) cases(tree, nodes$names %in% group))

  if (length(deviant_groups) > 0) {
    if (length(deviant_groups) > 26) {
      warning("More than 26 deviant groups found (", length(deviant_groups),
              "); only the first 26 are labeled A-Z.", call. = FALSE)
      deviant_groups <- deviant_groups[1:26]
    }
    group_roots <- as.integer(names(deviant_groups))
    names(group_roots) <- names(deviant_groups) <- LETTERS[seq_along(deviant_groups)]
  } else {
    group_roots <- integer(0)
  }

  unique_deviants <- setdiff(deviants, unlist(deviant_groups))

  dtree <- list(
    tree = tree,
    sorted_PD = sorted_PD,
    deviant_groups = deviant_groups,
    group_roots = group_roots,
    unique_deviants = unique_deviants,
    deviant_nodes = nodes$deviants
  )

  class(dtree) <- c("coa_dtree", class(dtree))
  dtree
}

# =============================================================================
# STEP 3: UNSTABLE PARAMETERS
# =============================================================================

#' Assess Parameter Instability
#'
#' Identifies unstable model parameters by removing each deviant group
#' and re-estimating the model, then computing the difference in specified
#' parameters.
#'
#' @param seminr_model An estimated SEMinR model.
#' @param deviant_groups A `coa_dtree` object from [deviance_tree()], or a
#'   named list of integer vectors of case indices. When a `coa_dtree` is
#'   passed, the `$deviant_groups` element is extracted automatically.
#' @param params Character vector of model parameters to diff
#'   (default `"path_coef"`).
#'
#' @return An object of class `coa_unstable`: a named list (one entry per group)
#'   each containing `cases` and `param_diffs`.
#'
#' @seealso [assess_coa()], [deviance_tree()]
#'
#' @export
unstable_params <- function(seminr_model,
                            deviant_groups,
                            params = "path_coef") {

  # Accept a coa_dtree object (from deviance_tree()) and unwrap
  if (inherits(deviant_groups, "coa_dtree")) {
    deviant_groups <- deviant_groups$deviant_groups
  }

  if (length(deviant_groups) == 0) {
    result <- list()
    class(result) <- c("coa_unstable", class(result))
    return(result)
  }

  result <- lapply(deviant_groups, function(group_cases) {
    diffs <- param_diffs(group_cases, seminr_model, params)
    list(cases = group_cases, param_diffs = diffs)
  })

  class(result) <- c("coa_unstable", class(result))
  result
}

# =============================================================================
# TREE EXTRACTION HELPERS
# =============================================================================

#' Trace the path from root (node 1) to a given node in an rpart binary tree.
#'
#' In rpart's node numbering, a node's parent is node_id %/% 2:
#' even nodes (2k) are left children, odd nodes (2k+1) are right children.
#' This function recursively walks up to the root and returns the full path.
#' @noRd
path_to <- function(node_id) {
  if (node_id[1] != 1)
    c(Recall(if (node_id %% 2 == 0L) node_id / 2 else (node_id - 1) / 2), node_id)
  else node_id
}

#' @noRd
cases <- function(tree, logical_frame) {
  which(tree$where %in% which(logical_frame))
}

#' @noRd
main_ancestors <- function(parent_ids) {
  ids <- as.integer(parent_ids)
  ancestor_ids <- unique(vapply(ids, function(id) {
    min(ids[ids %in% path_to(id)])
  }, integer(1)))
  as.character(ancestor_ids)
}

#' Extract deviant node information from an rpart tree frame.
#'
#' Classifies each node as leaf vs internal, deviant vs non-deviant (based on
#' whether its mean PD falls outside dev_interval), then:
#' 1. Identifies deviant leaves (individual terminal nodes with extreme PD)
#' 2. Identifies deviant parents (internal nodes whose subtrees are deviant)
#' 3. Finds the highest ancestor of each deviant parent to form groups
#' 4. Maps each group ancestor to its descendant leaf nodes
#' @noRd
extract_nodes <- function(frame, dev_interval) {
  # Classify every node in the tree

  is_leaf <- frame$var == "<leaf>"
  is_deviant <- frame$yval < dev_interval[1] | frame$yval > dev_interval[2]
  is_deviant_leaf <- is_deviant & is_leaf
  is_deviant_parent <- is_deviant & !is_leaf

  names <- row.names(frame)
  leaves <- frame[is_leaf, ]
  deviants <- frame[is_deviant, ]

  leaf_ids <- row.names(leaves)
  dev_parent_ids <- row.names(frame[is_deviant_parent, ])

  # Find the highest (most ancestral) deviant parent for each group,
  # then collect all leaves under each ancestor to form deviant groups
  if (length(dev_parent_ids) == 0) {
    dev_ancestor_ids <- character(0)
    dev_parent_leaves <- list()
  } else {
    dev_ancestor_ids <- main_ancestors(dev_parent_ids)
    dev_parent_leaves <- leaves_from_nodes(dev_ancestor_ids, leaf_ids)
  }

  list(
    is_leaf = is_leaf,
    is_deviant_leaf = is_deviant_leaf,
    names = names,
    leaves = leaves,
    deviants = deviants,
    dev_ancestor_ids = dev_ancestor_ids,
    dev_parent_leaves = dev_parent_leaves
  )
}

#' Find all leaf nodes that descend from each parent node.
#'
#' For each parent, traces root-to-leaf paths for every leaf and checks
#' whether the parent appears on that path (i.e., the leaf is a descendant).
#' @noRd
leaves_from_nodes <- function(parent_ids, leaf_ids) {
  # Pre-compute root-to-leaf paths for all leaves
  node_paths <- lapply(as.integer(leaf_ids), path_to)
  names(node_paths) <- leaf_ids

  # For each parent, find leaves whose path passes through it
  paths_list <- lapply(parent_ids, function(node_id) {
    nid <- as.integer(node_id)
    matching <- which(vapply(node_paths, function(p) nid %in% p, logical(1)))
    names(matching)
  })
  names(paths_list) <- parent_ids
  paths_list
}

#' Re-estimate model without specified cases and compute parameter differences.
#' @noRd
param_diffs <- function(remove_cases, pls_model, params = "path_coef") {
  reduced_data <- pls_model$data[-remove_cases, ]
  # suppressMessages hides the "Generating the seminr model" console output
  reduced_model <- suppressMessages(
    seminr::rerun(pls_model, data = reduced_data)
  )

  diffs <- lapply(params, function(param) {
    reduced_model[[param]] - pls_model[[param]]
  })
  names(diffs) <- params
  diffs
}

# =============================================================================
# RULES EXTRACTION
# =============================================================================

#' Extract Split Rules for Deviant Groups
#'
#' Returns the decision tree split criteria that define deviant group(s).
#' When called with a single `coa_analysis` or `coa_dtree` object, returns
#' rules for all deviant groups. When called with a group name and a
#' `coa_analysis` object, returns rules for that specific group.
#'
#' @param group_name A `coa_analysis` object (to get rules for all groups),
#'   a `coa_dtree` object, or a single uppercase letter identifying a specific
#'   group (e.g., \code{"A"}).
#' @param coa_result A `coa_analysis` object from [assess_coa()]. Required
#'   only when \code{group_name} is a character group letter.
#'
#' @return A data frame (or named list of data frames) with columns
#'   `construct`, `gte`, and `lt` describing the construct score ranges
#'   that define each group.
#'
#' @seealso [assess_coa()], [competes()]
#'
#' @export
group_rules <- function(group_name, coa_result = NULL) {
  # Convenience: accept coa_analysis or coa_dtree as single arg
  if (inherits(group_name, "coa_analysis")) {
    coa_result <- group_name
    return(group_rules_all(coa_result))
  }
  if (inherits(group_name, "coa_dtree")) {
    # Wrap dtree into a minimal coa_result-like structure
    coa_result <- list(deviance_tree = group_name)
    return(group_rules_all(coa_result))
  }

  dtree <- coa_result$deviance_tree
  group_root <- dtree$group_roots[[group_name]]
  node_path <- path_to(group_root)[-1]

  tree_info <- tree_split_index(dtree$tree)

  splits <- do.call(rbind, lapply(node_path, function(nid) {
    get_split_at_node(nid, dtree$tree, tree_info)
  }))

  splits_df <- data.frame(
    var   = splits$var,
    sign  = ifelse(splits$ncat > 0, ">=", "< "),
    value = splits$index,
    stringsAsFactors = FALSE
  )

  consolidate_all_rules(splits_df)
}

#' @noRd
group_rules_all <- function(coa_result) {
  dtree <- coa_result$deviance_tree
  group_names <- names(dtree$deviant_groups)
  if (length(group_names) == 0) return(list())
  rules <- lapply(group_names, function(g) group_rules(g, coa_result))
  names(rules) <- group_names
  rules
}

#' Report Competing Split Criteria
#'
#' Shows all competing split variables at tree nodes, ranked by improvement.
#' Useful for understanding which constructs nearly split instead of the
#' chosen variable.
#'
#' When called with a single `coa_dtree` object, returns competing splits
#' at all deviant group root nodes. When called with a node ID and a
#' `coa_dtree` object, returns competing splits at that specific node.
#'
#' @param node_id A `coa_dtree` object (to get competing splits for all
#'   group roots), or an integer node ID from the rpart tree.
#' @param dtree A `coa_dtree` object (from `coa_result$deviance_tree`).
#'   Required only when \code{node_id} is an integer.
#'
#' @return A data frame (or named list of data frames) with columns
#'   `criterion`, `sign`, `value`, and `improve`.
#'
#' @seealso [group_rules()], [assess_coa()]
#'
#' @export
competes <- function(node_id, dtree = NULL) {
  # Convenience: accept coa_dtree as single arg
  if (inherits(node_id, "coa_dtree")) {
    dtree <- node_id
    group_names <- names(dtree$deviant_groups)
    if (length(group_names) == 0) return(list())
    results <- lapply(group_names, function(g) {
      competes(dtree$group_roots[[g]], dtree)
    })
    names(results) <- group_names
    return(results)
  }

  if (node_id == 1) stop("No splits before root (node 1) of tree")

  tree <- dtree$tree
  tree_info <- tree_split_index(tree)
  is_odd <- node_id %% 2 == 1

  search_node <- if (is_odd) node_id - 1 else node_id
  frame_row <- match(search_node / 2, tree_info$node_ids)

  start <- tree_info$split_index[frame_row]
  end <- start + tree$frame$ncompete[frame_row]
  splits <- as.data.frame(tree$splits[start:end, ])

  if (is_odd) splits$ncat <- splits$ncat * -1

  data.frame(
    criterion = row.names(splits),
    sign      = ifelse(splits$ncat > 0, ">=", "< "),
    value     = splits$index,
    improve   = splits$improve,
    row.names = NULL,
    stringsAsFactors = FALSE
  )
}

#' Pre-compute tree frame metadata for split lookups
#' @noRd
tree_split_index <- function(tree) {
  frame <- tree$frame
  is_leaf_vec <- frame$var == "<leaf>"
  list(
    split_index = cumsum(c(1, frame$ncompete + frame$nsurrogate + !is_leaf_vec)),
    node_ids = as.integer(row.names(frame))
  )
}

#' Get the primary split at a given node
#' @noRd
get_split_at_node <- function(node_id, tree, tree_info) {
  is_odd <- node_id %% 2 == 1
  search_node <- if (is_odd) node_id - 1 else node_id
  frame_row <- match(search_node, tree_info$node_ids)
  split_row <- tree_info$split_index[frame_row - 1]

  split <- data.frame(
    var = rownames(tree$splits)[split_row],
    ncat = tree$splits[split_row, "ncat"],
    index = tree$splits[split_row, "index"],
    row.names = NULL,
    stringsAsFactors = FALSE
  )
  if (is_odd) split$ncat <- split$ncat * -1
  split
}

#' @noRd
consolidate_all_rules <- function(splits_df) {
  vars <- unique(splits_df$var)
  criteria <- as.data.frame(do.call(rbind, lapply(vars, consolidate_rule, splits_df)))
  rownames(criteria) <- NULL
  criteria
}

#' @noRd
consolidate_rule <- function(construct, splits_df) {
  split_rows <- splits_df[splits_df$var == construct, ]

  gt_rule <- split_rows[split_rows$sign == ">=", ]
  if (nrow(gt_rule) > 1) gt_rule <- gt_rule[gt_rule$value == max(gt_rule$value), ]
  lt_rule <- split_rows[split_rows$sign == "< ", ]
  if (nrow(lt_rule) > 1) lt_rule <- lt_rule[lt_rule$value == min(lt_rule$value), ]

  data.frame(
    construct = construct,
    gte = if (nrow(gt_rule) == 0) NA_real_ else gt_rule$value,
    lt  = if (nrow(lt_rule) == 0) NA_real_ else lt_rule$value,
    stringsAsFactors = FALSE
  )
}

# =============================================================================
# S3 METHODS
# =============================================================================

#' @export
print.coa_analysis <- function(x, ...) {
  cat("Composite Overfit Analysis (COA)\n")
  cat("================================\n")
  cat("Focal construct:", x$focal_construct, "\n")
  cat("Deviance bounds:", x$deviance_bounds[1], "-", x$deviance_bounds[2], "\n")
  cat("Observations:", nrow(x$pls_model$data), "\n\n")

  pd <- x$predictive_deviance
  cat("Prediction Metrics:\n")
  cat("  In-sample MSE: ", format(pd$IS_MSE, digits = 4), "\n")
  cat("  Out-of-sample MSE:", format(pd$OOS_MSE, digits = 4), "\n")
  cat("  Overfit ratio:    ", format(pd$overfit_ratio, digits = 4), "\n\n")

  dt <- x$deviance_tree
  n_groups <- length(dt$deviant_groups)
  n_unique <- length(dt$unique_deviants)
  cat("Deviant Cases:\n")
  cat("  Groups:", n_groups)
  if (n_groups > 0) {
    group_sizes <- vapply(dt$deviant_groups, length, integer(1))
    cat(" (", paste(names(dt$deviant_groups), "=", group_sizes, collapse = ", "), ")")
  }
  cat("\n")
  cat("  Unique deviants:", n_unique, "\n")

  invisible(x)
}

#' @export
summary.coa_analysis <- function(object, ...) {
  pd <- object$predictive_deviance
  dt <- object$deviance_tree

  metrics <- data.frame(
    Metric = c("IS_MSE", "OOS_MSE", "Overfit_Ratio"),
    Value = c(pd$IS_MSE, pd$OOS_MSE, pd$overfit_ratio)
  )

  group_summary <- NULL
  if (length(dt$deviant_groups) > 0) {
    group_summary <- data.frame(
      Group = names(dt$deviant_groups),
      Size = vapply(dt$deviant_groups, length, integer(1)),
      Root_Node = dt$group_roots,
      row.names = NULL
    )
  }

  unstable_summary <- NULL
  if (length(object$unstable) > 0) {
    max_diffs <- vapply(object$unstable, function(g) {
      if ("path_coef" %in% names(g$param_diffs)) {
        max(abs(g$param_diffs$path_coef), na.rm = TRUE)
      } else {
        NA_real_
      }
    }, numeric(1))
    unstable_summary <- data.frame(
      Group = names(object$unstable),
      Max_Path_Diff = max_diffs,
      row.names = NULL
    )
  }

  result <- list(
    focal_construct = object$focal_construct,
    deviance_bounds = object$deviance_bounds,
    n_observations = nrow(object$pls_model$data),
    metrics = metrics,
    group_summary = group_summary,
    unstable_summary = unstable_summary,
    n_unique_deviants = length(dt$unique_deviants)
  )

  class(result) <- c("summary.coa_analysis", class(result))
  result
}

#' @export
print.summary.coa_analysis <- function(x, ...) {
  cat("Composite Overfit Analysis (COA) Summary\n")
  cat("=========================================\n")
  cat("Focal construct:", x$focal_construct, "\n")
  cat("Observations:", x$n_observations, "\n")
  cat("Deviance bounds:", x$deviance_bounds[1], "-", x$deviance_bounds[2], "\n\n")

  cat("Prediction Metrics:\n")
  print(x$metrics, row.names = FALSE)
  cat("\n")

  if (!is.null(x$group_summary)) {
    cat("Deviant Groups:\n")
    print(x$group_summary, row.names = FALSE)
    cat("\n")
  }

  cat("Unique deviants:", x$n_unique_deviants, "\n\n")

  if (!is.null(x$unstable_summary)) {
    cat("Parameter Instability (max |path_coef diff|):\n")
    print(x$unstable_summary, row.names = FALSE)
  }

  invisible(x)
}

# =============================================================================
# PLOT METHODS
# =============================================================================

#' Plot COA Results
#'
#' @param x A `coa_analysis` object from `assess_coa()`.
#' @param type One of `"pd"` (predictive deviance scatter), `"groups"`
#'   (average construct scores per group), or `"tree"` (rpart tree diagram).
#' @param ... Additional arguments passed to the underlying plot function.
#'
#' @export
plot.coa_analysis <- function(x, type = c("pd", "groups", "tree"), ...) {
  type <- match.arg(type)

  switch(type,
    pd     = plot_pd(x, ...),
    groups = plot_group_scores(x, ...),
    tree   = plot_tree(x, ...)
  )
}

#' @noRd
plot_pd <- function(coa, ...) {
  pd <- coa$predictive_deviance
  dt <- coa$deviance_tree
  n <- length(pd$PD)

  # Assign group membership to each case
  group_id <- rep(0L, n)
  for (i in seq_along(dt$deviant_groups)) {
    group_id[dt$deviant_groups[[i]]] <- i + 1L
  }
  group_id[dt$unique_deviants] <- 1L

  pd_df <- data.frame(pd = pd$PD, group = group_id)
  pd_df <- pd_df[order(pd_df$pd), ]
  pd_df$order <- seq_len(n)

  dev_quantiles <- stats::quantile(pd$PD, probs = coa$deviance_bounds)

  old_par <- par(no.readonly = TRUE)
  on.exit(par(old_par))

  plot(NULL, NULL,
       xlim = c(1, n), ylim = range(pd$PD),
       frame.plot = FALSE,
       xlab = "Cases", ylab = "Predictive Deviance",
       main = paste("Predictive Deviance:", coa$focal_construct),
       ...)
  abline(h = dev_quantiles, col = "darkgray", lty = 2)

  non_dev <- pd_df$group == 0
  points(pd_df$order[non_dev], pd_df$pd[non_dev], pch = 19, col = "lightgray", cex = 0.6)

  if (any(!non_dev)) {
    n_group_colors <- max(0, max(pd_df$group) - 1)
    cols <- if (n_group_colors > 0) {
      c("black", palette.colors(n = n_group_colors, palette = "Set1"))
    } else {
      "black"
    }
    is_unique <- pd_df$group == 1
    is_group <- !non_dev & !is_unique

    if (any(is_unique)) {
      points(pd_df$order[is_unique], pd_df$pd[is_unique], pch = 8, col = "black", cex = 0.8)
    }
    if (any(is_group)) {
      gcols <- cols[pd_df$group[is_group]]
      points(pd_df$order[is_group], pd_df$pd[is_group], pch = 19, col = gcols, cex = 0.8)

      group_labels <- LETTERS[pd_df$group[is_group] - 1]
      text(pd_df$order[is_group], pd_df$pd[is_group],
           labels = group_labels, pos = 3, cex = 0.6, col = gcols)
    }

    # Legend
    group_names <- names(dt$deviant_groups)
    if (length(group_names) > 0) {
      legend_labels <- group_names
      legend_cols <- cols[2:(length(group_names) + 1)]
      legend_pch <- rep(19, length(group_names))
      if (length(dt$unique_deviants) > 0) {
        legend_labels <- c(legend_labels, "Unique")
        legend_cols <- c(legend_cols, "black")
        legend_pch <- c(legend_pch, 8)
      }
      legend("topleft", legend = legend_labels, col = legend_cols,
             pch = legend_pch, cex = 0.7, bty = "n")
    }
  }
}

#' @noRd
plot_group_scores <- function(coa, remove = NULL, ...) {
  dt <- coa$deviance_tree
  if (length(dt$deviant_groups) == 0) {
    message("No deviant groups to plot.")
    return(invisible(NULL))
  }

  group_scores <- vapply(names(dt$deviant_groups), function(gname) {
    scores <- coa$pls_model$construct_scores[dt$deviant_groups[[gname]], , drop = FALSE]
    colMeans(scores)
  }, numeric(ncol(coa$pls_model$construct_scores)))

  if (!is.null(remove)) {
    group_scores <- group_scores[!rownames(group_scores) %in% remove, , drop = FALSE]
  }

  # Move focal construct to end
  constructs <- rownames(group_scores)
  outcome_idx <- which(constructs == coa$focal_construct)
  if (length(outcome_idx) == 1 && outcome_idx < length(constructs)) {
    reorder <- c(seq_along(constructs)[-outcome_idx], outcome_idx)
    group_scores <- group_scores[reorder, , drop = FALSE]
  }

  num_constructs <- nrow(group_scores)
  ylim_range <- max(3.5, max(abs(group_scores), na.rm = TRUE) + 0.5)

  old_par <- par(no.readonly = TRUE)
  on.exit(par(old_par))

  cols <- palette.colors(n = ncol(group_scores), palette = "Set1")

  matplot(group_scores, type = "b", pch = colnames(group_scores),
          ylim = c(-ylim_range, ylim_range),
          lty = "solid", col = cols,
          xaxt = "n", bty = "n",
          ylab = "Average Construct Score",
          main = "Deviant Group Construct Scores", ...)

  # 50% zone shading
  polygon(x = c(0, num_constructs + 1, num_constructs + 1, 0),
          y = c(1.15, 1.15, -1.15, -1.15),
          col = adjustcolor("gray", alpha.f = 0.1), border = NA)
  polygon(x = c(0, num_constructs + 1, num_constructs + 1, 0),
          y = c(0.67, 0.67, -0.67, -0.67),
          col = adjustcolor("gray", alpha.f = 0.15), border = NA)

  # Separator before outcome
  if (length(outcome_idx) == 1) {
    abline(v = num_constructs - 0.5, col = "gray80", lwd = 2)
  }

  # Redraw lines on top of shading
  matlines(group_scores, type = "b", pch = colnames(group_scores),
           lty = "solid", col = cols)

  construct_labels <- rownames(group_scores)
  font_vec <- rep(1, num_constructs)
  if (length(outcome_idx) == 1) font_vec[num_constructs] <- 2
  axis(1, at = seq_len(num_constructs), labels = construct_labels, font = font_vec)

  legend("topright", legend = colnames(group_scores), col = cols,
         lty = 1, pch = colnames(group_scores), cex = 0.7, bty = "n")
}

#' @noRd
plot_tree <- function(coa, ...) {
  tree <- coa$deviance_tree$tree
  plot(tree, uniform = TRUE, main = paste("Deviance Tree:", coa$focal_construct), ...)
  text(tree, use.n = TRUE, cex = 0.7)
}

library(seminr)

# ============================================================================
# Setup: Estimate models once, reuse across all tests
# ============================================================================

set.seed(123)

# Mediation model: COMP, LIKE -> CUSA -> CUSL
corp_rep_mm <- constructs(
  composite("COMP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  composite("CUSA", single_item("cusa")),
  composite("CUSL", multi_items("cusl_", 1:3))
)

corp_rep_sm <- relationships(
  paths(from = c("COMP", "LIKE"), to = "CUSA"),
  paths(from = "CUSA", to = "CUSL")
)

pls_model <- estimate_pls(
  data = corp_rep_data,
  measurement_model = corp_rep_mm,
  structural_model  = corp_rep_sm,
  missing = mean_replacement,
  missing_value = "-99"
)

# Pre-compute predictions once (expensive step)
pls_predictions <- predict_pls(
  pls_model,
  technique = predict_DA,
  noFolds = 10,
  reps = 1,
  cores = 1
)

# Pre-compute COA components for reuse
pd_result <- predictive_deviance(
  pls_model,
  focal_construct = "CUSL",
  predict_model = pls_predictions
)

dt_result <- deviance_tree(pd_result)

# ============================================================================
# Unit tests: path_to (tree traversal helper)
# ============================================================================

test_that("path_to computes correct root-to-node path for even node", {
  expect_equal(path_to(468), c(1, 3, 7, 14, 29, 58, 117, 234, 468))
})

test_that("path_to computes correct root-to-node path for odd node", {
  expect_equal(path_to(469), c(1, 3, 7, 14, 29, 58, 117, 234, 469))
})

test_that("path_to returns root for node 1", {
  expect_equal(path_to(1), 1)
})

# ============================================================================
# Unit tests: main_ancestors (deviant group identification)
# ============================================================================

test_that("main_ancestors removes descendant nodes", {
  parent_ids <- c("4", "40", "81", "12", "24", "119", "239", "31")
  result <- main_ancestors(parent_ids)
  expect_equal(result, c("4", "40", "12", "119", "31"))
})

test_that("main_ancestors handles multi-level descendants", {
  parent_ids <- c("2", "12", "24", "204", "26", "29", "117", "469", "15", "31", "62", "124", "63")
  result <- main_ancestors(parent_ids)
  expect_equal(result, c("2", "12", "26", "29", "15"))
})

test_that("main_ancestors returns single node unchanged", {
  expect_equal(main_ancestors("5"), "5")
})

# ============================================================================
# Unit tests: predictive_deviance
# ============================================================================

test_that("predictive_deviance returns correct structure", {
  expect_type(pd_result, "list")
  expect_s3_class(pd_result, "coa_deviance")
  expect_true(all(c("PD", "pd_data", "IS_MSE", "OOS_MSE",
                     "overfit_ratio", "fitted_score",
                     "predicted_score") %in% names(pd_result)))
})

test_that("PD equals fitted minus predicted", {
  expected_pd <- pd_result$fitted_score - pd_result$predicted_score
  expect_equal(pd_result$PD, expected_pd)
})

test_that("PD length matches number of observations", {
  expect_equal(length(pd_result$PD), nrow(pls_model$data))
})

test_that("pd_data contains all construct scores plus PD column", {
  construct_names <- colnames(pls_model$construct_scores)
  expect_true(all(construct_names %in% colnames(pd_result$pd_data)))
  expect_true("PD" %in% colnames(pd_result$pd_data))
})

test_that("overfit_ratio has correct formula", {
  expected_ratio <- (pd_result$OOS_MSE - pd_result$IS_MSE) / pd_result$IS_MSE
  expect_equal(pd_result$overfit_ratio, expected_ratio)
})

test_that("MSE values are non-negative", {
  expect_true(pd_result$IS_MSE >= 0)
  expect_true(pd_result$OOS_MSE >= 0)
})

test_that("predictive_deviance accepts pre-computed predict_model", {
  skip_on_cran()
  pd_precomp <- predictive_deviance(
    pls_model,
    focal_construct = "CUSL",
    predict_model = pls_predictions
  )
  pd_fresh <- predictive_deviance(
    pls_model,
    focal_construct = "CUSL",
    technique = predict_DA,
    noFolds = 10,
    reps = 1,
    cores = 1,
    seed = 123
  )

  # Structure should be identical
  expect_s3_class(pd_precomp, "coa_deviance")
  expect_equal(length(pd_precomp$PD), length(pd_fresh$PD))
})

# ============================================================================
# Unit tests: deviance_tree
# ============================================================================

test_that("deviance_tree returns correct structure", {
  expect_type(dt_result, "list")
  expect_s3_class(dt_result, "coa_dtree")
  expect_true(all(c("tree", "deviant_groups", "group_roots",
                     "unique_deviants", "sorted_PD") %in% names(dt_result)))
})

test_that("deviance_tree produces an rpart tree", {
  expect_s3_class(dt_result$tree, "rpart")
})

test_that("deviant_groups are named with uppercase letters", {
  group_names <- names(dt_result$deviant_groups)
  if (length(group_names) > 0) {
    expect_true(all(group_names %in% LETTERS))
  }
})

test_that("group_roots match deviant_groups names", {
  expect_equal(names(dt_result$group_roots), names(dt_result$deviant_groups))
})

test_that("unique_deviants are not in any deviant group", {
  all_grouped <- unlist(dt_result$deviant_groups)
  overlap <- intersect(dt_result$unique_deviants, all_grouped)
  expect_equal(length(overlap), 0)
})

test_that("deviance_tree respects custom bounds", {
  dt_wide <- deviance_tree(pd_result, deviance_bounds = c(0.1, 0.9))
  dt_narrow <- deviance_tree(pd_result, deviance_bounds = c(0.01, 0.99))

  # Wider bounds should produce more deviants
  n_wide <- length(unlist(dt_wide$deviant_groups)) + length(dt_wide$unique_deviants)
  n_narrow <- length(unlist(dt_narrow$deviant_groups)) + length(dt_narrow$unique_deviants)
  expect_true(n_wide >= n_narrow)
})

test_that("sorted_PD is sorted in decreasing order", {
  expect_equal(dt_result$sorted_PD, sort(dt_result$sorted_PD, decreasing = TRUE))
})

# ============================================================================
# Unit tests: unstable_params
# ============================================================================

test_that("unstable_params returns correct structure", {
  skip_if(length(dt_result$deviant_groups) == 0, "No deviant groups found")

  unstable <- unstable_params(
    pls_model,
    deviant_groups = dt_result$deviant_groups,
    params = "path_coef"
  )

  expect_type(unstable, "list")
  expect_s3_class(unstable, "coa_unstable")
  expect_equal(length(unstable), length(dt_result$deviant_groups))
})

test_that("unstable_params diffs have correct dimensions", {
  skip_if(length(dt_result$deviant_groups) == 0, "No deviant groups found")

  unstable <- unstable_params(
    pls_model,
    deviant_groups = dt_result$deviant_groups,
    params = "path_coef"
  )

  first_diff <- unstable[[1]]$param_diffs$path_coef
  expect_equal(dim(first_diff), dim(pls_model$path_coef))
})

test_that("unstable_params works with multiple param types", {
  skip_if(length(dt_result$deviant_groups) == 0, "No deviant groups found")

  unstable <- unstable_params(
    pls_model,
    deviant_groups = dt_result$deviant_groups,
    params = c("path_coef", "outer_weights")
  )

  first_group <- unstable[[1]]$param_diffs
  expect_true("path_coef" %in% names(first_group))
  expect_true("outer_weights" %in% names(first_group))
})

# ============================================================================
# Integration: assess_coa
# ============================================================================

test_that("assess_coa returns correct S3 class and structure", {
  skip_on_cran()
  result <- assess_coa(
    pls_model,
    focal_construct = "CUSL",
    noFolds = 10, reps = 1, cores = 1,
    seed = 123
  )

  expect_s3_class(result, "coa_analysis")
  expect_true(all(c("pls_model", "focal_construct", "deviance_bounds",
                     "predictive_deviance", "deviance_tree",
                     "unstable") %in% names(result)))
})

test_that("assess_coa accepts pre-computed predict_model", {
  result <- assess_coa(
    pls_model,
    focal_construct = "CUSL",
    predict_model = pls_predictions,
    seed = 123
  )

  expect_s3_class(result, "coa_analysis")
  expect_s3_class(result$predictive_deviance, "coa_deviance")
  expect_s3_class(result$deviance_tree, "coa_dtree")
})

test_that("assess_coa is reproducible with same seed", {
  skip_on_cran()
  r1 <- assess_coa(pls_model, "CUSL",
                    noFolds = 10, reps = 1, cores = 1, seed = 42)
  r2 <- assess_coa(pls_model, "CUSL",
                    noFolds = 10, reps = 1, cores = 1, seed = 42)

  expect_equal(r1$predictive_deviance$PD, r2$predictive_deviance$PD)
})

test_that("assess_coa works with predict_EA technique", {
  skip_on_cran()
  result <- assess_coa(
    pls_model,
    focal_construct = "CUSL",
    technique = predict_EA,
    noFolds = 10, reps = 1, cores = 1,
    seed = 123
  )

  expect_s3_class(result, "coa_analysis")
})

# ============================================================================
# Input validation
# ============================================================================

test_that("assess_coa rejects non-seminr model", {
  expect_warning(
    result <- assess_coa(list(not = "a_model"), focal_construct = "X"),
    "only works with SEMinR models"
  )
  expect_null(result)
})

test_that("assess_coa rejects HOC model", {
  hoc_mm <- constructs(
    composite("COMP", multi_items("comp_", 1:3)),
    composite("LIKE", multi_items("like_", 1:3)),
    higher_composite("COKE", c("COMP", "LIKE"), two_stage),
    composite("CUSA", single_item("cusa")),
    composite("CUSL", multi_items("cusl_", 1:3))
  )
  hoc_sm <- relationships(
    paths(from = "COKE", to = "CUSA"),
    paths(from = "CUSA", to = "CUSL")
  )
  hoc_model <- estimate_pls(
    data = corp_rep_data,
    measurement_model = hoc_mm,
    structural_model  = hoc_sm,
    missing = mean_replacement,
    missing_value = "-99"
  )

  expect_warning(
    result <- assess_coa(hoc_model, focal_construct = "CUSL"),
    "higher-order"
  )
  expect_null(result)
})

test_that("assess_coa errors on invalid focal construct", {
  expect_error(
    assess_coa(pls_model, focal_construct = "NONEXISTENT",
               noFolds = 10, reps = 1, cores = 1, seed = 123),
    "focal_construct"
  )
})

test_that("assess_coa errors on invalid deviance_bounds", {
  expect_error(
    assess_coa(pls_model, focal_construct = "CUSL",
               deviance_bounds = c(0.975, 0.025)),
    "deviance_bounds"
  )
  expect_error(
    assess_coa(pls_model, focal_construct = "CUSL",
               deviance_bounds = c(-0.1, 1.2)),
    "deviance_bounds"
  )
  expect_error(
    assess_coa(pls_model, focal_construct = "CUSL",
               deviance_bounds = c(0.5)),
    "deviance_bounds"
  )
})

test_that("assess_coa errors on invalid params", {
  expect_error(
    assess_coa(pls_model, focal_construct = "CUSL",
               params = "nonexistent_param",
               predict_model = pls_predictions, seed = 123),
    "Invalid params"
  )
})

# ============================================================================
# Rules extraction, S3 methods, and plots (shared fixture)
# ============================================================================

# Compute once for rules, S3, and plot tests
coa_fixture <- assess_coa(
  pls_model, "CUSL",
  predict_model = pls_predictions, seed = 123
)

test_that("group_rules returns data frame with correct columns", {
  skip_if(length(coa_fixture$deviance_tree$deviant_groups) == 0,
          "No deviant groups found")

  group_name <- names(coa_fixture$deviance_tree$deviant_groups)[1]
  rules <- group_rules(group_name, coa_fixture)

  expect_s3_class(rules, "data.frame")
  expect_true(all(c("construct", "gte", "lt") %in% colnames(rules)))
})

test_that("competes returns data frame with correct columns", {
  skip_if(length(coa_fixture$deviance_tree$deviant_groups) == 0,
          "No deviant groups found")

  group_name <- names(coa_fixture$deviance_tree$deviant_groups)[1]
  node_id <- coa_fixture$deviance_tree$group_roots[[group_name]]
  comp <- competes(node_id, coa_fixture$deviance_tree)

  expect_s3_class(comp, "data.frame")
  expect_true(all(c("criterion", "sign", "value", "improve") %in% colnames(comp)))
})

test_that("competes errors for root node", {
  expect_error(competes(1, coa_fixture$deviance_tree), "root")
})

test_that("print.coa_analysis runs without error", {
  expect_output(print(coa_fixture))
})

test_that("summary.coa_analysis returns summary object", {
  s <- summary(coa_fixture)
  expect_type(s, "list")
  expect_s3_class(s, "summary.coa_analysis")
})

test_that("plot.coa_analysis type='pd' runs without error", {
  expect_no_error(plot(coa_fixture, type = "pd"))
})

test_that("plot.coa_analysis type='groups' runs without error", {
  skip_if(length(coa_fixture$deviance_tree$deviant_groups) == 0,
          "No deviant groups found")
  expect_no_error(plot(coa_fixture, type = "groups"))
})

test_that("plot.coa_analysis type='tree' runs without error", {
  expect_no_error(plot(coa_fixture, type = "tree"))
})

test_that("plot.coa_analysis defaults to 'pd' type", {
  expect_no_error(plot(coa_fixture))
})

# ============================================================================
# Edge case: HOC rejection (fast — no CV needed)
# ============================================================================

test_that("assess_coa returns NULL for HOC repeated indicators", {
  mm_hoc <- constructs(
    composite("Image", multi_items("IMAG", 1:5)),
    composite("Value", multi_items("PERV", 1:2)),
    higher_composite("Rep", c("Image", "Value"), method = two_stage),
    composite("Satisfaction", multi_items("CUSA", 1:3)),
    composite("Loyalty", multi_items("CUSL", 1:3))
  )
  sm_hoc <- relationships(
    paths(from = "Rep", to = "Satisfaction"),
    paths(from = "Satisfaction", to = "Loyalty")
  )
  model_hoc <- estimate_pls(data = mobi, measurement_model = mm_hoc,
                             structural_model = sm_hoc)

  expect_warning(
    result <- assess_coa(model_hoc, focal_construct = "Loyalty",
                          noFolds = 10, cores = 1, seed = 123),
    "higher-order"
  )
  expect_null(result)
})

# ============================================================================
# Extended edge-case tests
# ============================================================================
# These tests estimate multiple model variants with full cross-validation.
# They are slow (~2-5 min) and skipped during R CMD check / CRAN.
# Run locally with: devtools::test(filter = "extended")

library(seminr)

skip_on_cran()
skip_if(identical(Sys.getenv("NOT_CRAN"), ""), "Skipping extended tests (set NOT_CRAN=true to run)")

# ============================================================================
# COA edge cases
# ============================================================================

test_that("assess_coa works with two-stage moderation", {
  mm_mod <- constructs(
    composite("Image", multi_items("IMAG", 1:5)),
    composite("Value", multi_items("PERV", 1:2)),
    composite("Satisfaction", multi_items("CUSA", 1:3)),
    interaction_term(iv = "Image", moderator = "Value", method = two_stage)
  )
  sm_mod <- relationships(
    paths(from = c("Image", "Value", "Image*Value"), to = "Satisfaction")
  )
  model_mod <- estimate_pls(data = mobi, measurement_model = mm_mod,
                            structural_model = sm_mod)
  result <- assess_coa(model_mod, focal_construct = "Satisfaction",
                        noFolds = 10, cores = 1, seed = 123)

  expect_s3_class(result, "coa_analysis")
  expect_true(result$predictive_deviance$overfit_ratio >= 0)
})

test_that("assess_coa works with mediated moderation", {
  mm_medmod <- constructs(
    composite("Image", multi_items("IMAG", 1:5)),
    composite("Value", multi_items("PERV", 1:2)),
    composite("Satisfaction", multi_items("CUSA", 1:3)),
    composite("Loyalty", multi_items("CUSL", 1:3)),
    interaction_term(iv = "Image", moderator = "Value", method = two_stage)
  )
  sm_medmod <- relationships(
    paths(from = c("Image", "Value", "Image*Value"), to = "Satisfaction"),
    paths(from = "Satisfaction", to = "Loyalty")
  )
  model_medmod <- estimate_pls(data = mobi, measurement_model = mm_medmod,
                               structural_model = sm_medmod)
  result <- assess_coa(model_medmod, focal_construct = "Loyalty",
                        noFolds = 10, cores = 1, seed = 123)

  expect_s3_class(result, "coa_analysis")
  expect_s3_class(result$deviance_tree, "coa_dtree")
})

test_that("assess_coa works with serial mediation", {
  mm_ser <- constructs(
    composite("Image", multi_items("IMAG", 1:5)),
    composite("Quality", multi_items("PERQ", 1:7)),
    composite("Satisfaction", multi_items("CUSA", 1:3)),
    composite("Loyalty", multi_items("CUSL", 1:3))
  )
  sm_ser <- relationships(
    paths(from = "Image", to = "Quality"),
    paths(from = c("Image", "Quality"), to = "Satisfaction"),
    paths(from = "Satisfaction", to = "Loyalty")
  )
  model_ser <- estimate_pls(data = mobi, measurement_model = mm_ser,
                            structural_model = sm_ser)
  result <- assess_coa(model_ser, focal_construct = "Loyalty",
                        noFolds = 10, cores = 1, seed = 123)

  expect_s3_class(result, "coa_analysis")
  expect_equal(length(result$predictive_deviance$PD), nrow(mobi))
})

test_that("assess_coa works with mode_B composites", {
  mm_b <- constructs(
    composite("Image", multi_items("IMAG", 1:5), weights = mode_B),
    composite("Value", multi_items("PERV", 1:2), weights = mode_B),
    composite("Satisfaction", multi_items("CUSA", 1:3)),
    composite("Loyalty", multi_items("CUSL", 1:3))
  )
  sm_b <- relationships(
    paths(from = c("Image", "Value"), to = "Satisfaction"),
    paths(from = "Satisfaction", to = "Loyalty")
  )
  model_b <- estimate_pls(data = mobi, measurement_model = mm_b,
                           structural_model = sm_b)
  result <- assess_coa(model_b, focal_construct = "Loyalty",
                        noFolds = 10, cores = 1, seed = 123)

  expect_s3_class(result, "coa_analysis")
})

test_that("assess_coa works with reflective constructs (PLSc)", {
  mm_ref <- constructs(
    reflective("Image", multi_items("IMAG", 1:5)),
    reflective("Value", multi_items("PERV", 1:2)),
    reflective("Satisfaction", multi_items("CUSA", 1:3)),
    reflective("Loyalty", multi_items("CUSL", 1:3))
  )
  sm_ref <- relationships(
    paths(from = c("Image", "Value"), to = "Satisfaction"),
    paths(from = "Satisfaction", to = "Loyalty")
  )
  model_ref <- estimate_pls(data = mobi, measurement_model = mm_ref,
                             structural_model = sm_ref)
  result <- assess_coa(model_ref, focal_construct = "Loyalty",
                        noFolds = 10, cores = 1, seed = 123)

  expect_s3_class(result, "coa_analysis")
})

test_that("assess_coa works with predict_EA on mediated moderation", {
  mm_ea <- constructs(
    composite("Image", multi_items("IMAG", 1:5)),
    composite("Value", multi_items("PERV", 1:2)),
    composite("Satisfaction", multi_items("CUSA", 1:3)),
    composite("Loyalty", multi_items("CUSL", 1:3)),
    interaction_term(iv = "Image", moderator = "Value", method = two_stage)
  )
  sm_ea <- relationships(
    paths(from = c("Image", "Value", "Image*Value"), to = "Satisfaction"),
    paths(from = "Satisfaction", to = "Loyalty")
  )
  model_ea <- estimate_pls(data = mobi, measurement_model = mm_ea,
                            structural_model = sm_ea)
  result <- assess_coa(model_ea, focal_construct = "Loyalty",
                        technique = predict_EA,
                        noFolds = 10, cores = 1, seed = 123)

  expect_s3_class(result, "coa_analysis")
})

# ============================================================================
# NCA extended tests (with significance testing)
# ============================================================================

test_that("NCA reproducibility with significance testing", {
  mobi_mm <- constructs(
    composite("Image",        multi_items("IMAG", 1:5)),
    composite("Value",        multi_items("PERV", 1:2)),
    composite("Satisfaction", multi_items("CUSA", 1:3))
  )
  mobi_sm <- relationships(
    paths(from = c("Image", "Value"), to = "Satisfaction")
  )
  pls_model <- estimate_pls(data = mobi, measurement_model = mobi_mm,
                             structural_model = mobi_sm)

  r1 <- assess_nca(pls_model, target = "Satisfaction",
                    test.rep = 200, seed = 42)
  r2 <- assess_nca(pls_model, target = "Satisfaction",
                    test.rep = 200, seed = 42)

  expect_equal(r1$effect_sizes, r2$effect_sizes)
})

# ============================================================================
# CVPAT extended: technique comparison and testtype variants
# ============================================================================

test_that("assess_cvpat EA vs DA give different results", {
  corp_rep_mm <- constructs(
    composite("COMP", multi_items("comp_", 1:3)),
    composite("LIKE", multi_items("like_", 1:3)),
    composite("CUSA", single_item("cusa")),
    composite("CUSL", multi_items("cusl_", 1:3))
  )
  sm <- relationships(
    paths(from = c("COMP", "LIKE"), to = "CUSA"),
    paths(from = "CUSA", to = "CUSL")
  )
  model <- estimate_pls(data = corp_rep_data, measurement_model = corp_rep_mm,
                         structural_model = sm, missing = mean_replacement,
                         missing_value = "-99")

  result_ea <- assess_cvpat(model, technique = predict_EA,
                             nboot = 20, seed = 123, cores = 1)
  result_da <- assess_cvpat(model, technique = predict_DA,
                             nboot = 20, seed = 123, cores = 1)

  expect_false(identical(
    as.numeric(result_ea$CVPAT_compare_LM),
    as.numeric(result_da$CVPAT_compare_LM)
  ))
})

# =============================================================================
# helpers.R - Internal helper functions for seminrExtras
# =============================================================================
# This file contains shared utility functions used across the package:
# - Model validation helpers
# - Endogenous construct extraction helpers
# - Loss calculation helpers for CVPAT
# - Bootstrap significance testing helpers
# =============================================================================

#' @importFrom stats pt
#' @importFrom stats t.test
#' @importFrom stats var
#' @importFrom stats sd
#' @importFrom stats median
#' @importFrom stats lm
#' @importFrom stats coef
#' @importFrom stats dnorm
#' @importFrom stats cov quantile p.adjust
#' @importFrom utils head combn
#' @importFrom seminr predict_DA
#' @importFrom seminr predict_EA
#' @importFrom seminr mean_replacement
#' @importFrom seminr estimate_pls
#' @importFrom seminr predict_pls
#' @importFrom rpart rpart
#' @importFrom graphics abline axis barplot legend lines matlines matplot
#'   par plot points polygon rect text
#' @importFrom grDevices adjustcolor palette palette.colors
#'
NULL

# =============================================================================
# VALIDATION HELPERS
# =============================================================================

#' @noRd
validate_seminr_model <- function(model, func_name = "This function") {
  if (!inherits(model, "seminr_model")) {
    warning(func_name, " only works with SEMinR models.", call. = FALSE)
    return(FALSE)
  }
  return(TRUE)
}

#' @noRd
has_higher_order <- function(model) {
  !is.null(model$hoc)
}

#' @noRd
validate_for_prediction <- function(model, func_name = "This function") {
  if (!validate_seminr_model(model, func_name)) {
    return(FALSE)
  }
  if (has_higher_order(model)) {
    warning("There is no published solution for applying PLSpredict to higher-order models.",
            call. = FALSE)
    return(FALSE)
  }
  return(TRUE)
}

# =============================================================================
# SEMINR INTERNAL REIMPLEMENTATIONS
# =============================================================================
# These replace seminr:::  calls to avoid CRAN NOTEs about unexported imports.

#' Extract unique endogenous (target) constructs from a structural model matrix.
#' @noRd
all_endogenous <- function(smMatrix) {
  unique(smMatrix[, "target"])
}

#' Get the measurement indicator names for a construct.
#' @noRd
items_of_construct <- function(construct, model) {
  model$mmMatrix[model$mmMatrix[, 1] == construct, 2]
}

#' Compute bootstrap percentile confidence interval for a path.
#' @noRd
conf_int <- function(boot_array, from, to, through = NULL, alpha = 0.05) {
  if (is.null(through)) {
    coefficient <- boot_array[from, to, ]
  } else {
    coefficient <- boot_array[from, through, ] * boot_array[through, to, ]
  }
  quantile(coefficient, probs = c(alpha / 2, 1 - alpha / 2))
}

#' Assign the table_output class to a matrix for seminr-compatible printing.
#' @noRd
convert_to_table_output <- function(matrix) {
  class(matrix) <- c("table_output", class(matrix))
  matrix
}

# =============================================================================
# ENDOGENOUS CONSTRUCT HELPERS
# =============================================================================

#' @noRd
get_endogenous_constructs <- function(model) {
  all_endogenous(model$smMatrix)
}

#' @noRd
get_endogenous_items <- function(model, constructs = NULL) {
  if (is.null(constructs)) {
    constructs <- get_endogenous_constructs(model)
  }
  # Interaction constructs have no measurement indicators — exclude them
  constructs <- constructs[!grepl("\\*", constructs)]
  unlist(lapply(constructs, function(x) {
    items_of_construct(construct = x, model = model)
  }))
}

# =============================================================================
# LOSS CALCULATION HELPERS
# =============================================================================

#' @noRd
lv_loss <- function(construct, model, error) {
  items <- items_of_construct(construct = construct, model = model)
  if (length(dim(error)) > 1) {
    loss <- rowMeans(error[, items, drop = FALSE]^2)
  } else {
    loss <- as.vector(error[, items, drop = FALSE]^2)
  }
  return(loss)
}

#' @noRd
overall_loss <- function(error) {
  if (length(dim(error)) > 1) {
    return(rowMeans(error))
  }
  return(error)
}

#' @noRd
calculate_lv_losses <- function(constructs, model, error_matrix) {
  losses <- do.call("cbind", lapply(constructs, function(x) {
    lv_loss(construct = x, model = model, error = error_matrix)
  }))
  colnames(losses) <- constructs
  return(losses)
}

# =============================================================================
# BOOTSTRAP CVPAT HELPERS
# =============================================================================

#' @noRd
bootstrap_cvpat <- function(loss_m1, loss_m2, testtype = "two.sided", nboot = 2000) {

  n <- length(loss_m1)

  org_t_test <- t.test(loss_m2, loss_m1,
                       alternative = testtype,
                       paired = TRUE)$statistic

  org_d_bar <- mean(loss_m2 - loss_m1)

  d_null <- loss_m2 - loss_m1 - org_d_bar
  d <- loss_m2 - loss_m1

  boot_d_bar <- rep(0, nboot)
  m_losses <- cbind(loss_m1, loss_m2)
  t_stat <- rep(0, nboot)

  for (b in seq_len(nboot)) {
    boot_sample <- m_losses[sample(seq_along(d), length(d), replace = TRUE), ]

    t_stat[b] <- t.test(boot_sample[, 2], boot_sample[, 1],
                        mu = mean(d),
                        alternative = testtype,
                        paired = TRUE)$statistic

    boot_d_bar[b] <- mean(sample(d_null, length(d_null), replace = TRUE))
  }

  sorted_t_stat <- sort(t_stat, decreasing = FALSE)
  sorted_boot_d_bar <- sort(boot_d_bar, decreasing = FALSE)

  std <- sqrt(var(boot_d_bar))

  if (std < .Machine$double.eps || is.na(std)) {
    t_stat_boot_var <- NA
    warning("Bootstrap variance near zero; t-statistic set to NA", call. = FALSE)
  } else {
    t_stat_boot_var <- org_d_bar / std
  }

  if (testtype == "two.sided") {
    p_value_perc_t <- (sum(sorted_t_stat > abs(org_t_test)) +
                       sum(sorted_t_stat <= -abs(org_t_test))) / nboot
    p_value_perc_d <- (sum(sorted_boot_d_bar > abs(org_d_bar)) +
                       sum(sorted_boot_d_bar <= -abs(org_d_bar))) / nboot
    p_value_var_t <- 2 * pt(-abs(t_stat_boot_var), (n - 1), lower.tail = TRUE)
  }

  if (testtype == "greater") {
    idx_t <- which(sorted_t_stat > org_t_test)
    idx_d <- which(sorted_boot_d_bar > org_d_bar)

    if (length(idx_t) == 0) {
      p_value_perc_t <- 0
    } else {
      p_value_perc_t <- 1 - (head(idx_t, 1) - 1) / (nboot + 1)
    }

    if (length(idx_d) == 0) {
      p_value_perc_d <- 0
    } else {
      p_value_perc_d <- 1 - (head(idx_d, 1) - 1) / (nboot + 1)
    }
    p_value_var_t <- pt(t_stat_boot_var, (n - 1), lower.tail = FALSE)
  }

  results <- data.frame(
    "Std. T value" = org_t_test,
    "Std. P value" = p_value_perc_t,
    "Boot T value" = as.numeric(t_stat_boot_var),
    "Boot P Value" = as.numeric(p_value_var_t),
    "Perc. P Value" = as.numeric(p_value_perc_d),
    check.names = FALSE
  )

  return(results)
}

#' @noRd
cvpat_per_construct <- function(loss_one, loss_two, testtype = "two.sided", nboot = 2000) {
  constructs <- colnames(loss_one)
  results <- as.data.frame(matrix(nrow = 0, ncol = 6))

  for (construct in constructs) {
    boot_result <- bootstrap_cvpat(loss_one[, construct],
                                   loss_two[, construct],
                                   testtype = testtype,
                                   nboot = nboot)
    results <- rbind(results, c(construct, unlist(boot_result)))
  }

  colnames(results) <- c("Construct", "Std. T value", "Std. P value",
                         "Boot T value", "Boot P Value", "Perc. P Value")
  return(results)
}

# =============================================================================
# feature_pcm.R - Predictive Contribution of the Mediator (PCM)
# =============================================================================
# Evaluates the predictive contribution of mediating constructs by comparing
# predictions from Direct Antecedents (DA) vs Earliest Antecedents (EA)
# approaches on isolated sub-models.
#
# References:
# - Danks, N. P. (2021). The Piggy in the Middle: The Role of Mediators in
#   PLS-SEM Prediction. The DATA BASE for Advances in Information Systems,
#   52(SI), 24-42.
# - Shmueli, G., Sarstedt, M., Hair, J. F., Cheah, J.-H., Ting, H.,
#   Vaithilingam, S. & Ringle, C. M. (2019). Predictive Model Assessment in
#   PLS-SEM: Guidelines for Using PLSpredict. European Journal of Marketing,
#   53(11), 2322-2347.
# =============================================================================

# =============================================================================
# INTERNAL HELPERS
# =============================================================================

#' Detect the single final endogenous construct in a model.
#'
#' A "final" endogenous construct is one that appears as a target in the
#' structural model but never as a source.
#' @noRd
detect_final_endogenous <- function(model) {
  sm <- model$smMatrix
  endogenous <- unique(unname(sm[, "target"]))
  sources    <- unique(unname(sm[, "source"]))
  final_endo <- setdiff(endogenous, sources)

  if (length(final_endo) == 1) return(final_endo)
  if (length(final_endo) == 0) {
    stop("No final endogenous construct found (all endogenous constructs ",
         "are also sources). Please specify 'target'.", call. = FALSE)
  }
  stop("Multiple final endogenous constructs found: ",
       paste(final_endo, collapse = ", "),
       ". Please specify 'target'.", call. = FALSE)
}

#' Find all simple mediation paths (X -> M -> Y) to a target.
#'
#' A mediation path exists when a direct predictor of the target (M) is itself
#' endogenous (has its own antecedent X). Interaction constructs and
#' higher-order constructs are excluded (HOC items are LOC construct names,
#' not raw indicators, so sub-model reconstruction is not supported).
#' @noRd
find_mediation_paths <- function(model, target) {
  sm <- model$smMatrix

  # Direct predictors of target, excluding interaction constructs
  direct_preds <- unname(sm[sm[, "target"] == target, "source"])
  direct_preds <- direct_preds[!grepl("\\*", direct_preds)]

  paths <- list()
  for (mediator in direct_preds) {
    antecedents <- unname(sm[sm[, "target"] == mediator, "source"])
    antecedents <- antecedents[!grepl("\\*", antecedents)]

    for (antecedent in antecedents) {
      # Skip paths involving HOC constructs (cannot build sub-model)
      triple <- c(antecedent, mediator, target)
      hoc_in_triple <- sapply(triple, is_hoc_construct, model = model)
      if (any(hoc_in_triple)) {
        hoc_names <- triple[hoc_in_triple]
        warning(sprintf("Skipping %s -> %s -> %s: higher-order construct(s) %s ",
                        antecedent, mediator, target,
                        paste(hoc_names, collapse = ", ")),
                "cannot be used in PCM sub-models.", call. = FALSE)
        next
      }

      paths[[length(paths) + 1]] <- list(
        antecedent = antecedent,
        mediator   = mediator,
        target     = target
      )
    }
  }
  paths
}

#' Build an isolated sub-model for a single mediation triple.
#'
#' Creates and estimates a partial mediation model (X -> M, M -> Y, X -> Y)
#' containing only the three specified constructs.
#' @noRd
build_isolated_sub_model <- function(model, antecedent, mediator, target) {
  constructs_to_keep <- c(antecedent, mediator, target)

  # Reconstruct measurement model from the parent model's mmMatrix
  meas_specs <- list()
  for (cname in constructs_to_keep) {
    indicators <- items_of_construct(cname, model)
    mode <- get_construct_mode(cname, model)

    if (mode == "C") {
      meas_specs[[length(meas_specs) + 1]] <-
        seminr::reflective(cname, indicators)
    } else if (mode == "B") {
      meas_specs[[length(meas_specs) + 1]] <-
        seminr::composite(cname, indicators, weights = seminr::mode_B)
    } else {
      meas_specs[[length(meas_specs) + 1]] <-
        seminr::composite(cname, indicators, weights = seminr::mode_A)
    }
  }
  measurement_model <- do.call(seminr::constructs, meas_specs)

  # Partial mediation structural model: X -> M, M -> Y, X -> Y
  structural_model <- seminr::relationships(
    seminr::paths(from = antecedent, to = mediator),
    seminr::paths(from = c(antecedent, mediator), to = target)
  )

  # Re-estimate on original data
  suppressMessages(
    estimate_pls(
      data = model$rawdata,
      measurement_model = measurement_model,
      structural_model = structural_model
    )
  )
}

#' Compute PCM for a single mediation path.
#'
#' Builds an isolated sub-model, runs DA and EA predictions, and computes the
#' PCM metric per indicator of the target construct.
#' @noRd
compute_pcm_for_path <- function(model, antecedent, mediator, target,
                                 noFolds, reps) {
  # Build isolated sub-model
  sub_model <- build_isolated_sub_model(model, antecedent, mediator, target)

  # Run DA and EA cross-validated predictions
  pred_da <- suppressMessages(
    predict_pls(sub_model, technique = predict_DA,
                noFolds = noFolds, reps = reps)
  )
  pred_ea <- suppressMessages(
    predict_pls(sub_model, technique = predict_EA,
                noFolds = noFolds, reps = reps)
  )

  # Get summary metrics
  sum_da <- summary(pred_da)
  sum_ea <- summary(pred_ea)

  # Extract target indicators
  target_indicators <- items_of_construct(target, sub_model)

  # Get RMSE and MAE for target indicators (metrics = rows, indicators = cols)
  rmse_da <- sum_da$PLS_out_of_sample["RMSE", target_indicators]
  mae_da  <- sum_da$PLS_out_of_sample["MAE",  target_indicators]
  rmse_ea <- sum_ea$PLS_out_of_sample["RMSE", target_indicators]
  mae_ea  <- sum_ea$PLS_out_of_sample["MAE",  target_indicators]

  # PCM = (METRIC_EA - METRIC_DA) / METRIC_EA  (equation 2)
  pcm_rmse <- (rmse_ea - rmse_da) / rmse_ea
  pcm_mae  <- (mae_ea  - mae_da)  / mae_ea

  # Build results matrix
  result_matrix <- cbind(
    RMSE_DA  = rmse_da,
    RMSE_EA  = rmse_ea,
    PCM_RMSE = pcm_rmse,
    MAE_DA   = mae_da,
    MAE_EA   = mae_ea,
    PCM_MAE  = pcm_mae
  )
  rownames(result_matrix) <- target_indicators

  list(
    antecedent = antecedent,
    mediator   = mediator,
    target     = target,
    results    = result_matrix,
    pcm_rmse   = as.vector(pcm_rmse),
    pcm_mae    = as.vector(pcm_mae)
  )
}

#' Check whether a construct is a higher-order construct (HOC).
#'
#' HOC items are LOC construct names (not raw indicators), identifiable by
#' being column names in the outer_weights matrix.
#' @noRd
is_hoc_construct <- function(construct, model) {
  items <- items_of_construct(construct, model)
  construct_names <- colnames(model$outer_weights)
  any(items %in% construct_names)
}

#' Classify a PCM value using rules of thumb from Danks (2021).
#' @noRd
classify_pcm <- function(pcm_value) {
  if (is.na(pcm_value)) return("NA")
  if (pcm_value < 0)    return("Negative")
  if (pcm_value < 0.05) return("Weak")
  if (pcm_value < 0.10) return("Moderate")
  return("Strong")
}

# =============================================================================
# EXPORTED FUNCTIONS
# =============================================================================

#' Predictive Contribution of the Mediator (PCM)
#'
#' Evaluates the predictive contribution of mediating constructs in a PLS-SEM
#' model. For each mediation path (antecedent -> mediator -> target), computes
#' the PCM metric by comparing cross-validated predictions from the Direct
#' Antecedents (DA) and Earliest Antecedents (EA) approaches on an isolated
#' partial mediation sub-model.
#'
#' @param seminr_model A PLS model estimated by \code{seminr::estimate_pls()}.
#' @param target Character string specifying the outcome construct. If
#'   \code{NULL}, auto-detects when a single final endogenous construct exists.
#' @param noFolds Integer, number of folds for cross-validation (default 10).
#' @param reps Integer, number of cross-validation repetitions (default 10).
#'
#' @return An object of class \code{pcm_analysis} containing:
#' \describe{
#'   \item{target}{The outcome construct name.}
#'   \item{mediation_paths}{List of identified mediation triples
#'     (antecedent, mediator, target).}
#'   \item{pcm_results}{List of per-path results, each with RMSE/MAE for DA
#'     and EA approaches plus PCM values per indicator.}
#'   \item{noFolds}{Number of cross-validation folds used.}
#'   \item{reps}{Number of cross-validation repetitions used.}
#' }
#'
#' @details
#' The PCM metric (Danks, 2021) quantifies the predictive improvement due to
#' the mediator. It is computed by isolating each mediation path into a partial
#' mediation sub-model (X -> M -> Y with direct path X -> Y), then comparing
#' out-of-sample predictions from two approaches:
#'
#' \itemize{
#'   \item \strong{DA (Direct Antecedents)}: Predicts Y using its direct
#'     structural predictors (M and X).
#'   \item \strong{EA (Earliest Antecedents)}: Predicts Y using only the
#'     earliest antecedent (X), propagated through the structural model.
#' }
#'
#' \deqn{PCM = \frac{METRIC_{EA} - METRIC_{DA}}{METRIC_{EA}}}
#'
#' Rules of thumb for PCM interpretation:
#' \itemize{
#'   \item \strong{0 to 0.05}: Weak predictive contribution
#'   \item \strong{0.05 to 0.10}: Moderate predictive contribution
#'   \item \strong{Greater than 0.10}: Strong predictive contribution
#'   \item \strong{Negative}: Mediator damages predictive accuracy
#' }
#'
#' @references Danks, N. P. (2021). The Piggy in the Middle: The Role of
#' Mediators in PLS-SEM Prediction. \emph{The DATA BASE for Advances in
#' Information Systems}, 52(SI), 24-42.
#'
#' @seealso \code{\link[seminr]{predict_pls}} for the underlying prediction
#'   methods.
#'
#' @examples
#' \donttest{
#' library(seminr)
#' data(mobi)
#'
#' # Simple mediation: Image -> Satisfaction -> Loyalty
#' mobi_mm <- constructs(
#'   composite("Image",        multi_items("IMAG", 1:5)),
#'   composite("Satisfaction", multi_items("CUSA", 1:3)),
#'   composite("Loyalty",      multi_items("CUSL", 1:3))
#' )
#' mobi_sm <- relationships(
#'   paths(from = "Image", to = "Satisfaction"),
#'   paths(from = "Satisfaction", to = "Loyalty"),
#'   paths(from = "Image", to = "Loyalty")
#' )
#' model <- estimate_pls(mobi, mobi_mm, mobi_sm)
#'
#' # Compute PCM
#' pcm <- assess_pcm(model, target = "Loyalty", noFolds = 10, reps = 10)
#' pcm
#' summary(pcm)
#' }
#'
#' @export
assess_pcm <- function(seminr_model, target = NULL, noFolds = 10, reps = 10) {
  # --- Validation ---
  if (!validate_seminr_model(seminr_model, "assess_pcm")) return(NULL)
  if (!validate_for_prediction(seminr_model, "assess_pcm")) return(NULL)

  if (!is.numeric(noFolds) || noFolds < 2) {
    stop("noFolds must be an integer >= 2.", call. = FALSE)
  }
  if (!is.numeric(reps) || reps < 1) {
    stop("reps must be an integer >= 1.", call. = FALSE)
  }

  # --- Identify target ---
  if (is.null(target)) {
    target <- detect_final_endogenous(seminr_model)
  }
  if (!(target %in% colnames(seminr_model$path_coef))) {
    stop("Target '", target, "' not found in model constructs.", call. = FALSE)
  }

  # --- Find mediation paths ---
  mediation_paths <- find_mediation_paths(seminr_model, target)
  if (length(mediation_paths) == 0) {
    stop("No mediation paths found for target '", target,
         "'. The target must have at least one predictor that is itself ",
         "endogenous.", call. = FALSE)
  }

  # --- Compute PCM per path ---
  pcm_results <- vector("list", length(mediation_paths))
  for (i in seq_along(mediation_paths)) {
    path <- mediation_paths[[i]]
    message(sprintf("Computing PCM for %s -> %s -> %s ...",
                    path$antecedent, path$mediator, path$target))
    pcm_results[[i]] <- compute_pcm_for_path(
      seminr_model, path$antecedent, path$mediator, path$target,
      noFolds, reps
    )
  }

  # --- Return ---
  output <- list(
    target          = target,
    mediation_paths = mediation_paths,
    pcm_results     = pcm_results,
    noFolds         = noFolds,
    reps            = reps
  )
  class(output) <- c("pcm_analysis", class(output))
  output
}

# =============================================================================
# S3 METHODS
# =============================================================================

#' @export
print.pcm_analysis <- function(x, ...) {
  cat("Predictive Contribution of the Mediator (PCM)\n")
  cat("==============================================\n")
  cat("Target:", x$target, "\n")
  cat("Cross-validation:", x$noFolds, "folds,", x$reps, "reps\n")
  cat("Mediation paths:", length(x$mediation_paths), "\n\n")

  for (i in seq_along(x$pcm_results)) {
    res  <- x$pcm_results[[i]]
    path <- x$mediation_paths[[i]]
    cat(sprintf("  %s -> %s -> %s\n",
                path$antecedent, path$mediator, path$target))

    avg_pcm <- mean(res$pcm_rmse)
    cat(sprintf("    Avg PCM (RMSE): %7.4f  [%s]\n",
                avg_pcm, classify_pcm(avg_pcm)))
    avg_pcm_mae <- mean(res$pcm_mae)
    cat(sprintf("    Avg PCM (MAE):  %7.4f  [%s]\n\n",
                avg_pcm_mae, classify_pcm(avg_pcm_mae)))
  }
  invisible(x)
}

#' @export
summary.pcm_analysis <- function(object, ...) {
  out <- list(
    target          = object$target,
    noFolds         = object$noFolds,
    reps            = object$reps,
    mediation_paths = object$mediation_paths,
    pcm_results     = object$pcm_results
  )
  class(out) <- c("summary.pcm_analysis", class(out))
  out
}

#' @export
print.summary.pcm_analysis <- function(x, ...) {
  cat("Predictive Contribution of the Mediator (PCM)\n")
  cat("==============================================\n")
  cat("Target:", x$target, "\n")
  cat("Cross-validation:", x$noFolds, "folds,", x$reps, "reps\n\n")

  for (i in seq_along(x$pcm_results)) {
    res  <- x$pcm_results[[i]]
    path <- x$mediation_paths[[i]]

    cat(sprintf("Mediation: %s -> %s -> %s\n",
                path$antecedent, path$mediator, path$target))
    cat(paste(rep("-", 60), collapse = ""), "\n")

    # Header
    cat(sprintf("  %-12s %8s %8s %9s %8s %8s %9s  %s\n",
                "Indicator", "RMSE_DA", "RMSE_EA", "PCM_RMSE",
                "MAE_DA", "MAE_EA", "PCM_MAE", "Conclusion"))

    display <- res$results
    for (j in seq_len(nrow(display))) {
      cat(sprintf("  %-12s %8.4f %8.4f %9.4f %8.4f %8.4f %9.4f  %s\n",
                  rownames(display)[j],
                  display[j, "RMSE_DA"],  display[j, "RMSE_EA"],
                  display[j, "PCM_RMSE"],
                  display[j, "MAE_DA"],   display[j, "MAE_EA"],
                  display[j, "PCM_MAE"],
                  classify_pcm(display[j, "PCM_RMSE"])))
    }
    cat("\n")
  }

  cat("PCM thresholds: < 0 Negative | 0-0.05 Weak | 0.05-0.10 Moderate | > 0.10 Strong\n")
  cat("Reference: Danks (2021), The DATA BASE for Advances in IS, 52(SI), 24-42.\n")
  invisible(x)
}

#' Plot PCM Results
#'
#' Creates a grouped barplot of PCM values for each mediation path, with one bar
#' per indicator and threshold lines for the weak/moderate/strong classification
#' boundaries.
#'
#' @param x An object of class \code{pcm_analysis} from \code{\link{assess_pcm}}.
#' @param metric Character, either \code{"RMSE"} (default) or \code{"MAE"}.
#' @param legend_pos Character giving the legend position (e.g. \code{"topright"},
#'   \code{"topleft"}, \code{"bottomright"}). Set to \code{NULL} or \code{FALSE}
#'   to suppress the legend entirely. Default \code{"topright"}.
#' @param bar_col Colour for positive-PCM bars (default \code{"steelblue"}).
#' @param neg_col Colour for negative-PCM bars (default \code{"tomato"}).
#' @param cex.labels Numeric expansion factor for the value labels printed on
#'   each bar. Default 0.8.
#' @param cex.legend Numeric expansion factor for the legend text. Default 0.75.
#' @param ... Additional graphical parameters passed to \code{\link[graphics]{barplot}}.
#'
#' @examples
#' \donttest{
#' # Customise legend placement and bar colours
#' # plot(pcm, legend_pos = "topleft", bar_col = "dodgerblue")
#'
#' # Suppress legend entirely
#' # plot(pcm, legend_pos = NULL)
#' }
#'
#' @export
plot.pcm_analysis <- function(x, metric = "RMSE",
                              legend_pos = "topright",
                              bar_col = "steelblue",
                              neg_col = "tomato",
                              cex.labels = 0.8,
                              cex.legend = 0.75, ...) {
  metric <- match.arg(metric, c("RMSE", "MAE"))
  pcm_col <- if (metric == "RMSE") "PCM_RMSE" else "PCM_MAE"

  n_paths <- length(x$pcm_results)

  # Set up multi-panel layout if multiple paths
  if (n_paths > 1) {
    old_par <- par(mfrow = c(1, n_paths), mar = c(5, 4, 4, 1))
    on.exit(par(old_par))
  }

  show_legend <- !is.null(legend_pos) && !identical(legend_pos, FALSE)

  for (i in seq_along(x$pcm_results)) {
    res  <- x$pcm_results[[i]]
    path <- x$mediation_paths[[i]]
    pcm_vals <- res$results[, pcm_col]

    # Bar colours: user-defined positive / negative
    bar_cols <- ifelse(pcm_vals >= 0,
                       adjustcolor(bar_col, 0.7),
                       adjustcolor(neg_col, 0.7))

    y_range <- range(c(pcm_vals, 0, 0.12))
    y_lim   <- c(min(y_range[1], -0.02), max(y_range[2] * 1.15, 0.12))

    bp <- barplot(pcm_vals,
                  names.arg = rownames(res$results),
                  col = bar_cols,
                  ylim = y_lim,
                  ylab = paste0("PCM (", metric, ")"),
                  main = sprintf("%s -> %s -> %s",
                                 path$antecedent, path$mediator, path$target),
                  las = 2, border = NA, ...)

    # Threshold lines
    abline(h = 0,    lty = 1, col = "grey40")
    abline(h = 0.05, lty = 2, col = "orange")
    abline(h = 0.10, lty = 2, col = "darkgreen")

    # Value labels on bars
    text(bp, pcm_vals, labels = sprintf("%.3f", pcm_vals),
         pos = ifelse(pcm_vals >= 0, 3, 1), cex = cex.labels)

    # Legend (only on first panel, if requested)
    if (show_legend && i == 1) {
      legend(legend_pos,
             legend = c("Weak (< 0.05)", "Moderate (0.05-0.10)", "Strong (> 0.10)"),
             lty = c(0, 2, 2),
             col = c(NA, "orange", "darkgreen"),
             pch = c(15, NA, NA),
             pt.cex = 1.5,
             bty = "n", cex = cex.legend)
    }
  }
}

# =============================================================================
# Tests for PCM (Predictive Contribution of the Mediator)
# =============================================================================
library(testthat)
library(seminr)
library(seminrExtras)

# =============================================================================
# FIXTURES: Pre-compute models once
# =============================================================================

# Simple mediation model: Image -> Satisfaction -> Loyalty
mobi_mm_simple <- constructs(
  composite("Image",        multi_items("IMAG", 1:5)),
  composite("Satisfaction", multi_items("CUSA", 1:3)),
  composite("Loyalty",      multi_items("CUSL", 1:3))
)
mobi_sm_simple <- relationships(
  paths(from = "Image", to = "Satisfaction"),
  paths(from = "Satisfaction", to = "Loyalty"),
  paths(from = "Image", to = "Loyalty")
)
pls_simple <- estimate_pls(mobi, mobi_mm_simple, mobi_sm_simple)

# Multi-mediator model: ECSI paths to Loyalty
mobi_mm_multi <- constructs(
  composite("Image",        multi_items("IMAG", 1:5)),
  composite("Expectation",  multi_items("CUEX", 1:3)),
  composite("Value",        multi_items("PERV", 1:2)),
  composite("Satisfaction", multi_items("CUSA", 1:3)),
  composite("Loyalty",      multi_items("CUSL", 1:3))
)
mobi_sm_multi <- relationships(
  paths(from = "Image",       to = c("Expectation", "Satisfaction", "Loyalty")),
  paths(from = "Expectation", to = c("Value", "Satisfaction")),
  paths(from = "Value",       to = "Satisfaction"),
  paths(from = "Satisfaction", to = "Loyalty")
)
pls_multi <- estimate_pls(mobi, mobi_mm_multi, mobi_sm_multi)

# =============================================================================
# Run PCM on fixtures (low reps for speed)
# =============================================================================
pcm_simple <- assess_pcm(pls_simple, target = "Loyalty", noFolds = 10, reps = 2)
pcm_multi  <- assess_pcm(pls_multi, target = "Loyalty", noFolds = 10, reps = 2)

# =============================================================================
# TESTS: Internal helpers
# =============================================================================

test_that("detect_final_endogenous identifies correct construct", {
  result <- seminrExtras:::detect_final_endogenous(pls_simple)
  expect_equal(result, "Loyalty")

  result2 <- seminrExtras:::detect_final_endogenous(pls_multi)
  expect_equal(result2, "Loyalty")
})

test_that("find_mediation_paths finds correct paths for simple model", {
  paths <- seminrExtras:::find_mediation_paths(pls_simple, "Loyalty")
  expect_length(paths, 1)
  expect_equal(paths[[1]]$antecedent, "Image")
  expect_equal(paths[[1]]$mediator, "Satisfaction")
  expect_equal(paths[[1]]$target, "Loyalty")
})

test_that("find_mediation_paths finds correct paths for multi-mediator model", {
  paths <- seminrExtras:::find_mediation_paths(pls_multi, "Loyalty")
  # Satisfaction has 3 antecedents: Image, Expectation, Value
  expect_length(paths, 3)

  antecedents <- sapply(paths, function(p) p$antecedent)
  mediators   <- sapply(paths, function(p) p$mediator)
  expect_true(all(mediators == "Satisfaction"))
  expect_true("Image" %in% antecedents)
  expect_true("Expectation" %in% antecedents)
  expect_true("Value" %in% antecedents)
})

test_that("find_mediation_paths returns empty for non-mediated construct", {
  # Satisfaction has no mediation paths to Loyalty in simple model
  # because its antecedent (Image) does not have its own antecedent
  paths <- seminrExtras:::find_mediation_paths(pls_simple, "Satisfaction")
  expect_length(paths, 0)
})

test_that("classify_pcm returns correct classifications", {
  expect_equal(seminrExtras:::classify_pcm(-0.05), "Negative")
  expect_equal(seminrExtras:::classify_pcm(0.00),  "Weak")
  expect_equal(seminrExtras:::classify_pcm(0.03),  "Weak")
  expect_equal(seminrExtras:::classify_pcm(0.049), "Weak")
  expect_equal(seminrExtras:::classify_pcm(0.05),  "Moderate")
  expect_equal(seminrExtras:::classify_pcm(0.07),  "Moderate")
  expect_equal(seminrExtras:::classify_pcm(0.10),  "Strong")
  expect_equal(seminrExtras:::classify_pcm(0.20),  "Strong")
  expect_equal(seminrExtras:::classify_pcm(NA),    "NA")
})

# =============================================================================
# TESTS: build_isolated_sub_model
# =============================================================================

test_that("build_isolated_sub_model creates valid sub-model", {
  sub <- seminrExtras:::build_isolated_sub_model(
    pls_simple, "Image", "Satisfaction", "Loyalty"
  )
  expect_s3_class(sub, "seminr_model")
  expect_equal(sort(colnames(sub$construct_scores)),
               sort(c("Image", "Satisfaction", "Loyalty")))

  # Should have 3 structural paths: Image->Satisfaction, Satisfaction->Loyalty, Image->Loyalty
  sm <- sub$smMatrix
  expect_equal(nrow(sm), 3)
})

test_that("build_isolated_sub_model preserves measurement mode", {
  # Use a model with mode_B construct
  mm_mixed <- constructs(
    composite("Image",        multi_items("IMAG", 1:5), weights = mode_B),
    composite("Satisfaction", multi_items("CUSA", 1:3)),
    composite("Loyalty",      multi_items("CUSL", 1:3))
  )
  sm_mixed <- relationships(
    paths(from = "Image", to = "Satisfaction"),
    paths(from = "Satisfaction", to = "Loyalty"),
    paths(from = "Image", to = "Loyalty")
  )
  model_mixed <- estimate_pls(mobi, mm_mixed, sm_mixed)

  sub <- seminrExtras:::build_isolated_sub_model(
    model_mixed, "Image", "Satisfaction", "Loyalty"
  )

  # Image should be mode B in the sub-model
  mode <- sub$mmMatrix[sub$mmMatrix[, "construct"] == "Image", "type"]
  expect_true(all(mode == "B"))
})

# =============================================================================
# TESTS: assess_pcm output structure
# =============================================================================

test_that("assess_pcm returns correct class", {
  expect_s3_class(pcm_simple, "pcm_analysis")
})

test_that("assess_pcm output has expected components", {
  expect_equal(pcm_simple$target, "Loyalty")
  expect_equal(pcm_simple$noFolds, 10)
  expect_equal(pcm_simple$reps, 2)
  expect_length(pcm_simple$mediation_paths, 1)
  expect_length(pcm_simple$pcm_results, 1)
})

test_that("PCM results have correct structure per path", {
  res <- pcm_simple$pcm_results[[1]]
  expect_equal(res$antecedent, "Image")
  expect_equal(res$mediator, "Satisfaction")
  expect_equal(res$target, "Loyalty")

  # Results matrix should have 3 rows (CUSL1-3) and 6 columns
  expect_equal(nrow(res$results), 3)
  expect_equal(ncol(res$results), 6)
  expect_equal(colnames(res$results),
               c("RMSE_DA", "RMSE_EA", "PCM_RMSE", "MAE_DA", "MAE_EA", "PCM_MAE"))
  expect_equal(sort(rownames(res$results)), sort(c("CUSL1", "CUSL2", "CUSL3")))
})

test_that("PCM values are finite and within expected range", {
  for (res in pcm_simple$pcm_results) {
    expect_true(all(is.finite(res$pcm_rmse)))
    expect_true(all(is.finite(res$pcm_mae)))
    # PCM should typically be between -1 and 1
    expect_true(all(res$pcm_rmse > -1 & res$pcm_rmse < 1))
    expect_true(all(res$pcm_mae  > -1 & res$pcm_mae  < 1))
  }
})

test_that("PCM RMSE and MAE are derived correctly from DA and EA metrics", {
  res <- pcm_simple$pcm_results[[1]]
  mat <- res$results
  # PCM = (EA - DA) / EA
  expected_pcm_rmse <- (mat[, "RMSE_EA"] - mat[, "RMSE_DA"]) / mat[, "RMSE_EA"]
  expected_pcm_mae  <- (mat[, "MAE_EA"]  - mat[, "MAE_DA"])  / mat[, "MAE_EA"]
  expect_equal(as.vector(mat[, "PCM_RMSE"]), as.vector(expected_pcm_rmse))
  expect_equal(as.vector(mat[, "PCM_MAE"]),  as.vector(expected_pcm_mae))
})

test_that("DA RMSE and MAE are strictly positive", {
  for (res in pcm_simple$pcm_results) {
    expect_true(all(res$results[, "RMSE_DA"] > 0))
    expect_true(all(res$results[, "MAE_DA"]  > 0))
    expect_true(all(res$results[, "RMSE_EA"] > 0))
    expect_true(all(res$results[, "MAE_EA"]  > 0))
  }
})

# =============================================================================
# TESTS: Multi-mediator model
# =============================================================================

test_that("assess_pcm handles multiple mediation paths", {
  expect_length(pcm_multi$pcm_results, 3)
  mediators <- sapply(pcm_multi$pcm_results, function(r) r$mediator)
  expect_true(all(mediators == "Satisfaction"))
})

test_that("Multi-mediator PCM has results per indicator per path", {
  for (res in pcm_multi$pcm_results) {
    expect_equal(nrow(res$results), 3)  # CUSL1, CUSL2, CUSL3
    expect_equal(ncol(res$results), 6)
  }
})

# =============================================================================
# TESTS: Auto-detection and validation
# =============================================================================

test_that("assess_pcm auto-detects single final endogenous target", {
  pcm_auto <- assess_pcm(pls_simple, noFolds = 10, reps = 2)
  expect_equal(pcm_auto$target, "Loyalty")
})

test_that("assess_pcm rejects non-seminr model", {
  expect_warning(
    result <- assess_pcm(list(not = "a model")),
    "only works with SEMinR"
  )
  expect_null(result)
})

test_that("assess_pcm rejects invalid target", {
  expect_error(
    assess_pcm(pls_simple, target = "Nonexistent"),
    "not found in model"
  )
})

test_that("assess_pcm rejects target with no mediation", {
  # Satisfaction is endogenous but its predictor (Image) has no antecedents
  expect_error(
    assess_pcm(pls_simple, target = "Satisfaction"),
    "No mediation paths"
  )
})

test_that("assess_pcm rejects invalid noFolds", {
  expect_error(assess_pcm(pls_simple, noFolds = 1), "noFolds must be")
  expect_error(assess_pcm(pls_simple, noFolds = "a"), "noFolds must be")
})

test_that("assess_pcm rejects invalid reps", {
  expect_error(assess_pcm(pls_simple, reps = 0), "reps must be")
})

# =============================================================================
# TESTS: S3 methods
# =============================================================================

test_that("print.pcm_analysis produces output", {
  output <- capture.output(print(pcm_simple))
  expect_true(any(grepl("Predictive Contribution", output)))
  expect_true(any(grepl("Loyalty", output)))
  expect_true(any(grepl("Image -> Satisfaction -> Loyalty", output)))
  expect_true(any(grepl("PCM", output)))
})

test_that("summary.pcm_analysis returns correct class", {
  s <- summary(pcm_simple)
  expect_s3_class(s, "summary.pcm_analysis")
})

test_that("print.summary.pcm_analysis shows detailed results", {
  s <- summary(pcm_simple)
  output <- capture.output(print(s))
  expect_true(any(grepl("RMSE_DA", output)))
  expect_true(any(grepl("RMSE_EA", output)))
  expect_true(any(grepl("PCM_RMSE", output)))
  expect_true(any(grepl("CUSL", output)))
  expect_true(any(grepl("Danks", output)))
})

# =============================================================================
# TESTS: Interaction construct exclusion
# =============================================================================

test_that("PCM excludes interaction constructs from mediation paths", {
  mobi_mm_mod <- constructs(
    composite("Image",        multi_items("IMAG", 1:5)),
    composite("Value",        multi_items("PERV", 1:2)),
    composite("Satisfaction", multi_items("CUSA", 1:3)),
    composite("Loyalty",      multi_items("CUSL", 1:3)),
    interaction_term(iv = "Image", moderator = "Value")
  )
  mobi_sm_mod <- relationships(
    paths(from = "Image", to = "Satisfaction"),
    paths(from = "Value", to = "Satisfaction"),
    paths(from = "Image*Value", to = "Satisfaction"),
    paths(from = "Satisfaction", to = "Loyalty"),
    paths(from = "Image", to = "Loyalty")
  )
  model_mod <- estimate_pls(mobi, mobi_mm_mod, mobi_sm_mod)

  paths <- seminrExtras:::find_mediation_paths(model_mod, "Loyalty")
  mediators <- sapply(paths, function(p) p$mediator)
  antecedents <- sapply(paths, function(p) p$antecedent)

  # No interaction construct should appear as mediator or antecedent

  expect_false(any(grepl("\\*", mediators)))
  expect_false(any(grepl("\\*", antecedents)))
})

test_that("PCM works end-to-end with moderation model", {
  skip_on_cran()
  mobi_mm_mod <- constructs(
    composite("Image",        multi_items("IMAG", 1:5)),
    composite("Value",        multi_items("PERV", 1:2)),
    composite("Satisfaction", multi_items("CUSA", 1:3)),
    composite("Loyalty",      multi_items("CUSL", 1:3)),
    interaction_term(iv = "Image", moderator = "Value")
  )
  mobi_sm_mod <- relationships(
    paths(from = "Image", to = "Satisfaction"),
    paths(from = "Value", to = "Satisfaction"),
    paths(from = "Image*Value", to = "Satisfaction"),
    paths(from = "Satisfaction", to = "Loyalty"),
    paths(from = "Image", to = "Loyalty")
  )
  model_mod <- estimate_pls(mobi, mobi_mm_mod, mobi_sm_mod)

  pcm_mod <- assess_pcm(model_mod, target = "Loyalty", noFolds = 10, reps = 2)
  expect_s3_class(pcm_mod, "pcm_analysis")
  # Should find mediation paths through Satisfaction, excluding Image*Value
  expect_true(length(pcm_mod$pcm_results) >= 1)
  for (res in pcm_mod$pcm_results) {
    expect_false(grepl("\\*", res$mediator))
    expect_false(grepl("\\*", res$antecedent))
  }
})

# =============================================================================
# TESTS: HOC guard
# =============================================================================

test_that("PCM skips paths involving HOC constructs with warning", {
  # Two-stage HOC model: Image (HOC) with LOCs
  mobi_mm_hoc <- constructs(
    composite("Tangibles",  multi_items("IMAG", 1:3)),
    composite("Intangibles", multi_items("IMAG", 4:5)),
    higher_composite("Image", c("Tangibles", "Intangibles")),
    composite("Satisfaction", multi_items("CUSA", 1:3)),
    composite("Loyalty",      multi_items("CUSL", 1:3))
  )
  mobi_sm_hoc <- relationships(
    paths(from = "Image", to = "Satisfaction"),
    paths(from = "Satisfaction", to = "Loyalty"),
    paths(from = "Image", to = "Loyalty")
  )
  model_hoc <- estimate_pls(mobi, mobi_mm_hoc, mobi_sm_hoc)

  # Should warn about HOC and return no valid paths
  expect_warning(
    paths <- seminrExtras:::find_mediation_paths(model_hoc, "Loyalty"),
    "higher-order"
  )
  expect_length(paths, 0)
})

# =============================================================================
# TESTS: Plot method
# =============================================================================

test_that("plot.pcm_analysis runs without error", {
  expect_no_error(plot(pcm_simple))
})

test_that("plot.pcm_analysis accepts MAE metric", {
  expect_no_error(plot(pcm_simple, metric = "MAE"))
})

test_that("plot.pcm_analysis handles multiple paths", {
  expect_no_error(plot(pcm_multi))
})

# =============================================================================
# feature_fimix.R - FIMIX-PLS (Finite Mixture PLS)
# =============================================================================
# Implements EM-based latent class segmentation for PLS-SEM structural models.
# Observations are probabilistically assigned to K segments, each with
# segment-specific structural path coefficients.
#
# References:
# - Hahn, C., Johnson, M. D., Herrmann, A., & Huber, F. (2002). Capturing
#   Customer Heterogeneity using a Finite Mixture PLS Approach. Schmalenbach
#   Business Review, 54, 243-269.
# - Sarstedt, M., Becker, J.-M., Ringle, C. M. & Schwaiger, M. (2011).
#   Uncovering and Treating Unobserved Heterogeneity with FIMIX-PLS.
#   Schmalenbach Business Review, 63(1), 34-62.
# - Hair, J. F. et al. (2016). Identifying and Treating Unobserved
#   Heterogeneity with FIMIX-PLS: Part I. EBR, 28(1), 63-76.
# - Ringle, C. M., Sarstedt, M. & Mooi, E. A. (2010). Response-Based
#   Segmentation Using Finite Mixture PLS. In Data Mining, Springer, 19-49.
# =============================================================================

# =============================================================================
# INTERNAL HELPERS
# =============================================================================

#' Log-sum-exp trick for numerical stability.
#' @noRd
log_sum_exp <- function(x) {
  m <- max(x)
  if (is.infinite(m)) return(m)
  m + log(sum(exp(x - m)))
}

#' Extract structural equations from a seminr model.
#'
#' Returns a list of equations, each with target name, predictor names,
#' y vector (endogenous scores), and X matrix (intercept + predictor scores).
#' @noRd
extract_structural_equations <- function(model) {
  sm <- model$smMatrix
  scores <- model$construct_scores
  endogenous <- unique(sm[, "target"])

  equations <- list()
  for (target in endogenous) {
    predictors <- unname(sm[sm[, "target"] == target, "source"])
    y <- scores[, target]
    X <- cbind(1, scores[, predictors, drop = FALSE])
    colnames(X)[1] <- "(Intercept)"

    equations[[target]] <- list(
      target = target,
      predictors = predictors,
      y = y,
      X = X
    )
  }

  equations
}

#' Initialize random posterior probabilities via Dirichlet-like sampling.
#' @noRd
init_random_posteriors <- function(N, K) {
  # Sample from Dirichlet(1,...,1) = normalized exponentials of uniform
  raw <- matrix(stats::rexp(N * K, rate = 1), nrow = N, ncol = K)
  raw / rowSums(raw)
}

#' Weighted OLS for one structural equation in one segment.
#'
#' @param y Numeric vector of endogenous construct scores.
#' @param X Matrix with intercept column + predictor scores.
#' @param weights Numeric vector of observation weights (posterior probs).
#' @return List with coefficients and residual variance.
#' @noRd
weighted_ols <- function(y, X, weights) {
  W <- as.vector(weights)
  XtWX <- crossprod(X * W, X)
  XtWy <- crossprod(X * W, y)

  beta <- tryCatch(
    as.vector(solve(XtWX, XtWy)),
    error = function(e) rep(NA_real_, ncol(X))
  )

  if (any(is.na(beta))) {
    return(list(coefficients = beta,
                variance = NA_real_))
  }

  resid <- as.vector(y - X %*% beta)
  sigma2 <- sum(W * resid^2) / sum(W)
  # Floor variance to prevent degenerate densities
  sigma2 <- max(sigma2, .Machine$double.eps * 100)

  list(coefficients = beta, variance = sigma2)
}

#' Count the number of free parameters in a FIMIX model.
#'
#' Q = K * sum_j(p_j + 2) + (K - 1)
#' where p_j = number of predictors for equation j, +1 intercept, +1 variance.
#' @noRd
count_fimix_parameters <- function(equations, K) {
  params_per_eq <- vapply(equations, function(eq) {
    length(eq$predictors) + 2  # coefficients + intercept + variance
  }, numeric(1))

  K * sum(params_per_eq) + (K - 1)
}

#' Compute normed entropy statistic.
#'
#' EN = 1 - (-sum P*log(P)) / (N * log(K))
#' Closer to 1 = better classification quality.
#' @noRd
compute_entropy <- function(posteriors, K) {
  if (K <= 1) return(NA_real_)

  N <- nrow(posteriors)
  # Clamp to avoid log(0)
  P <- pmax(posteriors, .Machine$double.eps)
  raw_entropy <- -sum(P * log(P))
  1 - raw_entropy / (N * log(K))
}

#' Compute all FIMIX information criteria.
#' @noRd
compute_fimix_criteria <- function(lnL, Q, N, K, posteriors) {
  c(
    lnL  = lnL,
    AIC  = -2 * lnL + 2 * Q,
    AIC3 = -2 * lnL + 3 * Q,
    AIC4 = -2 * lnL + 4 * Q,
    BIC  = -2 * lnL + Q * log(N),
    CAIC = -2 * lnL + Q * (log(N) + 1),
    HQ   = -2 * lnL + 2 * Q * log(log(N)),
    MDL5 = -2 * lnL + (Q / 2) * log(N),
    EN   = compute_entropy(posteriors, K)
  )
}

#' Run a single EM iteration set for FIMIX-PLS.
#'
#' @param equations List from extract_structural_equations().
#' @param K Number of segments.
#' @param max_iter Maximum EM iterations.
#' @param stop_criterion Convergence threshold on log-likelihood change.
#' @param posteriors_init N x K matrix of initial posterior probabilities.
#' @return List with final parameters, posteriors, log-likelihood, convergence info.
#' @noRd
run_fimix_em <- function(equations, K, max_iter, stop_criterion, posteriors_init) {
  N <- length(equations[[1]]$y)
  J <- length(equations)
  posteriors <- posteriors_init
  lnL_old <- -Inf
  converged <- FALSE
  iter <- 0L

  # Storage for segment parameters
  # segment_params[[eq_name]][[k]] = list(coefficients, variance)
  segment_params <- list()

  for (it in seq_len(max_iter)) {
    iter <- it

    # --- M-step ---
    N_k <- colSums(posteriors)
    pi_k <- N_k / N

    # Check for degenerate segments
    min_effective <- min(N_k)
    if (min_effective < 1) {
      return(list(converged = FALSE, lnL = -Inf, iter = iter))
    }

    for (eq_name in names(equations)) {
      eq <- equations[[eq_name]]
      segment_params[[eq_name]] <- list()

      for (k in seq_len(K)) {
        result <- weighted_ols(eq$y, eq$X, posteriors[, k])
        if (any(is.na(result$coefficients))) {
          return(list(converged = FALSE, lnL = -Inf, iter = iter))
        }
        segment_params[[eq_name]][[k]] <- result
      }
    }

    # --- E-step ---
    # Compute log component densities: log_dens[i, k]
    log_dens <- matrix(0, nrow = N, ncol = K)

    for (k in seq_len(K)) {
      log_dens[, k] <- log(pi_k[k])
      for (eq_name in names(equations)) {
        eq <- equations[[eq_name]]
        sp <- segment_params[[eq_name]][[k]]
        mu <- as.vector(eq$X %*% sp$coefficients)
        sigma <- sqrt(sp$variance)
        log_dens[, k] <- log_dens[, k] + dnorm(eq$y, mean = mu,
                                                  sd = sigma, log = TRUE)
      }
    }

    # Posteriors via log-sum-exp
    log_denom <- apply(log_dens, 1, log_sum_exp)
    posteriors <- exp(log_dens - log_denom)

    # Clamp to avoid exact 0/1
    posteriors <- pmax(posteriors, .Machine$double.eps)
    posteriors <- posteriors / rowSums(posteriors)

    # --- Convergence check ---
    lnL_new <- sum(log_denom)
    if (abs(lnL_new - lnL_old) < stop_criterion) {
      converged <- TRUE
      lnL_old <- lnL_new
      break
    }
    lnL_old <- lnL_new
  }

  list(
    converged = converged,
    lnL = lnL_old,
    iter = iter,
    posteriors = posteriors,
    pi_k = pi_k,
    segment_params = segment_params
  )
}

#' Build segment-specific path coefficient matrices from EM results.
#'
#' Each segment gets a matrix with the same structure as model$path_coef,
#' but with segment-specific coefficients filled in.
#' @noRd
build_segment_path_matrices <- function(equations, segment_params, model, K) {
  template <- model$path_coef * 0  # Same structure, all zeros
  construct_names <- colnames(template)

  segment_paths <- list()
  segment_intercepts <- list()

  for (k in seq_len(K)) {
    path_mat <- template
    intercepts <- numeric(length(equations))
    names(intercepts) <- names(equations)

    for (eq_name in names(equations)) {
      eq <- equations[[eq_name]]
      sp <- segment_params[[eq_name]][[k]]
      coefs <- sp$coefficients

      intercepts[eq_name] <- coefs[1]  # First coefficient is intercept

      for (p_idx in seq_along(eq$predictors)) {
        pred <- eq$predictors[p_idx]
        if (pred %in% construct_names && eq$target %in% construct_names) {
          path_mat[pred, eq$target] <- coefs[p_idx + 1]
        }
      }
    }

    segment_paths[[k]] <- path_mat
    segment_intercepts[[k]] <- intercepts
  }

  list(paths = segment_paths, intercepts = segment_intercepts)
}

# =============================================================================
# MAIN ENTRY POINTS
# =============================================================================

#' FIMIX-PLS Analysis for PLS-SEM
#'
#' \code{assess_fimix} runs the FIMIX-PLS (Finite Mixture PLS) procedure to
#' identify unobserved heterogeneity in PLS-SEM structural models. The EM
#' algorithm probabilistically assigns observations to \code{K} latent
#' segments, each with segment-specific structural path coefficients.
#'
#' FIMIX-PLS operates on construct scores from the estimated PLS model.
#' It fits a mixture of regressions to each structural equation simultaneously,
#' estimating segment-specific intercepts, path coefficients, and residual
#' variances. Multiple random starts (\code{nstart}) are used to avoid local
#' optima; the solution with the highest log-likelihood is retained.
#'
#' @param seminr_model An estimated SEMinR model from \code{estimate_pls()}.
#' @param K Integer; the number of segments (default 2). Must be >= 2.
#' @param nstart Integer; number of random EM starts (default 10).
#' @param max_iter Integer; maximum EM iterations per start (default 5000).
#' @param stop_criterion Numeric; convergence threshold on log-likelihood
#'   change (default 1e-6).
#' @param seed Random seed for reproducibility (default 123).
#'
#' @return An object of class \code{fimix_analysis} containing:
#'   \item{K}{Number of segments}
#'   \item{segment_proportions}{Named numeric vector of mixing proportions}
#'   \item{posterior}{N x K matrix of posterior segment probabilities}
#'   \item{segment_assignment}{Integer vector of hard assignments (argmax)}
#'   \item{segment_paths}{List of K path coefficient matrices}
#'   \item{segment_intercepts}{List of K intercept vectors}
#'   \item{segment_variances}{J x K matrix of residual variances}
#'   \item{log_likelihood}{Final log-likelihood}
#'   \item{n_parameters}{Number of free parameters}
#'   \item{info_criteria}{Named vector of information criteria}
#'   \item{converged}{Logical; whether EM converged}
#'   \item{iterations}{Number of EM iterations}
#'   \item{n_starts_completed}{Number of random starts completed}
#'   \item{pls_model}{The original estimated seminr model}
#'   \item{n_obs}{Sample size}
#'
#' @references
#' Hahn, C., Johnson, M. D., Herrmann, A., & Huber, F. (2002). Capturing
#' Customer Heterogeneity using a Finite Mixture PLS Approach. Schmalenbach
#' Business Review, 54, 243-269.
#'
#' Sarstedt, M., Becker, J.-M., Ringle, C. M. & Schwaiger, M. (2011).
#' Uncovering and Treating Unobserved Heterogeneity with FIMIX-PLS: Which
#' Model Selection Criterion Provides an Appropriate Number of Segments?
#' Schmalenbach Business Review, 63(1), 34-62.
#'
#' @examples
#' library(seminr)
#' library(seminrExtras)
#'
#' mobi_mm <- constructs(
#'   composite("Image",        multi_items("IMAG", 1:5)),
#'   composite("Expectation",  multi_items("CUEX", 1:3)),
#'   composite("Value",        multi_items("PERV", 1:2)),
#'   composite("Satisfaction", multi_items("CUSA", 1:3)),
#'   composite("Loyalty",      multi_items("CUSL", 1:3))
#' )
#'
#' mobi_sm <- relationships(
#'   paths(from = "Image",       to = c("Expectation", "Satisfaction", "Loyalty")),
#'   paths(from = "Expectation", to = c("Value", "Satisfaction")),
#'   paths(from = "Value",       to = "Satisfaction"),
#'   paths(from = "Satisfaction",to = "Loyalty")
#' )
#'
#' mobi_pls <- estimate_pls(data = mobi,
#'                           measurement_model = mobi_mm,
#'                           structural_model  = mobi_sm)
#'
#' \donttest{
#' fimix_result <- assess_fimix(mobi_pls, K = 2, nstart = 5, seed = 123)
#' print(fimix_result)
#' summary(fimix_result)
#' }
#'
#' @seealso \code{\link{assess_fimix_compare}} for comparing multiple K values
#'
#' @export
assess_fimix <- function(seminr_model,
                          K = 2,
                          nstart = 10,
                          max_iter = 5000,
                          stop_criterion = 1e-6,
                          seed = 123) {

  # ---------------------------------------------------------------------------
  # Step 1: Validate inputs
  # ---------------------------------------------------------------------------
  if (!validate_seminr_model(seminr_model, "assess_fimix")) return(NULL)

  if (!is.numeric(K) || length(K) != 1 || K < 2 || K != as.integer(K)) {
    stop("K must be an integer >= 2.", call. = FALSE)
  }

  if (!is.numeric(nstart) || length(nstart) != 1 || nstart < 1) {
    stop("nstart must be an integer >= 1.", call. = FALSE)
  }

  # ---------------------------------------------------------------------------
  # Step 2: Extract structural equations
  # ---------------------------------------------------------------------------
  equations <- extract_structural_equations(seminr_model)

  if (length(equations) == 0) {
    stop("No endogenous constructs found in the structural model.", call. = FALSE)
  }

  N <- nrow(seminr_model$construct_scores)
  Q <- count_fimix_parameters(equations, K)

  # Minimum observations per segment heuristic

  max_predictors <- max(vapply(equations, function(eq) length(eq$predictors), integer(1)))
  min_segment_obs <- max(max_predictors + 2, 10)
  if (N / K < min_segment_obs) {
    warning("Sample size (", N, ") may be too small for K = ", K,
            " segments. Minimum ~", min_segment_obs, " observations per segment ",
            "recommended.", call. = FALSE)
  }

  # ---------------------------------------------------------------------------
  # Step 3: Run EM with multiple random starts
  # ---------------------------------------------------------------------------
  set.seed(seed)

  best_result <- NULL
  best_lnL <- -Inf
  n_completed <- 0L

  for (r in seq_len(nstart)) {
    posteriors_init <- init_random_posteriors(N, K)
    result <- run_fimix_em(equations, K, max_iter, stop_criterion,
                            posteriors_init)
    n_completed <- n_completed + 1L

    if (result$converged && result$lnL > best_lnL) {
      best_result <- result
      best_lnL <- result$lnL
    }
  }

  if (is.null(best_result)) {
    warning("FIMIX-PLS did not converge for K = ", K,
            " in any of ", nstart, " random starts. ",
            "Consider reducing K or increasing max_iter.", call. = FALSE)
    return(NULL)
  }

  # ---------------------------------------------------------------------------
  # Step 4: Construct return object
  # ---------------------------------------------------------------------------
  # Build segment path matrices and intercepts
  built <- build_segment_path_matrices(equations, best_result$segment_params,
                                        seminr_model, K)

  # Segment variances: J x K matrix
  J <- length(equations)
  var_matrix <- matrix(NA_real_, nrow = J, ncol = K,
                        dimnames = list(names(equations),
                                        paste0("Segment_", seq_len(K))))
  for (eq_name in names(equations)) {
    for (k in seq_len(K)) {
      var_matrix[eq_name, k] <- best_result$segment_params[[eq_name]][[k]]$variance
    }
  }

  # Segment assignments (hard clustering)
  assignment <- apply(best_result$posteriors, 1, which.max)

  # Segment proportions
  pi_k <- best_result$pi_k
  names(pi_k) <- paste0("Segment_", seq_len(K))

  # Segment sizes (hard assignment counts)
  segment_sizes <- tabulate(assignment, nbins = K)
  names(segment_sizes) <- paste0("Segment_", seq_len(K))

  # Information criteria
  info_criteria <- compute_fimix_criteria(best_result$lnL, Q, N, K,
                                           best_result$posteriors)

  # Column names for posteriors
  colnames(best_result$posteriors) <- paste0("Segment_", seq_len(K))

  result <- list(
    K                   = K,
    segment_proportions = pi_k,
    segment_sizes       = segment_sizes,
    posterior           = best_result$posteriors,
    segment_assignment  = assignment,
    segment_paths       = built$paths,
    segment_intercepts  = built$intercepts,
    segment_variances   = var_matrix,
    log_likelihood      = best_result$lnL,
    n_parameters        = Q,
    info_criteria       = info_criteria,
    converged           = best_result$converged,
    iterations          = best_result$iter,
    n_starts_completed  = n_completed,
    pls_model           = seminr_model,
    n_obs               = N
  )

  class(result) <- c("fimix_analysis", class(result))
  result
}

#' Compare FIMIX-PLS Solutions Across Multiple K Values
#'
#' \code{assess_fimix_compare} runs \code{\link{assess_fimix}} for each value
#' of K in \code{K_range} and produces a comparison table of information
#' criteria to guide segment number selection.
#'
#' The recommended approach is to inspect AIC3 and CAIC jointly: when both
#' agree on K, accuracy reaches approximately 84\% (Sarstedt et al., 2011).
#' AIC4 is the best-performing single criterion. Entropy (EN) indicates
#' classification quality but should not be used alone to select K.
#'
#' @inheritParams assess_fimix
#' @param K_range Integer vector of K values to evaluate (default 2:5).
#'
#' @return An object of class \code{fimix_comparison} containing:
#'   \item{solutions}{Named list of \code{fimix_analysis} objects (one per K)}
#'   \item{fit_table}{Data frame with K as rows and criteria as columns}
#'   \item{K_range}{The evaluated K values}
#'   \item{pls_model}{The original model}
#'
#' @references
#' Sarstedt, M., Becker, J.-M., Ringle, C. M. & Schwaiger, M. (2011).
#' Uncovering and Treating Unobserved Heterogeneity with FIMIX-PLS: Which
#' Model Selection Criterion Provides an Appropriate Number of Segments?
#' Schmalenbach Business Review, 63(1), 34-62.
#'
#' @examples
#' library(seminr)
#' library(seminrExtras)
#'
#' mobi_mm <- constructs(
#'   composite("Image",        multi_items("IMAG", 1:5)),
#'   composite("Expectation",  multi_items("CUEX", 1:3)),
#'   composite("Value",        multi_items("PERV", 1:2)),
#'   composite("Satisfaction", multi_items("CUSA", 1:3)),
#'   composite("Loyalty",      multi_items("CUSL", 1:3))
#' )
#'
#' mobi_sm <- relationships(
#'   paths(from = "Image",       to = c("Expectation", "Satisfaction", "Loyalty")),
#'   paths(from = "Expectation", to = c("Value", "Satisfaction")),
#'   paths(from = "Value",       to = "Satisfaction"),
#'   paths(from = "Satisfaction",to = "Loyalty")
#' )
#'
#' mobi_pls <- estimate_pls(data = mobi,
#'                           measurement_model = mobi_mm,
#'                           structural_model  = mobi_sm)
#'
#' \donttest{
#' fimix_comp <- assess_fimix_compare(mobi_pls, K_range = 2:4,
#'                                      nstart = 5, seed = 123)
#' print(fimix_comp)
#' plot(fimix_comp)
#' }
#'
#' @seealso \code{\link{assess_fimix}} for single-K FIMIX analysis
#'
#' @export
assess_fimix_compare <- function(seminr_model,
                                   K_range = 2:5,
                                   nstart = 10,
                                   max_iter = 5000,
                                   stop_criterion = 1e-6,
                                   seed = 123) {

  if (!validate_seminr_model(seminr_model, "assess_fimix_compare")) return(NULL)

  if (!is.numeric(K_range) || length(K_range) < 1 || any(K_range < 2)) {
    stop("K_range must be a vector of integers >= 2.", call. = FALSE)
  }

  solutions <- list()
  criteria_list <- list()

  for (k in K_range) {
    sol <- assess_fimix(seminr_model, K = k, nstart = nstart,
                          max_iter = max_iter,
                          stop_criterion = stop_criterion,
                          seed = seed)
    sol_name <- paste0("K", k)
    solutions[[sol_name]] <- sol

    if (!is.null(sol)) {
      criteria_list[[sol_name]] <- c(K = k, sol$info_criteria)
    } else {
      criteria_list[[sol_name]] <- c(K = k,
                                      lnL = NA, AIC = NA, AIC3 = NA,
                                      AIC4 = NA, BIC = NA, CAIC = NA,
                                      HQ = NA, MDL5 = NA, EN = NA)
    }
  }

  fit_table <- as.data.frame(do.call(rbind, criteria_list))
  rownames(fit_table) <- NULL

  result <- list(
    solutions = solutions,
    fit_table = fit_table,
    K_range   = K_range,
    pls_model = seminr_model
  )

  class(result) <- c("fimix_comparison", class(result))
  result
}

# =============================================================================
# S3 METHODS — fimix_analysis
# =============================================================================

#' @export
print.fimix_analysis <- function(x, ...) {
  cat("FIMIX-PLS Analysis\n")
  cat("==================\n")
  cat("Segments:", x$K, "\n")
  cat("Observations:", x$n_obs, "\n")
  cat("Converged:", if (x$converged) "Yes" else "No",
      "(", x$iterations, "iterations )\n")
  cat("Random starts:", x$n_starts_completed, "\n")

  cat("\nSegment Proportions:\n")
  seg_tbl <- data.frame(
    Segment = paste0("Segment ", seq_len(x$K)),
    Proportion = round(x$segment_proportions, 4),
    Size = x$segment_sizes,
    check.names = FALSE,
    row.names = NULL
  )
  print(seg_tbl, row.names = FALSE, ...)

  cat("\nFit Criteria:\n")
  ic <- x$info_criteria
  ic_tbl <- data.frame(
    lnL = round(ic["lnL"], 2),
    AIC = round(ic["AIC"], 2),
    AIC3 = round(ic["AIC3"], 2),
    AIC4 = round(ic["AIC4"], 2),
    BIC = round(ic["BIC"], 2),
    CAIC = round(ic["CAIC"], 2),
    EN = round(ic["EN"], 4),
    check.names = FALSE,
    row.names = NULL
  )
  print(ic_tbl, row.names = FALSE, ...)

  cat("\nSegment Path Coefficients:\n")
  for (k in seq_len(x$K)) {
    cat("\n  Segment", k, ":\n")
    pm <- x$segment_paths[[k]]
    # Print only non-zero entries
    nonzero <- which(pm != 0, arr.ind = TRUE)
    if (nrow(nonzero) > 0) {
      path_tbl <- data.frame(
        From = rownames(pm)[nonzero[, 1]],
        To = colnames(pm)[nonzero[, 2]],
        Coefficient = round(pm[nonzero], 4),
        check.names = FALSE,
        row.names = NULL
      )
      print(path_tbl, row.names = FALSE, ...)
    }
  }

  invisible(x)
}

#' @export
summary.fimix_analysis <- function(object, ...) {
  result <- list(
    K                   = object$K,
    n_obs               = object$n_obs,
    converged           = object$converged,
    iterations          = object$iterations,
    n_starts            = object$n_starts_completed,
    segment_proportions = object$segment_proportions,
    segment_sizes       = object$segment_sizes,
    segment_paths       = object$segment_paths,
    segment_intercepts  = object$segment_intercepts,
    segment_variances   = object$segment_variances,
    info_criteria       = object$info_criteria,
    n_parameters        = object$n_parameters
  )

  class(result) <- c("summary.fimix_analysis", class(result))
  result
}

#' @export
print.summary.fimix_analysis <- function(x, ...) {
  cat("FIMIX-PLS Analysis Summary\n")
  cat("==========================\n")
  cat("Segments:", x$K, "\n")
  cat("Observations:", x$n_obs, "\n")
  cat("Free parameters:", x$n_parameters, "\n")
  cat("Converged:", if (x$converged) "Yes" else "No",
      "(", x$iterations, "iterations )\n")
  cat("Random starts:", x$n_starts, "\n")

  cat("\nSegment Proportions:\n")
  print(round(x$segment_proportions, 4), ...)

  cat("\nFit Criteria:\n")
  print(round(x$info_criteria, 4), ...)

  for (k in seq_len(x$K)) {
    cat("\n--- Segment", k, "---\n")

    cat("Intercepts:\n")
    print(round(x$segment_intercepts[[k]], 4), ...)

    cat("Path Coefficients:\n")
    pm <- x$segment_paths[[k]]
    nonzero <- which(pm != 0, arr.ind = TRUE)
    if (nrow(nonzero) > 0) {
      path_tbl <- data.frame(
        From = rownames(pm)[nonzero[, 1]],
        To = colnames(pm)[nonzero[, 2]],
        Coefficient = round(pm[nonzero], 4),
        check.names = FALSE,
        row.names = NULL
      )
      print(path_tbl, row.names = FALSE, ...)
    }

    cat("Residual Variances:\n")
    print(round(x$segment_variances[, k], 4), ...)
  }

  invisible(x)
}

#' Plot FIMIX-PLS Results
#'
#' @param x A \code{fimix_analysis} object from \code{assess_fimix()}.
#' @param type One of \code{"segments"} (segment proportion bar plot) or
#'   \code{"paths"} (grouped bar plot of path coefficients across segments).
#' @param ... Additional arguments passed to \code{plot()}.
#'
#' @export
plot.fimix_analysis <- function(x,
                                 type = c("segments", "paths"),
                                 ...) {
  type <- match.arg(type)

  if (type == "segments") {
    old_par <- par(no.readonly = TRUE)
    on.exit(par(old_par))
    par(mar = c(5, 5, 4, 2))

    cols <- if (x$K <= 8) {
      palette.colors(x$K, palette = "Set2")
    } else {
      palette.colors(n = x$K)
    }

    barplot(x$segment_proportions * 100,
            names.arg = paste("Seg.", seq_len(x$K)),
            col = cols, ylim = c(0, 100),
            ylab = "Proportion (%)",
            main = paste("FIMIX-PLS: Segment Proportions (K =", x$K, ")"),
            ...)

  } else if (type == "paths") {
    # Collect all non-zero path labels and their segment-specific values
    pm_ref <- x$segment_paths[[1]]
    nonzero <- which(pm_ref != 0, arr.ind = TRUE)
    if (nrow(nonzero) == 0) {
      message("No non-zero path coefficients to plot.")
      return(invisible(x))
    }

    path_labels <- paste(rownames(pm_ref)[nonzero[, 1]], "->",
                         colnames(pm_ref)[nonzero[, 2]])

    coef_matrix <- matrix(NA_real_, nrow = length(path_labels), ncol = x$K)
    for (k in seq_len(x$K)) {
      coef_matrix[, k] <- x$segment_paths[[k]][nonzero]
    }
    rownames(coef_matrix) <- path_labels
    colnames(coef_matrix) <- paste("Seg.", seq_len(x$K))

    old_par <- par(no.readonly = TRUE)
    on.exit(par(old_par))
    par(mar = c(8, 5, 4, 2))

    cols <- if (x$K <= 8) {
      palette.colors(x$K, palette = "Set2")
    } else {
      palette.colors(n = x$K)
    }

    barplot(t(coef_matrix), beside = TRUE, col = cols,
            las = 2, ylab = "Path Coefficient",
            main = paste("FIMIX-PLS: Path Coefficients (K =", x$K, ")"),
            ...)

    legend("topright", legend = colnames(coef_matrix),
           fill = cols, cex = 0.8, bty = "n")
  }
}

# =============================================================================
# S3 METHODS — fimix_comparison
# =============================================================================

#' @export
print.fimix_comparison <- function(x, ...) {
  cat("FIMIX-PLS Segment Selection\n")
  cat("===========================\n")
  cat("K range:", min(x$K_range), "to", max(x$K_range), "\n\n")

  cat("Fit Criteria:\n")
  ft <- x$fit_table
  ft$K <- as.integer(ft$K)
  for (col_name in setdiff(names(ft), "K")) {
    if (col_name == "EN") {
      ft[[col_name]] <- round(ft[[col_name]], 4)
    } else {
      ft[[col_name]] <- round(ft[[col_name]], 2)
    }
  }
  print(ft, row.names = FALSE, ...)

  # Identify minimum for each criterion (lower = better, except EN: higher = better)
  cat("\nBest K by criterion:\n")
  for (crit in c("AIC", "AIC3", "AIC4", "BIC", "CAIC")) {
    vals <- ft[[crit]]
    if (all(is.na(vals))) next
    best_k <- ft$K[which.min(vals)]
    cat("  ", crit, ":", best_k, "\n")
  }
  en_vals <- ft[["EN"]]
  if (!all(is.na(en_vals))) {
    cat("   EN (best):", ft$K[which.max(en_vals)], "\n")
  }

  invisible(x)
}

#' @export
summary.fimix_comparison <- function(object, ...) {
  result <- list(
    fit_table = object$fit_table,
    K_range   = object$K_range,
    solutions = object$solutions
  )
  class(result) <- c("summary.fimix_comparison", class(result))
  result
}

#' @export
print.summary.fimix_comparison <- function(x, ...) {
  cat("FIMIX-PLS Comparison Summary\n")
  cat("============================\n\n")

  cat("Fit Table:\n")
  ft <- x$fit_table
  for (col_name in setdiff(names(ft), "K")) {
    if (col_name == "EN") {
      ft[[col_name]] <- round(ft[[col_name]], 4)
    } else {
      ft[[col_name]] <- round(ft[[col_name]], 2)
    }
  }
  print(ft, row.names = FALSE, ...)

  # Per-K summaries
  for (sol_name in names(x$solutions)) {
    sol <- x$solutions[[sol_name]]
    if (is.null(sol)) next

    cat("\n--- K =", sol$K, "---\n")
    cat("Converged:", if (sol$converged) "Yes" else "No", "\n")
    cat("Segment sizes:", paste(sol$segment_sizes, collapse = ", "), "\n")
    cat("EN:", round(sol$info_criteria["EN"], 4), "\n")
  }

  invisible(x)
}

#' Plot FIMIX-PLS Comparison
#'
#' @param x A \code{fimix_comparison} object from \code{assess_fimix_compare()}.
#' @param type One of \code{"criteria"} (line plot of IC vs K) or
#'   \code{"entropy"} (EN vs K).
#' @param criteria Character vector of criteria to plot when
#'   \code{type = "criteria"} (default: \code{c("AIC3", "AIC4", "BIC", "CAIC")}).
#' @param ... Additional arguments passed to \code{plot()}.
#'
#' @export
plot.fimix_comparison <- function(x,
                                    type = c("criteria", "entropy"),
                                    criteria = c("AIC3", "AIC4", "BIC", "CAIC"),
                                    ...) {
  type <- match.arg(type)
  ft <- x$fit_table

  if (type == "criteria") {
    old_par <- par(no.readonly = TRUE)
    on.exit(par(old_par))
    par(mar = c(5, 5, 4, 8), xpd = TRUE)

    valid_criteria <- intersect(criteria, names(ft))
    if (length(valid_criteria) == 0) {
      stop("No valid criteria found to plot.", call. = FALSE)
    }

    vals <- ft[, valid_criteria, drop = FALSE]
    K_vals <- ft$K

    cols <- if (length(valid_criteria) <= 8) {
      palette.colors(length(valid_criteria), palette = "Set1")
    } else {
      seq_len(length(valid_criteria))
    }

    ylim <- range(vals, na.rm = TRUE)
    ylim <- ylim + c(-1, 1) * diff(ylim) * 0.05

    plot(K_vals, vals[[1]], type = "b", pch = 19, col = cols[1],
         xlim = range(K_vals), ylim = ylim,
         xlab = "Number of Segments (K)", ylab = "Information Criterion",
         main = "FIMIX-PLS: Model Selection Criteria",
         xaxt = "n", ...)
    axis(1, at = K_vals)

    if (length(valid_criteria) > 1) {
      for (i in 2:length(valid_criteria)) {
        lines(K_vals, vals[[i]], type = "b", pch = 19, col = cols[i])
      }
    }

    legend("topright", inset = c(-0.25, 0),
           legend = valid_criteria, col = cols, lty = 1, pch = 19,
           cex = 0.8, bty = "n")

  } else if (type == "entropy") {
    old_par <- par(no.readonly = TRUE)
    on.exit(par(old_par))
    par(mar = c(5, 5, 4, 2))

    K_vals <- ft$K
    en_vals <- ft$EN

    plot(K_vals, en_vals, type = "b", pch = 19, col = "steelblue",
         xlim = range(K_vals), ylim = c(0, 1),
         xlab = "Number of Segments (K)", ylab = "Normed Entropy (EN)",
         main = "FIMIX-PLS: Classification Quality",
         xaxt = "n", ...)
    axis(1, at = K_vals)
    abline(h = 0.5, col = "gray70", lty = 2)
    text(max(K_vals), 0.5, "EN = 0.50", pos = 1, cex = 0.8, col = "gray50")
  }
}

# =============================================================================
# Tests for FIMIX-PLS (Finite Mixture PLS)
# =============================================================================

# --- Setup: pre-compute models and results -----------------------------------
library(seminr)

mobi_mm <- constructs(
  composite("Image",        multi_items("IMAG", 1:5)),
  composite("Expectation",  multi_items("CUEX", 1:3)),
  composite("Value",        multi_items("PERV", 1:2)),
  composite("Satisfaction", multi_items("CUSA", 1:3)),
  composite("Loyalty",      multi_items("CUSL", 1:3))
)
mobi_sm <- relationships(
  paths(from = "Image",       to = c("Expectation", "Satisfaction", "Loyalty")),
  paths(from = "Expectation", to = c("Value", "Satisfaction")),
  paths(from = "Value",       to = "Satisfaction"),
  paths(from = "Satisfaction", to = "Loyalty")
)
pls_model <- estimate_pls(data = mobi, measurement_model = mobi_mm,
                           structural_model = mobi_sm)

# Pre-compute FIMIX for K=2 (reused across tests)
fimix_k2 <- assess_fimix(pls_model, K = 2, nstart = 5, seed = 123)

# =============================================================================
# STRUCTURE TESTS
# =============================================================================

test_that("assess_fimix returns fimix_analysis object", {
  expect_s3_class(fimix_k2, "fimix_analysis")
})

test_that("fimix result contains all expected components", {
  expected <- c("K", "segment_proportions", "segment_sizes", "posterior",
                "segment_assignment", "segment_paths", "segment_intercepts",
                "segment_variances", "log_likelihood", "n_parameters",
                "info_criteria", "converged", "iterations",
                "n_starts_completed", "pls_model", "n_obs")
  expect_true(all(expected %in% names(fimix_k2)))
})

test_that("K is stored correctly", {
  expect_equal(fimix_k2$K, 2)
})

test_that("n_obs matches model data", {
  expect_equal(fimix_k2$n_obs, nrow(pls_model$construct_scores))
})

# =============================================================================
# SEGMENT PROPORTION AND ASSIGNMENT TESTS
# =============================================================================

test_that("segment proportions sum to 1", {
  expect_equal(sum(fimix_k2$segment_proportions), 1, tolerance = 1e-10)
})

test_that("segment proportions are all positive", {
  expect_true(all(fimix_k2$segment_proportions > 0))
})

test_that("segment sizes sum to N", {
  expect_equal(sum(fimix_k2$segment_sizes), fimix_k2$n_obs)
})

test_that("segment assignments are integers from 1 to K", {
  expect_true(all(fimix_k2$segment_assignment %in% 1:2))
})

test_that("segment assignment length matches N", {
  expect_equal(length(fimix_k2$segment_assignment), fimix_k2$n_obs)
})

# =============================================================================
# POSTERIOR PROBABILITY TESTS
# =============================================================================

test_that("posterior matrix has correct dimensions", {
  expect_equal(dim(fimix_k2$posterior), c(fimix_k2$n_obs, 2))
})

test_that("posterior rows sum to 1", {
  row_sums <- rowSums(fimix_k2$posterior)
  expect_equal(row_sums, rep(1, fimix_k2$n_obs), tolerance = 1e-10)
})

test_that("posterior values are in [0, 1]", {
  expect_true(all(fimix_k2$posterior >= 0))
  expect_true(all(fimix_k2$posterior <= 1))
})

test_that("hard assignment matches argmax of posterior", {
  argmax <- apply(fimix_k2$posterior, 1, which.max)
  expect_equal(fimix_k2$segment_assignment, argmax)
})

# =============================================================================
# PATH COEFFICIENT TESTS
# =============================================================================

test_that("segment_paths is a list of length K", {
  expect_true(is.list(fimix_k2$segment_paths))
  expect_equal(length(fimix_k2$segment_paths), 2)
})

test_that("segment path matrices have same dimensions as model path_coef", {
  for (k in 1:2) {
    expect_equal(dim(fimix_k2$segment_paths[[k]]),
                 dim(pls_model$path_coef))
    expect_equal(rownames(fimix_k2$segment_paths[[k]]),
                 rownames(pls_model$path_coef))
    expect_equal(colnames(fimix_k2$segment_paths[[k]]),
                 colnames(pls_model$path_coef))
  }
})

test_that("segment path coefficients are non-zero where model has paths", {
  nonzero_mask <- pls_model$path_coef != 0
  for (k in 1:2) {
    # All structural paths in the model should have coefficients
    seg_coefs <- fimix_k2$segment_paths[[k]][nonzero_mask]
    expect_true(all(seg_coefs != 0))
  }
})

test_that("segment path coefficients are zero where model has no paths", {
  zero_mask <- pls_model$path_coef == 0
  for (k in 1:2) {
    seg_zeros <- fimix_k2$segment_paths[[k]][zero_mask]
    expect_true(all(seg_zeros == 0))
  }
})

test_that("segment intercepts have correct structure", {
  expect_equal(length(fimix_k2$segment_intercepts), 2)
  endogenous <- seminr:::all_endogenous(pls_model$smMatrix)
  for (k in 1:2) {
    expect_equal(length(fimix_k2$segment_intercepts[[k]]), length(endogenous))
    expect_true(all(names(fimix_k2$segment_intercepts[[k]]) %in% endogenous))
  }
})

# =============================================================================
# SEGMENT VARIANCE TESTS
# =============================================================================

test_that("segment_variances has correct dimensions", {
  endogenous <- seminr:::all_endogenous(pls_model$smMatrix)
  expect_equal(nrow(fimix_k2$segment_variances), length(endogenous))
  expect_equal(ncol(fimix_k2$segment_variances), 2)
})

test_that("segment variances are positive", {
  expect_true(all(fimix_k2$segment_variances > 0))
})

# =============================================================================
# INFORMATION CRITERIA TESTS
# =============================================================================

test_that("info_criteria contains expected names", {
  expected <- c("lnL", "AIC", "AIC3", "AIC4", "BIC", "CAIC", "HQ", "MDL5", "EN")
  expect_true(all(expected %in% names(fimix_k2$info_criteria)))
})

test_that("all criteria are finite", {
  expect_true(all(is.finite(fimix_k2$info_criteria)))
})

test_that("AIC < AIC3 < AIC4 (by formula)", {
  ic <- fimix_k2$info_criteria
  expect_true(ic["AIC"] < ic["AIC3"])
  expect_true(ic["AIC3"] < ic["AIC4"])
})

test_that("entropy is in [0, 1]", {
  en <- fimix_k2$info_criteria["EN"]
  expect_true(en >= 0 && en <= 1)
})

test_that("log-likelihood is negative", {
  expect_true(fimix_k2$log_likelihood < 0)
})

test_that("n_parameters is positive integer", {
  expect_true(fimix_k2$n_parameters > 0)
  expect_equal(fimix_k2$n_parameters, as.integer(fimix_k2$n_parameters))
})

# =============================================================================
# CONVERGENCE TESTS
# =============================================================================

test_that("EM converges for K=2", {
  expect_true(fimix_k2$converged)
})

test_that("iterations is positive", {
  expect_true(fimix_k2$iterations > 0)
})

test_that("n_starts_completed equals nstart", {
  expect_equal(fimix_k2$n_starts_completed, 5L)
})

# =============================================================================
# REPRODUCIBILITY TESTS
# =============================================================================

test_that("results are deterministic with same seed", {
  r1 <- assess_fimix(pls_model, K = 2, nstart = 3, seed = 42)
  r2 <- assess_fimix(pls_model, K = 2, nstart = 3, seed = 42)
  expect_equal(r1$log_likelihood, r2$log_likelihood)
  expect_equal(r1$segment_proportions, r2$segment_proportions)
  expect_equal(r1$segment_assignment, r2$segment_assignment)
})

# =============================================================================
# INPUT VALIDATION TESTS
# =============================================================================

test_that("non-seminr model returns NULL with warning", {
  expect_warning(result <- assess_fimix("not a model", K = 2),
                 "SEMinR models")
  expect_null(result)
})

test_that("K < 2 raises error", {
  expect_error(assess_fimix(pls_model, K = 1), "K must be an integer >= 2")
})

test_that("K = 0 raises error", {
  expect_error(assess_fimix(pls_model, K = 0), "K must be an integer >= 2")
})

test_that("non-integer K raises error", {
  expect_error(assess_fimix(pls_model, K = 2.5), "K must be an integer >= 2")
})

test_that("nstart < 1 raises error", {
  expect_error(assess_fimix(pls_model, K = 2, nstart = 0),
               "nstart must be an integer >= 1")
})

# =============================================================================
# S3 METHOD TESTS
# =============================================================================

test_that("print.fimix_analysis returns invisibly", {
  expect_invisible(print(fimix_k2))
})

test_that("print output contains key information", {
  out <- capture.output(print(fimix_k2))
  out_text <- paste(out, collapse = "\n")

  expect_true(grepl("FIMIX-PLS Analysis", out_text))
  expect_true(grepl("Segments: 2", out_text))
  expect_true(grepl("Converged: Yes", out_text))
  expect_true(grepl("Segment Proportions", out_text))
  expect_true(grepl("Fit Criteria", out_text))
  expect_true(grepl("Segment Path Coefficients", out_text))
})

test_that("summary returns summary.fimix_analysis object", {
  s <- summary(fimix_k2)
  expect_s3_class(s, "summary.fimix_analysis")
})

test_that("summary print includes segment details", {
  out <- capture.output(print(summary(fimix_k2)))
  out_text <- paste(out, collapse = "\n")

  expect_true(grepl("Summary", out_text))
  expect_true(grepl("Segment 1", out_text))
  expect_true(grepl("Intercept", out_text))
  expect_true(grepl("Residual Variance", out_text))
})

test_that("plot.fimix_analysis runs without error", {
  expect_no_error(plot(fimix_k2, type = "segments"))
  expect_no_error(plot(fimix_k2, type = "paths"))
})

# =============================================================================
# COMPARISON TESTS
# =============================================================================

test_that("assess_fimix_compare returns fimix_comparison object", {
  comp <- assess_fimix_compare(pls_model, K_range = 2:3,
                                 nstart = 3, seed = 123)
  expect_s3_class(comp, "fimix_comparison")
})

test_that("comparison contains solutions for each K", {
  comp <- assess_fimix_compare(pls_model, K_range = 2:3,
                                 nstart = 3, seed = 123)
  expect_true("K2" %in% names(comp$solutions))
  expect_true("K3" %in% names(comp$solutions))
})

test_that("comparison fit_table has correct structure", {
  comp <- assess_fimix_compare(pls_model, K_range = 2:3,
                                 nstart = 3, seed = 123)
  expect_true("K" %in% names(comp$fit_table))
  expect_true("AIC" %in% names(comp$fit_table))
  expect_true("BIC" %in% names(comp$fit_table))
  expect_true("EN" %in% names(comp$fit_table))
  expect_equal(nrow(comp$fit_table), 2)
})

test_that("comparison print shows best K by criterion", {
  comp <- assess_fimix_compare(pls_model, K_range = 2:3,
                                 nstart = 3, seed = 123)
  out <- capture.output(print(comp))
  out_text <- paste(out, collapse = "\n")

  expect_true(grepl("FIMIX-PLS Segment Selection", out_text))
  expect_true(grepl("Best K by criterion", out_text))
})

test_that("comparison plot runs without error", {
  comp <- assess_fimix_compare(pls_model, K_range = 2:3,
                                 nstart = 3, seed = 123)
  expect_no_error(plot(comp, type = "criteria"))
  expect_no_error(plot(comp, type = "entropy"))
})

test_that("comparison K_range validation works", {
  expect_error(assess_fimix_compare(pls_model, K_range = 1),
               "K_range must be a vector of integers >= 2")
})

# =============================================================================
# MODEL TYPE TESTS
# =============================================================================

test_that("FIMIX works with mediation model", {
  skip_on_cran()
  mm_med <- constructs(
    composite("Image",        multi_items("IMAG", 1:5)),
    composite("Expectation",  multi_items("CUEX", 1:3)),
    composite("Satisfaction", multi_items("CUSA", 1:3)),
    composite("Loyalty",      multi_items("CUSL", 1:3))
  )
  sm_med <- relationships(
    paths(from = "Image", to = c("Expectation", "Loyalty")),
    paths(from = "Expectation", to = "Satisfaction"),
    paths(from = "Satisfaction", to = "Loyalty")
  )
  pls_med <- estimate_pls(data = mobi, measurement_model = mm_med,
                            structural_model = sm_med)

  result <- assess_fimix(pls_med, K = 2, nstart = 3, seed = 42)

  expect_s3_class(result, "fimix_analysis")
  expect_true(result$converged)
  expect_equal(sum(result$segment_sizes), nrow(mobi))
})

test_that("FIMIX works with moderation model (two-stage)", {
  skip_on_cran()
  mm_mod <- constructs(
    composite("Image",        multi_items("IMAG", 1:5)),
    composite("Value",        multi_items("PERV", 1:2)),
    composite("Satisfaction", multi_items("CUSA", 1:3)),
    composite("Loyalty",      multi_items("CUSL", 1:3)),
    interaction_term(iv = "Image", moderator = "Value", method = two_stage)
  )
  sm_mod <- relationships(
    paths(from = c("Image", "Value", "Image*Value"), to = "Satisfaction"),
    paths(from = "Satisfaction", to = "Loyalty")
  )
  pls_mod <- estimate_pls(data = mobi, measurement_model = mm_mod,
                            structural_model = sm_mod)

  result <- assess_fimix(pls_mod, K = 2, nstart = 3, seed = 42)

  expect_s3_class(result, "fimix_analysis")
  expect_true(result$converged)
  # Interaction construct should be a predictor in segment paths
  expect_true(any(result$segment_paths[[1]]["Image*Value", ] != 0))
})

test_that("FIMIX works with HOC two-stage model", {
  skip_on_cran()
  mm_hoc <- constructs(
    composite("Image",        multi_items("IMAG", 1:5)),
    composite("Expectation",  multi_items("CUEX", 1:3)),
    higher_composite("Quality", c("Image", "Expectation"), method = two_stage),
    composite("Satisfaction", multi_items("CUSA", 1:3)),
    composite("Loyalty",      multi_items("CUSL", 1:3))
  )
  sm_hoc <- relationships(
    paths(from = "Quality", to = c("Satisfaction", "Loyalty")),
    paths(from = "Satisfaction", to = "Loyalty")
  )
  pls_hoc <- estimate_pls(data = mobi, measurement_model = mm_hoc,
                            structural_model = sm_hoc)

  result <- assess_fimix(pls_hoc, K = 2, nstart = 3, seed = 42)

  expect_s3_class(result, "fimix_analysis")
  expect_true(result$converged)
  expect_true("Quality" %in% rownames(result$segment_paths[[1]]))
})

# =============================================================================
# SEGMENT DIFFERENTIATION TESTS
# =============================================================================

test_that("segments have different path coefficients", {
  # At least some paths should differ between segments
  diffs <- abs(fimix_k2$segment_paths[[1]] - fimix_k2$segment_paths[[2]])
  nonzero_mask <- pls_model$path_coef != 0
  max_diff <- max(diffs[nonzero_mask])
  expect_true(max_diff > 0.01)
})

# =============================================================================
# feature_pos.R - Prediction-Oriented Segmentation (PLS-POS)
# =============================================================================
# This file implements PLS-POS, a deterministic hill-climbing approach to
# uncovering unobserved heterogeneity in PLS-SEM models, as described in:
#
# Becker, J.-M., Rai, A., Ringle, C. M., & Voelckner, F. (2013). Discovering
# Unobserved Heterogeneity in Structural Equation Models to Avert Validity
# Threats. MIS Quarterly, 37(3), 665-694.
#
# PLS-POS maximizes the sum of R-squared values across all endogenous
# constructs across K segments. Unlike FIMIX-PLS, it makes no distributional
# assumptions, uses hard (deterministic) segment assignment, and can detect
# heterogeneity in both structural and formative measurement models.
#
# Algorithm overview (Appendix B of Becker et al., 2013):
#   1. Create random initial partition of N observations into K groups
#   2. Estimate group-specific PLS models
#   3. Compute distance measure for every observation to every group
#   4. Reassign observations one at a time if it improves the objective
#   5. Repeat until no improving move exists (convergence)
#   6. Use multiple random starts to avoid local optima
# =============================================================================

# =============================================================================
# Internal helpers
# =============================================================================

#' Get endogenous constructs for the PLS-POS objective criterion.
#'
#' Excludes interaction constructs (names containing "*") because their
#' R-squared values are not meaningfully interpretable for segmentation.
#' @noRd
pos_endogenous <- function(model) {
  endo <- get_endogenous_constructs(model)
  endo[!grepl("\\*", endo)]
}

#' Estimate group-specific PLS models for each segment.
#'
#' Re-estimates the full PLS model on each segment's subset of data using
#' \code{seminr::rerun()}. Returns NULL (whole list) if any segment has
#' fewer than 2 observations; returns NULL for individual segments that
#' fail estimation.
#' @noRd
estimate_segment_models <- function(model, assignment, K) {
  segment_models <- vector("list", K)
  for (k in seq_len(K)) {
    idx <- which(assignment == k)
    if (length(idx) < 2) return(NULL)
    segment_data <- model$data[idx, , drop = FALSE]
    segment_models[[k]] <- tryCatch(
      suppressMessages(seminr::rerun(model, data = segment_data)),
      error = function(e) NULL
    )
  }
  segment_models
}

#' Compute the PLS-POS objective criterion.
#'
#' The objective is the sum of R-squared across all endogenous constructs
#' across all K segments: OBJ = sum_{k=1}^{K} sum_{b in endo} R^2_{b,k}.
#' Returns -Inf if any segment model is NULL (failed estimation).
#'
#' Note: seminr's rSquared is a matrix with rows "Rsq"/"AdjRsq" and
#' construct names as columns, so we index as rs["Rsq", endogenous].
#' @noRd
compute_pos_objective <- function(segment_models, endogenous) {
  total <- 0
  for (k in seq_along(segment_models)) {
    m <- segment_models[[k]]
    if (is.null(m)) return(-Inf)
    vals <- m$rSquared["Rsq", endogenous]
    total <- total + sum(vals, na.rm = TRUE)
  }
  total
}

#' Compute squared structural residuals for every observation.
#'
#' For each observation i, endogenous construct b, and segment k, computes
#' the squared structural residual: e^2_{i,b,k} = (y_{i,b} - yhat_{i,b,k})^2
#' where yhat uses global construct scores with segment-specific path
#' coefficients. This follows the PLS-POS distance measure definition
#' (Becker et al., 2013, Appendix B).
#'
#' @return An N x B x K array of squared residuals.
#' @noRd
compute_structural_residuals <- function(model, segment_models, endogenous) {
  N <- nrow(model$construct_scores)
  K <- length(segment_models)
  B <- length(endogenous)
  sm <- model$smMatrix
  scores <- model$construct_scores

  residuals <- array(NA_real_, dim = c(N, B, K))

  for (k in seq_len(K)) {
    seg_model <- segment_models[[k]]
    if (is.null(seg_model)) next

    for (b_idx in seq_len(B)) {
      b <- endogenous[b_idx]
      # Identify direct predictors of this endogenous construct
      predictors <- unname(sm[sm[, "target"] == b, "source"])
      if (length(predictors) == 0) next

      # Predicted value = X %*% beta_k (segment-specific coefficients)
      path_coefs <- seg_model$path_coef[predictors, b]
      yhat <- as.vector(scores[, predictors, drop = FALSE] %*% path_coefs)
      y <- scores[, b]
      residuals[, b_idx, k] <- (y - yhat)^2
    }
  }
  residuals
}

#' Compute the PLS-POS distance matrix D (N x K).
#'
#' For each observation i and segment k, the distance is:
#'   D_{i,k} = sum_b sqrt( e^2_{i,b,k} / sum_i(e^2_{i,b,k}) )
#'
#' This normalizes squared residuals by their column total so that each
#' endogenous construct contributes equally regardless of scale, then
#' aggregates across constructs. Smaller distance = better fit to segment k.
#' @noRd
compute_pos_distances <- function(sq_residuals) {
  N <- dim(sq_residuals)[1]
  B <- dim(sq_residuals)[2]
  K <- dim(sq_residuals)[3]

  distances <- matrix(0, nrow = N, ncol = K)

  for (k in seq_len(K)) {
    for (b in seq_len(B)) {
      e2_i <- sq_residuals[, b, k]
      sum_e2 <- sum(e2_i, na.rm = TRUE)
      # Guard against zero denominator (perfect fit for all observations)
      if (sum_e2 < .Machine$double.eps) next
      distances[, k] <- distances[, k] + sqrt(e2_i / sum_e2)
    }
  }
  distances
}

#' Build sorted candidate list for reassignment.
#'
#' For each observation, computes the distance improvement from moving it
#' from its current segment to its best alternative segment. Only observations
#' with positive improvement potential (d_current > d_best_alt) are included.
#'
#' @return A data.frame sorted by descending improvement potential, or NULL
#'   if no candidates have positive improvement.
#' @noRd
build_candidate_list <- function(distances, assignment, K) {
  N <- nrow(distances)
  obs_vec <- integer(0)
  from_vec <- integer(0)
  to_vec <- integer(0)
  diff_vec <- numeric(0)

  for (i in seq_len(N)) {
    current_k <- assignment[i]
    d_current <- distances[i, current_k]

    # Find the alternative segment with smallest distance
    alt_dists <- distances[i, ]
    alt_dists[current_k] <- Inf
    best_alt <- which.min(alt_dists)
    d_alt <- alt_dists[best_alt]
    diff_val <- d_current - d_alt

    if (diff_val > 0) {
      obs_vec <- c(obs_vec, i)
      from_vec <- c(from_vec, current_k)
      to_vec <- c(to_vec, best_alt)
      diff_vec <- c(diff_vec, diff_val)
    }
  }

  if (length(obs_vec) == 0) return(NULL)

  candidates <- data.frame(
    obs = obs_vec, from = from_vec, to = to_vec, diff = diff_vec,
    stringsAsFactors = FALSE
  )
  candidates[order(candidates$diff, decreasing = TRUE), ]
}

#' Create a random initial partition ensuring minimum segment size.
#'
#' First guarantees min_size observations per segment, then randomly assigns
#' the remaining observations. The final assignment is shuffled to randomize
#' observation order within segments.
#' @noRd
random_partition <- function(N, K, min_size) {
  guaranteed <- rep(seq_len(K), each = min_size)
  remaining <- N - K * min_size
  if (remaining > 0) {
    extra <- sample(seq_len(K), remaining, replace = TRUE)
    assignment <- sample(c(guaranteed, extra))
  } else {
    assignment <- sample(guaranteed)
  }
  assignment
}

#' Get segment-appropriate palette colors.
#' @noRd
pos_palette <- function(K) {
  if (K <= 8) {
    palette.colors(K, palette = "Set2")
  } else {
    palette.colors(n = K)
  }
}

# =============================================================================
# Main exported functions
# =============================================================================

#' PLS-POS: Prediction-Oriented Segmentation
#'
#' Performs prediction-oriented segmentation (PLS-POS) to uncover unobserved
#' heterogeneity in PLS-SEM models. PLS-POS uses a deterministic hill-climbing
#' approach to maximize the sum of R-squared values across all endogenous
#' constructs across K segments (Becker et al., 2013).
#'
#' Unlike FIMIX-PLS, PLS-POS makes no distributional assumptions and can
#' detect heterogeneity in both structural and (formative) measurement models.
#'
#' @param seminr_model An estimated SEMinR model from \code{estimate_pls()}.
#' @param K Integer >= 2. Number of segments to extract.
#' @param nstart Number of random starting partitions (default 10). The best
#'   solution (highest objective) is returned.
#' @param max_iter Maximum hill-climbing iterations per start (default 100).
#' @param search_depth Maximum number of candidates to evaluate per iteration
#'   before accepting the first improvement (default = N, the sample size).
#'   Use a smaller value for faster but potentially less optimal results.
#' @param min_segment_size Minimum observations per segment. Default is
#'   \code{max(10, max_predictors + 2)} where max_predictors is the largest
#'   number of predictors for any endogenous construct.
#' @param seed Random seed for reproducibility.
#'
#' @return An S3 object of class \code{"pos_analysis"} with components:
#'   \describe{
#'     \item{K}{Number of segments}
#'     \item{segment_assignment}{Integer vector of segment assignments (length N)}
#'     \item{segment_sizes}{Named integer vector of segment sizes}
#'     \item{segment_models}{List of K re-estimated seminr model objects}
#'     \item{segment_rsquared}{Matrix of R-squared values (endogenous x K)}
#'     \item{segment_paths}{List of K path coefficient matrices}
#'     \item{objective}{Sum of R-squared (objective criterion)}
#'     \item{converged}{Whether the best start converged}
#'     \item{iterations}{Number of iterations in the best start}
#'     \item{nstart}{Number of random starts attempted}
#'     \item{all_objectives}{Objective values for all starts}
#'   }
#'
#' @seealso [assess_fimix()] for probabilistic (EM-based) segmentation
#'
#' @references
#' Becker, J.-M., Rai, A., Ringle, C. M., & Voelckner, F. (2013). Discovering
#' Unobserved Heterogeneity in Structural Equation Models to Avert Validity
#' Threats. \emph{MIS Quarterly}, 37(3), 665-694.
#'
#' @examples
#' \donttest{
#' library(seminr)
#'
#' mobi_mm <- constructs(
#'   composite("Image",        multi_items("IMAG", 1:5)),
#'   composite("Expectation",  multi_items("CUEX", 1:3)),
#'   composite("Satisfaction", multi_items("CUSA", 1:3)),
#'   composite("Loyalty",      multi_items("CUSL", 1:3))
#' )
#' mobi_sm <- relationships(
#'   paths(from = "Image",       to = c("Expectation", "Satisfaction", "Loyalty")),
#'   paths(from = "Expectation", to = "Satisfaction"),
#'   paths(from = "Satisfaction", to = "Loyalty")
#' )
#' pls_model <- estimate_pls(data = mobi, measurement_model = mobi_mm,
#'                            structural_model = mobi_sm)
#'
#' pos_result <- assess_pos(pls_model, K = 2, nstart = 2, max_iter = 10,
#'                           seed = 123)
#' print(pos_result)
#' summary(pos_result)
#' }
#'
#' @export
assess_pos <- function(seminr_model,
                       K = 2,
                       nstart = 10,
                       max_iter = 100,
                       search_depth = NULL,
                       min_segment_size = NULL,
                       seed = 123) {
  # ---------------------------------------------------------------------------
  # Step 0: Validate inputs
  # ---------------------------------------------------------------------------
  if (!validate_seminr_model(seminr_model, "assess_pos")) {
    return(NULL)
  }

  if (!is.numeric(K) || length(K) != 1 || K < 2 || K != as.integer(K))
    stop("K must be an integer >= 2.")
  K <- as.integer(K)

  N <- nrow(seminr_model$construct_scores)
  endogenous <- pos_endogenous(seminr_model)
  if (length(endogenous) == 0)
    stop("No endogenous constructs found in the model.")

  # Default min_segment_size: enough for OLS estimation in each segment
  sm <- seminr_model$smMatrix
  max_preds <- max(vapply(endogenous, function(b) {
    sum(sm[, "target"] == b)
  }, integer(1)))
  if (is.null(min_segment_size)) {
    min_segment_size <- max(10L, max_preds + 2L)
  }

  if (N < K * min_segment_size) {
    stop("Sample size (", N, ") is too small for K=", K,
         " segments with min_segment_size=", min_segment_size, ".")
  }

  if (is.null(search_depth)) search_depth <- N

  # ---------------------------------------------------------------------------
  # Step 1: Multi-start hill-climbing (Algorithm Steps 1-6, Appendix B)
  # ---------------------------------------------------------------------------
  set.seed(seed)

  best_objective <- -Inf
  best_result <- NULL
  all_objectives <- numeric(nstart)

  for (r in seq_len(nstart)) {

    # --- Step 1.1: Random initial partition ---
    assignment <- random_partition(N, K, min_segment_size)

    # --- Step 1.2: Estimate initial segment models ---
    seg_models <- estimate_segment_models(seminr_model, assignment, K)
    if (is.null(seg_models) || any(vapply(seg_models, is.null, logical(1)))) {
      all_objectives[r] <- -Inf
      next
    }

    # --- Step 1.3: Distance-based initial reassignment ---
    # Assign each observation to the segment with smallest distance
    sq_resid <- compute_structural_residuals(seminr_model, seg_models, endogenous)
    dists <- compute_pos_distances(sq_resid)
    assignment <- apply(dists, 1, which.min)

    # Verify min segment sizes are still met after reassignment
    seg_sizes <- tabulate(assignment, nbins = K)
    if (any(seg_sizes < min_segment_size)) {
      all_objectives[r] <- -Inf
      next
    }

    # --- Step 1.4: Re-estimate after initial assignment ---
    seg_models <- estimate_segment_models(seminr_model, assignment, K)
    if (is.null(seg_models) || any(vapply(seg_models, is.null, logical(1)))) {
      all_objectives[r] <- -Inf
      next
    }

    obj <- compute_pos_objective(seg_models, endogenous)
    converged <- FALSE
    iter <- 0

    # --- Steps 2-6: Hill-climbing loop ---
    # Reassign ONE observation at a time, only if it improves the objective.
    # This ensures monotonic improvement of the objective criterion.
    for (iter_idx in seq_len(max_iter)) {
      iter <- iter_idx

      # Step 2: Compute distances for current partition
      sq_resid <- compute_structural_residuals(seminr_model, seg_models, endogenous)
      dists <- compute_pos_distances(sq_resid)

      # Step 3: Build candidate list sorted by improvement potential
      candidates <- build_candidate_list(dists, assignment, K)
      if (is.null(candidates) || nrow(candidates) == 0) {
        converged <- TRUE
        break
      }

      # Step 4-5: Try reassigning the most promising candidates
      improved <- FALSE
      n_tried <- 0
      for (c_idx in seq_len(nrow(candidates))) {
        if (n_tried >= search_depth) break
        n_tried <- n_tried + 1

        cand <- candidates[c_idx, ]
        from_k <- cand$from
        to_k <- cand$to

        # Enforce minimum segment size constraint
        from_size <- sum(assignment == from_k)
        if (from_size - 1 < min_segment_size) next

        # Tentative reassignment: move this observation
        new_assignment <- assignment
        new_assignment[cand$obs] <- to_k

        # Re-estimate only the two affected segments (efficiency)
        new_seg_models <- seg_models
        for (affected_k in c(from_k, to_k)) {
          idx <- which(new_assignment == affected_k)
          seg_data <- seminr_model$data[idx, , drop = FALSE]
          new_model <- tryCatch(
            suppressMessages(seminr::rerun(seminr_model, data = seg_data)),
            error = function(e) NULL
          )
          if (is.null(new_model)) break
          new_seg_models[[affected_k]] <- new_model
        }

        # Skip if estimation failed for either affected segment
        if (any(vapply(new_seg_models[c(from_k, to_k)], is.null, logical(1)))) next

        # Step 6: Accept move only if objective improves
        new_obj <- compute_pos_objective(new_seg_models, endogenous)
        if (new_obj > obj) {
          assignment <- new_assignment
          seg_models <- new_seg_models
          obj <- new_obj
          improved <- TRUE
          break
        }
      }

      # No improving move found = converged for this start
      if (!improved) {
        converged <- TRUE
        break
      }
    }

    all_objectives[r] <- obj

    # Keep track of the best solution across all starts
    if (obj > best_objective) {
      best_objective <- obj
      best_result <- list(
        assignment = assignment,
        seg_models = seg_models,
        objective = obj,
        converged = converged,
        iterations = iter
      )
    }
  }

  if (is.null(best_result)) {
    stop("PLS-POS failed to find a valid segmentation across all ", nstart,
         " random starts. Consider increasing min_segment_size or decreasing K.")
  }

  # ---------------------------------------------------------------------------
  # Step 7: Build result object from best solution
  # ---------------------------------------------------------------------------
  assignment <- best_result$assignment
  seg_models <- best_result$seg_models
  seg_sizes <- as.integer(tabulate(assignment, nbins = K))
  names(seg_sizes) <- paste("Segment", seq_len(K))

  # R-squared matrix: endogenous constructs (rows) x segments (cols)
  seg_rsq <- matrix(NA_real_, nrow = length(endogenous), ncol = K,
                    dimnames = list(endogenous, paste("Segment", seq_len(K))))
  for (k in seq_len(K)) {
    seg_rsq[, k] <- seg_models[[k]]$rSquared["Rsq", endogenous]
  }

  # Extract path coefficient matrices for convenient access
  seg_paths <- lapply(seg_models, function(m) m$path_coef)

  result <- list(
    K = K,
    segment_assignment = assignment,
    segment_sizes = seg_sizes,
    segment_models = seg_models,
    segment_rsquared = seg_rsq,
    segment_paths = seg_paths,
    objective = best_result$objective,
    converged = best_result$converged,
    iterations = best_result$iterations,
    nstart = nstart,
    all_objectives = all_objectives,
    endogenous = endogenous,
    pls_model = seminr_model,
    n_obs = N
  )
  class(result) <- c("pos_analysis", class(result))
  result
}

#' Compare PLS-POS Solutions Across K Values
#'
#' Runs \code{\link{assess_pos}} for each K in \code{K_range} and returns a
#' comparison table of objective criterion (sum of R-squared) values. This
#' helps identify the optimal number of segments.
#'
#' @inheritParams assess_pos
#' @param K_range Integer vector of K values to compare (default \code{2:5}).
#'
#' @return An S3 object of class \code{"pos_comparison"} with components:
#'   \describe{
#'     \item{solutions}{Named list of \code{pos_analysis} objects}
#'     \item{fit_table}{Data frame comparing K values}
#'     \item{K_range}{The K values compared}
#'   }
#'
#' @seealso [assess_fimix_compare()] for comparing FIMIX solutions across K
#'
#' @examples
#' \donttest{
#' library(seminr)
#'
#' mobi_mm <- constructs(
#'   composite("Image",        multi_items("IMAG", 1:5)),
#'   composite("Expectation",  multi_items("CUEX", 1:3)),
#'   composite("Satisfaction", multi_items("CUSA", 1:3)),
#'   composite("Loyalty",      multi_items("CUSL", 1:3))
#' )
#' mobi_sm <- relationships(
#'   paths(from = "Image",       to = c("Expectation", "Satisfaction", "Loyalty")),
#'   paths(from = "Expectation", to = "Satisfaction"),
#'   paths(from = "Satisfaction", to = "Loyalty")
#' )
#' pls_model <- estimate_pls(data = mobi, measurement_model = mobi_mm,
#'                            structural_model = mobi_sm)
#'
#' pos_comp <- assess_pos_compare(pls_model, K_range = 2:3,
#'                                 nstart = 2, max_iter = 10, seed = 123)
#' print(pos_comp)
#' plot(pos_comp)
#' }
#'
#' @export
assess_pos_compare <- function(seminr_model,
                               K_range = 2:5,
                               nstart = 10,
                               max_iter = 100,
                               search_depth = NULL,
                               min_segment_size = NULL,
                               seed = 123) {
  if (!validate_seminr_model(seminr_model, "assess_pos_compare")) {
    return(NULL)
  }

  solutions <- list()
  fit_rows <- list()

  for (K in K_range) {
    sol <- tryCatch(
      assess_pos(seminr_model, K = K, nstart = nstart, max_iter = max_iter,
                 search_depth = search_depth, min_segment_size = min_segment_size,
                 seed = seed),
      error = function(e) {
        message("K=", K, ": ", e$message)
        NULL
      }
    )

    label <- paste0("K", K)
    solutions[[label]] <- sol

    if (!is.null(sol)) {
      avg_rsq <- mean(colSums(sol$segment_rsquared, na.rm = TRUE))
      fit_rows[[label]] <- data.frame(
        K = K,
        Sum_R2 = sol$objective,
        Avg_R2_per_segment = avg_rsq,
        Converged = sol$converged,
        Iterations = sol$iterations,
        stringsAsFactors = FALSE
      )
    } else {
      fit_rows[[label]] <- data.frame(
        K = K, Sum_R2 = NA_real_, Avg_R2_per_segment = NA_real_,
        Converged = NA, Iterations = NA_integer_,
        stringsAsFactors = FALSE
      )
    }
  }

  fit_table <- do.call(rbind, fit_rows)
  rownames(fit_table) <- NULL

  result <- list(
    solutions = solutions,
    fit_table = fit_table,
    K_range = K_range,
    pls_model = seminr_model
  )
  class(result) <- c("pos_comparison", class(result))
  result
}

#' Extract Segment-Specific PLS Models from PLS-POS Results
#'
#' Returns a list of fully re-estimated PLS models, one per segment, using the
#' hard segment assignment from PLS-POS.
#'
#' @param pos_result A \code{pos_analysis} object from \code{assess_pos()}.
#'
#' @return A named list of K seminr model objects.
#'
#' @seealso [assess_pos()] for running PLS-POS segmentation
#'
#' @export
pos_segments <- function(pos_result) {
  if (!inherits(pos_result, "pos_analysis"))
    stop("pos_result must be a pos_analysis object from assess_pos().")
  pos_result$segment_models
}

# =============================================================================
# S3 methods: print, summary, plot
# =============================================================================

#' @export
print.pos_analysis <- function(x, ...) {
  cat("PLS-POS Analysis\n")
  cat("================\n")
  cat("Segments:", x$K, "\n")
  cat("Observations:", x$n_obs, "\n")
  cat("Converged:", ifelse(x$converged, "Yes", "No"),
      "(", x$iterations, "iterations )\n")
  cat("Random starts:", x$nstart, "\n")
  cat("Objective (Sum R\u00b2):", round(x$objective, 4), "\n\n")

  # Segment sizes table
  cat("Segment Sizes:\n")
  size_df <- data.frame(
    Segment = names(x$segment_sizes),
    Size = as.integer(x$segment_sizes),
    Proportion = round(x$segment_sizes / x$n_obs, 4),
    stringsAsFactors = FALSE
  )
  print(size_df, row.names = FALSE)

  # R-squared comparison: segment vs global
  cat("\nR\u00b2 per Endogenous Construct:\n")
  rsq_df <- as.data.frame(round(x$segment_rsquared, 4))
  rsq_df$Global <- round(x$pls_model$rSquared["Rsq", x$endogenous], 4)
  print(rsq_df)

  # Segment-specific path coefficients
  cat("\nSegment Path Coefficients:\n")
  for (k in seq_len(x$K)) {
    cat("\n  Segment", k, ":\n")
    pm <- x$segment_paths[[k]]
    nonzero <- which(pm != 0, arr.ind = TRUE)
    if (nrow(nonzero) == 0) next
    path_df <- data.frame(
      From = rownames(pm)[nonzero[, 1]],
      To = colnames(pm)[nonzero[, 2]],
      Coefficient = round(pm[nonzero], 4),
      stringsAsFactors = FALSE
    )
    print(path_df, row.names = FALSE)
  }

  invisible(x)
}

#' @export
summary.pos_analysis <- function(object, ...) {
  result <- list(
    K = object$K,
    n_obs = object$n_obs,
    objective = object$objective,
    converged = object$converged,
    iterations = object$iterations,
    nstart = object$nstart,
    all_objectives = object$all_objectives,
    segment_sizes = object$segment_sizes,
    segment_rsquared = object$segment_rsquared,
    segment_paths = object$segment_paths,
    global_rsquared = object$pls_model$rSquared,
    global_paths = object$pls_model$path_coef,
    endogenous = object$endogenous
  )
  class(result) <- c("summary.pos_analysis", class(result))
  result
}

#' @export
print.summary.pos_analysis <- function(x, digits = 4, ...) {
  cat("PLS-POS Analysis \u2014 Detailed Summary\n")
  cat("======================================\n")
  cat("Segments:", x$K, "| Observations:", x$n_obs, "\n")
  cat("Objective (Sum R\u00b2):", round(x$objective, digits), "\n")
  cat("Converged:", ifelse(x$converged, "Yes", "No"),
      "(", x$iterations, "iterations )\n")
  cat("Random starts:", x$nstart, "\n")
  cat("Start objectives:", paste(round(x$all_objectives, digits), collapse = ", "), "\n\n")

  # Segment size breakdown
  cat("Segment Sizes:\n")
  for (k in seq_along(x$segment_sizes)) {
    cat("  Segment ", k, ": ", x$segment_sizes[k],
        " (", round(100 * x$segment_sizes[k] / x$n_obs, 1), "%)\n", sep = "")
  }

  # Side-by-side R-squared: segment vs global
  cat("\nR\u00b2 Comparison (Segment vs Global):\n")
  rsq_df <- as.data.frame(round(x$segment_rsquared, digits))
  rsq_df$Global <- round(x$global_rsquared["Rsq", x$endogenous], digits)
  print(rsq_df)

  # Side-by-side path coefficients: global + each segment
  cat("\nPath Coefficients per Segment:\n")
  global_pm <- x$global_paths
  nonzero <- which(global_pm != 0, arr.ind = TRUE)
  if (nrow(nonzero) > 0) {
    path_labels <- paste(rownames(global_pm)[nonzero[, 1]], "->",
                         colnames(global_pm)[nonzero[, 2]])
    coef_df <- data.frame(Path = path_labels, stringsAsFactors = FALSE)
    coef_df$Global <- round(global_pm[nonzero], digits)
    for (k in seq_along(x$segment_paths)) {
      coef_df[[paste0("Seg.", k)]] <- round(x$segment_paths[[k]][nonzero], digits)
    }
    print(coef_df, row.names = FALSE)
  }

  invisible(x)
}

#' @export
plot.pos_analysis <- function(x,
                              type = c("segments", "rsquared", "paths"),
                              ...) {
  type <- match.arg(type)
  cols <- pos_palette(x$K)

  if (type == "segments") {
    old_par <- par(no.readonly = TRUE)
    on.exit(par(old_par))
    par(mar = c(5, 5, 4, 2))

    barplot(x$segment_sizes / x$n_obs * 100,
            names.arg = paste("Seg.", seq_len(x$K)),
            col = cols, ylim = c(0, 100),
            ylab = "Proportion (%)",
            main = paste("PLS-POS: Segment Proportions (K =", x$K, ")"),
            ...)

  } else if (type == "rsquared") {
    old_par <- par(no.readonly = TRUE)
    on.exit(par(old_par))
    par(mar = c(8, 5, 4, 2))

    barplot(t(x$segment_rsquared), beside = TRUE, col = cols,
            las = 2, ylab = expression(R^2),
            main = paste("PLS-POS: R\u00b2 per Construct (K =", x$K, ")"),
            ...)
    legend("topright", legend = paste("Seg.", seq_len(x$K)),
           fill = cols, bty = "n")

  } else if (type == "paths") {
    pm_ref <- x$segment_paths[[1]]
    nonzero <- which(pm_ref != 0, arr.ind = TRUE)
    if (nrow(nonzero) == 0) {
      message("No non-zero path coefficients to plot.")
      return(invisible(x))
    }

    path_labels <- paste(rownames(pm_ref)[nonzero[, 1]], "->",
                         colnames(pm_ref)[nonzero[, 2]])
    coef_matrix <- matrix(NA_real_, nrow = length(path_labels), ncol = x$K)
    for (k in seq_len(x$K)) {
      coef_matrix[, k] <- x$segment_paths[[k]][nonzero]
    }
    rownames(coef_matrix) <- path_labels
    colnames(coef_matrix) <- paste("Seg.", seq_len(x$K))

    old_par <- par(no.readonly = TRUE)
    on.exit(par(old_par))
    par(mar = c(8, 5, 4, 2))

    barplot(t(coef_matrix), beside = TRUE, col = cols,
            las = 2, ylab = "Path Coefficient",
            main = paste("PLS-POS: Path Coefficients (K =", x$K, ")"),
            ...)
    legend("topright", legend = colnames(coef_matrix),
           fill = cols, bty = "n")
  }

  invisible(x)
}

#' @export
print.pos_comparison <- function(x, ...) {
  cat("PLS-POS Comparison\n")
  cat("==================\n")
  cat("K range:", paste(x$K_range, collapse = ", "), "\n\n")
  print(x$fit_table, row.names = FALSE)
  invisible(x)
}

#' @export
summary.pos_comparison <- function(object, ...) {
  object
}

#' @export
plot.pos_comparison <- function(x, ...) {
  ft <- x$fit_table
  valid <- !is.na(ft$Sum_R2)

  if (sum(valid) < 2) {
    message("Not enough valid solutions to plot.")
    return(invisible(x))
  }

  old_par <- par(no.readonly = TRUE)
  on.exit(par(old_par))
  par(mar = c(5, 5, 4, 2))

  plot(ft$K[valid], ft$Sum_R2[valid],
       type = "b", pch = 19, lwd = 2,
       xlab = "Number of Segments (K)",
       ylab = expression("Objective (Sum " * R^2 * ")"),
       main = "PLS-POS: Objective Criterion vs K",
       xaxt = "n", ...)
  axis(1, at = ft$K[valid])

  invisible(x)
}

# =============================================================================
# Tests for PLS-POS (Prediction-Oriented Segmentation)
# =============================================================================

# --- Setup: pre-compute models and results -----------------------------------
library(seminr)

mobi_mm <- constructs(
  composite("Image",        multi_items("IMAG", 1:5)),
  composite("Expectation",  multi_items("CUEX", 1:3)),
  composite("Value",        multi_items("PERV", 1:2)),
  composite("Satisfaction", multi_items("CUSA", 1:3)),
  composite("Loyalty",      multi_items("CUSL", 1:3))
)
mobi_sm <- relationships(
  paths(from = "Image",       to = c("Expectation", "Satisfaction", "Loyalty")),
  paths(from = "Expectation", to = c("Value", "Satisfaction")),
  paths(from = "Value",       to = "Satisfaction"),
  paths(from = "Satisfaction", to = "Loyalty")
)
pls_model <- estimate_pls(data = mobi, measurement_model = mobi_mm,
                           structural_model = mobi_sm)

# Pre-compute PLS-POS for K=2 (reused across tests)
pos_k2 <- assess_pos(pls_model, K = 2, nstart = 3, max_iter = 20, seed = 123)

# =============================================================================
# STRUCTURE TESTS
# =============================================================================

test_that("assess_pos returns pos_analysis object", {
  expect_s3_class(pos_k2, "pos_analysis")
})

test_that("pos result contains all expected components", {
  expected <- c("K", "segment_assignment", "segment_sizes", "segment_models",
                "segment_rsquared", "segment_paths", "objective",
                "converged", "iterations", "nstart", "all_objectives",
                "endogenous", "pls_model", "n_obs")
  expect_true(all(expected %in% names(pos_k2)))
})

test_that("K is stored correctly", {
  expect_equal(pos_k2$K, 2)
})

test_that("n_obs matches model data", {
  expect_equal(pos_k2$n_obs, nrow(pls_model$construct_scores))
})

# =============================================================================
# SEGMENT ASSIGNMENT AND SIZE TESTS
# =============================================================================

test_that("segment assignments are integers from 1 to K", {
  expect_true(all(pos_k2$segment_assignment %in% 1:2))
})

test_that("segment assignment length matches N", {
  expect_equal(length(pos_k2$segment_assignment), pos_k2$n_obs)
})

test_that("segment sizes sum to N", {
  expect_equal(sum(pos_k2$segment_sizes), pos_k2$n_obs)
})

test_that("segment sizes match assignment counts", {
  counts <- tabulate(pos_k2$segment_assignment, nbins = 2)
  expect_equal(as.integer(pos_k2$segment_sizes), counts)
})

test_that("all segments are non-empty", {
  expect_true(all(pos_k2$segment_sizes > 0))
})

# =============================================================================
# OBJECTIVE CRITERION TESTS
# =============================================================================

test_that("objective is finite and positive", {
  expect_true(is.finite(pos_k2$objective))
  expect_true(pos_k2$objective > 0)
})

test_that("objective equals sum of segment R-squared values", {
  expected_obj <- sum(pos_k2$segment_rsquared, na.rm = TRUE)
  expect_equal(pos_k2$objective, expected_obj, tolerance = 0.01)
})

test_that("all_objectives has length nstart", {
  expect_equal(length(pos_k2$all_objectives), pos_k2$nstart)
})

test_that("best objective is max of all_objectives", {
  valid <- pos_k2$all_objectives[is.finite(pos_k2$all_objectives)]
  expect_equal(pos_k2$objective, max(valid))
})

# =============================================================================
# SEGMENT MODEL TESTS
# =============================================================================

test_that("segment_models is a list of length K", {
  expect_true(is.list(pos_k2$segment_models))
  expect_equal(length(pos_k2$segment_models), 2)
})

test_that("segment models are seminr model objects", {
  for (k in 1:2) {
    expect_true(inherits(pos_k2$segment_models[[k]], "seminr_model"))
  }
})

test_that("segment path matrices have same structure as global model", {
  for (k in 1:2) {
    expect_equal(dim(pos_k2$segment_paths[[k]]),
                 dim(pls_model$path_coef))
    expect_equal(rownames(pos_k2$segment_paths[[k]]),
                 rownames(pls_model$path_coef))
    expect_equal(colnames(pos_k2$segment_paths[[k]]),
                 colnames(pls_model$path_coef))
  }
})

test_that("segment path coefficients are non-zero where model has paths", {
  nonzero_mask <- pls_model$path_coef != 0
  for (k in 1:2) {
    seg_coefs <- pos_k2$segment_paths[[k]][nonzero_mask]
    expect_true(all(seg_coefs != 0))
  }
})

# =============================================================================
# R-SQUARED TESTS
# =============================================================================

test_that("segment_rsquared has correct dimensions", {
  endogenous <- pos_k2$endogenous
  expect_equal(nrow(pos_k2$segment_rsquared), length(endogenous))
  expect_equal(ncol(pos_k2$segment_rsquared), 2)
})

test_that("R-squared values are in [0, 1]", {
  expect_true(all(pos_k2$segment_rsquared >= 0, na.rm = TRUE))
  expect_true(all(pos_k2$segment_rsquared <= 1, na.rm = TRUE))
})

test_that("endogenous constructs exclude interaction terms", {
  expect_false(any(grepl("\\*", pos_k2$endogenous)))
})

# =============================================================================
# CONVERGENCE TESTS
# =============================================================================

test_that("iterations is a positive integer", {
  expect_true(pos_k2$iterations > 0)
})

test_that("nstart is stored correctly", {
  expect_equal(pos_k2$nstart, 3L)
})

# =============================================================================
# REPRODUCIBILITY TESTS
# =============================================================================

test_that("results are deterministic with same seed", {
  r1 <- assess_pos(pls_model, K = 2, nstart = 2, max_iter = 10, seed = 42)
  r2 <- assess_pos(pls_model, K = 2, nstart = 2, max_iter = 10, seed = 42)
  expect_equal(r1$objective, r2$objective)
  expect_equal(r1$segment_assignment, r2$segment_assignment)
  expect_equal(r1$segment_sizes, r2$segment_sizes)
})

# =============================================================================
# SEGMENT DIFFERENTIATION TESTS
# =============================================================================

test_that("segments have different path coefficients", {
  diffs <- abs(pos_k2$segment_paths[[1]] - pos_k2$segment_paths[[2]])
  nonzero_mask <- pls_model$path_coef != 0
  max_diff <- max(diffs[nonzero_mask])
  expect_true(max_diff > 0.01)
})

# =============================================================================
# INPUT VALIDATION TESTS
# =============================================================================

test_that("non-seminr model returns NULL with warning", {
  expect_warning(result <- assess_pos("not a model", K = 2),
                 "SEMinR models")
  expect_null(result)
})

test_that("K < 2 raises error", {
  expect_error(assess_pos(pls_model, K = 1), "K must be an integer >= 2")
})

test_that("K = 0 raises error", {
  expect_error(assess_pos(pls_model, K = 0), "K must be an integer >= 2")
})

test_that("non-integer K raises error", {
  expect_error(assess_pos(pls_model, K = 2.5), "K must be an integer >= 2")
})

test_that("too many segments for sample size raises error", {
  expect_error(assess_pos(pls_model, K = 50, min_segment_size = 20),
               "too small")
})

# =============================================================================
# pos_segments() TESTS
# =============================================================================

test_that("pos_segments returns list of segment models", {
  segs <- pos_segments(pos_k2)
  expect_true(is.list(segs))
  expect_equal(length(segs), 2)
  for (k in 1:2) {
    expect_true(inherits(segs[[k]], "seminr_model"))
  }
})

test_that("pos_segments rejects non-pos_analysis input", {
  expect_error(pos_segments("not_pos"), "pos_analysis object")
})

# =============================================================================
# S3 METHOD TESTS
# =============================================================================

test_that("print.pos_analysis returns invisibly", {
  expect_invisible(print(pos_k2))
})

test_that("print output contains key information", {
  out <- capture.output(print(pos_k2))
  out_text <- paste(out, collapse = "\n")

  expect_true(grepl("PLS-POS Analysis", out_text))
  expect_true(grepl("Segments: 2", out_text))
  expect_true(grepl("Objective", out_text))
  expect_true(grepl("Segment Sizes", out_text))
  expect_true(grepl("Path Coefficients", out_text))
})

test_that("summary returns summary.pos_analysis object", {
  s <- summary(pos_k2)
  expect_s3_class(s, "summary.pos_analysis")
})

test_that("summary print includes detailed output", {
  out <- capture.output(print(summary(pos_k2)))
  out_text <- paste(out, collapse = "\n")

  expect_true(grepl("Detailed Summary", out_text))
  expect_true(grepl("Global", out_text))
  expect_true(grepl("Path Coefficients per Segment", out_text))
})

test_that("plot.pos_analysis runs without error for all types", {
  expect_no_error(plot(pos_k2, type = "segments"))
  expect_no_error(plot(pos_k2, type = "rsquared"))
  expect_no_error(plot(pos_k2, type = "paths"))
})

# =============================================================================
# COMPARISON TESTS
# =============================================================================

test_that("assess_pos_compare returns pos_comparison object", {
  comp <- assess_pos_compare(pls_model, K_range = 2:3,
                              nstart = 2, max_iter = 10, seed = 123)
  expect_s3_class(comp, "pos_comparison")
})

test_that("comparison contains solutions for each K", {
  comp <- assess_pos_compare(pls_model, K_range = 2:3,
                              nstart = 2, max_iter = 10, seed = 123)
  expect_true("K2" %in% names(comp$solutions))
  expect_true("K3" %in% names(comp$solutions))
})

test_that("comparison fit_table has correct structure", {
  comp <- assess_pos_compare(pls_model, K_range = 2:3,
                              nstart = 2, max_iter = 10, seed = 123)
  expect_true("K" %in% names(comp$fit_table))
  expect_true("Sum_R2" %in% names(comp$fit_table))
  expect_true("Converged" %in% names(comp$fit_table))
  expect_equal(nrow(comp$fit_table), 2)
})

test_that("objective increases with more segments", {
  comp <- assess_pos_compare(pls_model, K_range = 2:3,
                              nstart = 2, max_iter = 10, seed = 123)
  r2_vals <- comp$fit_table$Sum_R2
  expect_true(r2_vals[2] >= r2_vals[1])
})

test_that("comparison print runs without error", {
  comp <- assess_pos_compare(pls_model, K_range = 2:3,
                              nstart = 2, max_iter = 10, seed = 123)
  expect_output(print(comp), "PLS-POS Comparison")
})

test_that("comparison plot runs without error", {
  comp <- assess_pos_compare(pls_model, K_range = 2:3,
                              nstart = 2, max_iter = 10, seed = 123)
  expect_no_error(plot(comp))
})

# =============================================================================
# MODEL TYPE TESTS: MEDIATION
# =============================================================================

test_that("PLS-POS works with mediation model", {
  skip_on_cran()
  mm_med <- constructs(
    composite("Image",        multi_items("IMAG", 1:5)),
    composite("Expectation",  multi_items("CUEX", 1:3)),
    composite("Satisfaction", multi_items("CUSA", 1:3)),
    composite("Loyalty",      multi_items("CUSL", 1:3))
  )
  sm_med <- relationships(
    paths(from = "Image", to = c("Expectation", "Loyalty")),
    paths(from = "Expectation", to = "Satisfaction"),
    paths(from = "Satisfaction", to = "Loyalty")
  )
  pls_med <- estimate_pls(data = mobi, measurement_model = mm_med,
                            structural_model = sm_med)

  result <- assess_pos(pls_med, K = 2, nstart = 2, max_iter = 10, seed = 42)

  expect_s3_class(result, "pos_analysis")
  expect_equal(sum(result$segment_sizes), nrow(mobi))
  expect_true(result$objective > 0)
  # Mediated endogenous constructs should be in the objective
  expect_true("Satisfaction" %in% result$endogenous)
  expect_true("Loyalty" %in% result$endogenous)
})

# =============================================================================
# MODEL TYPE TESTS: MODERATION
# =============================================================================

test_that("PLS-POS works with moderation model (two-stage)", {
  skip_on_cran()
  mm_mod <- constructs(
    composite("Image",        multi_items("IMAG", 1:5)),
    composite("Value",        multi_items("PERV", 1:2)),
    composite("Satisfaction", multi_items("CUSA", 1:3)),
    composite("Loyalty",      multi_items("CUSL", 1:3)),
    interaction_term(iv = "Image", moderator = "Value", method = two_stage)
  )
  sm_mod <- relationships(
    paths(from = c("Image", "Value", "Image*Value"), to = "Satisfaction"),
    paths(from = "Satisfaction", to = "Loyalty")
  )
  pls_mod <- estimate_pls(data = mobi, measurement_model = mm_mod,
                            structural_model = sm_mod)

  result <- assess_pos(pls_mod, K = 2, nstart = 2, max_iter = 10, seed = 42)

  expect_s3_class(result, "pos_analysis")
  expect_equal(sum(result$segment_sizes), nrow(mobi))

  # Interaction construct should NOT be in endogenous list
  expect_false("Image*Value" %in% result$endogenous)

  # Satisfaction and Loyalty should be endogenous
  expect_true("Satisfaction" %in% result$endogenous)
  expect_true("Loyalty" %in% result$endogenous)

  # Interaction construct should still appear as a predictor in segment paths
  expect_true("Image*Value" %in% rownames(result$segment_paths[[1]]))
})

# =============================================================================
# MODEL TYPE TESTS: HOC (Higher-Order Constructs)
# =============================================================================

test_that("PLS-POS works with HOC two-stage model", {
  skip_on_cran()
  mm_hoc <- constructs(
    composite("Image",        multi_items("IMAG", 1:5)),
    composite("Expectation",  multi_items("CUEX", 1:3)),
    higher_composite("Quality", c("Image", "Expectation"), method = two_stage),
    composite("Satisfaction", multi_items("CUSA", 1:3)),
    composite("Loyalty",      multi_items("CUSL", 1:3))
  )
  sm_hoc <- relationships(
    paths(from = "Quality", to = c("Satisfaction", "Loyalty")),
    paths(from = "Satisfaction", to = "Loyalty")
  )
  pls_hoc <- estimate_pls(data = mobi, measurement_model = mm_hoc,
                            structural_model = sm_hoc)

  result <- assess_pos(pls_hoc, K = 2, nstart = 2, max_iter = 10, seed = 42)

  expect_s3_class(result, "pos_analysis")
  expect_equal(sum(result$segment_sizes), nrow(mobi))
  expect_true(result$objective > 0)

  # Quality (HOC) should appear as predictor in segment paths
  expect_true("Quality" %in% rownames(result$segment_paths[[1]]))

  # Satisfaction and Loyalty are endogenous
  expect_true("Satisfaction" %in% result$endogenous)
  expect_true("Loyalty" %in% result$endogenous)
})

# =============================================================================
# INTERNAL HELPER TESTS
# =============================================================================

test_that("pos_endogenous excludes interaction constructs", {
  mm_mod <- constructs(
    composite("Image",        multi_items("IMAG", 1:5)),
    composite("Value",        multi_items("PERV", 1:2)),
    composite("Satisfaction", multi_items("CUSA", 1:3)),
    interaction_term(iv = "Image", moderator = "Value", method = two_stage)
  )
  sm_mod <- relationships(
    paths(from = c("Image", "Value", "Image*Value"), to = "Satisfaction")
  )
  pls_mod <- estimate_pls(data = mobi, measurement_model = mm_mod,
                            structural_model = sm_mod)

  endo <- seminrExtras:::pos_endogenous(pls_mod)
  expect_true("Satisfaction" %in% endo)
  expect_false("Image*Value" %in% endo)
})

test_that("compute_pos_objective returns sum of R-squared", {
  seg_models <- pos_k2$segment_models
  endogenous <- pos_k2$endogenous
  obj <- seminrExtras:::compute_pos_objective(seg_models, endogenous)
  expect_true(is.numeric(obj))
  expect_true(obj > 0)
})

test_that("compute_pos_objective returns -Inf for NULL model", {
  obj <- seminrExtras:::compute_pos_objective(list(NULL, NULL), pos_k2$endogenous)
  expect_equal(obj, -Inf)
})

test_that("random_partition produces valid assignment", {
  asgn <- seminrExtras:::random_partition(100, 3, 10)
  expect_equal(length(asgn), 100)
  expect_true(all(asgn %in% 1:3))
  # Each segment has at least min_size
  expect_true(all(tabulate(asgn, nbins = 3) >= 10))
})

test_that("compute_pos_distances returns N x K matrix", {
  sq_resid <- seminrExtras:::compute_structural_residuals(
    pls_model, pos_k2$segment_models, pos_k2$endogenous)
  dists <- seminrExtras:::compute_pos_distances(sq_resid)
  expect_equal(nrow(dists), nrow(pls_model$construct_scores))
  expect_equal(ncol(dists), 2)
  expect_true(all(dists >= 0))
})

# =============================================================================
# feature_congruence.R - Congruence Coefficient Testing
# =============================================================================
# This file implements bootstrap-based congruence coefficient testing for
# assessing measure congruence in nomological networks, as described in:
#
# Franke, G. R., Sarstedt, M., & Danks, N. P. (2021). "Assessing measure
# congruence in nomological networks." Journal of Business Research, 130, 318-334.
#
# The congruence coefficient measures the similarity between two constructs
# in terms of their correlational patterns. A coefficient close to 1 indicates
# high congruence (the constructs behave similarly in the nomological network).
# =============================================================================

#' Bootstrap congruence coefficient test
#'
#' `congruence_test` conducts a bootstrapped significance test of congruence
#' coefficients between all pairs of constructs in a PLS-SEM model.
#'
#' The congruence coefficient (rc) measures how similarly two constructs
#' relate to other constructs in the model. Values close to 1 indicate high
#' congruence. The test evaluates H0: rc < threshold (default = 1).
#'
#' @param seminr_model The SEMinR model for congruence analysis
#' @param nboot The number of bootstrap subsamples to execute (defaults to 2000).
#' @param seed The seed for reproducibility (defaults to 123).
#' @param alpha The required level of alpha for statistical testing (defaults
#'   to 0.05). Used to compute confidence intervals.
#' @param threshold The threshold with which to compare significance testing.
#'   H0: rc < threshold (defaults to 1).
#'
#' @return A list containing a matrix of congruence coefficients and
#'   significance test results for all construct pairs.
#'
#' @seealso [assess_cvpat()] and [assess_cvpat_compare()] for predictive validity testing
#'
#' @references Franke, G. R., Sarstedt, M., & Danks, N. P. (2021). Assessing
#' measure congruence in nomological networks. Journal of Business Research,
#' 130, 318-334.
#'
#' @examples
#' # Load libraries
#' library(seminr)
#' library(seminrExtras)
#'
#' # Create measurement model ----
#' corp_rep_mm <- constructs(
#'   composite("QUAL", multi_items("qual_", 1:8), weights = mode_B),
#'   composite("PERF", multi_items("perf_", 1:5), weights = mode_B),
#'   composite("CSOR", multi_items("csor_", 1:5), weights = mode_B),
#'   composite("ATTR", multi_items("attr_", 1:3), weights = mode_B),
#'   composite("COMP", multi_items("comp_", 1:3)),
#'   composite("LIKE", multi_items("like_", 1:3))
#' )
#'
#' # Create structural model ----
#' corp_rep_sm <- relationships(
#'   paths(from = c("QUAL", "PERF", "CSOR", "ATTR"), to = c("COMP", "LIKE"))
#' )
#'
#' # Estimate the model ----
#' corp_rep_pls_model <- estimate_pls(
#'   data = corp_rep_data,
#'   measurement_model = corp_rep_mm,
#'   structural_model  = corp_rep_sm,
#'   missing = mean_replacement,
#'  missing_value = "-99")
#'
#' # Assess the base model ----
#' congruence_test(seminr_model = corp_rep_pls_model,
#'                 nboot = 20,
#'                 seed = 123,
#'                 alpha = 0.05,
#'                 threshold = 1)
#'
#' @export
congruence_test <- function(seminr_model,
                            nboot = 2000,
                            seed = 123,
                            alpha = 0.05,
                            threshold = 1) {

  # Set seed for reproducibility of bootstrap resampling
  set.seed(seed)

  # ---------------------------------------------------------------------------
  # Step 1: Validate the model
  # ---------------------------------------------------------------------------
  if (!validate_seminr_model(seminr_model, "congruence_test")) {
    return(NULL)
  }

  # Get all construct names from the model
  construct_names <- colnames(seminr_model$construct_scores)

  # ---------------------------------------------------------------------------
  # Step 2: Define the congruence coefficient calculation
  # ---------------------------------------------------------------------------
  # The congruence coefficient (rc) measures pattern similarity between two
  # vectors. Formula: rc = sum(X*Y) / sqrt(sum(X^2) * sum(Y^2))
  # This is essentially a cosine similarity applied to correlation patterns.
  calc_congruence <- function(mat, X, Y) {
    return(sum(mat[, X] * mat[, Y]) / sqrt(sum(mat[, X]^2) * sum(mat[, Y]^2)))
  }

  # ---------------------------------------------------------------------------
  # Step 3: Generate all pairwise construct combinations
  # ---------------------------------------------------------------------------
  # We test congruence for every unique pair of constructs
  combns <- t(utils::combn(construct_names, 2))

  # ---------------------------------------------------------------------------
  # Step 4: Bootstrap loop - resample and compute congruence coefficients
  # ---------------------------------------------------------------------------
  # Create 3D array to store bootstrap results: [constructs x constructs x iterations]
  ret_array <- array(NA,
                     dim = list(length(construct_names), length(construct_names), nboot),
                     dimnames = list(construct_names, construct_names, 1:nboot))

  for (iter in seq_len(nboot)) {
    # Resample data with replacement and re-estimate the model
    resampled_data <- seminr_model$rawdata[
      sample(nrow(seminr_model$rawdata), nrow(seminr_model$rawdata), replace = TRUE),
    ]
    it_model <- suppressMessages(seminr::rerun(seminr_model, data = resampled_data))

    # Compute correlation matrix of construct scores for this bootstrap sample
    ret_mat <- stats::cor(it_model$construct_scores)

    # Replace diagonal with rhoC (composite reliability) values
    # This creates a matrix where diagonal = reliability, off-diagonal = correlations
    diag(ret_mat) <- seminr::rhoC_AVE(x = it_model)[colnames(ret_mat), 1]

    # Calculate congruence coefficient for each construct pair
    # Store in upper triangle of the result array
    ret_array[, , iter][upper.tri(ret_mat)] <- apply(combns, 1, function(x) {
      calc_congruence(ret_mat, x[1], x[2])
    })
  }

  # ---------------------------------------------------------------------------
  # Step 5: Calculate original (non-bootstrap) congruence coefficients
  # ---------------------------------------------------------------------------
  # Compute correlation matrix from original model
  cor_mat <- stats::cor(seminr_model$construct_scores)

  # Replace diagonal with rhoC values
  diag(cor_mat) <- seminr::rhoC_AVE(x = seminr_model)[colnames(ret_mat), 1]

  # Prepare matrix for original estimates (upper triangle only)
  original_matrix <- cor_mat
  original_matrix[lower.tri(original_matrix)] <- 0
  diag(original_matrix) <- 0

  # Calculate congruence coefficients for original data
  original_matrix[upper.tri(original_matrix)] <- apply(combns, 1, function(x) {
    calc_congruence(cor_mat, x[1], x[2])
  })

  # ---------------------------------------------------------------------------
  # Step 6: Compute bootstrap statistics for each construct pair
  # ---------------------------------------------------------------------------
  boot_array <- ret_array

  # Initialize result vectors
  Path <- c()        # Construct pair labels (e.g., "QUAL -> PERF")
  original <- c()    # Original congruence coefficient
  boot_mean <- c()   # Difference from threshold
  boot_SD <- c()     # Bootstrap standard deviation
  t_stat <- c()      # T-statistic for significance test
  lower <- c()       # Lower confidence interval bound
  upper <- c()       # Upper confidence interval bound

  # Calculate alpha/2 for confidence interval labels
  alpha_text <- alpha / 2 * 100

  # Use upper.tri mask to iterate over construct pairs
  ut_mask <- upper.tri(original_matrix)

  # Loop through upper triangle to extract results for each pair
  for (i in seq_len(nrow(original_matrix))) {
    for (j in seq_len(ncol(original_matrix))) {
      if (ut_mask[i, j]) {
        # Store construct pair label
        Path <- append(Path, paste(rownames(original_matrix)[i], " -> ",
                                   colnames(original_matrix)[j]))

        # Store original coefficient
        original <- append(original, original_matrix[i, j])

        # Compute difference from threshold (for significance test)
        # Diff = threshold - |original| (positive if below threshold)
        boot_mean <- append(boot_mean, (threshold - abs(original_matrix[i, j])))

        # Bootstrap standard deviation of congruence coefficient
        boot_SD <- append(boot_SD, stats::sd(boot_array[i, j, ]))

        # Compute t-statistic: (threshold - |rc|) / SE
        # Guard against division by near-zero SD (indicates perfect stability)
        if (stats::sd(boot_array[i, j, ]) < .Machine$double.eps) {
          t_stat <- append(t_stat, NA)
        } else {
          t_stat <- append(t_stat, (threshold - abs(original_matrix[i, j])) /
                             stats::sd(boot_array[i, j, ]))
        }

        # Compute bootstrap confidence intervals using seminr's internal function
        ci <- conf_int(boot_array,
                                from = rownames(original_matrix)[i],
                                to = colnames(original_matrix)[j],
                                alpha = alpha)
        lower <- append(lower, ci[[1]])
        upper <- append(upper, ci[[2]])
      }
    }
  }

  # ---------------------------------------------------------------------------
  # Step 7: Compile and format results
  # ---------------------------------------------------------------------------
  return_matrix <- cbind(original, boot_mean, boot_SD, t_stat, lower, upper)

  colnames(return_matrix) <- c("Original Est.",        # Congruence coefficient
                               "Diff",                  # threshold - |rc|
                               "Bootstrap SD",          # Standard error
                               "T Stat.",               # t-statistic
                               paste(alpha_text, "% CI", sep = ""),   # Lower CI
                               paste((100 - alpha_text), "% CI", sep = ""))  # Upper CI
  rownames(return_matrix) <- Path

  # Convert to table_output class for consistent printing
  return_matrix <- convert_to_table_output(return_matrix)

  return(list(results = return_matrix))
}

library(seminr)

# Setup: Create a basic model for testing
set.seed(123)

corp_rep_mm <- constructs(
  composite("COMP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  composite("CUSA", single_item("cusa")),
  composite("CUSL", multi_items("cusl_", 1:3))
)

corp_rep_sm <- relationships(
  paths(from = c("COMP", "LIKE"), to = c("CUSA")),
  paths(from = c("CUSA"), to = c("CUSL"))
)

test_model <- estimate_pls(
  data = corp_rep_data,
  measurement_model = corp_rep_mm,
  structural_model  = corp_rep_sm,
  missing = mean_replacement,
  missing_value = "-99"
)

# ============================================================================
# Basic Functionality Tests
# ============================================================================

test_that("congruence_test returns proper structure", {
  result <- congruence_test(test_model, nboot = 50, seed = 123)

  # Should return a list

  expect_type(result, "list")


  # Should have 'results' element

  expect_named(result, "results")

  # Results should be a matrix with table_output class
  expect_true(inherits(result$results, "matrix"))
  expect_true("table_output" %in% class(result$results))
})

test_that("congruence_test returns correct dimensions", {
  result <- congruence_test(test_model, nboot = 50, seed = 123)

  # Number of rows should equal number of construct pairs
  n_constructs <- ncol(test_model$construct_scores)
  expected_pairs <- choose(n_constructs, 2)
  expect_equal(nrow(result$results), expected_pairs)

  # Should have 6 columns
  expect_equal(ncol(result$results), 6)
})

test_that("congruence_test returns correct column names", {
  result <- congruence_test(test_model, nboot = 50, seed = 123, alpha = 0.05)

  expected_cols <- c("Original Est.", "Diff", "Bootstrap SD", "T Stat.", "2.5% CI", "97.5% CI")
  expect_equal(colnames(result$results), expected_cols)
})

test_that("congruence_test row names contain construct pairs", {
  result <- congruence_test(test_model, nboot = 50, seed = 123)

  # Row names should contain " -> " pattern
  expect_true(all(grepl(" -> ", rownames(result$results))))
})

# ============================================================================
# Reproducibility Tests
# ============================================================================

test_that("congruence_test is reproducible with same seed", {
  result1 <- congruence_test(test_model, nboot = 50, seed = 42)
  result2 <- congruence_test(test_model, nboot = 50, seed = 42)

  expect_equal(result1$results, result2$results)
})

test_that("congruence_test differs with different seeds", {
  result1 <- congruence_test(test_model, nboot = 50, seed = 42)
  result2 <- congruence_test(test_model, nboot = 50, seed = 99)

  # Bootstrap SD and CI columns should differ
  expect_false(identical(result1$results[, "Bootstrap SD"], result2$results[, "Bootstrap SD"]))
})

# ============================================================================
# Parameter Variation Tests
# ============================================================================

test_that("congruence_test respects alpha parameter for CI columns", {
  result_05 <- congruence_test(test_model, nboot = 50, seed = 123, alpha = 0.05)
  result_10 <- congruence_test(test_model, nboot = 50, seed = 123, alpha = 0.10)

  # Column names should reflect alpha

  expect_true("2.5% CI" %in% colnames(result_05$results))
  expect_true("97.5% CI" %in% colnames(result_05$results))
  expect_true("5% CI" %in% colnames(result_10$results))
  expect_true("95% CI" %in% colnames(result_10$results))
})

test_that("congruence_test works with different nboot values", {
  result_small <- congruence_test(test_model, nboot = 20, seed = 123)
  result_large <- congruence_test(test_model, nboot = 100, seed = 123)

  # Both should return valid results

  expect_true(all(is.finite(result_small$results[, "Original Est."])))
  expect_true(all(is.finite(result_large$results[, "Original Est."])))

  # Original estimates should be identical (same model)
  expect_equal(result_small$results[, "Original Est."], result_large$results[, "Original Est."])
})

test_that("congruence_test works with custom threshold", {
  result <- congruence_test(test_model, nboot = 50, seed = 123, threshold = 0.9)

  # Diff column should reflect threshold - original
  # Diff = threshold - abs(original)
  expect_true(all(is.finite(result$results[, "Diff"])))
})

# ============================================================================
# Input Validation Tests
# ============================================================================

test_that("congruence_test rejects non-seminr model objects", {
  expect_warning(
    result <- congruence_test(list(not = "a_model"), nboot = 50),
    "only works with SEMinR models"
  )
  expect_null(result)
})

test_that("congruence_test rejects NULL input", {
  expect_warning(
    result <- congruence_test(NULL, nboot = 50),
    "only works with SEMinR models"
  )
  expect_null(result)
})

test_that("congruence_test rejects data frame input", {
  expect_warning(
    result <- congruence_test(data.frame(x = 1:10), nboot = 50),
    "only works with SEMinR models"
  )
  expect_null(result)
})

# ============================================================================
# Output Value Tests
# ============================================================================

test_that("congruence_test original estimates are bounded", {
  result <- congruence_test(test_model, nboot = 50, seed = 123)

  # Congruence coefficients should be between -1 and 1
  expect_true(all(result$results[, "Original Est."] >= -1))
  expect_true(all(result$results[, "Original Est."] <= 1))
})

test_that("congruence_test bootstrap SD is positive", {
  result <- congruence_test(test_model, nboot = 50, seed = 123)

  expect_true(all(result$results[, "Bootstrap SD"] >= 0))
})

test_that("congruence_test confidence intervals are ordered correctly", {
  result <- congruence_test(test_model, nboot = 100, seed = 123, alpha = 0.05)

  lower_ci <- result$results[, "2.5% CI"]
  upper_ci <- result$results[, "97.5% CI"]

  # Lower CI should be less than or equal to upper CI
  expect_true(all(lower_ci <= upper_ci))
})

# =============================================================================
# feature_cvpat.R - Cross-Validated Predictive Ability Test (CVPAT)
# =============================================================================
# This file implements CVPAT for PLS-SEM models, as described in:
#
# - Liengaard et al. (2021). "Prediction: coveted, yet forsaken? Introducing a
#   cross-validated predictive ability test in partial least squares path
#   modeling." Decision Sciences, 52(2), 362-392.
#
# - Sharma et al. (2023). "Predictive model assessment and selection in
#   composite-based modeling using PLS-SEM: extensions and guidelines for
#   using CVPAT." European Journal of Marketing, 57(6), 1662-1677.
#
# CVPAT compares the predictive loss of PLS models against benchmarks (LM, IA)
# or against each other using bootstrap-based significance testing.
# =============================================================================

#' Compare CVPAT loss between two PLS models
#'
#' `assess_cvpat_compare` conducts a CVPAT significance test of loss between
#' two models. Use this to determine which of two competing models has
#' superior predictive performance.
#'
#' @param established_model The base seminr model for CVPAT comparison.
#' @param alternative_model The alternate seminr model for CVPAT comparison.
#' @param testtype Either "two.sided" (default) or "greater".
#' @param nboot The number of bootstrap subsamples to execute (defaults to 2000).
#' @param seed The seed for reproducibility (defaults to 123).
#' @param technique predict_EA or predict_DA (default).
#' @param noFolds Number of folds for k-fold cross validation.
#' @param reps Number of repetitions for cross validation.
#' @param cores Number of cores for parallelization.
#'
#' @return A matrix of the estimated loss and results of significance testing.
#'
#' @seealso [assess_cvpat()] for single model assessment against benchmarks
#'
#' @references Sharma, P. N., Liengaard, B. D., Hair, J. F., Sarstedt, M., &
#' Ringle, C. M. (2023). Predictive model assessment and selection in
#' composite-based modeling using PLS-SEM: extensions and guidelines for
#' using CVPAT. European journal of marketing, 57(6), 1662-1677.
#'
#' Liengaard, B. D., Sharma, P. N., Hult, G. T. M., Jensen, M. B.,
#' Sarstedt, M., Hair, J. F., & Ringle, C. M. (2021). Prediction: coveted,
#' yet forsaken? Introducing a cross‐validated predictive ability test in
#' partial least squares path modeling. Decision Sciences, 52(2), 362-392.
#'
#' @examples
#' # Load libraries
#' library(seminr)
#' library(seminrExtras)
#'
#' # Create measurement model ----
#' corp_rep_mm_ext <- constructs(
#'  composite("QUAL", multi_items("qual_", 1:8), weights = mode_B),
#'  composite("PERF", multi_items("perf_", 1:5), weights = mode_B),
#'  composite("CSOR", multi_items("csor_", 1:5), weights = mode_B),
#'  composite("ATTR", multi_items("attr_", 1:3), weights = mode_B),
#'  composite("COMP", multi_items("comp_", 1:3)),
#'  composite("LIKE", multi_items("like_", 1:3))
#' )
#'
#' alt_mm <- constructs(
#'  composite("QUAL", multi_items("qual_", 1:8), weights = mode_B),
#'  composite("PERF", multi_items("perf_", 1:5), weights = mode_B),
#'  composite("CSOR", multi_items("csor_", 1:5), weights = mode_B),
#'  composite("COMP", multi_items("comp_", 1:3)),
#'  composite("LIKE", multi_items("like_", 1:3))
#' )
#'
#' # Create structural model ----
#'
#' corp_rep_sm_ext <- relationships(
#'  paths(from = c("QUAL", "PERF", "CSOR", "ATTR"), to = c("COMP", "LIKE"))
#' )
#' alt_sm <- relationships(
#'  paths(from = c("QUAL", "PERF", "CSOR"), to = c("COMP", "LIKE"))
#' )
#'
#' # Estimate the model ----
#' established_model <- estimate_pls(
#'  data = corp_rep_data,
#'  measurement_model = corp_rep_mm_ext,
#'  structural_model  = corp_rep_sm_ext,
#'  missing = mean_replacement,
#'  missing_value = "-99")
#'
#' alternative_model <- estimate_pls(
#'  data = corp_rep_data,
#'  measurement_model = alt_mm,
#'  structural_model  = alt_sm,
#'  missing = mean_replacement,
#'  missing_value = "-99")
#'
#' # Function to compare the Loss of two models
#' assess_cvpat_compare(established_model,
#'                     alternative_model ,
#'                     testtype = "two.sided",
#'                     nboot = 20,
#'                     seed = 123,
#'                     technique = predict_DA,
#'                     noFolds = 5,
#'                     reps = 1,
#'                     cores = 1)
#'
#' @export
assess_cvpat_compare <- function(established_model,
                                 alternative_model,
                                 testtype = "two.sided",
                                 nboot = 2000,
                                 seed = 123,
                                 technique = predict_DA,
                                 noFolds = NULL,
                                 reps = NULL,
                                 cores = NULL) {

  # ---------------------------------------------------------------------------
  # Step 1: Validate inputs
  # ---------------------------------------------------------------------------
  if (!validate_for_prediction(established_model, "assess_cvpat_compare")) {
    return(NULL)
  }
  if (!validate_for_prediction(alternative_model, "assess_cvpat_compare")) {
    return(NULL)
  }

  # Set seed for reproducibility of cross-validation and bootstrap
  set.seed(seed)

  # ---------------------------------------------------------------------------
  # Step 2: Extract endogenous constructs and their measurement items
  # ---------------------------------------------------------------------------
  # Endogenous constructs are those we predict (have incoming paths)
  endo_lvs1 <- get_endogenous_constructs(established_model)
  endo_lvs2 <- get_endogenous_constructs(alternative_model)
  endo_mvs1 <- get_endogenous_items(established_model, endo_lvs1)
  endo_mvs2 <- get_endogenous_items(alternative_model, endo_lvs2)

  # ---------------------------------------------------------------------------
  # Step 3: Validate model compatibility
  # ---------------------------------------------------------------------------
  # CVPAT comparison requires identical endogenous constructs and items
  # (only the structural paths to exogenous constructs can differ)
  same_mvs <- all(endo_mvs1 %in% endo_mvs2) && all(endo_mvs2 %in% endo_mvs1)
  same_lvs <- all(endo_lvs1 %in% endo_lvs2) && all(endo_lvs2 %in% endo_lvs1)

  if (!(same_mvs && same_lvs)) {
    stop("CVPAT can only be applied to models with identical endogenous constructs and measures",
         call. = FALSE)
  }

  # ---------------------------------------------------------------------------
  # Step 4: Generate out-of-sample predictions via k-fold cross-validation
  # ---------------------------------------------------------------------------
  # predict_pls uses k-fold CV to generate unbiased prediction errors
  pls_predict_one <- predict_pls(established_model,
                                 technique = technique,
                                 noFolds = noFolds,
                                 reps = reps,
                                 cores = cores)
  pls_predict_two <- predict_pls(alternative_model,
                                 technique = technique,
                                 noFolds = noFolds,
                                 reps = reps,
                                 cores = cores)

  # ---------------------------------------------------------------------------
  # Step 5: Extract prediction errors (residuals)
  # ---------------------------------------------------------------------------
  # Residuals = actual - predicted; used to calculate loss
  pls_error_one <- as.matrix(pls_predict_one$items$PLS_out_of_sample_residuals)
  colnames(pls_error_one) <- endo_mvs1
  pls_error_two <- as.matrix(pls_predict_two$items$PLS_out_of_sample_residuals)
  colnames(pls_error_two) <- endo_mvs2

  # Ensure correct column ordering
  pls_error_one <- pls_error_one[, endo_mvs1, drop = FALSE]
  pls_error_two <- pls_error_two[, endo_mvs2, drop = FALSE]

  # ---------------------------------------------------------------------------
  # Step 6: Calculate construct-level losses (mean squared error)
  # ---------------------------------------------------------------------------
  # Loss aggregates item-level squared errors to construct level
  lv_losses_one <- calculate_lv_losses(endo_lvs1, established_model, pls_error_one)
  lv_losses_two <- calculate_lv_losses(endo_lvs2, alternative_model, pls_error_two)

  # ---------------------------------------------------------------------------
  # Step 7: Calculate overall model loss
  # ---------------------------------------------------------------------------
  # Overall loss = mean of construct losses (equal weighting)
  pls_overall_one <- overall_loss(lv_losses_one)
  pls_overall_two <- overall_loss(lv_losses_two)

  # ---------------------------------------------------------------------------
  # Step 8: Perform CVPAT comparison based on construct overlap
  # ---------------------------------------------------------------------------
  if (identical(endo_lvs1, endo_lvs2)) {
    # Case A: 100% overlap - direct comparison of all constructs
    pls_v_pls_overall <- bootstrap_cvpat(pls_overall_one, pls_overall_two,
                                         testtype = testtype, nboot = nboot)
    lv_cvpat <- cvpat_per_construct(loss_one = lv_losses_one, loss_two = lv_losses_two,
                                    testtype = testtype, nboot = nboot)
    # Build results matrix: columns = [Loss1, Loss2, Diff, bootstrap stats]
    mat_one <- cbind(colMeans(lv_losses_one), colMeans(lv_losses_two),
                     colMeans(lv_losses_one) - colMeans(lv_losses_two),
                     lv_cvpat[, -1])
  } else {
    # Case B: Partial overlap - compare only shared constructs
    overlap <- intersect(endo_lvs1, endo_lvs2)

    if (length(overlap) == 0) {
      # No shared constructs - cannot compare
      return(list(results = "Cannot compare directly"))
    }

    message("Not all endogenous vars co-occur in models 1 and 2. Only comparing overlap.")

    pls_v_pls_overall <- bootstrap_cvpat(pls_overall_one, pls_overall_two,
                                         testtype = testtype, nboot = nboot)
    lv_cvpat <- cvpat_per_construct(loss_one = lv_losses_one[, overlap, drop = FALSE],
                                    loss_two = lv_losses_two[, overlap, drop = FALSE],
                                    testtype = testtype, nboot = nboot)
    mat_one <- cbind(colMeans(lv_losses_one)[overlap], colMeans(lv_losses_two)[overlap],
                     colMeans(lv_losses_one)[overlap] - colMeans(lv_losses_two)[overlap],
                     lv_cvpat[, -1, drop = FALSE])
  }

  # ---------------------------------------------------------------------------
  # Step 9: Add overall row and format output
  # ---------------------------------------------------------------------------
  mat_one <- rbind(mat_one, unlist(c(mean(pls_overall_one),
                                     mean(pls_overall_two),
                                     mean(pls_overall_one) - mean(pls_overall_two),
                                     pls_v_pls_overall)))

  # Convert to numeric matrix with proper row/column names
  mat_out <- matrix(as.numeric(unlist(mat_one)), nrow = nrow(mat_one))
  rownames(mat_out) <- rownames(mat_one)
  rownames(mat_out)[nrow(mat_one)] <- "Overall"

  # Select final columns: Loss1, Loss2, Diff, Boot T, Boot P
  # (columns 4-5 are Std T and Std P which we exclude)
  mat_out <- mat_out[, c(1, 2, 3, 6, 7)]

  colnames(mat_out) <- c("Base Model Loss", "Alt Model Loss", "Diff",
                         "Boot T value", "Boot P Value")

  # Add metadata for print method
  comment(mat_out) <- "CVPAT as per Sharma, Liengaard, Hair, Sarstedt, & Ringle, (2023).
  Both models under comparison have identical endogenous constructs with identical measurement models.
  Purely exogenous constructs can differ in regards to their relationships with both nomological
  partners and measurement indicators."
  class(mat_out) <- c("table_output", class(mat_out))

  return(mat_out)
}

# =============================================================================
# Single Model CVPAT Assessment
# =============================================================================

#' Assess single model CVPAT against benchmarks
#'
#' `assess_cvpat` conducts a single model CVPAT assessment against item average
#' and linear model prediction benchmarks. Use this to determine whether your
#' PLS model has meaningful predictive power.
#'
#' Two benchmarks are used:
#' - **Linear Model (LM)**: Multiple regression predicting each item
#' - **Indicator Average (IA)**: Naive benchmark using training set means
#'
#' If PLS loss < LM loss (significantly), the structural model adds predictive value.
#' If PLS loss < IA loss (significantly), the model has basic predictive relevance.
#'
#' @param seminr_model The SEMinR model for CVPAT analysis
#' @param testtype Either "two.sided" (default) or "greater".
#' @param nboot The number of bootstrap subsamples to execute (defaults to 2000).
#' @param seed The seed for reproducibility (defaults to 123).
#' @param technique predict_EA or predict_DA (default).
#' @param noFolds Number of folds for k-fold cross validation.
#' @param reps Number of repetitions for cross validation.
#' @param cores Number of cores for parallelization.
#'
#' @return A list containing two matrices: CVPAT_compare_LM (comparison with
#'   linear model) and CVPAT_compare_IA (comparison with indicator average).
#'
#' @seealso [assess_cvpat_compare()] for comparing two PLS models
#'
#' @references Sharma, P. N., Liengaard, B. D., Hair, J. F., Sarstedt, M., &
#' Ringle, C. M. (2023). Predictive model assessment and selection in
#' composite-based modeling using PLS-SEM: extensions and guidelines for
#' using CVPAT. European journal of marketing, 57(6), 1662-1677.
#'
#' Liengaard, B. D., Sharma, P. N., Hult, G. T. M., Jensen, M. B.,
#' Sarstedt, M., Hair, J. F., & Ringle, C. M. (2021). Prediction: coveted,
#' yet forsaken? Introducing a cross‐validated predictive ability test in
#' partial least squares path modeling. Decision Sciences, 52(2), 362-392.
#'
#' @examples
#' # Load libraries
#' library(seminr)
#' library(seminrExtras)
#'
#' # Create measurement model ----
#' corp_rep_mm_ext <- constructs(
#'   composite("QUAL", multi_items("qual_", 1:8), weights = mode_B),
#'   composite("PERF", multi_items("perf_", 1:5), weights = mode_B),
#'   composite("CSOR", multi_items("csor_", 1:5), weights = mode_B),
#'   composite("ATTR", multi_items("attr_", 1:3), weights = mode_B),
#'   composite("COMP", multi_items("comp_", 1:3)),
#'   composite("LIKE", multi_items("like_", 1:3))
#' )
#'
#' # Create structural model ----
#' corp_rep_sm_ext <- relationships(
#'   paths(from = c("QUAL", "PERF", "CSOR", "ATTR"), to = c("COMP", "LIKE"))
#' )
#'
#' # Estimate the model ----
#' corp_rep_pls_model_ext <- estimate_pls(
#'   data = corp_rep_data,
#'   measurement_model = corp_rep_mm_ext,
#'   structural_model  = corp_rep_sm_ext,
#'   missing = mean_replacement,
#'   missing_value = "-99")
#'
#' # Assess the base model ----
#' assess_cvpat(seminr_model = corp_rep_pls_model_ext,
#'              testtype = "two.sided",
#'              nboot = 20,
#'              seed = 123,
#'              technique = predict_DA,
#'              noFolds = 5,
#'              reps = 1,
#'              cores = 1)
#'
#' @export
assess_cvpat <- function(seminr_model,
                         testtype = "two.sided",
                         nboot = 2000,
                         seed = 123,
                         technique = predict_DA,
                         noFolds = NULL,
                         reps = NULL,
                         cores = NULL) {

  # Set seed for reproducibility
  set.seed(seed)

  # ---------------------------------------------------------------------------
  # Step 1: Validate the model
  # ---------------------------------------------------------------------------
  if (!validate_for_prediction(seminr_model, "assess_cvpat")) {
    return(NULL)
  }

  # ---------------------------------------------------------------------------
  # Step 2: Extract endogenous constructs and items
  # ---------------------------------------------------------------------------
  endo_lvs <- get_endogenous_constructs(seminr_model)
  endo_mvs <- get_endogenous_items(seminr_model, endo_lvs)

  # ---------------------------------------------------------------------------
  # Step 3: Calculate Indicator Average (IA) benchmark errors
  # ---------------------------------------------------------------------------
  # IA benchmark: predict each item using its training set mean
  # This is the simplest possible prediction - no model structure used
  ia_means <- seminr_model$meanData[endo_mvs, drop = FALSE]

  # IA error = actual - mean (for each item)
  if (length(endo_mvs) > 1) {
    # sweep subtracts ia_means from each row across columns
    ia_pred_error <- sweep(seminr_model$data[, endo_mvs, drop = FALSE], 2, ia_means)
  } else {
    # Single item case
    ia_pred_error <- seminr_model$data[, endo_mvs, drop = FALSE] - ia_means
  }

  # ---------------------------------------------------------------------------
  # Step 4: Generate PLS and LM predictions via cross-validation
  # ---------------------------------------------------------------------------
  # predict_pls returns both PLS predictions and LM (linear model) predictions
  # LM benchmark: multiple regression predicting each item from other items
  pls_predict <- suppressWarnings(predict_pls(model = seminr_model,
                                              technique = technique,
                                              noFolds = noFolds,
                                              reps = reps,
                                              cores = cores))

  # Extract prediction errors (residuals)
  pls_error <- as.matrix(pls_predict$items$PLS_out_of_sample_residuals)
  colnames(pls_error) <- endo_mvs
  lm_error <- as.matrix(pls_predict$items$lm_out_of_sample_residuals)
  colnames(lm_error) <- endo_mvs

  # Ensure correct column ordering
  pls_error <- pls_error[, endo_mvs, drop = FALSE]
  lm_error <- lm_error[, endo_mvs, drop = FALSE]

  # ---------------------------------------------------------------------------
  # Step 5: Calculate construct-level losses for all three methods
  # ---------------------------------------------------------------------------
  lv_losses_ia <- calculate_lv_losses(endo_lvs, seminr_model, ia_pred_error)
  lv_losses_lm <- calculate_lv_losses(endo_lvs, seminr_model, lm_error)
  lv_losses_pls <- calculate_lv_losses(endo_lvs, seminr_model, pls_error)

  # ---------------------------------------------------------------------------
  # Step 6: Calculate overall model losses
  # ---------------------------------------------------------------------------
  ia_overall <- overall_loss(lv_losses_ia)
  lm_overall <- overall_loss(lv_losses_lm)
  pls_overall <- overall_loss(lv_losses_pls)

  # ---------------------------------------------------------------------------
  # Step 7: Bootstrap significance tests (PLS vs each benchmark)
  # ---------------------------------------------------------------------------
  # Test 1: PLS vs IA (does model beat naive mean prediction?)
  pls_v_ia_overall <- bootstrap_cvpat(pls_overall, ia_overall,
                                      testtype = testtype, nboot = nboot)
  # Test 2: PLS vs LM (does structural model add value over regression?)
  pls_v_lm_overall <- bootstrap_cvpat(pls_overall, lm_overall,
                                      testtype = testtype, nboot = nboot)

  # Per-construct bootstrap tests
  ia_cvpat <- cvpat_per_construct(loss_one = lv_losses_pls, loss_two = lv_losses_ia,
                                  testtype = testtype, nboot = nboot)
  lm_cvpat <- cvpat_per_construct(loss_one = lv_losses_pls, loss_two = lv_losses_lm,
                                  testtype = testtype, nboot = nboot)

  # ---------------------------------------------------------------------------
  # Step 8: Build output matrix for LM comparison
  # ---------------------------------------------------------------------------
  mat_lm <- cbind(colMeans(lv_losses_pls), colMeans(lv_losses_lm),
                  colMeans(lv_losses_pls) - colMeans(lv_losses_lm),
                  lm_cvpat[, -1, drop = FALSE])
  colnames(mat_lm)[1:3] <- c("PLS Loss", "LM Loss", "Diff")

  # Add overall row
  mat_lm <- rbind(mat_lm, unlist(c(mean(pls_overall), mean(lm_overall),
                                   mean(pls_overall) - mean(lm_overall),
                                   pls_v_lm_overall)))
  mat_lm <- apply(mat_lm, 2, as.numeric)
  rownames(mat_lm) <- c(endo_lvs, "Overall")

  # Select columns: Loss1, Loss2, Diff, Boot T, Boot P
  mat_lm <- mat_lm[, c(1, 2, 3, 6, 7)]

  # ---------------------------------------------------------------------------
  # Step 9: Build output matrix for IA comparison
  # ---------------------------------------------------------------------------
  mat_ia <- cbind(colMeans(lv_losses_pls), colMeans(lv_losses_ia),
                  colMeans(lv_losses_pls) - colMeans(lv_losses_ia),
                  ia_cvpat[, -1, drop = FALSE])
  colnames(mat_ia)[1:3] <- c("PLS Loss", "IA Loss", "Diff")

  # Add overall row
  mat_ia <- rbind(mat_ia, unlist(c(mean(pls_overall), mean(ia_overall),
                                   mean(pls_overall) - mean(ia_overall),
                                   pls_v_ia_overall)))
  mat_ia <- apply(mat_ia, 2, as.numeric)
  rownames(mat_ia) <- c(endo_lvs, "Overall")

  # Select columns: Loss1, Loss2, Diff, Boot T, Boot P
  mat_ia <- mat_ia[, c(1, 2, 3, 6, 7)]

  # ---------------------------------------------------------------------------
  # Step 10: Add metadata and return
  # ---------------------------------------------------------------------------
  comment(mat_lm) <- comment(mat_ia) <- "CVPAT as per Sharma et al. (2023)."
  class(mat_lm) <- class(mat_ia) <- c("table_output", class(mat_lm))

  return(list(CVPAT_compare_LM = mat_lm,
              CVPAT_compare_IA = mat_ia))
}

library(seminr)
set.seed(123)

# Heavy CV/bootstrap setup — only run off-CRAN. Each assess_cvpat* call below
# runs a 10-fold × 10-rep CV (100 model fits) plus bootstrap; on CRAN we skip.
if (identical(Sys.getenv("NOT_CRAN"), "true")) {

  corp_rep <- corp_rep_data

  # Create measurement model ----
  corp_rep_mm <- constructs(
    composite("COMP", multi_items("comp_", 1:3)),
    composite("LIKE", multi_items("like_", 1:3)),
    composite("CUSA", single_item("cusa")),
    composite("CUSL", multi_items("cusl_", 1:3))
  )

  # Create structural model ----
  sm_one <- relationships(
    paths(from = c("COMP", "LIKE"), to = c("CUSA")),
    paths(from = c("CUSA"), to = c("CUSL")))

  sm_two <- relationships(
    paths(from = c("COMP", "LIKE"), to = c("CUSA", "CUSL")),
    paths(from = c("CUSA"), to = c("CUSL")))


  model_one <- estimate_pls(
    data = corp_rep,
    measurement_model = corp_rep_mm,
    structural_model  = sm_one,
    missing = mean_replacement,
    missing_value = "-99")

  model_two<- estimate_pls(
    data = corp_rep,
    measurement_model = corp_rep_mm,
    structural_model  = sm_two,
    missing = mean_replacement,
    missing_value = "-99")

  # Function to compare the Loss of two models
  Results1 <- assess_cvpat_compare(established_model = model_one,
                                   alternative_model = model_two,
                                   testtype = "two.sided",
                                   nboot = 200,
                                   technique = predict_EA,
                                   noFolds = NULL,
                                   reps = NULL,
                                   cores = 1)

  # Assess the base model ----
  Results2 <- as.data.frame(assess_cvpat(model_one,
                                         technique = predict_EA,
                                         noFolds = NULL,
                                         reps = NULL,
                                         cores = 1))


  ## Output originally created using following lines
  # write.csv(cbind(Results1,Results2), file = "tests/fixtures/cvpat1.csv")

  # Load controls
  cvpat_control <- as.matrix(read.csv(file = paste(test_folder,"cvpat1.csv", sep = ""), row.names = 1))
}

# Testing
test_that("CVPAT compares models correctly\n", {
  skip_on_cran()
  expect_equal(as.numeric(unlist(Results1)), as.numeric(cvpat_control[,1:5]), tolerance = 0.1)
})

test_that("CVPAT assesses model correctly\n", {
  skip_on_cran()
  expect_equal(as.numeric(unlist(Results2)), as.numeric(cvpat_control[,6:15]), tolerance = 0.1)
})

# Now check that CVPAT can generalize ----
# Higher Composite ----
if (identical(Sys.getenv("NOT_CRAN"), "true")) {

  corp_rep_mm <- constructs(
    composite("COMP", multi_items("comp_", 1:3)),
    composite("LIKE", multi_items("like_", 1:3)),
    higher_composite("COKE",c("COMP", "LIKE"),two_stage ),
    composite("CUSA", single_item("cusa")),
    composite("CUSL", multi_items("cusl_", 1:3))
  )

  # Create structural model ----
  sm_one <- relationships(
    paths(from = c("COKE"), to = c("CUSA")),
    paths(from = c("CUSA"), to = c("CUSL")))

  sm_two <- relationships(
    paths(from = c("COMP", "LIKE"), to = c("CUSA", "CUSL")),
    paths(from = c("CUSA"), to = c("CUSL")))


  model_one <- estimate_pls(
    data = corp_rep_data,
    measurement_model = corp_rep_mm,
    structural_model  = sm_one,
    missing = mean_replacement,
    missing_value = "-99")

  model_two <- estimate_pls(
    data = corp_rep_data,
    measurement_model = corp_rep_mm,
    structural_model  = sm_two,
    missing = mean_replacement,
    missing_value = "-99")

  # Function to compare the Loss of two models
  Results1 <- assess_cvpat_compare(established_model = model_one,
                                   alternative_model = model_two,
                                   testtype = "two.sided",
                                   nboot = 200,
                                   technique = predict_EA,
                                   noFolds = NULL,
                                   reps = NULL,
                                   cores = 1)

  # Assess the base model ----
  Results2 <- assess_cvpat(model_one,
                           technique = predict_EA,
                           noFolds = NULL,
                           reps = NULL,
                           cores = 1)
}

test_that("CVPAT_compare can handle higher_composite\n", {
  skip_on_cran()
  expect_null(Results1)
})

test_that("CVPAT_assess can handle higher_composite\n", {
  skip_on_cran()
  expect_null(Results2)
})

library(seminr)

# ============================================================================
# Setup: Create models and pre-compute results for testing
# ============================================================================
# Heavy CV/bootstrap setup — only run off-CRAN. Each assess_cvpat* call
# performs 10-fold × 10-rep CV (100 model fits) plus bootstrap.

set.seed(123)

if (identical(Sys.getenv("NOT_CRAN"), "true")) {

  corp_rep_mm <- constructs(
    composite("COMP", multi_items("comp_", 1:3)),
    composite("LIKE", multi_items("like_", 1:3)),
    composite("CUSA", single_item("cusa")),
    composite("CUSL", multi_items("cusl_", 1:3))
  )

  sm_one <- relationships(
    paths(from = c("COMP", "LIKE"), to = c("CUSA")),
    paths(from = c("CUSA"), to = c("CUSL"))
  )

  sm_two <- relationships(
    paths(from = c("COMP", "LIKE"), to = c("CUSA", "CUSL")),
    paths(from = c("CUSA"), to = c("CUSL"))
  )

  model_one <- estimate_pls(
    data = corp_rep_data,
    measurement_model = corp_rep_mm,
    structural_model  = sm_one,
    missing = mean_replacement,
    missing_value = "-99"
  )

  model_two <- estimate_pls(
    data = corp_rep_data,
    measurement_model = corp_rep_mm,
    structural_model  = sm_two,
    missing = mean_replacement,
    missing_value = "-99"
  )

  # Pre-compute assess_cvpat once (expensive: runs k-fold CV + bootstrap)
  cvpat_result <- assess_cvpat(model_one, nboot = 50, seed = 123, cores = 1)

  # Pre-compute assess_cvpat_compare once
  compare_result <- assess_cvpat_compare(
    established_model = model_one,
    alternative_model = model_two,
    nboot = 50, seed = 123, cores = 1
  )
}

# ============================================================================
# assess_cvpat - Output Structure Tests
# ============================================================================

test_that("assess_cvpat returns list with two elements", {
  skip_on_cran()
  expect_type(cvpat_result, "list")
  expect_named(cvpat_result, c("CVPAT_compare_LM", "CVPAT_compare_IA"))
})

test_that("assess_cvpat returns matrices with table_output class", {
  skip_on_cran()
  expect_true(inherits(cvpat_result$CVPAT_compare_LM, "matrix"))
  expect_true(inherits(cvpat_result$CVPAT_compare_IA, "matrix"))
  expect_true("table_output" %in% class(cvpat_result$CVPAT_compare_LM))
  expect_true("table_output" %in% class(cvpat_result$CVPAT_compare_IA))
})

test_that("assess_cvpat returns correct column names", {
  skip_on_cran()
  lm_cols <- c("PLS Loss", "LM Loss", "Diff", "Boot T value", "Boot P Value")
  ia_cols <- c("PLS Loss", "IA Loss", "Diff", "Boot T value", "Boot P Value")

  expect_equal(colnames(cvpat_result$CVPAT_compare_LM), lm_cols)
  expect_equal(colnames(cvpat_result$CVPAT_compare_IA), ia_cols)
})

test_that("assess_cvpat returns correct row names with Overall", {
  skip_on_cran()
  expect_true("Overall" %in% rownames(cvpat_result$CVPAT_compare_LM))
  expect_true("Overall" %in% rownames(cvpat_result$CVPAT_compare_IA))
  expect_true("CUSA" %in% rownames(cvpat_result$CVPAT_compare_LM))
  expect_true("CUSL" %in% rownames(cvpat_result$CVPAT_compare_LM))
})

test_that("assess_cvpat has comment attribute", {
  skip_on_cran()
  expect_false(is.null(comment(cvpat_result$CVPAT_compare_LM)))
  expect_false(is.null(comment(cvpat_result$CVPAT_compare_IA)))
  expect_true(grepl("CVPAT", comment(cvpat_result$CVPAT_compare_LM)))
})

# ============================================================================
# assess_cvpat - Reproducibility Tests
# ============================================================================

test_that("assess_cvpat is reproducible with same seed", {
  skip_on_cran()
  result2 <- assess_cvpat(model_one, nboot = 50, seed = 123, cores = 1)

  expect_equal(as.numeric(cvpat_result$CVPAT_compare_LM),
               as.numeric(result2$CVPAT_compare_LM))
  expect_equal(as.numeric(cvpat_result$CVPAT_compare_IA),
               as.numeric(result2$CVPAT_compare_IA))
})

test_that("assess_cvpat differs with different seeds", {
  skip_on_cran()
  result2 <- assess_cvpat(model_one, nboot = 50, seed = 99, cores = 1)

  expect_false(identical(
    cvpat_result$CVPAT_compare_LM[, "Boot T value"],
    result2$CVPAT_compare_LM[, "Boot T value"]
  ))
})

# ============================================================================
# assess_cvpat - Input Validation Tests
# ============================================================================

test_that("assess_cvpat rejects non-seminr model objects", {
  skip_on_cran()
  expect_warning(
    result <- assess_cvpat(list(not = "a_model"), nboot = 50, cores = 1),
    "only works with SEMinR models"
  )
  expect_null(result)
})

test_that("assess_cvpat rejects data frame input", {
  skip_on_cran()
  expect_warning(
    result <- assess_cvpat(data.frame(x = 1:10), nboot = 50, cores = 1),
    "only works with SEMinR models"
  )
  expect_null(result)
})

# ============================================================================
# assess_cvpat - Output Value Tests
# ============================================================================

test_that("assess_cvpat loss values are positive", {
  skip_on_cran()
  expect_true(all(cvpat_result$CVPAT_compare_LM[, "PLS Loss"] >= 0))
  expect_true(all(cvpat_result$CVPAT_compare_LM[, "LM Loss"] >= 0))
  expect_true(all(cvpat_result$CVPAT_compare_IA[, "PLS Loss"] >= 0))
  expect_true(all(cvpat_result$CVPAT_compare_IA[, "IA Loss"] >= 0))
})

test_that("assess_cvpat p-values are between 0 and 1", {
  skip_on_cran()
  expect_true(all(cvpat_result$CVPAT_compare_LM[, "Boot P Value"] >= 0))
  expect_true(all(cvpat_result$CVPAT_compare_LM[, "Boot P Value"] <= 1))
  expect_true(all(cvpat_result$CVPAT_compare_IA[, "Boot P Value"] >= 0))
  expect_true(all(cvpat_result$CVPAT_compare_IA[, "Boot P Value"] <= 1))
})

test_that("assess_cvpat Diff has correct sign relationship to losses", {
  skip_on_cran()
  lm_diff <- cvpat_result$CVPAT_compare_LM[, "PLS Loss"] -
             cvpat_result$CVPAT_compare_LM[, "LM Loss"]
  expect_equal(as.numeric(cvpat_result$CVPAT_compare_LM[, "Diff"]),
               as.numeric(lm_diff), tolerance = 0.01)
})

# ============================================================================
# assess_cvpat_compare - Output Structure Tests
# ============================================================================

test_that("assess_cvpat_compare returns matrix with table_output class", {
  skip_on_cran()
  expect_true(inherits(compare_result, "matrix"))
  expect_true("table_output" %in% class(compare_result))
})

test_that("assess_cvpat_compare returns correct column names", {
  skip_on_cran()
  expected_cols <- c("Base Model Loss", "Alt Model Loss", "Diff",
                     "Boot T value", "Boot P Value")
  expect_equal(colnames(compare_result), expected_cols)
})

test_that("assess_cvpat_compare returns correct row names with Overall", {
  skip_on_cran()
  expect_true("Overall" %in% rownames(compare_result))
})

test_that("assess_cvpat_compare has comment attribute", {
  skip_on_cran()
  expect_false(is.null(comment(compare_result)))
  expect_true(grepl("CVPAT", comment(compare_result)))
})

# ============================================================================
# assess_cvpat_compare - Reproducibility Tests
# ============================================================================

test_that("assess_cvpat_compare is reproducible with same seed", {
  skip_on_cran()
  result2 <- assess_cvpat_compare(model_one, model_two,
                                   nboot = 50, seed = 123, cores = 1)
  expect_equal(as.numeric(compare_result), as.numeric(result2))
})

test_that("assess_cvpat_compare differs with different seeds", {
  skip_on_cran()
  result2 <- assess_cvpat_compare(model_one, model_two,
                                   nboot = 50, seed = 99, cores = 1)
  expect_false(identical(compare_result[, "Boot T value"],
                         result2[, "Boot T value"]))
})

# ============================================================================
# assess_cvpat_compare - Output Value Tests
# ============================================================================

test_that("assess_cvpat_compare loss values are positive", {
  skip_on_cran()
  expect_true(all(compare_result[, "Base Model Loss"] >= 0))
  expect_true(all(compare_result[, "Alt Model Loss"] >= 0))
})

test_that("assess_cvpat_compare p-values are between 0 and 1", {
  skip_on_cran()
  expect_true(all(compare_result[, "Boot P Value"] >= 0))
  expect_true(all(compare_result[, "Boot P Value"] <= 1))
})

test_that("assess_cvpat_compare Diff equals Base minus Alt Loss", {
  skip_on_cran()
  expected_diff <- compare_result[, "Base Model Loss"] -
                   compare_result[, "Alt Model Loss"]
  expect_equal(compare_result[, "Diff"], expected_diff, tolerance = 1e-10)
})

# ============================================================================
# assess_cvpat_compare - Model Validation Tests
# ============================================================================

test_that("assess_cvpat_compare stops with mismatched endogenous constructs", {
  skip_on_cran()
  mm_diff <- constructs(
    composite("COMP", multi_items("comp_", 1:3)),
    composite("LIKE", multi_items("like_", 1:3)),
    composite("CUSA", single_item("cusa"))
  )
  sm_diff <- relationships(
    paths(from = c("COMP", "LIKE"), to = c("CUSA"))
  )
  model_diff <- estimate_pls(
    data = corp_rep_data,
    measurement_model = mm_diff,
    structural_model  = sm_diff,
    missing = mean_replacement,
    missing_value = "-99"
  )

  expect_error(
    assess_cvpat_compare(model_one, model_diff, nboot = 50, cores = 1),
    "identical endogenous"
  )
})

# ============================================================================
# Prediction Technique and Testtype Tests (compute fresh, minimal reps)
# ============================================================================

test_that("assess_cvpat works with predict_EA technique", {
  skip_on_cran()
  result <- assess_cvpat(model_one, technique = predict_EA,
                          nboot = 20, seed = 123, cores = 1)
  expect_type(result, "list")
  expect_true(all(is.finite(as.numeric(result$CVPAT_compare_LM))))
})

test_that("assess_cvpat_compare works with predict_EA technique", {
  skip_on_cran()
  result <- assess_cvpat_compare(model_one, model_two,
                                  technique = predict_EA,
                                  nboot = 20, seed = 123, cores = 1)
  expect_true(inherits(result, "matrix"))
  expect_true(all(is.finite(as.numeric(result))))
})

test_that("assess_cvpat works with testtype greater", {
  skip_on_cran()
  result <- assess_cvpat(model_one, testtype = "greater",
                          nboot = 20, seed = 123, cores = 1)
  expect_type(result, "list")
  expect_true(all(is.finite(as.numeric(result$CVPAT_compare_LM))))
})

test_that("assess_cvpat_compare works with testtype greater", {
  skip_on_cran()
  result <- assess_cvpat_compare(model_one, model_two,
                                  testtype = "greater",
                                  nboot = 20, seed = 123, cores = 1)
  expect_true(inherits(result, "matrix"))
  expect_true(all(is.finite(as.numeric(result))))
})

# =============================================================================
# feature_nca.R - Necessary Condition Analysis (NCA) for PLS-SEM
# =============================================================================
# Implements CE-FDH and CR-FDH ceiling analysis internally.
#
# References:
# - Dul, J. (2016). "Necessary Condition Analysis (NCA): Logic and
#   Methodology of 'Necessary but Not Sufficient' Causality."
#   Organizational Research Methods, 19(1), 10-52.
# - Richter, N. F., Schubring, S., Hauff, S., Ringle, C. M., &
#   Sarstedt, M. (2020). "When predictors of outcomes are necessary:
#   guidelines for the combined use of PLS-SEM and NCA."
#   Industrial Management & Data Systems, 120(12), 2243-2267.
# =============================================================================

# =============================================================================
# INTERNAL NCA ALGORITHMS
# =============================================================================
# Self-contained CE-FDH and CR-FDH implementations. These cover the two
# most commonly used ceiling techniques in PLS-SEM NCA applications.

INTERNAL_CEILINGS <- c("ce_fdh", "cr_fdh")

#' Compute CE-FDH (Ceiling Envelopment - Free Disposal Hull) ceiling.
#' Returns sorted unique x values and their non-decreasing ceiling y values.
#' The ceiling at any x is the maximum y among all observations with x_obs <= x.
#' @noRd
compute_ce_fdh <- function(x, y) {
  ux <- sort(unique(x))
  max_y_at_x <- vapply(ux, function(xi) max(y[x == xi]), numeric(1))
  list(x = ux, ceiling_y = cummax(max_y_at_x))
}

#' Get CE-FDH peer coordinates (points defining the ceiling envelope).
#' A peer is a unique-x level whose max y actually pushes the ceiling
#' upward (exceeds the previous cummax). Points on flat parts of the
#' step function are excluded.
#' @noRd
get_ce_fdh_peers <- function(x, y) {
  ux <- sort(unique(x))
  max_y_at_x <- vapply(ux, function(xi) max(y[x == xi]), numeric(1))
  cy <- cummax(max_y_at_x)
  # A peer must set a NEW cummax: either first point, or exceeds previous cummax
  is_peer <- c(TRUE, max_y_at_x[-1] > cy[-length(cy)])
  data.frame(x = ux[is_peer], y = cy[is_peer])
}

#' Compute CE-FDH effect size d = ceiling_zone / scope.
#' The ceiling zone is the empty area above the step-function ceiling
#' within the scope rectangle [min(x),max(x)] x [min(y),max(y)].
#' @noRd
ce_fdh_effect_size <- function(x, y) {
  x_range <- range(x)
  y_range <- range(y)
  scope <- diff(x_range) * diff(y_range)
  if (scope < .Machine$double.eps) return(0)

  ceil <- compute_ce_fdh(x, y)
  ux <- ceil$x
  cy <- ceil$ceiling_y
  k <- length(ux)
  if (k < 2) return(0)

  # For each interval [ux[i], ux[i+1]), the ceiling is at cy[i].
  # Area above = width * (max_y - cy[i]).
  ceiling_zone <- sum(diff(ux) * (y_range[2] - cy[-k]))
  ceiling_zone / scope
}

#' Compute area above a line y = a + b*x within a rectangle.
#' Handles all cases: positive/negative/zero slope, line partially
#' or fully outside the scope.
#' @noRd
line_ceiling_zone <- function(a, b, x_min, x_max, y_min, y_max) {
  scope_height <- y_max - y_min

  if (abs(b) < .Machine$double.eps) {
    if (a >= y_max) return(0)
    if (a <= y_min) return((x_max - x_min) * scope_height)
    return((x_max - x_min) * (y_max - a))
  }

  # Break points where the line crosses y_min or y_max
  x_cross <- c((y_min - a) / b, (y_max - a) / b)
  inner <- x_cross[x_cross > x_min & x_cross < x_max]
  breaks <- sort(unique(c(x_min, inner, x_max)))

  total <- 0
  for (i in seq_len(length(breaks) - 1)) {
    xl <- breaks[i]
    xr <- breaks[i + 1]
    line_at_mid <- a + b * (xl + xr) / 2

    if (line_at_mid >= y_max) {
      # Line above scope top: no ceiling zone
    } else if (line_at_mid <= y_min) {
      # Line below scope bottom: full ceiling zone
      total <- total + (xr - xl) * scope_height
    } else {
      # Line within scope: integrate y_max - (a + bx)
      total <- total + (y_max - a) * (xr - xl) - b / 2 * (xr^2 - xl^2)
    }
  }

  total
}

#' Compute CR-FDH (Ceiling Regression - Free Disposal Hull) effect size.
#' Fits OLS through CE-FDH peers; falls back to CE-FDH if < 2 peers.
#' @noRd
cr_fdh_effect_size <- function(x, y) {
  x_range <- range(x)
  y_range <- range(y)
  scope <- diff(x_range) * diff(y_range)
  if (scope < .Machine$double.eps) return(0)

  peers <- get_ce_fdh_peers(x, y)
  if (nrow(peers) < 2) return(ce_fdh_effect_size(x, y))

  cf <- coef(lm(y ~ x, data = peers))
  a <- unname(cf[1])
  b <- unname(cf[2])

  ceiling_zone <- line_ceiling_zone(a, b, x_range[1], x_range[2],
                                     y_range[1], y_range[2])
  max(0, min(ceiling_zone / scope, 1))
}

#' Dispatch to the correct effect size function.
#' @noRd
nca_effect_size <- function(x, y, ceiling_type) {
  switch(ceiling_type,
    ce_fdh = ce_fdh_effect_size(x, y),
    cr_fdh = cr_fdh_effect_size(x, y),
    stop("Supported ceiling techniques: ", paste(INTERNAL_CEILINGS, collapse = ", "),
         ". Got '", ceiling_type, "'.", call. = FALSE)
  )
}

#' Permutation test for NCA effect size significance.
#' Returns p-value as proportion of permuted d >= observed d.
#' @noRd
nca_permutation_test <- function(x, y, ceiling_type, observed_d, n_perm) {
  d_perm <- vapply(seq_len(n_perm), function(i) {
    nca_effect_size(x, sample(y), ceiling_type)
  }, numeric(1))
  mean(d_perm >= observed_d)
}

#' Compute bottleneck column for one predictor: required X percentage
#' at each Y level. NA indicates the condition is not necessary.
#' @noRd
compute_bottleneck_column <- function(x, y, ceiling_type, steps) {
  x_range <- range(x)
  y_range <- range(y)
  y_levels <- seq(0, 100, length.out = steps + 1)
  y_targets <- y_range[1] + y_levels / 100 * diff(y_range)

  if (diff(x_range) < .Machine$double.eps) {
    return(rep(NA_real_, length(y_levels)))
  }

  if (ceiling_type == "ce_fdh") {
    ceil <- compute_ce_fdh(x, y)
    x_needed <- vapply(y_targets, function(yt) {
      idx <- which(ceil$ceiling_y >= yt)
      if (length(idx) == 0) return(NA_real_)
      ceil$x[idx[1]]
    }, numeric(1))
  } else if (ceiling_type == "cr_fdh") {
    peers <- get_ce_fdh_peers(x, y)
    if (nrow(peers) < 2) {
      return(compute_bottleneck_column(x, y, "ce_fdh", steps))
    }
    cf <- coef(lm(y ~ x, data = peers))
    a <- unname(cf[1])
    b <- unname(cf[2])
    if (abs(b) < .Machine$double.eps) {
      return(rep(NA_real_, length(y_levels)))
    }
    x_needed <- (y_targets - a) / b
  } else {
    return(rep(NA_real_, length(y_levels)))
  }

  x_pct <- (x_needed - x_range[1]) / diff(x_range) * 100
  x_pct[x_pct < 0] <- NA_real_
  x_pct[x_pct > 100] <- NA_real_
  round(x_pct, 1)
}

#' Run complete internal NCA analysis for all predictors and ceilings.
#' @noRd
run_nca_internal <- function(data, x_names, y_name, ceilings, test.rep, steps) {
  y <- data[[y_name]]

  effect_sizes <- matrix(NA_real_, nrow = length(x_names), ncol = length(ceilings),
                          dimnames = list(x_names, ceilings))
  p_values <- if (test.rep > 0) {
    matrix(NA_real_, nrow = length(x_names), ncol = length(ceilings),
           dimnames = list(x_names, ceilings))
  } else NULL

  bottlenecks <- list()

  for (ceil in ceilings) {
    y_levels <- seq(0, 100, length.out = steps + 1)
    bn <- data.frame(Y = y_levels)
    colnames(bn)[1] <- y_name

    for (pred in x_names) {
      x_vec <- data[[pred]]
      d <- nca_effect_size(x_vec, y, ceil)
      effect_sizes[pred, ceil] <- d

      if (test.rep > 0) {
        p_values[pred, ceil] <- nca_permutation_test(x_vec, y, ceil, d, test.rep)
      }

      bn[[pred]] <- compute_bottleneck_column(x_vec, y, ceil, steps)
    }

    bottlenecks[[ceil]] <- bn
  }

  list(effect_sizes = effect_sizes, p_values = p_values, bottlenecks = bottlenecks)
}

# =============================================================================
# PREDICTOR EXTRACTION
# =============================================================================

#' Extract direct predictor construct names for a given target from the
#' structural model matrix. unname() strips matrix row names that would
#' cause matching issues downstream.
#' @noRd
get_direct_predictors <- function(model, target) {
  sm <- model$smMatrix
  unname(sm[sm[, "target"] == target, "source"])
}

# =============================================================================
# INPUT VALIDATION
# =============================================================================

#' Shared input validation for assess_nca() and assess_nca_esse().
#' Validates test.rep, target, and predictors. Returns the predictor vector
#' (auto-detected or validated). Stops on invalid input.
#' @noRd
validate_nca_inputs <- function(seminr_model, target, predictors, test.rep) {
  if (!is.numeric(test.rep) || length(test.rep) != 1 || test.rep < 0 ||
      test.rep != as.integer(test.rep)) {
    stop("test.rep must be a non-negative integer.", call. = FALSE)
  }

  construct_names <- colnames(seminr_model$construct_scores)
  if (!(target %in% construct_names)) {
    stop("target '", target, "' not found in model constructs: ",
         paste(construct_names, collapse = ", "), call. = FALSE)
  }

  if (is.null(predictors)) {
    predictors <- get_direct_predictors(seminr_model, target)
    if (length(predictors) == 0) {
      stop("No direct predictors found for target '", target,
           "' in the structural model.", call. = FALSE)
    }
  } else {
    invalid <- setdiff(predictors, construct_names)
    if (length(invalid) > 0) {
      stop("Predictor(s) not found in model constructs: ",
           paste(invalid, collapse = ", "), call. = FALSE)
    }
  }

  predictors
}

# =============================================================================
# MAIN ENTRY POINT
# =============================================================================

#' Necessary Condition Analysis (NCA) for PLS-SEM Models
#'
#' `assess_nca` conducts a Necessary Condition Analysis on construct scores
#' from an estimated PLS-SEM model. It identifies whether predictors are
#' necessary conditions for achieving a given level of the target construct.
#'
#' NCA complements PLS-SEM's sufficiency-based regression by testing whether
#' a certain level of a predictor is \emph{necessary} (but not sufficient)
#' for a certain level of the outcome. A predictor is considered a necessary
#' condition when it has an effect size (d) >= 0.1 and a significant p-value
#' (p < 0.05).
#'
#' The function extracts construct scores from the seminr model, auto-detects
#' direct predictors of the target from the structural model, and computes
#' ceiling envelopes using the specified techniques.
#'
#' @param seminr_model An estimated SEMinR model from \code{estimate_pls()}.
#' @param target Name of the endogenous (outcome) construct (character).
#' @param predictors Optional character vector of predictor construct names.
#'   If \code{NULL} (default), auto-detected from the structural model as
#'   direct predictors of \code{target}.
#' @param ceilings Character vector of ceiling techniques to use
#'   (default \code{c("ce_fdh", "cr_fdh")}).
#' @param test.rep Number of permutation test repetitions for significance
#'   testing (default 1000). Set to 0 to skip significance testing.
#' @param steps Number of steps in the bottleneck table (default 10).
#' @param seed Random seed for reproducibility (default 123).
#' @param ... Additional arguments (currently unused).
#'
#' @return An object of class \code{nca_analysis} containing:
#'   \item{nca_raw}{Reserved (always NULL)}
#'   \item{effect_sizes}{Matrix of effect sizes (d) per predictor and ceiling}
#'   \item{significance}{Matrix of p-values per predictor and ceiling (NULL if test.rep = 0)}
#'   \item{bottleneck}{List of bottleneck tables, one per ceiling technique}
#'   \item{pls_model}{The original estimated seminr model}
#'   \item{target}{Name of the target construct}
#'   \item{predictors}{Character vector of predictor names used}
#'   \item{ceilings}{Character vector of ceiling techniques used}
#'
#' @references
#' Dul, J. (2016). Necessary Condition Analysis (NCA): Logic and Methodology
#' of 'Necessary but Not Sufficient' Causality. Organizational Research
#' Methods, 19(1), 10-52.
#'
#' Richter, N. F., Schubring, S., Hauff, S., Ringle, C. M., & Sarstedt, M.
#' (2020). When predictors of outcomes are necessary: guidelines for the
#' combined use of PLS-SEM and NCA. Industrial Management & Data Systems,
#' 120(12), 2243-2267.
#'
#' @examples
#' library(seminr)
#' library(seminrExtras)
#'
#' mobi_mm <- constructs(
#'   composite("Image",        multi_items("IMAG", 1:5)),
#'   composite("Value",        multi_items("PERV", 1:2)),
#'   composite("Satisfaction", multi_items("CUSA", 1:3)),
#'   composite("Loyalty",      multi_items("CUSL", 1:3))
#' )
#'
#' mobi_sm <- relationships(
#'   paths(from = c("Image", "Value"), to = "Satisfaction"),
#'   paths(from = "Satisfaction", to = "Loyalty")
#' )
#'
#' mobi_pls <- estimate_pls(data = mobi,
#'                           measurement_model = mobi_mm,
#'                           structural_model  = mobi_sm)
#'
#' \donttest{
#' nca_result <- assess_nca(mobi_pls,
#'                           target = "Satisfaction",
#'                           test.rep = 100)
#' print(nca_result)
#' summary(nca_result)
#' plot(nca_result, type = "effects")
#' }
#'
#' @export
assess_nca <- function(seminr_model,
                        target,
                        predictors = NULL,
                        ceilings = c("ce_fdh", "cr_fdh"),
                        test.rep = 1000,
                        steps = 10,
                        seed = 123,
                        ...) {

  # ---------------------------------------------------------------------------
  # Step 1: Validate inputs
  # ---------------------------------------------------------------------------
  if (!validate_seminr_model(seminr_model, "assess_nca")) {
    return(NULL)
  }
  predictors <- validate_nca_inputs(seminr_model, target, predictors, test.rep)

  # ---------------------------------------------------------------------------
  # Step 2: Run NCA analysis on construct scores
  # ---------------------------------------------------------------------------
  scores <- as.data.frame(seminr_model$construct_scores)

  set.seed(seed)

  nca_result <- run_nca_internal(scores, predictors, target, ceilings,
                                  test.rep, steps)
  effect_sizes <- nca_result$effect_sizes
  significance <- nca_result$p_values
  bottleneck   <- nca_result$bottlenecks
  nca_raw      <- NULL

  # ---------------------------------------------------------------------------
  # Step 3: Format and identify necessary conditions
  # ---------------------------------------------------------------------------
  comment(effect_sizes) <- "NCA Effect Sizes (d >= 0.1 indicates a necessary condition)"
  class(effect_sizes) <- c("table_output", class(effect_sizes))

  if (!is.null(significance)) {
    comment(significance) <- "NCA Significance (permutation test p-values)"
    class(significance) <- c("table_output", class(significance))
  }

  # Identify necessary conditions: d >= 0.1 AND p < 0.05 (Dul, 2016)
  necessary <- identify_necessary(effect_sizes, significance)
  necessary_predictors <- rownames(effect_sizes)[apply(necessary, 1, any)]

  result <- list(
    nca_raw      = nca_raw,
    effect_sizes = effect_sizes,
    significance = significance,
    bottleneck   = bottleneck,
    necessary_predictors = necessary_predictors,
    pls_model    = seminr_model,
    target       = target,
    predictors   = predictors,
    ceilings     = ceilings
  )

  class(result) <- c("nca_analysis", class(result))
  result
}

# =============================================================================
# NECESSARY CONDITION IDENTIFICATION
# =============================================================================

#' Identify necessary conditions per Dul (2016): a predictor is necessary
#' when effect size d >= 0.1 AND permutation p-value < 0.05.
#' When significance is NULL (no permutation test), uses d threshold only.
#' @noRd
identify_necessary <- function(effect_sizes, significance, d_threshold = 0.1,
                                p_threshold = 0.05) {
  if (is.null(significance)) {
    return(effect_sizes >= d_threshold)
  }
  (effect_sizes >= d_threshold) & (significance < p_threshold)
}

# =============================================================================
# S3 METHODS
# =============================================================================

#' @export
print.nca_analysis <- function(x, ...) {
  cat("Necessary Condition Analysis (NCA)\n")
  cat("==================================\n")
  cat("Target:", x$target, "\n")
  cat("Predictors:", paste(x$predictors, collapse = ", "), "\n")
  cat("Ceilings:", paste(x$ceilings, collapse = ", "), "\n")
  cat("Observations:", nrow(x$pls_model$construct_scores), "\n\n")

  cat("Effect Sizes (d):\n")
  print(round(x$effect_sizes, 4), ...)
  cat("\n")

  if (!is.null(x$significance)) {
    cat("Permutation p-values:\n")
    print(round(x$significance, 4), ...)
    cat("\n")
  }

  if (length(x$necessary_predictors) > 0) {
    cat("Necessary conditions (d >= 0.1, p < 0.05):",
        paste(x$necessary_predictors, collapse = ", "), "\n")
  } else {
    cat("No necessary conditions identified (d >= 0.1, p < 0.05)\n")
  }

  invisible(x)
}

#' @export
summary.nca_analysis <- function(object, ...) {
  result <- list(
    target       = object$target,
    predictors   = object$predictors,
    ceilings     = object$ceilings,
    n_obs        = nrow(object$pls_model$construct_scores),
    effect_sizes = object$effect_sizes,
    significance = object$significance,
    bottleneck   = object$bottleneck,
    necessary_predictors = object$necessary_predictors
  )

  class(result) <- c("summary.nca_analysis", class(result))
  result
}

#' @export
print.summary.nca_analysis <- function(x, ...) {
  cat("Necessary Condition Analysis (NCA) Summary\n")
  cat("============================================\n")
  cat("Target:", x$target, "\n")
  cat("Predictors:", paste(x$predictors, collapse = ", "), "\n")
  cat("Ceilings:", paste(x$ceilings, collapse = ", "), "\n")
  cat("Observations:", x$n_obs, "\n\n")

  cat("Effect Sizes (d):\n")
  print(round(x$effect_sizes, 4), ...)
  cat("\n")

  if (!is.null(x$significance)) {
    cat("Permutation p-values:\n")
    print(round(x$significance, 4), ...)
    cat("\n")
  }

  if (length(x$necessary_predictors) > 0) {
    cat("Necessary conditions (d >= 0.1, p < 0.05):",
        paste(x$necessary_predictors, collapse = ", "), "\n\n")
  } else {
    cat("No necessary conditions identified (d >= 0.1, p < 0.05)\n\n")
  }

  for (ceil in names(x$bottleneck)) {
    cat("Bottleneck table (", ceil, "):\n", sep = "")
    print(x$bottleneck[[ceil]], ...)
    cat("\n")
  }

  invisible(x)
}

#' Plot NCA Results
#'
#' @param x An \code{nca_analysis} object from \code{assess_nca()}.
#' @param type One of \code{"scatter"} (ceiling line scatter plots) or
#'   \code{"effects"} (bar plot of effect sizes).
#' @param ... Additional arguments passed to the underlying plot function.
#'
#' @export
plot.nca_analysis <- function(x, type = c("scatter", "effects"), ...) {
  type <- match.arg(type)

  switch(type,
    scatter = plot_nca_scatter(x, ...),
    effects = plot_nca_effects(x, ...)
  )
}

#' Scatter plot with ceiling lines.
#' @noRd
plot_nca_scatter <- function(nca_result, ...) {
  scores <- nca_result$pls_model$construct_scores
  n_pred <- length(nca_result$predictors)

  old_par <- par(no.readonly = TRUE)
  on.exit(par(old_par))

  if (n_pred > 1) {
    ncol_plot <- min(n_pred, 3)
    nrow_plot <- ceiling(n_pred / ncol_plot)
    par(mfrow = c(nrow_plot, ncol_plot))
  }

  for (pred in nca_result$predictors) {
    x <- scores[, pred]
    y <- scores[, nca_result$target]

    plot(x, y, pch = 19, col = adjustcolor("black", 0.4),
         xlab = pred, ylab = nca_result$target,
         main = paste("NCA:", pred, "->", nca_result$target),
         bty = "n")

    # CE-FDH ceiling: step function
    if ("ce_fdh" %in% nca_result$ceilings) {
      ceil <- compute_ce_fdh(x, y)
      lines(ceil$x, ceil$ceiling_y, type = "s", col = "red", lwd = 2)
    }

    # CR-FDH ceiling: regression line through peers
    if ("cr_fdh" %in% nca_result$ceilings) {
      peers <- get_ce_fdh_peers(x, y)
      if (nrow(peers) >= 2) {
        fit <- lm(y ~ x, data = peers)
        x_seq <- seq(min(x), max(x), length.out = 100)
        y_hat <- coef(fit)[1] + coef(fit)[2] * x_seq
        # Clip to scope
        y_hat <- pmin(pmax(y_hat, min(y)), max(y))
        lines(x_seq, y_hat, col = "blue", lwd = 2, lty = 2)
      }
    }

    # Legend
    ceil_names <- intersect(nca_result$ceilings, INTERNAL_CEILINGS)
    if (length(ceil_names) > 0) {
      cols <- c(ce_fdh = "red", cr_fdh = "blue")[ceil_names]
      ltys <- c(ce_fdh = 1, cr_fdh = 2)[ceil_names]
      legend("bottomright", legend = ceil_names, col = cols, lty = ltys,
             lwd = 2, cex = 0.7, bty = "n")
    }
  }
}

#' @noRd
plot_nca_effects <- function(nca_result, ...) {
  es <- nca_result$effect_sizes
  n_pred <- nrow(es)
  n_ceil <- ncol(es)

  if (n_pred == 0) {
    message("No predictors to plot.")
    return(invisible(NULL))
  }

  bar_width <- 0.8 / n_ceil
  positions <- seq_len(n_pred)

  old_par <- par(no.readonly = TRUE)
  on.exit(par(old_par))

  cols <- if (n_ceil <= 2) {
    c("steelblue", "coral")[seq_len(n_ceil)]
  } else {
    palette.colors(n = n_ceil, palette = "Set1")
  }

  ylim_max <- max(0.5, max(es, na.rm = TRUE) * 1.2)

  par(mar = c(5, 4, 4, 2) + 0.1)
  plot(NULL, NULL,
       xlim = c(0.5, n_pred + 0.5),
       ylim = c(0, ylim_max),
       xaxt = "n", bty = "n",
       xlab = "", ylab = "Effect Size (d)",
       main = paste("NCA Effect Sizes:", nca_result$target),
       ...)

  abline(h = 0.1, col = "darkgray", lty = 2)
  text(n_pred + 0.4, 0.1, "d = 0.1", cex = 0.7, col = "darkgray", pos = 3)

  for (j in seq_len(n_ceil)) {
    offset <- (j - (n_ceil + 1) / 2) * bar_width
    for (i in seq_len(n_pred)) {
      val <- es[i, j]
      if (!is.na(val)) {
        rect(positions[i] + offset - bar_width / 2,
             0,
             positions[i] + offset + bar_width / 2,
             val,
             col = cols[j], border = NA)
      }
    }
  }

  axis(1, at = positions, labels = rownames(es), las = 2)

  if (n_ceil > 1) {
    legend("topright", legend = colnames(es), fill = cols,
           cex = 0.7, bty = "n")
  }
}

# =============================================================================
# NCA-ESSE: EFFECT SIZE SENSITIVITY EXTENSION
# =============================================================================
# Implements the NCA-ESSE method from:
#
# - Becker, J.-M., Richter, N. F., Ringle, C. M., & Sarstedt, M. (2026).
#   "Must-have, or maybe not? A sensitivity-based extension to necessary
#   condition analysis." Journal of Business Research, 206, 115920.
#
# NCA-ESSE assesses the sensitivity of NCA effect sizes to extreme response
# patterns by systematically varying the ECDF ceiling threshold. It compares
# empirical effect size changes against a theoretical benchmark (joint uniform
# distribution) to determine whether a necessary condition is robust.
# =============================================================================

#' Compute the joint empirical CDF for NCA: P(X <= x_i, Y >= y_i).
#'
#' For each observation (x_i, y_i), computes the proportion of observations
#' with X <= x_i AND Y >= y_i. Low values indicate extreme upper-left
#' combinations (low X, high Y) that populate the NCA ceiling zone.
#' @noRd
compute_ecdf_nca <- function(x, y) {
  n <- length(x)
  vapply(seq_len(n), function(i) {
    sum(x <= x[i] & y >= y[i])
  }, numeric(1)) / n
}

#' Compute theoretical CE-FDH benchmark effect size under joint uniform.
#'
#' Under a joint uniform distribution, the CE-FDH effect size at ECDF
#' threshold t is d = t(1 - ln(t)). Derived from the area of the region
#' \{(x,y) : x(1-y) <= t\} on the unit square (Becker et al., 2026).
#' @noRd
benchmark_effect_size <- function(t) {
  ifelse(t == 0, 0, t * (1 - log(t)))
}

#' NCA with Effect Size Sensitivity Extension (NCA-ESSE)
#'
#' \code{assess_nca_esse} applies the NCA-ESSE method (Becker et al., 2026)
#' to assess how sensitive NCA effect sizes are to extreme response patterns.
#' It systematically varies the ECDF ceiling threshold and compares empirical
#' effect size changes against a theoretical benchmark (joint uniform
#' distribution) to determine whether a necessary condition is robust.
#'
#' The method proceeds in three steps:
#' \enumerate{
#'   \item Run standard NCA (threshold 0\%) to identify potential necessary
#'     conditions
#'   \item Apply NCA-ESSE: vary the ECDF threshold from 0\% to a maximum
#'     (default 5\%), computing CE-FDH effect sizes at each level and
#'     comparing against the uniform benchmark
#'   \item (Optional) Select a threshold where the empirical effect size
#'     meaningfully exceeds the benchmark for further evaluation
#' }
#'
#' At each threshold \code{t}, observations whose joint ECDF_NCA value
#' (P(X <= x, Y >= y)) is at most \code{t} are treated as extreme and
#' excluded before computing the CE-FDH ceiling. The uniform benchmark
#' d = t(1 - ln(t)) gives the expected effect size if no necessity exists.
#'
#' @param seminr_model An estimated SEMinR model from \code{estimate_pls()}.
#' @param target Name of the endogenous (outcome) construct.
#' @param predictors Optional character vector of predictor construct names.
#'   If \code{NULL} (default), auto-detected from the structural model.
#' @param thresholds Numeric vector of ECDF thresholds to evaluate
#'   (default \code{seq(0, 0.05, by = 0.005)}). Values must be in [0, 1].
#' @param ceiling Ceiling technique (default \code{"ce_fdh"}). The analytical
#'   benchmark is derived for CE-FDH; other techniques will trigger a warning.
#' @param test.rep Number of permutation test repetitions at each threshold
#'   (default 0). Set > 0 to obtain p-values, but note this multiplies
#'   computation time by the number of thresholds.
#' @param steps Number of steps in the bottleneck table (default 10).
#' @param seed Random seed for reproducibility (default 123).
#' @param ... Additional arguments (currently unused).
#'
#' @return An object of class \code{nca_esse} containing:
#'   \item{effect_sizes}{Matrix of empirical effect sizes (thresholds x predictors)}
#'   \item{benchmark}{Matrix of theoretical benchmark effect sizes}
#'   \item{delta}{Matrix of empirical minus benchmark differences}
#'   \item{significance}{Matrix of p-values (NULL if test.rep = 0)}
#'   \item{pls_model}{The original estimated seminr model}
#'   \item{target}{Name of the target construct}
#'   \item{predictors}{Character vector of predictor names}
#'   \item{thresholds}{Numeric vector of ECDF thresholds used}
#'   \item{ceiling}{Ceiling technique used}
#'   \item{n_obs}{Number of observations}
#'
#' @seealso \code{\link{assess_nca}} for standard NCA analysis
#'
#' @references
#' Becker, J.-M., Richter, N. F., Ringle, C. M., & Sarstedt, M. (2026).
#' Must-have, or maybe not? A sensitivity-based extension to necessary
#' condition analysis. Journal of Business Research, 206, 115920.
#'
#' @examples
#' library(seminr)
#' library(seminrExtras)
#'
#' mobi_mm <- constructs(
#'   composite("Image",        multi_items("IMAG", 1:5)),
#'   composite("Value",        multi_items("PERV", 1:2)),
#'   composite("Satisfaction", multi_items("CUSA", 1:3))
#' )
#'
#' mobi_sm <- relationships(
#'   paths(from = c("Image", "Value"), to = "Satisfaction")
#' )
#'
#' mobi_pls <- estimate_pls(data = mobi,
#'                           measurement_model = mobi_mm,
#'                           structural_model  = mobi_sm)
#'
#' \donttest{
#' esse_result <- assess_nca_esse(mobi_pls,
#'                                 target = "Satisfaction",
#'                                 seed = 123)
#' print(esse_result)
#' plot(esse_result, type = "sensitivity")
#' }
#'
#' @export
assess_nca_esse <- function(seminr_model,
                             target,
                             predictors = NULL,
                             thresholds = seq(0, 0.05, by = 0.005),
                             ceiling = "ce_fdh",
                             test.rep = 0,
                             steps = 10,
                             seed = 123,
                             ...) {

  # ---------------------------------------------------------------------------
  # Step 1: Validate inputs
  # ---------------------------------------------------------------------------
  if (!validate_seminr_model(seminr_model, "assess_nca_esse")) return(NULL)

  if (any(thresholds < 0) || any(thresholds > 1)) {
    stop("thresholds must be between 0 and 1.", call. = FALSE)
  }
  predictors <- validate_nca_inputs(seminr_model, target, predictors, test.rep)

  if (ceiling != "ce_fdh") {
    warning("NCA-ESSE benchmark is derived for CE-FDH (Becker et al., 2026). ",
            "Benchmark may not be directly comparable with '", ceiling, "'.",
            call. = FALSE)
  }

  if (!(ceiling %in% INTERNAL_CEILINGS)) {
    stop("Supported ceiling techniques: ", paste(INTERNAL_CEILINGS, collapse = ", "),
         ". Got '", ceiling, "'.", call. = FALSE)
  }

  scores <- as.data.frame(seminr_model$construct_scores)
  n_obs <- nrow(scores)

  set.seed(seed)

  # ---------------------------------------------------------------------------
  # Step 2: Initialize result matrices
  # ---------------------------------------------------------------------------
  threshold_labels <- paste0(thresholds * 100, "%")
  empirical <- matrix(NA_real_, nrow = length(thresholds), ncol = length(predictors),
                       dimnames = list(threshold_labels, predictors))
  significance <- if (test.rep > 0) {
    matrix(NA_real_, nrow = length(thresholds), ncol = length(predictors),
           dimnames = list(threshold_labels, predictors))
  } else NULL

  # ---------------------------------------------------------------------------
  # Step 3: Compute empirical effect sizes at each ECDF threshold
  # ---------------------------------------------------------------------------
  for (p_idx in seq_along(predictors)) {
    pred <- predictors[p_idx]
    x <- scores[[pred]]
    y <- scores[[target]]

    ecdf_nca <- compute_ecdf_nca(x, y)

    for (t_idx in seq_along(thresholds)) {
      t_val <- thresholds[t_idx]

      # At threshold 0, keep all observations (standard NCA)
      if (t_val == 0) {
        filtered_scores <- scores
      } else {
        keep <- ecdf_nca > t_val
        filtered_scores <- scores[keep, , drop = FALSE]
      }

      if (nrow(filtered_scores) < 10) {
        warning("Fewer than 10 observations at threshold ", threshold_labels[t_idx],
                " for ", pred, "; skipping.", call. = FALSE)
        next
      }

      filtered_x <- filtered_scores[[pred]]
      filtered_y <- filtered_scores[[target]]

      d <- nca_effect_size(filtered_x, filtered_y, ceiling)
      empirical[t_idx, p_idx] <- d

      if (test.rep > 0) {
        significance[t_idx, p_idx] <- nca_permutation_test(
          filtered_x, filtered_y, ceiling, d, test.rep
        )
      }
    }
  }

  # ---------------------------------------------------------------------------
  # Step 4: Compute theoretical benchmark (joint uniform distribution)
  # ---------------------------------------------------------------------------
  benchmark <- matrix(benchmark_effect_size(thresholds),
                       nrow = length(thresholds), ncol = length(predictors),
                       dimnames = list(threshold_labels, predictors))

  # ---------------------------------------------------------------------------
  # Step 5: Compute sensitivity (empirical - benchmark)
  # ---------------------------------------------------------------------------
  delta <- empirical - benchmark

  result <- list(
    effect_sizes = empirical,
    benchmark = benchmark,
    delta = delta,
    significance = significance,
    pls_model = seminr_model,
    target = target,
    predictors = predictors,
    thresholds = thresholds,
    ceiling = ceiling,
    n_obs = n_obs
  )

  class(result) <- c("nca_esse", class(result))
  result
}

# =============================================================================
# NCA-ESSE S3 METHODS
# =============================================================================

#' @export
print.nca_esse <- function(x, ...) {
  cat("NCA-ESSE: Effect Size Sensitivity Extension\n")
  cat("=============================================\n")
  cat("Target:", x$target, "\n")
  cat("Predictors:", paste(x$predictors, collapse = ", "), "\n")
  cat("Ceiling:", x$ceiling, "\n")
  cat("Observations:", x$n_obs, "\n")
  cat("Thresholds:", paste0(min(x$thresholds) * 100, "% to ",
      max(x$thresholds) * 100, "%"), "\n\n")

  cat("Empirical effect sizes by ECDF threshold:\n")
  print(round(x$effect_sizes, 4), ...)
  cat("\n")

  cat("Benchmark (uniform) effect sizes:\n")
  bench_vec <- x$benchmark[, 1]
  names(bench_vec) <- rownames(x$benchmark)
  print(round(bench_vec, 4), ...)
  cat("\n")

  cat("Sensitivity (empirical - benchmark):\n")
  print(round(x$delta, 4), ...)

  if (!is.null(x$significance)) {
    cat("\nPermutation p-values:\n")
    print(round(x$significance, 4), ...)
  }

  invisible(x)
}

#' @export
summary.nca_esse <- function(object, ...) {
  tables <- lapply(object$predictors, function(pred) {
    emp <- object$effect_sizes[, pred]
    bench <- object$benchmark[, pred]
    df <- data.frame(
      ECDF_threshold = object$thresholds,
      Empirical_d = emp,
      Benchmark_d = bench,
      Difference = emp - bench,
      row.names = NULL
    )
    if (!is.null(object$significance)) {
      df$p_value <- object$significance[, pred]
    }
    df
  })
  names(tables) <- object$predictors

  result <- list(
    target = object$target,
    predictors = object$predictors,
    ceiling = object$ceiling,
    n_obs = object$n_obs,
    tables = tables
  )

  class(result) <- c("summary.nca_esse", class(result))
  result
}

#' @export
print.summary.nca_esse <- function(x, ...) {
  cat("NCA-ESSE Summary (Becker et al., 2026)\n")
  cat("=======================================\n")
  cat("Target:", x$target, "\n")
  cat("Ceiling:", x$ceiling, "\n")
  cat("Observations:", x$n_obs, "\n\n")

  for (pred in x$predictors) {
    cat("Predictor:", pred, "\n")
    cat(strrep("-", 60), "\n")
    print(round(x$tables[[pred]], 4), row.names = FALSE, ...)
    cat("\n")
  }

  invisible(x)
}

#' Plot NCA-ESSE Results
#'
#' @param x An \code{nca_esse} object from \code{assess_nca_esse()}.
#' @param type One of \code{"sensitivity"} (effect size vs threshold, Fig. 4
#'   in Becker et al.) or \code{"difference"} (incremental empirical minus
#'   benchmark difference, Fig. 6 in Becker et al.).
#' @param ... Additional arguments passed to the underlying plot function.
#'
#' @export
plot.nca_esse <- function(x, type = c("sensitivity", "difference"), ...) {
  type <- match.arg(type)

  switch(type,
    sensitivity = plot_esse_sensitivity(x, ...),
    difference  = plot_esse_difference(x, ...)
  )
}

#' Sensitivity plot (Fig. 4): empirical and benchmark effect sizes vs threshold.
#' @noRd
plot_esse_sensitivity <- function(esse, ...) {
  n_pred <- length(esse$predictors)
  thresholds <- esse$thresholds

  old_par <- par(no.readonly = TRUE)
  on.exit(par(old_par))

  if (n_pred > 1) {
    ncol_plot <- min(n_pred, 3)
    nrow_plot <- ceiling(n_pred / ncol_plot)
    par(mfrow = c(nrow_plot, ncol_plot))
  }

  for (pred in esse$predictors) {
    emp <- esse$effect_sizes[, pred]
    bench <- esse$benchmark[, pred]
    ylim <- c(0, max(c(emp, bench, 0.1), na.rm = TRUE) * 1.2)

    plot(thresholds, emp, type = "b", pch = 19,
         ylim = ylim, xlab = "ECDF threshold",
         ylab = "NCA effect size (d)",
         main = paste("NCA-ESSE:", pred, "->", esse$target),
         bty = "n", ...)
    lines(thresholds, bench, type = "b", pch = 17, lty = 2, col = "gray50")
    abline(h = 0.1, col = "darkgray", lty = 3)
    legend("topleft", legend = c("Empirical", "Benchmark (uniform)"),
           pch = c(19, 17), lty = c(1, 2), col = c("black", "gray50"),
           cex = 0.7, bty = "n")
  }
}

#' Difference plot (Fig. 6): incremental empirical - benchmark difference.
#' @noRd
plot_esse_difference <- function(esse, ...) {
  n_pred <- length(esse$predictors)
  thresholds <- esse$thresholds[-1]

  old_par <- par(no.readonly = TRUE)
  on.exit(par(old_par))

  if (n_pred > 1) {
    ncol_plot <- min(n_pred, 3)
    nrow_plot <- ceiling(n_pred / ncol_plot)
    par(mfrow = c(nrow_plot, ncol_plot))
  }

  for (pred in esse$predictors) {
    emp <- esse$effect_sizes[, pred]
    bench <- esse$benchmark[, pred]

    delta_emp <- diff(emp)
    delta_bench <- diff(bench)
    delta_diff <- delta_emp - delta_bench

    ylim <- range(c(delta_diff, 0), na.rm = TRUE)
    ylim <- ylim + c(-1, 1) * max(0.05, diff(ylim) * 0.1)

    plot(thresholds, delta_diff, type = "b", pch = 19,
         ylim = ylim, xlab = "ECDF threshold",
         ylab = expression(Delta ~ "Empirical" - Delta ~ "Benchmark"),
         main = paste("ESSE Difference:", pred, "->", esse$target),
         bty = "n", ...)
    abline(h = 0, col = "darkgray", lty = 2)
  }
}

library(seminr)

# ============================================================================
# Setup: Estimate model once, reuse across all tests
# ============================================================================

set.seed(123)

mobi_mm <- constructs(
  composite("Image",        multi_items("IMAG", 1:5)),
  composite("Value",        multi_items("PERV", 1:2)),
  composite("Satisfaction", multi_items("CUSA", 1:3)),
  composite("Loyalty",      multi_items("CUSL", 1:3))
)

mobi_sm <- relationships(
  paths(from = c("Image", "Value"), to = "Satisfaction"),
  paths(from = "Satisfaction", to = "Loyalty")
)

pls_model <- estimate_pls(
  data = mobi,
  measurement_model = mobi_mm,
  structural_model  = mobi_sm
)

# Pre-compute NCA result once (minimal reps for speed)
nca_result <- assess_nca(pls_model,
                          target = "Satisfaction",
                          ceilings = c("ce_fdh", "cr_fdh"),
                          test.rep = 0,
                          seed = 123)

# ============================================================================
# Structure tests
# ============================================================================

test_that("assess_nca returns correct S3 class and elements", {
  expect_s3_class(nca_result, "nca_analysis")
  expected_names <- c("nca_raw", "effect_sizes", "significance",
                      "bottleneck", "necessary_predictors", "pls_model",
                      "target", "predictors", "ceilings")
  expect_true(all(expected_names %in% names(nca_result)))
})

test_that("nca_raw is NULL when using internal ceilings", {
  expect_null(nca_result$nca_raw)
})

test_that("effect_sizes matrix has correct dimensions and names", {
  expect_equal(dim(nca_result$effect_sizes), c(2, 2))
  expect_equal(rownames(nca_result$effect_sizes), c("Image", "Value"))
  expect_equal(colnames(nca_result$effect_sizes), c("ce_fdh", "cr_fdh"))
  expect_true("table_output" %in% class(nca_result$effect_sizes))
})

# ============================================================================
# Auto-detection tests
# ============================================================================

test_that("assess_nca auto-detects direct predictors from smMatrix", {
  expect_equal(nca_result$predictors, c("Image", "Value"))
})

test_that("assess_nca auto-detects single predictor correctly", {
  result <- assess_nca(pls_model, target = "Loyalty", test.rep = 0, seed = 123)
  expect_equal(result$predictors, "Satisfaction")
})

test_that("assess_nca accepts explicit predictors", {
  result <- assess_nca(pls_model, target = "Satisfaction",
                        predictors = "Image", test.rep = 0, seed = 123)
  expect_equal(result$predictors, "Image")
  expect_equal(nrow(result$effect_sizes), 1)
})

# ============================================================================
# Effect sizes tests
# ============================================================================

test_that("effect sizes are non-negative and bounded by 1", {
  expect_true(all(nca_result$effect_sizes >= 0, na.rm = TRUE))
  expect_true(all(nca_result$effect_sizes <= 1, na.rm = TRUE))
})

test_that("CE-FDH effect size >= CR-FDH for same predictor", {
  # CE-FDH (step function) always has ceiling zone >= CR-FDH (regression line)
  for (pred in nca_result$predictors) {
    expect_true(nca_result$effect_sizes[pred, "ce_fdh"] >=
                nca_result$effect_sizes[pred, "cr_fdh"] - 1e-10)
  }
})

# ============================================================================
# Significance and bottleneck tests
# ============================================================================

test_that("significance is NULL when test.rep = 0", {
  expect_null(nca_result$significance)
})

test_that("bottleneck is a list keyed by ceiling technique", {
  expect_type(nca_result$bottleneck, "list")
  expect_equal(sort(names(nca_result$bottleneck)),
               sort(c("ce_fdh", "cr_fdh")))
})

test_that("bottleneck tables have correct structure", {
  for (ceil in names(nca_result$bottleneck)) {
    bn <- nca_result$bottleneck[[ceil]]
    expect_s3_class(bn, "data.frame")
    expect_true("Satisfaction" %in% colnames(bn))
    expect_true(all(nca_result$predictors %in% colnames(bn)))
    expect_equal(nrow(bn), 11)  # 0, 10, 20, ..., 100
  }
})

# ============================================================================
# Ceiling options tests
# ============================================================================

test_that("assess_nca works with single ceiling technique", {
  result <- assess_nca(pls_model, target = "Satisfaction",
                        ceilings = "ce_fdh", test.rep = 0, seed = 123)
  expect_equal(ncol(result$effect_sizes), 1)
  expect_equal(colnames(result$effect_sizes), "ce_fdh")
})

# ============================================================================
# Validation tests
# ============================================================================

test_that("assess_nca warns on non-seminr model", {
  expect_warning(
    result <- assess_nca(list(not = "a model"), target = "X"),
    "only works with SEMinR models"
  )
  expect_null(result)
})

test_that("assess_nca errors on invalid target", {
  expect_error(assess_nca(pls_model, target = "NONEXISTENT"), "target")
})

test_that("assess_nca errors on invalid predictor", {
  expect_error(
    assess_nca(pls_model, target = "Satisfaction",
                predictors = c("Image", "FAKE")),
    "not found in model constructs"
  )
})

# ============================================================================
# S3 methods tests
# ============================================================================

test_that("print.nca_analysis runs without error", {
  expect_output(print(nca_result), "Necessary Condition Analysis")
})

test_that("summary.nca_analysis returns correct class with expected elements", {
  s <- summary(nca_result)
  expect_s3_class(s, "summary.nca_analysis")
  expect_true(all(c("bottleneck", "necessary_predictors") %in% names(s)))
})

test_that("necessary_predictors is precomputed in result", {
  expect_type(nca_result$necessary_predictors, "character")
})

test_that("plot.nca_analysis type='effects' runs without error", {
  expect_no_error(plot(nca_result, type = "effects"))
})

test_that("plot.nca_analysis type='scatter' runs without error (internal)", {
  expect_no_error(plot(nca_result, type = "scatter"))
})

# ============================================================================
# Significance testing (only when test.rep > 0)
# ============================================================================

test_that("significance matrix is correct when test.rep > 0", {
  result_sig <- assess_nca(pls_model, target = "Satisfaction",
                            test.rep = 50, seed = 123)
  expect_false(is.null(result_sig$significance))
  expect_equal(dim(result_sig$significance), dim(result_sig$effect_sizes))
  expect_true(all(result_sig$significance >= 0 &
                    result_sig$significance <= 1, na.rm = TRUE))
})

# ============================================================================
# Edge case: test.rep validation
# ============================================================================

test_that("assess_nca errors on negative test.rep", {
  expect_error(
    assess_nca(pls_model, target = "Satisfaction", test.rep = -1),
    "non-negative integer"
  )
})

test_that("assess_nca errors on non-integer test.rep", {
  expect_error(
    assess_nca(pls_model, target = "Satisfaction", test.rep = 1.5),
    "non-negative integer"
  )
})

# ============================================================================
# Edge case: HOC model (should work -- NCA uses construct scores directly)
# ============================================================================

test_that("assess_nca works with higher-order construct model", {
  hoc_mm <- constructs(
    composite("Image", multi_items("IMAG", 1:5)),
    composite("Value", multi_items("PERV", 1:2)),
    higher_composite("Rep", c("Image", "Value"), method = two_stage),
    composite("Satisfaction", multi_items("CUSA", 1:3)),
    composite("Loyalty", multi_items("CUSL", 1:3))
  )
  hoc_sm <- relationships(
    paths(from = "Rep", to = "Satisfaction"),
    paths(from = "Satisfaction", to = "Loyalty")
  )
  hoc_model <- estimate_pls(data = mobi, measurement_model = hoc_mm,
                             structural_model = hoc_sm)

  result <- assess_nca(hoc_model, target = "Satisfaction", test.rep = 0, seed = 123)

  expect_s3_class(result, "nca_analysis")
  expect_equal(result$predictors, "Rep")
})

# ============================================================================
# Internal NCA algorithm tests
# ============================================================================

test_that("CE-FDH effect size is correct for known case", {
  # Perfect diagonal: (0,0), (1,1) -- full ceiling zone
  # Scope = 1, ceiling zone = (1-0)*(1-0) = 1 (step at x=0 has ceiling_y=0)
  x <- c(0, 1)
  y <- c(0, 1)
  d <- seminrExtras:::ce_fdh_effect_size(x, y)
  expect_equal(d, 1.0)
})

test_that("CE-FDH effect size is zero when no empty space exists", {
  # Upper-left point present: (0, 1), (1, 0)
  # ceiling at x=0 is 1, ceiling at x=1 is 1 (cummax) -> no empty space
  x <- c(0, 1)
  y <- c(1, 0)
  d <- seminrExtras:::ce_fdh_effect_size(x, y)
  expect_equal(d, 0.0)
})

test_that("CE-FDH effect size is bounded [0, 1]", {
  set.seed(42)
  x <- rnorm(100)
  y <- rnorm(100)
  d <- seminrExtras:::ce_fdh_effect_size(x, y)
  expect_true(d >= 0 && d <= 1)
})

test_that("CR-FDH effect size is bounded [0, 1]", {
  set.seed(42)
  x <- rnorm(100)
  y <- rnorm(100)
  d <- seminrExtras:::cr_fdh_effect_size(x, y)
  expect_true(d >= 0 && d <= 1)
})

test_that("CE-FDH handles constant x (zero scope) gracefully", {
  d <- seminrExtras:::ce_fdh_effect_size(rep(1, 10), 1:10)
  expect_equal(d, 0)
})

test_that("CE-FDH handles constant y (zero scope) gracefully", {
  d <- seminrExtras:::ce_fdh_effect_size(1:10, rep(5, 10))
  expect_equal(d, 0)
})

test_that("bottleneck has correct length and bounded values", {
  x <- pls_model$construct_scores[, "Image"]
  y <- pls_model$construct_scores[, "Satisfaction"]
  bn <- seminrExtras:::compute_bottleneck_column(x, y, "ce_fdh", 10)
  expect_equal(length(bn), 11)
  # First value (0% of Y) should be 0 or NA
  expect_true(is.na(bn[1]) || bn[1] == 0)
  # Non-NA values should be in [0, 100]
  non_na <- bn[!is.na(bn)]
  expect_true(all(non_na >= 0 & non_na <= 100))
})

# ============================================================================
# NCA-ESSE tests
# ============================================================================

# Pre-compute ESSE result once (no permutation tests for speed)
esse_result <- assess_nca_esse(pls_model,
                                target = "Satisfaction",
                                thresholds = seq(0, 0.05, by = 0.01),
                                seed = 123)

test_that("assess_nca_esse returns correct S3 class and elements", {
  expect_s3_class(esse_result, "nca_esse")
  expected_names <- c("effect_sizes", "benchmark", "delta", "significance",
                      "pls_model", "target", "predictors", "thresholds",
                      "ceiling", "n_obs")
  expect_true(all(expected_names %in% names(esse_result)))
})

test_that("ESSE effect_sizes matrix has correct dimensions", {
  expect_equal(nrow(esse_result$effect_sizes), 6)  # 0, 1, 2, 3, 4, 5%
  expect_equal(ncol(esse_result$effect_sizes), 2)  # Image, Value
  expect_equal(colnames(esse_result$effect_sizes), c("Image", "Value"))
})

test_that("ESSE benchmark matches analytical formula d = t(1 - ln(t))", {
  thresholds <- esse_result$thresholds
  expected <- ifelse(thresholds == 0, 0, thresholds * (1 - log(thresholds)))
  expect_equal(unname(esse_result$benchmark[, 1]), expected)
})

test_that("ESSE delta equals empirical minus benchmark", {
  expect_equal(esse_result$delta,
               esse_result$effect_sizes - esse_result$benchmark)
})

test_that("ESSE effect sizes are non-negative", {
  expect_true(all(esse_result$effect_sizes >= 0, na.rm = TRUE))
})

test_that("ESSE standard NCA (threshold 0%) matches assess_nca()", {
  # Effect sizes at threshold 0% should match standard NCA
  standard_d <- nca_result$effect_sizes[, "ce_fdh"]
  esse_d <- esse_result$effect_sizes["0%", ]
  expect_equal(esse_d, standard_d, tolerance = 1e-10)
})

test_that("ESSE significance is NULL when test.rep = 0", {
  expect_null(esse_result$significance)
})

test_that("ESSE effect sizes increase with higher thresholds", {
  for (pred in esse_result$predictors) {
    d <- esse_result$effect_sizes[, pred]
    expect_true(d[length(d)] >= d[1])
  }
})

test_that("ESSE errors on invalid thresholds", {
  expect_error(
    assess_nca_esse(pls_model, target = "Satisfaction", thresholds = c(-0.1, 0.5)),
    "thresholds"
  )
})

test_that("ESSE print method runs without error", {
  expect_output(print(esse_result), "NCA-ESSE")
})

test_that("ESSE summary returns correct class", {
  s <- summary(esse_result)
  expect_s3_class(s, "summary.nca_esse")
  expect_true(all(c("tables", "target", "predictors") %in% names(s)))
})

test_that("ESSE plot sensitivity runs without error", {
  expect_no_error(plot(esse_result, type = "sensitivity"))
})

test_that("ESSE plot difference runs without error", {
  expect_no_error(plot(esse_result, type = "difference"))
})

test_that("ESSE warns for non-CE-FDH ceiling", {
  expect_warning(
    assess_nca_esse(pls_model, target = "Satisfaction",
                     ceiling = "cr_fdh",
                     thresholds = c(0, 0.01), seed = 123),
    "CE-FDH"
  )
})

# ============================================================================
# Different dataset
# ============================================================================

test_that("assess_nca works with corp_rep_data model", {
  corp_mm <- constructs(
    composite("COMP", multi_items("comp_", 1:3)),
    composite("LIKE", multi_items("like_", 1:3)),
    composite("CUSA", single_item("cusa"))
  )
  corp_sm <- relationships(
    paths(from = c("COMP", "LIKE"), to = "CUSA")
  )
  corp_model <- estimate_pls(
    data = corp_rep_data,
    measurement_model = corp_mm,
    structural_model  = corp_sm,
    missing = mean_replacement,
    missing_value = "-99"
  )

  result <- assess_nca(corp_model, target = "CUSA", test.rep = 0, seed = 123)

  expect_s3_class(result, "nca_analysis")
  expect_equal(result$target, "CUSA")
  expect_equal(sort(result$predictors), sort(c("COMP", "LIKE")))
})

---
title: "seminrExtras"
author: "Nicholas .P. Danks and Soumya Ray"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{seminrExtras}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(seminr)
library(seminrExtras)

# set the theme for plotting output
seminr_theme_set(seminr_theme_academic())

# Create measurement model ----
corp_rep_mm_ext <- constructs(
  composite("QUAL", multi_items("qual_", 1:8), weights = mode_B),
  composite("PERF", multi_items("perf_", 1:5), weights = mode_B),
  composite("CSOR", multi_items("csor_", 1:5), weights = mode_B),
  composite("ATTR", multi_items("attr_", 1:3), weights = mode_B),
  composite("COMP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  composite("CUSA", single_item("cusa")),
  composite("CUSL", multi_items("cusl_", 1:3))
)

alt_mm <- constructs(
  composite("QUAL", multi_items("qual_", 1:8), weights = mode_B),
  composite("PERF", multi_items("perf_", 1:5), weights = mode_B),
  composite("CSOR", multi_items("csor_", 1:5), weights = mode_B),
  composite("ATTR", multi_items("attr_", 1:3), weights = mode_B),
  composite("COMP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  composite("CUSA", single_item("cusa")),
  composite("CUSL", multi_items("cusl_", 1:3))
)

# Create structural model ----
corp_rep_sm_ext <- relationships(
  paths(from = c("QUAL", "PERF", "CSOR", "ATTR"), to = c("COMP", "LIKE")),
  paths(from = c("COMP", "LIKE"), to = c("CUSA", "CUSL")),
  paths(from = c("CUSA"),         to = c("CUSL"))
)
alt_sm <- relationships(
  paths(from = c("QUAL", "PERF", "CSOR", "ATTR"), to = c("COMP", "LIKE")),
  paths(from = c("COMP", "LIKE"), to = c("CUSA")),
  paths(from = c("CUSA"),         to = c("CUSL"))
)


# Estimate the models ----
established_model <- estimate_pls(
  data = corp_rep_data,
  measurement_model = corp_rep_mm_ext,
  structural_model  = corp_rep_sm_ext,
  missing = mean_replacement,
  missing_value = "-99")

competing_model <- estimate_pls(
  data = corp_rep_data,
  measurement_model = alt_mm,
  structural_model  = alt_sm,
  missing = mean_replacement,
  missing_value = "-99")


```

SEMinR (Ray, Danks, & Calero Valdez, 2026) is a domain specific language for modeling and estimating structural equation models. This is a supplementary package for SEMinR and not a standalone package.
This package serves to provide additional extra methods and functions that can be used to analyze PLS-SEM models.

SEMinRExtras provides advanced SEM tools which are compatible with SEMinR. It implements several methods for evaluating PLS-SEM models, including CVPAT (Liengaard et al., 2021; Sharma et al., 2022), Combined Importance-Performance Map Analysis (cIPMA; Sarstedt et al., 2024; Hauff et al., 2024), Composite Overfit Analysis (COA), Necessary Condition Analysis (NCA; Dul, 2016; Richter et al., 2020), NCA-ESSE (Becker et al., 2026), Confirmatory Tetrad Analysis (CTA-PLS; Gudergan et al., 2008), FIMIX-PLS (Hahn et al., 2002; Sarstedt et al., 2011), PLS-POS (Becker et al., 2013), and congruence testing (Franke, Sarstedt, & Danks, 2021).

SEMinRExtras also serves to host the example models used in the PLS-SEM in R workbook (Hair et al., 2026).

### Functions

| Function | Description |
|---|---|
| `assess_cvpat()` | CVPAT against LM and IA benchmarks |
| `assess_cvpat_compare()` | Compare predictive loss of two PLS models |
| `assess_ipma()` | Importance-Performance Map Analysis (IPMA) |
| `assess_cipma()` | Combined IPMA with Necessary Condition Analysis (cIPMA) |
| `assess_coa()` | Composite Overfit Analysis (full pipeline) |
| `predictive_deviance()` | Compute predictive deviance scores |
| `deviance_tree()` | Identify deviant case groups via decision tree |
| `unstable_params()` | Parameter instability analysis |
| `group_rules()` | Extract decision rules for deviant groups |
| `competes()` | Show competing splits at tree nodes |
| `assess_nca()` | Necessary Condition Analysis for PLS-SEM |
| `assess_nca_esse()` | NCA with Effect Size Sensitivity Extension |
| `assess_cta()` | Confirmatory Tetrad Analysis (CTA-PLS) with indicator borrowing (Gudergan et al., 2008) |
| `assess_fimix()` | FIMIX-PLS latent class segmentation |
| `assess_fimix_compare()` | Compare FIMIX solutions across K values |
| `assess_pos()` | PLS-POS prediction-oriented segmentation (Becker et al., 2013) |
| `assess_pos_compare()` | Compare PLS-POS solutions across K values |
| `pos_segments()` | Extract segment-specific re-estimated PLS models |
| `congruence_test()` | Bootstrapped congruence coefficient testing |

## The demo files for Hair et al. (2026)  

In order to access the demo files for the textbook, you can run the ```demo()``` function after loading the SEMinRExtras package. 

- seminr-help-debugging
- seminr-pls-cvpat
- seminr-pls-cipma
- seminr-pls-coa
- seminr-pls-nca
- seminr-pls-fimix
- seminr-pls-cta
- seminr-pls-pos
- seminr-pls-congruence
- seminr-primer-v2-chap2
- seminr-primer-v2-chap3
- seminr-primer-v2-chap4
- seminr-primer-v2-chap5
- seminr-primer-v2-chap6 
- seminr-primer-v2-chap7 
- seminr-primer-v2-chap8

E.g. ```demo("seminr-pls-cvpat", package = "seminrExtras")```

## The Example model: Corporate Reputation

We are applying the CVPAT process to the corporate reputation example bundled with SEMinR. Since we will be comparing two models, we will first estimate and plot both models. Below you will find the competing and established models which will be compared. 

```{r, fig.show='hold'}
plot(established_model)
plot(competing_model)
```

## Example
```{r, eval=FALSE, echo=TRUE}

# Create measurement model ----
corp_rep_mm_ext <- constructs(
  composite("QUAL", multi_items("qual_", 1:8), weights = mode_B),
  composite("PERF", multi_items("perf_", 1:5), weights = mode_B),
  composite("CSOR", multi_items("csor_", 1:5), weights = mode_B),
  composite("ATTR", multi_items("attr_", 1:3), weights = mode_B),
  composite("COMP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  composite("CUSA", single_item("cusa")),
  composite("CUSL", multi_items("cusl_", 1:3))
)

alt_mm <- constructs(
  composite("QUAL", multi_items("qual_", 1:8), weights = mode_B),
  composite("PERF", multi_items("perf_", 1:5), weights = mode_B),
  composite("CSOR", multi_items("csor_", 1:5), weights = mode_B),
  composite("ATTR", multi_items("attr_", 1:3), weights = mode_B),
  composite("COMP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  composite("CUSA", single_item("cusa")),
  composite("CUSL", multi_items("cusl_", 1:3))
)

# Create structural model ----
corp_rep_sm_ext <- relationships(
  paths(from = c("QUAL", "PERF", "CSOR", "ATTR"), to = c("COMP", "LIKE")),
  paths(from = c("COMP", "LIKE"), to = c("CUSA", "CUSL")),
  paths(from = c("CUSA"),         to = c("CUSL"))
)
alt_sm <- relationships(
  paths(from = c("QUAL", "PERF", "CSOR", "ATTR"), to = c("COMP", "LIKE")),
  paths(from = c("COMP", "LIKE"), to = c("CUSA")),
  paths(from = c("CUSA"),         to = c("CUSL"))
)


# Estimate the models ----
established_model <- estimate_pls(
  data = corp_rep_data,
  measurement_model = corp_rep_mm_ext,
  structural_model  = corp_rep_sm_ext,
  missing = mean_replacement,
  missing_value = "-99")

competing_model <- estimate_pls(
  data = corp_rep_data,
  measurement_model = alt_mm,
  structural_model  = alt_sm,
  missing = mean_replacement,
  missing_value = "-99")

# Function to compare the Loss of two models
compare_results <- assess_cvpat_compare(established_model = established_model,
                                        alternative_model = competing_model,
                                        testtype = "two.sided",
                                        nboot = 2000,
                                        technique = predict_DA,
                                        seed = 123,
                                        noFolds = 10,
                                        reps = 10,
                                        cores = 1)


print(compare_results,
      digits = 3)

# Assess the base model ----
assess_results <- assess_cvpat(established_model,
                               seed = 123, 
                               cores = 1)
print(assess_results$CVPAT_compare_LM,
      digits = 3)
print(assess_results$CVPAT_compare_IA,
      digits = 3)

```

## First, conduct CVPAT analysis of the established model.

```{r, echo=TRUE}
# Assess the base model ----
assess_results <- assess_cvpat(established_model,
                               seed = 123, 
                               cores = 1)
print(assess_results$CVPAT_compare_LM,
      digits = 3)
print(assess_results$CVPAT_compare_IA,
      digits = 3)
```

The established model has significantly lower predictive loss compared to both the naive benchmark IA and the LM model. Thus, we can say that the established model has predictive relevance.   

Now we compare the results:

```{r, echo=TRUE}
# Function to compare the Loss of two models
compare_results <- assess_cvpat_compare(established_model = established_model,
                                        alternative_model = competing_model,
                                        testtype = "two.sided",
                                        nboot = 2000,
                                        technique = predict_DA,
                                        seed = 123,
                                        noFolds = 10,
                                        reps = 10,
                                        cores = 1)

print(compare_results,
      digits = 3)
```

The established model has significantly lower predictive loss compared to the competing model. Thus, we can say that the established model has superior predictive performance compared to the competing model.

## Combined Importance-Performance Map Analysis (cIPMA)

IPMA (Ringle & Sarstedt, 2016) extends PLS-SEM results by jointly considering the **importance** (unstandardized total effect) and **performance** (rescaled 0--100 mean score) of each construct in predicting a target outcome. cIPMA (Sarstedt et al., 2024; Hauff et al., 2024) further integrates NCA to distinguish which constructs are **necessary conditions** vs. merely sufficient predictors.

`assess_cipma()` computes both IPMA and NCA in one call, classifying constructs into four categories: Top priority (high importance + necessary), Important driver (high importance, not necessary), Bottleneck risk (low importance but necessary), and Low priority.

```{r, eval=FALSE, echo=TRUE}
library(seminr)
library(seminrExtras)

# Estimate a model
mobi_mm <- constructs(
  composite("Image",        multi_items("IMAG", 1:5)),
  composite("Expectation",  multi_items("CUEX", 1:3)),
  composite("Value",        multi_items("PERV", 1:2)),
  composite("Satisfaction", multi_items("CUSA", 1:3)),
  composite("Loyalty",      multi_items("CUSL", 1:3))
)

mobi_sm <- relationships(
  paths(from = "Image",        to = c("Expectation", "Satisfaction", "Loyalty")),
  paths(from = "Expectation",  to = c("Value", "Satisfaction")),
  paths(from = "Value",        to = "Satisfaction"),
  paths(from = "Satisfaction", to = "Loyalty")
)

mobi_pls <- estimate_pls(data = mobi,
                          measurement_model = mobi_mm,
                          structural_model  = mobi_sm)

# Run cIPMA (IPMA + NCA)
cipma_result <- assess_cipma(mobi_pls,
                              target = "Loyalty",
                              scale_min = 1,
                              scale_max = 10,
                              nca_test.rep = 1000,
                              seed = 123)

# View results
print(cipma_result)
summary(cipma_result)

# cIPMA map (importance vs. performance, with NCA overlay)
plot(cipma_result, type = "cipma")

# Standard IPMA map (without NCA distinction)
plot(cipma_result, type = "ipma")

# Use standardized total effects for importance axis
plot(cipma_result, importance_metric = "standardized")
```

All outer weights must be positive for valid performance rescaling. Interaction constructs are automatically excluded. The function warns if negative weights are detected and recommends reverse-coding indicators (Ringle & Sarstedt, 2016).

## Necessary Condition Analysis (NCA)

NCA (Dul, 2016) tests whether a predictor is a *necessary condition* for an outcome -- i.e., whether a certain level of X is required (but not sufficient) for a certain level of Y. This complements the sufficiency-based logic of PLS-SEM regression.

`assess_nca()` auto-detects direct predictors from the structural model and runs ceiling-based NCA analysis using internal CE-FDH and CR-FDH algorithms.

```{r, eval=FALSE, echo=TRUE}
library(seminr)
library(seminrExtras)

# Estimate a simple model
mobi_mm <- constructs(
  composite("Image",        multi_items("IMAG", 1:5)),
  composite("Value",        multi_items("PERV", 1:2)),
  composite("Satisfaction", multi_items("CUSA", 1:3)),
  composite("Loyalty",      multi_items("CUSL", 1:3))
)

mobi_sm <- relationships(
  paths(from = c("Image", "Value"), to = "Satisfaction"),
  paths(from = "Satisfaction", to = "Loyalty")
)

mobi_pls <- estimate_pls(data = mobi,
                          measurement_model = mobi_mm,
                          structural_model  = mobi_sm)

# Run NCA -- predictors auto-detected from structural model
nca_result <- assess_nca(mobi_pls,
                          target = "Satisfaction",
                          test.rep = 1000,
                          seed = 123)

# Effect sizes and significance
print(nca_result)

# Full summary with bottleneck tables
summary(nca_result)

# Visualize
plot(nca_result, type = "effects")   # Effect size bar chart
plot(nca_result, type = "scatter")   # Ceiling line scatter plots
```

A predictor is identified as a necessary condition when it has an effect size d >= 0.1 and a significant permutation test (p < 0.05), following guidelines from Richter et al. (2020).

## NCA-ESSE: Effect Size Sensitivity Extension

NCA-ESSE (Becker et al., 2026) extends standard NCA by assessing how sensitive effect sizes are to extreme response patterns. It systematically varies the ECDF ceiling threshold and compares empirical effect size changes against a theoretical benchmark derived from a joint uniform distribution.

```{r, eval=FALSE, echo=TRUE}
# Run NCA-ESSE on the same model
esse_result <- assess_nca_esse(mobi_pls,
                                target = "Satisfaction",
                                thresholds = seq(0, 0.05, by = 0.005),
                                seed = 123)

# View results
print(esse_result)

# Summary tables (Table A2 style)
summary(esse_result)

# Sensitivity plot (Fig. 4 in Becker et al., 2026)
plot(esse_result, type = "sensitivity")

# Difference plot (Fig. 6 in Becker et al., 2026)
plot(esse_result, type = "difference")
```

At each threshold `t`, observations whose joint ECDF_NCA value P(X <= x, Y >= y) is at most `t` are treated as extreme upper-left cases and excluded. The uniform benchmark d = t(1 - ln(t)) gives the expected effect size increase if no true necessity exists. Empirical effect sizes that consistently exceed the benchmark provide evidence for a robust necessary condition.

## Composite Overfit Analysis (COA)

COA detects observation-level overfitting in PLS composite models. It computes predictive deviance (the gap between in-sample fitted and out-of-sample predicted scores), then uses a decision tree to identify subgroups where the model overfits.

```{r, eval=FALSE, echo=TRUE}
library(seminr)
library(seminrExtras)

# Estimate a model
corp_mm <- constructs(
  composite("COMP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  composite("CUSA", single_item("cusa")),
  composite("CUSL", multi_items("cusl_", 1:3))
)

corp_sm <- relationships(
  paths(from = c("COMP", "LIKE"), to = "CUSA"),
  paths(from = "CUSA", to = "CUSL")
)

corp_model <- estimate_pls(
  data = corp_rep_data,
  measurement_model = corp_mm,
  structural_model  = corp_sm,
  missing = mean_replacement,
  missing_value = "-99")

# Run full COA pipeline
coa_result <- assess_coa(corp_model,
                          focal_construct = "CUSL",
                          noFolds = 10, reps = 1, cores = 1,
                          seed = 123)

# Print results
print(coa_result)
summary(coa_result)

# Visualize
plot(coa_result, type = "pd")      # Predictive deviance distribution
plot(coa_result, type = "tree")    # Decision tree
plot(coa_result, type = "groups")  # Deviant group highlights
```

COA proceeds in four steps: (1) compute predictive deviance via cross-validated prediction, (2) grow a decision tree to identify deviant subgroups, (3) analyze parameter instability by re-estimating the model without each deviant group, and (4) inspect group rules and competing splits.

## Confirmatory Tetrad Analysis (CTA-PLS)

CTA-PLS (Gudergan et al., 2008) empirically tests whether a construct's measurement model is consistent with a reflective (common factor) specification. Under a reflective model, all model-implied vanishing tetrads equal zero. If any tetrad is significantly non-zero (after multiple testing correction), the reflective specification is rejected in favour of a formative one.

CTA-PLS requires at least 4 indicators per construct. When `borrow = TRUE` (the default), constructs with only 2 or 3 indicators can still be tested by borrowing indicators from structurally connected constructs. The borrowing rules follow Gudergan et al. (2008, Table 1):

- **3 own indicators + 1 borrowed** from an adjacent reflective construct: all tetrads vanish under H0.
- **2 own indicators + 2 borrowed** from any adjacent construct: only the cross-tetrad tau_1342 vanishes.
- Formative focal constructs cannot be tested via borrowing (no vanishing tetrads exist under a formative model).

When `borrow = FALSE`, constructs with fewer than 4 indicators are skipped entirely.

```{r, eval=FALSE, echo=TRUE}
library(seminr)
library(seminrExtras)

mobi_mm <- constructs(
  composite("Image",        multi_items("IMAG", 1:5)),
  composite("Expectation",  multi_items("CUEX", 1:3)),
  composite("Value",        multi_items("PERV", 1:2)),
  composite("Satisfaction", multi_items("CUSA", 1:3)),
  composite("Loyalty",      multi_items("CUSL", 1:3))
)

mobi_sm <- relationships(
  paths(from = "Image",       to = c("Expectation", "Satisfaction", "Loyalty")),
  paths(from = "Expectation", to = c("Value", "Satisfaction")),
  paths(from = "Value",       to = "Satisfaction"),
  paths(from = "Satisfaction", to = "Loyalty")
)

mobi_pls <- estimate_pls(data = mobi,
                          measurement_model = mobi_mm,
                          structural_model  = mobi_sm)

# CTA-PLS with borrowing (default)
cta_result <- assess_cta(mobi_pls, nboot = 5000, seed = 123)
print(cta_result)
summary(cta_result)

# Without borrowing — only Image (5 indicators) is testable
cta_no_borrow <- assess_cta(mobi_pls, nboot = 5000, borrow = FALSE)
print(cta_no_borrow)

# Visualize adjusted p-values per construct
plot(cta_result)
```

The `print()` output shows a per-construct summary with the measurement mode, number of indicators (including borrowed), number of tetrads tested, how many are significant, and the verdict (reflective supported or rejected). Constructs that borrowed indicators are annotated with the donor construct. The `summary()` output additionally shows the individual tetrad estimates, bootstrap t-values, confidence intervals, and adjusted p-values.

Benjamini-Hochberg (BH) correction for multiple testing is applied by default (Cefis et al., 2025). Bonferroni correction and no correction are also available via the `correction` parameter.

## Unobserved Heterogeneity (FIMIX-PLS and PLS-POS)

FIMIX-PLS and PLS-POS are complementary approaches for detecting unobserved heterogeneity in PLS-SEM. FIMIX-PLS uses probabilistic (EM-based) assignment and assumes normally distributed residuals, while PLS-POS uses deterministic (hill-climbing) assignment with no distributional assumptions. Both methods partition the sample into K segments with segment-specific path coefficients.

### FIMIX-PLS (Finite Mixture PLS)

FIMIX-PLS (Hahn et al., 2002; Sarstedt et al., 2011) probabilistically assigns observations to K segments, each with segment-specific structural path coefficients.

`assess_fimix()` estimates a single K-segment solution, while `assess_fimix_compare()` compares solutions across K values using information criteria (AIC, BIC, CAIC, etc.) to help select the optimal number of segments.

```{r, eval=FALSE, echo=TRUE}
# FIMIX with K=2 segments (using the corporate reputation model from the setup)
fimix_k2 <- assess_fimix(established_model, K = 2, nstart = 10, seed = 123)
print(fimix_k2)
summary(fimix_k2)
plot(fimix_k2)

# Compare across K=2..4 using information criteria
fimix_compare <- assess_fimix_compare(established_model,
                                       K_range = 2:4,
                                       nstart = 10,
                                       seed = 123)
print(fimix_compare)
plot(fimix_compare)
```

### PLS-POS (Prediction-Oriented Segmentation)

PLS-POS (Becker et al., 2013) maximizes the sum of R-squared values across all endogenous constructs across K segments using deterministic hill-climbing. Unlike FIMIX-PLS, it makes no distributional assumptions and can detect heterogeneity in both structural and formative measurement models.

`assess_pos()` estimates a single K-segment solution with multiple random starts, while `assess_pos_compare()` compares solutions across K values using the objective criterion.

```{r, eval=FALSE, echo=TRUE}
# PLS-POS with K=2 segments
pos_k2 <- assess_pos(established_model, K = 2, nstart = 10, seed = 123)
print(pos_k2)
summary(pos_k2)
plot(pos_k2, type = "rsquared")
plot(pos_k2, type = "paths")

# Compare across K=2..4 to find optimal number of segments
pos_compare <- assess_pos_compare(established_model,
                                   K_range = 2:4,
                                   nstart = 10,
                                   seed = 123)
print(pos_compare)
plot(pos_compare)

# Extract segment-specific models for further analysis
seg_models <- pos_segments(pos_k2)
summary(seg_models[[1]])
```

PLS-POS assigns each observation to exactly one segment (hard assignment) and re-estimates the full PLS model within each segment. The algorithm iteratively reassigns observations one at a time, accepting only moves that improve the objective criterion. Multiple random starting partitions help avoid local optima.

## Predictive Contribution of the Mediator (PCM)

Mediators are a popular mechanism for adding nuance and greater explanatory power to causal models. However, mediators pose a special challenge to generating predictions as they serve a dual role of antecedent and outcome. The Predictive Contribution of the Mediator (PCM; Danks, 2021) evaluates whether a mediating construct improves out-of-sample predictive accuracy.

PCM compares predictions from two approaches on an isolated sub-model for each mediation path:

- **DA (Direct Antecedents)**: Uses direct predictors (including the mediator) to predict the outcome.
- **EA (Earliest Antecedents)**: Uses only the earliest antecedent, propagated through the structural model.

The PCM metric is computed as: `PCM = (METRIC_EA - METRIC_DA) / METRIC_EA`

Rules of thumb: PCM < 0.05 = Weak, 0.05-0.10 = Moderate, > 0.10 = Strong.

```{r, eval=FALSE, echo=TRUE}
# Specify a mediation model
mobi_mm <- constructs(
  composite("Image",        multi_items("IMAG", 1:5)),
  composite("Expectation",  multi_items("CUEX", 1:3)),
  composite("Value",        multi_items("PERV", 1:2)),
  composite("Satisfaction", multi_items("CUSA", 1:3)),
  composite("Loyalty",      multi_items("CUSL", 1:3))
)
mobi_sm <- relationships(
  paths(from = "Image",       to = c("Expectation", "Satisfaction", "Loyalty")),
  paths(from = "Expectation", to = c("Value", "Satisfaction")),
  paths(from = "Value",       to = "Satisfaction"),
  paths(from = "Satisfaction", to = "Loyalty")
)
pls_model <- estimate_pls(mobi, mobi_mm, mobi_sm)

# Compute PCM for all mediation paths to Loyalty
pcm_result <- assess_pcm(pls_model,
                         target  = "Loyalty",
                         noFolds = 10,
                         reps    = 10)

# Print concise overview
pcm_result

# Detailed per-indicator results
summary(pcm_result)

# Visual comparison of mediation paths
plot(pcm_result)
```

## Congruence Testing

Congruence testing (Franke, Sarstedt, & Danks, 2021) evaluates whether PLS composite weights are stable across bootstrap samples by computing congruence coefficients. A congruence coefficient close to 1 indicates that the composite weight pattern is robust.

```{r, eval=FALSE, echo=TRUE}
cong_result <- congruence_test(mobi_pls,
                                nboot = 2000,
                                seed = 123)
print(cong_result)
summary(cong_result)
```

# References
- Becker, J.-M., Rai, A., Ringle, C. M., & Voelckner, F. (2013). Discovering Unobserved Heterogeneity in Structural Equation Models to Avert Validity Threats. MIS Quarterly, 37(3), 665-694.
- Danks, N. P. (2021). The Piggy in the Middle: The Role of Mediators in PLS-SEM Prediction. The DATA BASE for Advances in Information Systems, 52(SI), 24-42.
- Becker, J.-M., Richter, N. F., Ringle, C. M., & Sarstedt, M. (2026). Must-have, or maybe not? A sensitivity-based extension to necessary condition analysis. Journal of Business Research, 206, 115920.
- Cefis, M., Angelelli, M., Carpita, M. & Ciavolino, E. (2025). Confirmatory Tetrad Analysis in PLS-SEM: A Multiple Testing Correction Perspective. Social Indicators Research (advance online).
- Dul, J. (2016). Necessary Condition Analysis (NCA): Logic and Methodology of 'Necessary but Not Sufficient' Causality. Organizational Research Methods, 19(1), 10-52.
- Franke, G. R., Sarstedt, M., & Danks, N. P. (2021). An empirical comparison of factor score estimation methods. Journal of Business Research, 130, 318-334.
- Gudergan, S. P., Ringle, C. M., Wende, S. & Will, A. (2008). Confirmatory Tetrad Analysis in PLS Path Modeling. Journal of Business Research, 61(12), 1238-1249.
- Hahn, C., Johnson, M. D., Herrmann, A., & Huber, F. (2002). Capturing Customer Heterogeneity using a Finite Mixture PLS Approach. Schmalenbach Business Review, 54, 243-269.
- Hair, J.F. (Jr), Hult, T.M., Ringle, C.M., Sarstedt, M., Danks, N.P., and Adler, S. (2026). Partial Least Squares Structural Equation Modeling (PLS-SEM) Using R (Second Edition) - A Workbook. Springer.
- Hauff, S., Richter, N. F., Sarstedt, M., & Ringle, C. M. (2024). Importance and Performance in PLS-SEM and NCA: Introducing the Combined Importance-Performance Map Analysis (cIPMA). Journal of Retailing and Consumer Services, 78, 103723.
- Liengaard, B. D., Sharma, P. N., Hult, G. T. M., Jensen, M. B., Sarstedt, M., Hair, J. F., & Ringle, C. M. (2021). Prediction: coveted, yet forsaken? Introducing a cross-validated predictive ability test in partial least squares path modeling. Decision Sciences, 52(2), 362-392.
- Ray, S., Danks, N.P., Calero Valdez, A. (2026) SEMinR: Domain-specific language for building, estimating, and visualizing structural equation models in R.
- Richter, N. F., Schubring, S., Hauff, S., Ringle, C. M., & Sarstedt, M. (2020). When predictors of outcomes are necessary: guidelines for the combined use of PLS-SEM and NCA. Industrial Management & Data Systems, 120(12), 2243-2267.
- Ringle, C. M. & Sarstedt, M. (2016). Gain More Insight from Your PLS-SEM Results: The Importance-Performance Map Analysis. Industrial Management & Data Systems, 119(9), 1865-1886.
- Sarstedt, M., Becker, J.-M., Ringle, C. M. & Schwaiger, M. (2011). Uncovering and Treating Unobserved Heterogeneity with FIMIX-PLS. Schmalenbach Business Review, 63(1), 34-62.
- Sarstedt, M., Richter, N. F., Hauff, S. & Ringle, C. M. (2024). Combined Importance-Performance Map Analysis (cIPMA): A SmartPLS 4 Tutorial. Journal of Marketing Analytics, 12, 746-760.
- Sharma, P. N., Liengaard, B. D., Hair, J. F., Sarstedt, M., & Ringle, C. M. (2022). Predictive model assessment and selection in composite-based modeling using PLS-SEM: extensions and guidelines for using CVPAT. European Journal of Marketing, 57(6), 1662-1677.

---
title: "seminrExtras"
author: "Nicholas .P. Danks and Soumya Ray"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{seminrExtras}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(seminr)
library(seminrExtras)

# set the theme for plotting output
seminr_theme_set(seminr_theme_academic())

# Create measurement model ----
corp_rep_mm_ext <- constructs(
  composite("QUAL", multi_items("qual_", 1:8), weights = mode_B),
  composite("PERF", multi_items("perf_", 1:5), weights = mode_B),
  composite("CSOR", multi_items("csor_", 1:5), weights = mode_B),
  composite("ATTR", multi_items("attr_", 1:3), weights = mode_B),
  composite("COMP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  composite("CUSA", single_item("cusa")),
  composite("CUSL", multi_items("cusl_", 1:3))
)

alt_mm <- constructs(
  composite("QUAL", multi_items("qual_", 1:8), weights = mode_B),
  composite("PERF", multi_items("perf_", 1:5), weights = mode_B),
  composite("CSOR", multi_items("csor_", 1:5), weights = mode_B),
  composite("ATTR", multi_items("attr_", 1:3), weights = mode_B),
  composite("COMP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  composite("CUSA", single_item("cusa")),
  composite("CUSL", multi_items("cusl_", 1:3))
)

# Create structural model ----
corp_rep_sm_ext <- relationships(
  paths(from = c("QUAL", "PERF", "CSOR", "ATTR"), to = c("COMP", "LIKE")),
  paths(from = c("COMP", "LIKE"), to = c("CUSA", "CUSL")),
  paths(from = c("CUSA"),         to = c("CUSL"))
)
alt_sm <- relationships(
  paths(from = c("QUAL", "PERF", "CSOR", "ATTR"), to = c("COMP", "LIKE")),
  paths(from = c("COMP", "LIKE"), to = c("CUSA")),
  paths(from = c("CUSA"),         to = c("CUSL"))
)


# Estimate the models ----
established_model <- estimate_pls(
  data = corp_rep_data,
  measurement_model = corp_rep_mm_ext,
  structural_model  = corp_rep_sm_ext,
  missing = mean_replacement,
  missing_value = "-99")

competing_model <- estimate_pls(
  data = corp_rep_data,
  measurement_model = alt_mm,
  structural_model  = alt_sm,
  missing = mean_replacement,
  missing_value = "-99")


```

SEMinR (Ray, Danks, & calero-Valdez, 2026) is a domain specific language for modeling and estimating structural equation models. This is a supplementary package for SEMinR and not a standalone package. 
This package serves to provide additional extra methods and functions that can be used to analyze PLS-SEM models. 

SEMinRExtras provides advanced SEM tools which are compatible with SEMinR. In the current version, we provide an implementation of the Cross-Validated Predictive Ability Test (CVPAT) as proposed by Liengaard et al. (2021) and Sharma et al. (2022).

SEMinRExtras also serves to host the example models used in the PLS-SEM in R workbook (Hair et al., 2026).

### New features implemented with this package

- assess_cvpat_compare
- assess_cvpat

## The demo files for Hair et al. (2026)  

In order to access the demo files for the textbook, you can run the ```demo()``` function after loading the SEMinRExtras package. 

- seminr-help-debugging
- seminr-pls-cvpat
- seminr-primer-v2-chap2
- seminr-primer-v2-chap3
- seminr-primer-v2-chap4
- seminr-primer-v2-chap5
- seminr-primer-v2-chap6 
- seminr-primer-v2-chap7 
- seminr-primer-v2-chap8

E.g. ```demo("seminr-pls-cvpat", package = "seminrExtras")```

## The Example model: Corporate Reputation

We are applying the CVPAT process to the corporate reputation example bundled with SEMinR. Since we will be comparing two models, we will first estimate and plot both models. Below you will find the competing and established models which will be compared. 

```{r, fig.show='hold'}
plot(established_model)
plot(competing_model)
```

## Example
```{r, eval=FALSE, echo=TRUE}

# Create measurement model ----
corp_rep_mm_ext <- constructs(
  composite("QUAL", multi_items("qual_", 1:8), weights = mode_B),
  composite("PERF", multi_items("perf_", 1:5), weights = mode_B),
  composite("CSOR", multi_items("csor_", 1:5), weights = mode_B),
  composite("ATTR", multi_items("attr_", 1:3), weights = mode_B),
  composite("COMP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  composite("CUSA", single_item("cusa")),
  composite("CUSL", multi_items("cusl_", 1:3))
)

alt_mm <- constructs(
  composite("QUAL", multi_items("qual_", 1:8), weights = mode_B),
  composite("PERF", multi_items("perf_", 1:5), weights = mode_B),
  composite("CSOR", multi_items("csor_", 1:5), weights = mode_B),
  composite("ATTR", multi_items("attr_", 1:3), weights = mode_B),
  composite("COMP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3)),
  composite("CUSA", single_item("cusa")),
  composite("CUSL", multi_items("cusl_", 1:3))
)

# Create structural model ----
corp_rep_sm_ext <- relationships(
  paths(from = c("QUAL", "PERF", "CSOR", "ATTR"), to = c("COMP", "LIKE")),
  paths(from = c("COMP", "LIKE"), to = c("CUSA", "CUSL")),
  paths(from = c("CUSA"),         to = c("CUSL"))
)
alt_sm <- relationships(
  paths(from = c("QUAL", "PERF", "CSOR", "ATTR"), to = c("COMP", "LIKE")),
  paths(from = c("COMP", "LIKE"), to = c("CUSA")),
  paths(from = c("CUSA"),         to = c("CUSL"))
)


# Estimate the models ----
established_model <- estimate_pls(
  data = corp_rep_data,
  measurement_model = corp_rep_mm_ext,
  structural_model  = corp_rep_sm_ext,
  missing = mean_replacement,
  missing_value = "-99")

competing_model <- estimate_pls(
  data = corp_rep_data,
  measurement_model = alt_mm,
  structural_model  = alt_sm,
  missing = mean_replacement,
  missing_value = "-99")

# Function to compare the Loss of two models
compare_results <- assess_cvpat_compare(established_model = established_model,
                                        alternative_model = competing_model,
                                        testtype = "two.sided",
                                        nboot = 2000,
                                        technique = predict_DA,
                                        seed = 123,
                                        noFolds = 10,
                                        reps = 10,
                                        cores = 1)


print(compare_results,
      digits = 3)

# Assess the base model ----
assess_results <- assess_cvpat(established_model,
                               seed = 123, 
                               cores = 1)
print(assess_results$CVPAT_compare_LM,
      digits = 3)
print(assess_results$CVPAT_compare_IA,
      digits = 3)

```

## First, conduct CVPAT analysis of the established model.

```{r, echo=TRUE}
# Assess the base model ----
assess_results <- assess_cvpat(established_model,
                               seed = 123, 
                               cores = 1)
print(assess_results$CVPAT_compare_LM,
      digits = 3)
print(assess_results$CVPAT_compare_IA,
      digits = 3)
```

The established model has significantly lower predictive loss compared to both the naive benchmark IA and the LM model. Thus, we can say that the established model has predictive relevance.   

Now we compare the results:

```{r, echo=TRUE}
# Function to compare the Loss of two models
compare_results <- assess_cvpat_compare(established_model = established_model,
                                        alternative_model = competing_model,
                                        testtype = "two.sided",
                                        nboot = 2000,
                                        technique = predict_DA,
                                        seed = 123,
                                        noFolds = 10,
                                        reps = 10,
                                        cores = 1)

print(compare_results, 
      cores = 1,
      digits = 3)
```

The established model has significantly lower predictive loss compared to the competing model. Thus, we can say that the established model has superior predictive performance compared to the competing model.


# References 
- Hair, J.F. (Jr), Hult, T.M., Ringle, C.M., Sarstedt, M., Danks, N.P., and Adler, S. (2026). Partial Least Squares Structural Equation Modeling (PLS-SEM) Using R (Second Edition) - A Workbook. Springer.
- Liengaard, B. D., Sharma, P. N., Hult, G. T. M., Jensen, M. B., Sarstedt, M., Hair, J. F., & Ringle, C. M. (2021). Prediction: coveted,yet forsaken? Introducing a cross‐validated predictive ability test in partial least squares path modeling. Decision Sciences, 52(2), 362-392.
- Ray, S., Danks, N.P., Calero Valdez, A. (2026) SEMinR: Domain-specific language for building, estimating, and visualizing structural equation models in R.
- Sharma, P. N., Liengaard, B. D., Hair, J. F., Sarstedt, M., & Ringle, C. M. (2022). Predictive model assessment and selection in composite-based modeling using PLS-SEM: extensions and guidelines for using CVPAT. European journal of marketing, 57(6), 1662-1677.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_congruence.R
\name{congruence_test}
\alias{congruence_test}
\title{SEMinR function to bootstrap calculate the congruence coefficient}
\usage{
congruence_test(
  seminr_model,
  nboot = 2000,
  seed = 123,
  alpha = 0.05,
  threshold = 1
)
}
\arguments{
\item{seminr_model}{The SEMinR model for CVPAT analysis}

\item{nboot}{The number of bootstrap subsamples to execute (defaults to 2000).}

\item{seed}{The seed for reproducibility (defaults to 123).}

\item{alpha}{The required level of alpha for statistical testing (defaults
to 0.05).}

\item{threshold}{The threshold with which to compare significance testing
H0: rc < 1 (defaults to 1).}
}
\value{
A matrix of the estimated congruence coefficient and results of
significance testing.
}
\description{
`congruence_test` conducts a single bootstrapped congruence test.
}
\examples{
# Load libraries
library(seminr)
library(seminrExtras)

# Create measurement model ----
corp_rep_mm <- constructs(
  composite("QUAL", multi_items("qual_", 1:8), weights = mode_B),
  composite("PERF", multi_items("perf_", 1:5), weights = mode_B),
  composite("CSOR", multi_items("csor_", 1:5), weights = mode_B),
  composite("ATTR", multi_items("attr_", 1:3), weights = mode_B),
  composite("COMP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3))
)

# Create structural model ----
corp_rep_sm <- relationships(
  paths(from = c("QUAL", "PERF", "CSOR", "ATTR"), to = c("COMP", "LIKE"))
)

# Estimate the model ----
corp_rep_pls_model <- estimate_pls(
  data = corp_rep_data,
  measurement_model = corp_rep_mm,
  structural_model  = corp_rep_sm,
  missing = mean_replacement,
 missing_value = "-99")

# Assess the base model ----
congruence_test(seminr_model = corp_rep_pls_model,
                nboot = 20,
                seed = 123,
                alpha = 0.05,
                threshold = 1)

}
\references{
Franke, G. R., Sarstedt, M., & Danks, N. P. (2021). Assessing
measure congruence in nomological networks. Journal of Business Research,
130, 318-334.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_coa.R
\name{group_rules}
\alias{group_rules}
\title{Extract Split Rules for Deviant Groups}
\usage{
group_rules(group_name, coa_result = NULL)
}
\arguments{
\item{group_name}{A `coa_analysis` object (to get rules for all groups),
a `coa_dtree` object, or a single uppercase letter identifying a specific
group (e.g., \code{"A"}).}

\item{coa_result}{A `coa_analysis` object from [assess_coa()]. Required
only when \code{group_name} is a character group letter.}
}
\value{
A data frame (or named list of data frames) with columns
  `construct`, `gte`, and `lt` describing the construct score ranges
  that define each group.
}
\description{
Returns the decision tree split criteria that define deviant group(s).
When called with a single `coa_analysis` or `coa_dtree` object, returns
rules for all deviant groups. When called with a group name and a
`coa_analysis` object, returns rules for that specific group.
}
\seealso{
[assess_coa()], [competes()]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_cta.R
\name{assess_cta}
\alias{assess_cta}
\title{Confirmatory Tetrad Analysis for PLS-SEM (CTA-PLS)}
\usage{
assess_cta(
  seminr_model,
  constructs = NULL,
  nboot = 5000,
  seed = 123,
  alpha = 0.05,
  correction = "BH",
  borrow = TRUE
)
}
\arguments{
\item{seminr_model}{A PLS-SEM model estimated by [seminr::estimate_pls()].}

\item{constructs}{Character vector of construct names to test. If `NULL`
(default), all constructs in the model are tested.}

\item{nboot}{Number of bootstrap subsamples (default 5000).}

\item{seed}{Seed for reproducibility (default 123).}

\item{alpha}{Significance level (default 0.05).}

\item{correction}{Multiple testing correction method: `"BH"`
(Benjamini-Hochberg, default), `"bonferroni"`, or `"none"`.}

\item{borrow}{Logical. If `TRUE` (default), constructs with 2--3
indicators borrow indicators from structurally connected constructs
to reach the minimum of 4 needed for tetrad testing (Gudergan et al.,
2008, Table 1). If `FALSE`, constructs with fewer than 4 indicators
are skipped.}
}
\value{
An S3 object of class `cta_analysis` containing:
\describe{
  \item{construct_results}{Data frame summarising each construct: mode,
    number of indicators, tetrads tested, significant tetrads, and verdict.}
  \item{tetrad_details}{Named list of per-construct data frames with
    individual tetrad estimates, bootstrap CIs, and adjusted p-values.}
  \item{nboot}{Number of bootstrap subsamples used.}
  \item{alpha}{Significance level used.}
  \item{correction}{Multiple testing correction method used.}
  \item{skipped}{Character vector of constructs that were skipped.}
  \item{borrowing}{Named list of borrowing details for constructs that
    borrowed indicators (donor name, mode, and vanishing pattern).}
}
}
\description{
`assess_cta` tests whether each construct's measurement model is consistent
with a reflective (common factor) specification. Under a reflective model,
all vanishing tetrads equal zero. If any tetrad is significantly non-zero
(after multiple testing correction), the reflective specification is rejected.
}
\details{
**Minimum indicator requirement:** Without borrowing, constructs with fewer
than 4 indicators are skipped. With `borrow = TRUE` (default), constructs
with 2--3 indicators borrow from structurally adjacent constructs.
A minimum of 2 own indicators is always required.

**Borrowing rules** (Gudergan et al., 2008, Table 1): For a reflective
focal construct with 3 own indicators, 1 indicator is borrowed from an
adjacent reflective construct; all tetrads vanish under H0. For 2 own
indicators, 2 are borrowed from any adjacent construct; only the
tau_1342 tetrad vanishes. Formative focal constructs cannot be tested
via borrowing.

For HOC constructs, the lower-order constructs
(LOCs) serve as "indicators"; at least 4 LOCs are required.

**Interaction constructs** (moderation terms with `*` in the name) are
automatically excluded because their measurement specification is determined
by the interaction method, not the data.

**Multiple testing correction:** The number of tetrads grows combinatorially
with indicators. Benjamini-Hochberg (BH) correction is recommended as the
default (Cefis et al., 2025). Bonferroni is more conservative. With only
4 indicators (2 tetrads), correction has minimal impact.

**Sample size:** CTA-PLS requires adequate sample sizes for reliable results.
A warning is issued if N < 200.
}
\examples{
library(seminr)
library(seminrExtras)

# Specify measurement model
mobi_mm <- constructs(
  composite("Image",        multi_items("IMAG", 1:5)),
  composite("Expectation",  multi_items("CUEX", 1:3)),
  composite("Value",        multi_items("PERV", 1:2)),
  composite("Satisfaction", multi_items("CUSA", 1:3)),
  composite("Loyalty",      multi_items("CUSL", 1:3))
)

# Specify structural model
mobi_sm <- relationships(
  paths(from = "Image",       to = c("Expectation", "Satisfaction", "Loyalty")),
  paths(from = "Expectation", to = c("Value", "Satisfaction")),
  paths(from = "Value",       to = "Satisfaction"),
  paths(from = "Satisfaction", to = "Loyalty")
)

# Estimate the model
pls_model <- estimate_pls(data = mobi, measurement_model = mobi_mm,
                           structural_model = mobi_sm)

# Run CTA-PLS with borrowing (low nboot for example speed)
cta <- assess_cta(pls_model, nboot = 50, seed = 123)
print(cta)
summary(cta)

# Without borrowing -- only constructs with >= 4 indicators are tested
cta_no_borrow <- assess_cta(pls_model, nboot = 50, borrow = FALSE)

}
\references{
Gudergan, S. P., Ringle, C. M., Wende, S. & Will, A. (2008). Confirmatory
Tetrad Analysis in PLS Path Modeling. *Journal of Business Research*, 61(12),
1238-1249.

Cefis, M., Angelelli, M., Carpita, M. & Ciavolino, E. (2025). Confirmatory
Tetrad Analysis in PLS-SEM: A Multiple Testing Correction Perspective.
*Social Indicators Research* (advance online).
}
\seealso{
[congruence_test()] for congruence coefficient testing
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_cvpat.R
\name{assess_cvpat_compare}
\alias{assess_cvpat_compare}
\title{SEMinR function to compare CVPAT loss of two models}
\usage{
assess_cvpat_compare(
  established_model,
  alternative_model,
  testtype = "two.sided",
  nboot = 2000,
  seed = 123,
  technique = predict_DA,
  noFolds = NULL,
  reps = NULL,
  cores = NULL
)
}
\arguments{
\item{established_model}{The base seminr model for CVPAT comparison.}

\item{alternative_model}{The alternate seminr model for CVPAT comparison.}

\item{testtype}{Either "two.sided" (default) or "greater".}

\item{nboot}{The number of bootstrap subsamples to execute (defaults to 2000).}

\item{seed}{The seed for reproducibility (defaults to 123).}

\item{technique}{predict_EA or predict_DA (default).}

\item{noFolds}{Number of folds for k-fold cross validation.}

\item{reps}{Number of repetitions for cross validation.}

\item{cores}{Number of cores for parallelization.}
}
\value{
A matrix of the estimated loss and results of significance testing.
}
\description{
`assess_cvpat_compare` conducts a CVPAT significance test of loss between
two models.
}
\examples{
# Load libraries
library(seminr)
library(seminrExtras)

# Create measurement model ----
corp_rep_mm_ext <- constructs(
 composite("QUAL", multi_items("qual_", 1:8), weights = mode_B),
 composite("PERF", multi_items("perf_", 1:5), weights = mode_B),
 composite("CSOR", multi_items("csor_", 1:5), weights = mode_B),
 composite("ATTR", multi_items("attr_", 1:3), weights = mode_B),
 composite("COMP", multi_items("comp_", 1:3)),
 composite("LIKE", multi_items("like_", 1:3))
)

alt_mm <- constructs(
 composite("QUAL", multi_items("qual_", 1:8), weights = mode_B),
 composite("PERF", multi_items("perf_", 1:5), weights = mode_B),
 composite("CSOR", multi_items("csor_", 1:5), weights = mode_B),
 composite("COMP", multi_items("comp_", 1:3)),
 composite("LIKE", multi_items("like_", 1:3))
)

# Create structural model ----

corp_rep_sm_ext <- relationships(
 paths(from = c("QUAL", "PERF", "CSOR", "ATTR"), to = c("COMP", "LIKE"))
)
alt_sm <- relationships(
 paths(from = c("QUAL", "PERF", "CSOR"), to = c("COMP", "LIKE"))
)

# Estimate the model ----
established_model <- estimate_pls(
 data = corp_rep_data,
 measurement_model = corp_rep_mm_ext,
 structural_model  = corp_rep_sm_ext,
 missing = mean_replacement,
 missing_value = "-99")

alternative_model <- estimate_pls(
 data = corp_rep_data,
 measurement_model = alt_mm,
 structural_model  = alt_sm,
 missing = mean_replacement,
 missing_value = "-99")

# Function to compare the Loss of two models
assess_cvpat_compare(established_model,
                    alternative_model ,
                    testtype = "two.sided",
                    nboot = 20,
                    seed = 123,
                    technique = predict_DA,
                    noFolds = 5,
                    reps = 1,
                    cores = 1)

}
\references{
Sharma, P. N., Liengaard, B. D., Hair, J. F., Sarstedt, M., &
Ringle, C. M. (2022). Predictive model assessment and selection in
composite-based modeling using PLS-SEM: extensions and guidelines for
using CVPAT. European journal of marketing, 57(6), 1662-1677.

Liengaard, B. D., Sharma, P. N., Hult, G. T. M., Jensen, M. B.,
Sarstedt, M., Hair, J. F., & Ringle, C. M. (2021). Prediction: coveted,
yet forsaken? Introducing a cross‐validated predictive ability test in
partial least squares path modeling. Decision Sciences, 52(2), 362-392.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_coa.R
\name{plot.coa_analysis}
\alias{plot.coa_analysis}
\title{Plot COA Results}
\usage{
\method{plot}{coa_analysis}(x, type = c("pd", "groups", "tree"), ...)
}
\arguments{
\item{x}{A `coa_analysis` object from `assess_coa()`.}

\item{type}{One of `"pd"` (predictive deviance scatter), `"groups"`
(average construct scores per group), or `"tree"` (rpart tree diagram).}

\item{...}{Additional arguments passed to the underlying plot function.}
}
\description{
Plot COA Results
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_nca.R
\name{plot.nca_analysis}
\alias{plot.nca_analysis}
\title{Plot NCA Results}
\usage{
\method{plot}{nca_analysis}(x, type = c("scatter", "effects"), ...)
}
\arguments{
\item{x}{An \code{nca_analysis} object from \code{assess_nca()}.}

\item{type}{One of \code{"scatter"} (ceiling line scatter plots) or
\code{"effects"} (bar plot of effect sizes).}

\item{...}{Additional arguments passed to the underlying plot function.}
}
\description{
Plot NCA Results
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_pos.R
\name{pos_segments}
\alias{pos_segments}
\title{Extract Segment-Specific PLS Models from PLS-POS Results}
\usage{
pos_segments(pos_result)
}
\arguments{
\item{pos_result}{A \code{pos_analysis} object from \code{assess_pos()}.}
}
\value{
A named list of K seminr model objects.
}
\description{
Returns a list of fully re-estimated PLS models, one per segment, using the
hard segment assignment from PLS-POS.
}
\seealso{
[assess_pos()] for running PLS-POS segmentation
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_coa.R
\name{assess_coa}
\alias{assess_coa}
\title{Composite Overfit Analysis (COA)}
\usage{
assess_coa(
  seminr_model,
  focal_construct,
  deviance_bounds = c(0.025, 0.975),
  params = "path_coef",
  technique = predict_DA,
  noFolds = 10,
  reps = 1,
  cores = NULL,
  seed = 123,
  predict_model = NULL
)
}
\arguments{
\item{seminr_model}{An estimated SEMinR model from `estimate_pls()`.}

\item{focal_construct}{Name of the endogenous construct to analyze (character).}

\item{deviance_bounds}{Two-element numeric vector of quantile bounds for
identifying deviant cases (default `c(0.025, 0.975)`).}

\item{params}{Character vector of model parameters to assess for instability.
Valid values include `"path_coef"`, `"outer_weights"`, `"outer_loadings"`,
`"rSquared"` (default `"path_coef"`).}

\item{technique}{Prediction technique: `predict_DA` (default) or `predict_EA`.}

\item{noFolds}{Number of folds for k-fold cross-validation (default 10).}

\item{reps}{Number of CV repetitions (default 1).}

\item{cores}{Number of cores for parallel CV (default NULL = sequential).}

\item{seed}{Random seed for reproducibility (default 123).}

\item{predict_model}{Optional pre-computed `predict_pls_model` object. If
provided, skips the cross-validation step (saves time when predictions
have already been computed).}
}
\value{
An object of class `coa_analysis` containing:
  \item{pls_model}{The original estimated model}
  \item{focal_construct}{Name of the analyzed construct}
  \item{deviance_bounds}{Quantile bounds used}
  \item{predictive_deviance}{Predictive deviance results (class `coa_deviance`)}
  \item{deviance_tree}{Deviance tree and deviant groups (class `coa_dtree`)}
  \item{unstable}{Parameter instability results (class `coa_unstable`)}
}
\description{
`assess_coa` conducts a full Composite Overfit Analysis on a PLS-SEM model.
It computes predictive deviance for a focal construct, identifies groups of
deviant cases using a decision tree, and assesses parameter instability.
}
\examples{
\donttest{
library(seminr)
library(seminrExtras)

mobi_mm <- constructs(
  composite("Image",  multi_items("IMAG", 1:5)),
  composite("Value",  multi_items("PERV", 1:2)),
  composite("Satisfaction", multi_items("CUSA", 1:3)),
  composite("Loyalty", multi_items("CUSL", 1:3))
)

mobi_sm <- relationships(
  paths(from = c("Image", "Value"), to = "Satisfaction"),
  paths(from = "Satisfaction", to = "Loyalty")
)

mobi_pls <- estimate_pls(
  data = mobi,
  measurement_model = mobi_mm,
  structural_model  = mobi_sm
)

coa_result <- assess_coa(mobi_pls,
                          focal_construct = "Loyalty",
                          noFolds = 10,
                          cores = 1,
                          seed = 123)
print(coa_result)
summary(coa_result)
plot(coa_result, type = "pd")
}

}
\seealso{
[predictive_deviance()], [deviance_tree()], [unstable_params()],
  [group_rules()], [competes()]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_pos.R
\name{assess_pos}
\alias{assess_pos}
\title{PLS-POS: Prediction-Oriented Segmentation}
\usage{
assess_pos(
  seminr_model,
  K = 2,
  nstart = 10,
  max_iter = 100,
  search_depth = NULL,
  min_segment_size = NULL,
  seed = 123
)
}
\arguments{
\item{seminr_model}{An estimated SEMinR model from \code{estimate_pls()}.}

\item{K}{Integer >= 2. Number of segments to extract.}

\item{nstart}{Number of random starting partitions (default 10). The best
solution (highest objective) is returned.}

\item{max_iter}{Maximum hill-climbing iterations per start (default 100).}

\item{search_depth}{Maximum number of candidates to evaluate per iteration
before accepting the first improvement (default = N, the sample size).
Use a smaller value for faster but potentially less optimal results.}

\item{min_segment_size}{Minimum observations per segment. Default is
\code{max(10, max_predictors + 2)} where max_predictors is the largest
number of predictors for any endogenous construct.}

\item{seed}{Random seed for reproducibility.}
}
\value{
An S3 object of class \code{"pos_analysis"} with components:
  \describe{
    \item{K}{Number of segments}
    \item{segment_assignment}{Integer vector of segment assignments (length N)}
    \item{segment_sizes}{Named integer vector of segment sizes}
    \item{segment_models}{List of K re-estimated seminr model objects}
    \item{segment_rsquared}{Matrix of R-squared values (endogenous x K)}
    \item{segment_paths}{List of K path coefficient matrices}
    \item{objective}{Sum of R-squared (objective criterion)}
    \item{converged}{Whether the best start converged}
    \item{iterations}{Number of iterations in the best start}
    \item{nstart}{Number of random starts attempted}
    \item{all_objectives}{Objective values for all starts}
  }
}
\description{
Performs prediction-oriented segmentation (PLS-POS) to uncover unobserved
heterogeneity in PLS-SEM models. PLS-POS uses a deterministic hill-climbing
approach to maximize the sum of R-squared values across all endogenous
constructs across K segments (Becker et al., 2013).
}
\details{
Unlike FIMIX-PLS, PLS-POS makes no distributional assumptions and can
detect heterogeneity in both structural and (formative) measurement models.
}
\examples{
\donttest{
library(seminr)

mobi_mm <- constructs(
  composite("Image",        multi_items("IMAG", 1:5)),
  composite("Expectation",  multi_items("CUEX", 1:3)),
  composite("Satisfaction", multi_items("CUSA", 1:3)),
  composite("Loyalty",      multi_items("CUSL", 1:3))
)
mobi_sm <- relationships(
  paths(from = "Image",       to = c("Expectation", "Satisfaction", "Loyalty")),
  paths(from = "Expectation", to = "Satisfaction"),
  paths(from = "Satisfaction", to = "Loyalty")
)
pls_model <- estimate_pls(data = mobi, measurement_model = mobi_mm,
                           structural_model = mobi_sm)

pos_result <- assess_pos(pls_model, K = 2, nstart = 2, max_iter = 10,
                          seed = 123)
print(pos_result)
summary(pos_result)
}

}
\references{
Becker, J.-M., Rai, A., Ringle, C. M., & Voelckner, F. (2013). Discovering
Unobserved Heterogeneity in Structural Equation Models to Avert Validity
Threats. \emph{MIS Quarterly}, 37(3), 665-694.
}
\seealso{
[assess_fimix()] for probabilistic (EM-based) segmentation
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_fimix.R
\name{assess_fimix_compare}
\alias{assess_fimix_compare}
\title{Compare FIMIX-PLS Solutions Across Multiple K Values}
\usage{
assess_fimix_compare(
  seminr_model,
  K_range = 2:5,
  nstart = 10,
  max_iter = 5000,
  stop_criterion = 1e-06,
  seed = 123
)
}
\arguments{
\item{seminr_model}{An estimated SEMinR model from \code{estimate_pls()}.}

\item{K_range}{Integer vector of K values to evaluate (default 2:5).}

\item{nstart}{Integer; number of random EM starts (default 10).}

\item{max_iter}{Integer; maximum EM iterations per start (default 5000).}

\item{stop_criterion}{Numeric; convergence threshold on log-likelihood
change (default 1e-6).}

\item{seed}{Random seed for reproducibility (default 123).}
}
\value{
An object of class \code{fimix_comparison} containing:
  \item{solutions}{Named list of \code{fimix_analysis} objects (one per K)}
  \item{fit_table}{Data frame with K as rows and criteria as columns}
  \item{K_range}{The evaluated K values}
  \item{pls_model}{The original model}
}
\description{
\code{assess_fimix_compare} runs \code{\link{assess_fimix}} for each value
of K in \code{K_range} and produces a comparison table of information
criteria to guide segment number selection.
}
\details{
The recommended approach is to inspect AIC3 and CAIC jointly: when both
agree on K, accuracy reaches approximately 84\% (Sarstedt et al., 2011).
AIC4 is the best-performing single criterion. Entropy (EN) indicates
classification quality but should not be used alone to select K.
}
\examples{
library(seminr)
library(seminrExtras)

mobi_mm <- constructs(
  composite("Image",        multi_items("IMAG", 1:5)),
  composite("Expectation",  multi_items("CUEX", 1:3)),
  composite("Value",        multi_items("PERV", 1:2)),
  composite("Satisfaction", multi_items("CUSA", 1:3)),
  composite("Loyalty",      multi_items("CUSL", 1:3))
)

mobi_sm <- relationships(
  paths(from = "Image",       to = c("Expectation", "Satisfaction", "Loyalty")),
  paths(from = "Expectation", to = c("Value", "Satisfaction")),
  paths(from = "Value",       to = "Satisfaction"),
  paths(from = "Satisfaction",to = "Loyalty")
)

mobi_pls <- estimate_pls(data = mobi,
                          measurement_model = mobi_mm,
                          structural_model  = mobi_sm)

\donttest{
fimix_comp <- assess_fimix_compare(mobi_pls, K_range = 2:4,
                                     nstart = 5, seed = 123)
print(fimix_comp)
plot(fimix_comp)
}

}
\references{
Sarstedt, M., Becker, J.-M., Ringle, C. M. & Schwaiger, M. (2011).
Uncovering and Treating Unobserved Heterogeneity with FIMIX-PLS: Which
Model Selection Criterion Provides an Appropriate Number of Segments?
Schmalenbach Business Review, 63(1), 34-62.
}
\seealso{
\code{\link{assess_fimix}} for single-K FIMIX analysis
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_fimix.R
\name{assess_fimix}
\alias{assess_fimix}
\title{FIMIX-PLS Analysis for PLS-SEM}
\usage{
assess_fimix(
  seminr_model,
  K = 2,
  nstart = 10,
  max_iter = 5000,
  stop_criterion = 1e-06,
  seed = 123
)
}
\arguments{
\item{seminr_model}{An estimated SEMinR model from \code{estimate_pls()}.}

\item{K}{Integer; the number of segments (default 2). Must be >= 2.}

\item{nstart}{Integer; number of random EM starts (default 10).}

\item{max_iter}{Integer; maximum EM iterations per start (default 5000).}

\item{stop_criterion}{Numeric; convergence threshold on log-likelihood
change (default 1e-6).}

\item{seed}{Random seed for reproducibility (default 123).}
}
\value{
An object of class \code{fimix_analysis} containing:
  \item{K}{Number of segments}
  \item{segment_proportions}{Named numeric vector of mixing proportions}
  \item{posterior}{N x K matrix of posterior segment probabilities}
  \item{segment_assignment}{Integer vector of hard assignments (argmax)}
  \item{segment_paths}{List of K path coefficient matrices}
  \item{segment_intercepts}{List of K intercept vectors}
  \item{segment_variances}{J x K matrix of residual variances}
  \item{log_likelihood}{Final log-likelihood}
  \item{n_parameters}{Number of free parameters}
  \item{info_criteria}{Named vector of information criteria}
  \item{converged}{Logical; whether EM converged}
  \item{iterations}{Number of EM iterations}
  \item{n_starts_completed}{Number of random starts completed}
  \item{pls_model}{The original estimated seminr model}
  \item{n_obs}{Sample size}
}
\description{
\code{assess_fimix} runs the FIMIX-PLS (Finite Mixture PLS) procedure to
identify unobserved heterogeneity in PLS-SEM structural models. The EM
algorithm probabilistically assigns observations to \code{K} latent
segments, each with segment-specific structural path coefficients.
}
\details{
FIMIX-PLS operates on construct scores from the estimated PLS model.
It fits a mixture of regressions to each structural equation simultaneously,
estimating segment-specific intercepts, path coefficients, and residual
variances. Multiple random starts (\code{nstart}) are used to avoid local
optima; the solution with the highest log-likelihood is retained.
}
\examples{
library(seminr)
library(seminrExtras)

mobi_mm <- constructs(
  composite("Image",        multi_items("IMAG", 1:5)),
  composite("Expectation",  multi_items("CUEX", 1:3)),
  composite("Value",        multi_items("PERV", 1:2)),
  composite("Satisfaction", multi_items("CUSA", 1:3)),
  composite("Loyalty",      multi_items("CUSL", 1:3))
)

mobi_sm <- relationships(
  paths(from = "Image",       to = c("Expectation", "Satisfaction", "Loyalty")),
  paths(from = "Expectation", to = c("Value", "Satisfaction")),
  paths(from = "Value",       to = "Satisfaction"),
  paths(from = "Satisfaction",to = "Loyalty")
)

mobi_pls <- estimate_pls(data = mobi,
                          measurement_model = mobi_mm,
                          structural_model  = mobi_sm)

\donttest{
fimix_result <- assess_fimix(mobi_pls, K = 2, nstart = 5, seed = 123)
print(fimix_result)
summary(fimix_result)
}

}
\references{
Hahn, C., Johnson, M. D., Herrmann, A., & Huber, F. (2002). Capturing
Customer Heterogeneity using a Finite Mixture PLS Approach. Schmalenbach
Business Review, 54, 243-269.

Sarstedt, M., Becker, J.-M., Ringle, C. M. & Schwaiger, M. (2011).
Uncovering and Treating Unobserved Heterogeneity with FIMIX-PLS: Which
Model Selection Criterion Provides an Appropriate Number of Segments?
Schmalenbach Business Review, 63(1), 34-62.
}
\seealso{
\code{\link{assess_fimix_compare}} for comparing multiple K values
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_coa.R
\name{predictive_deviance}
\alias{predictive_deviance}
\title{Compute Predictive Deviance}
\usage{
predictive_deviance(
  seminr_model,
  focal_construct,
  technique = predict_DA,
  noFolds = 10,
  reps = 1,
  cores = NULL,
  seed = 123,
  predict_model = NULL
)
}
\arguments{
\item{seminr_model}{An estimated SEMinR model.}

\item{focal_construct}{Name of the construct to analyze.}

\item{technique}{Prediction technique (default `predict_DA`).}

\item{noFolds}{Number of CV folds (default 10).}

\item{reps}{Number of CV repetitions (default 1).}

\item{cores}{Number of cores for parallel CV (default NULL).}

\item{seed}{Random seed (default 123).}

\item{predict_model}{Optional pre-computed `predict_pls_model` object.}
}
\value{
An object of class `coa_deviance` containing PD values, metrics,
  and the data frame used for tree construction.
}
\description{
Computes per-case predictive deviance (PD) for a focal construct using
k-fold cross-validated predictions from `predict_pls()`.
}
\details{
PD = in-sample fitted score - out-of-sample predicted score.
High PD indicates a case that is well-fitted in-sample but poorly predicted
out-of-sample, suggesting overfit.
}
\seealso{
[assess_coa()], [deviance_tree()]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_coa.R
\name{unstable_params}
\alias{unstable_params}
\title{Assess Parameter Instability}
\usage{
unstable_params(seminr_model, deviant_groups, params = "path_coef")
}
\arguments{
\item{seminr_model}{An estimated SEMinR model.}

\item{deviant_groups}{A `coa_dtree` object from [deviance_tree()], or a
named list of integer vectors of case indices. When a `coa_dtree` is
passed, the `$deviant_groups` element is extracted automatically.}

\item{params}{Character vector of model parameters to diff
(default `"path_coef"`).}
}
\value{
An object of class `coa_unstable`: a named list (one entry per group)
  each containing `cases` and `param_diffs`.
}
\description{
Identifies unstable model parameters by removing each deviant group
and re-estimating the model, then computing the difference in specified
parameters.
}
\seealso{
[assess_coa()], [deviance_tree()]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_fimix.R
\name{plot.fimix_analysis}
\alias{plot.fimix_analysis}
\title{Plot FIMIX-PLS Results}
\usage{
\method{plot}{fimix_analysis}(x, type = c("segments", "paths"), ...)
}
\arguments{
\item{x}{A \code{fimix_analysis} object from \code{assess_fimix()}.}

\item{type}{One of \code{"segments"} (segment proportion bar plot) or
\code{"paths"} (grouped bar plot of path coefficients across segments).}

\item{...}{Additional arguments passed to \code{plot()}.}
}
\description{
Plot FIMIX-PLS Results
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_nca.R
\name{assess_nca_esse}
\alias{assess_nca_esse}
\title{NCA with Effect Size Sensitivity Extension (NCA-ESSE)}
\usage{
assess_nca_esse(
  seminr_model,
  target,
  predictors = NULL,
  thresholds = seq(0, 0.05, by = 0.005),
  ceiling = "ce_fdh",
  test.rep = 0,
  steps = 10,
  seed = 123,
  ...
)
}
\arguments{
\item{seminr_model}{An estimated SEMinR model from \code{estimate_pls()}.}

\item{target}{Name of the endogenous (outcome) construct.}

\item{predictors}{Optional character vector of predictor construct names.
If \code{NULL} (default), auto-detected from the structural model.}

\item{thresholds}{Numeric vector of ECDF thresholds to evaluate
(default \code{seq(0, 0.05, by = 0.005)}). Values must be in [0, 1].}

\item{ceiling}{Ceiling technique (default \code{"ce_fdh"}). The analytical
benchmark is derived for CE-FDH; other techniques will trigger a warning.}

\item{test.rep}{Number of permutation test repetitions at each threshold
(default 0). Set > 0 to obtain p-values, but note this multiplies
computation time by the number of thresholds.}

\item{steps}{Number of steps in the bottleneck table (default 10).}

\item{seed}{Random seed for reproducibility (default 123).}

\item{...}{Additional arguments (currently unused).}
}
\value{
An object of class \code{nca_esse} containing:
  \item{effect_sizes}{Matrix of empirical effect sizes (thresholds x predictors)}
  \item{benchmark}{Matrix of theoretical benchmark effect sizes}
  \item{delta}{Matrix of empirical minus benchmark differences}
  \item{significance}{Matrix of p-values (NULL if test.rep = 0)}
  \item{pls_model}{The original estimated seminr model}
  \item{target}{Name of the target construct}
  \item{predictors}{Character vector of predictor names}
  \item{thresholds}{Numeric vector of ECDF thresholds used}
  \item{ceiling}{Ceiling technique used}
  \item{n_obs}{Number of observations}
}
\description{
\code{assess_nca_esse} applies the NCA-ESSE method (Becker et al., 2026)
to assess how sensitive NCA effect sizes are to extreme response patterns.
It systematically varies the ECDF ceiling threshold and compares empirical
effect size changes against a theoretical benchmark (joint uniform
distribution) to determine whether a necessary condition is robust.
}
\details{
The method proceeds in three steps:
\enumerate{
  \item Run standard NCA (threshold 0\%) to identify potential necessary
    conditions
  \item Apply NCA-ESSE: vary the ECDF threshold from 0\% to a maximum
    (default 5\%), computing CE-FDH effect sizes at each level and
    comparing against the uniform benchmark
  \item (Optional) Select a threshold where the empirical effect size
    meaningfully exceeds the benchmark for further evaluation
}

At each threshold \code{t}, observations whose joint ECDF_NCA value
(P(X <= x, Y >= y)) is at most \code{t} are treated as extreme and
excluded before computing the CE-FDH ceiling. The uniform benchmark
d = t(1 - ln(t)) gives the expected effect size if no necessity exists.
}
\examples{
library(seminr)
library(seminrExtras)

mobi_mm <- constructs(
  composite("Image",        multi_items("IMAG", 1:5)),
  composite("Value",        multi_items("PERV", 1:2)),
  composite("Satisfaction", multi_items("CUSA", 1:3))
)

mobi_sm <- relationships(
  paths(from = c("Image", "Value"), to = "Satisfaction")
)

mobi_pls <- estimate_pls(data = mobi,
                          measurement_model = mobi_mm,
                          structural_model  = mobi_sm)

\donttest{
esse_result <- assess_nca_esse(mobi_pls,
                                target = "Satisfaction",
                                seed = 123)
print(esse_result)
plot(esse_result, type = "sensitivity")
}

}
\references{
Becker, J.-M., Richter, N. F., Ringle, C. M., & Sarstedt, M. (2026).
Must-have, or maybe not? A sensitivity-based extension to necessary
condition analysis. Journal of Business Research, 206, 115920.
}
\seealso{
\code{\link{assess_nca}} for standard NCA analysis
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_cipma.R
\name{assess_ipma}
\alias{assess_ipma}
\title{Importance-Performance Map Analysis (IPMA) for PLS-SEM}
\usage{
assess_ipma(seminr_model, target, scale_min = 1, scale_max = 7, seed = 123)
}
\arguments{
\item{seminr_model}{An estimated SEMinR model from \code{estimate_pls()}.}

\item{target}{Name of the target (endogenous) construct for the IPMA.}

\item{scale_min}{Minimum of the measurement scale (default 1).}

\item{scale_max}{Maximum of the measurement scale (default 7).}

\item{seed}{Random seed for reproducibility (default 123).}
}
\value{
An object of class \code{cipma_analysis} (see
  \code{\link{assess_cipma}} for details). The \code{nca} element will
  be \code{NULL}.
}
\description{
\code{assess_ipma} conducts an Importance-Performance Map Analysis (IPMA)
for a given target construct. It computes each predecessor's importance
(unstandardized total effect on the target) and performance (rescaled
0--100 mean construct score), identifying constructs with high importance
but low performance as priority areas for improvement.
}
\details{
This is a convenience wrapper around \code{\link{assess_cipma}} with
\code{nca = FALSE}. Use \code{\link{assess_cipma}} instead if you also
want Necessary Condition Analysis (NCA) to produce a combined IPMA.
}
\examples{
library(seminr)
library(seminrExtras)

mobi_mm <- constructs(
  composite("Image",        multi_items("IMAG", 1:5)),
  composite("Expectation",  multi_items("CUEX", 1:3)),
  composite("Value",        multi_items("PERV", 1:2)),
  composite("Satisfaction", multi_items("CUSA", 1:3)),
  composite("Loyalty",      multi_items("CUSL", 1:3))
)

mobi_sm <- relationships(
  paths(from = "Image",       to = c("Expectation", "Satisfaction", "Loyalty")),
  paths(from = "Expectation", to = c("Value", "Satisfaction")),
  paths(from = "Value",       to = "Satisfaction"),
  paths(from = "Satisfaction",to = "Loyalty")
)

mobi_pls <- estimate_pls(data = mobi,
                          measurement_model = mobi_mm,
                          structural_model  = mobi_sm)

\donttest{
ipma_result <- assess_ipma(mobi_pls,
                            target = "Loyalty",
                            scale_min = 1,
                            scale_max = 10)
print(ipma_result)
plot(ipma_result)
}

}
\references{
Ringle, C. M. & Sarstedt, M. (2016). Gain More Insight from Your PLS-SEM
Results: The Importance-Performance Map Analysis. Industrial Management &
Data Systems, 119(9), 1865-1886.
}
\seealso{
\code{\link{assess_cipma}} for cIPMA (IPMA + NCA)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_coa.R
\name{competes}
\alias{competes}
\title{Report Competing Split Criteria}
\usage{
competes(node_id, dtree = NULL)
}
\arguments{
\item{node_id}{A `coa_dtree` object (to get competing splits for all
group roots), or an integer node ID from the rpart tree.}

\item{dtree}{A `coa_dtree` object (from `coa_result$deviance_tree`).
Required only when \code{node_id} is an integer.}
}
\value{
A data frame (or named list of data frames) with columns
  `criterion`, `sign`, `value`, and `improve`.
}
\description{
Shows all competing split variables at tree nodes, ranked by improvement.
Useful for understanding which constructs nearly split instead of the
chosen variable.
}
\details{
When called with a single `coa_dtree` object, returns competing splits
at all deviant group root nodes. When called with a node ID and a
`coa_dtree` object, returns competing splits at that specific node.
}
\seealso{
[group_rules()], [assess_coa()]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_cvpat.R
\name{assess_cvpat}
\alias{assess_cvpat}
\title{Assess single model CVPAT against benchmarks}
\usage{
assess_cvpat(
  seminr_model,
  testtype = "two.sided",
  nboot = 2000,
  seed = 123,
  technique = predict_DA,
  noFolds = NULL,
  reps = NULL,
  cores = NULL
)
}
\arguments{
\item{seminr_model}{The SEMinR model for CVPAT analysis}

\item{testtype}{Either "two.sided" (default) or "greater".}

\item{nboot}{The number of bootstrap subsamples to execute (defaults to 2000).}

\item{seed}{The seed for reproducibility (defaults to 123).}

\item{technique}{predict_EA or predict_DA (default).}

\item{noFolds}{Number of folds for k-fold cross validation.}

\item{reps}{Number of repetitions for cross validation.}

\item{cores}{Number of cores for parallelization.}
}
\value{
A list containing two matrices: CVPAT_compare_LM (comparison with
  linear model) and CVPAT_compare_IA (comparison with indicator average).
}
\description{
`assess_cvpat` conducts a single model CVPAT assessment against item average
and linear model prediction benchmarks. Use this to determine whether your
PLS model has meaningful predictive power.
}
\details{
Two benchmarks are used:
- **Linear Model (LM)**: Multiple regression predicting each item
- **Indicator Average (IA)**: Naive benchmark using training set means

If PLS loss < LM loss (significantly), the structural model adds predictive value.
If PLS loss < IA loss (significantly), the model has basic predictive relevance.
}
\examples{
# Load libraries
library(seminr)
library(seminrExtras)

# Create measurement model ----
corp_rep_mm_ext <- constructs(
  composite("QUAL", multi_items("qual_", 1:8), weights = mode_B),
  composite("PERF", multi_items("perf_", 1:5), weights = mode_B),
  composite("CSOR", multi_items("csor_", 1:5), weights = mode_B),
  composite("ATTR", multi_items("attr_", 1:3), weights = mode_B),
  composite("COMP", multi_items("comp_", 1:3)),
  composite("LIKE", multi_items("like_", 1:3))
)

# Create structural model ----
corp_rep_sm_ext <- relationships(
  paths(from = c("QUAL", "PERF", "CSOR", "ATTR"), to = c("COMP", "LIKE"))
)

# Estimate the model ----
corp_rep_pls_model_ext <- estimate_pls(
  data = corp_rep_data,
  measurement_model = corp_rep_mm_ext,
  structural_model  = corp_rep_sm_ext,
  missing = mean_replacement,
  missing_value = "-99")

# Assess the base model ----
assess_cvpat(seminr_model = corp_rep_pls_model_ext,
             testtype = "two.sided",
             nboot = 20,
             seed = 123,
             technique = predict_DA,
             noFolds = 5,
             reps = 1,
             cores = 1)

}
\references{
Sharma, P. N., Liengaard, B. D., Hair, J. F., Sarstedt, M., &
Ringle, C. M. (2023). Predictive model assessment and selection in
composite-based modeling using PLS-SEM: extensions and guidelines for
using CVPAT. European journal of marketing, 57(6), 1662-1677.

Liengaard, B. D., Sharma, P. N., Hult, G. T. M., Jensen, M. B.,
Sarstedt, M., Hair, J. F., & Ringle, C. M. (2021). Prediction: coveted,
yet forsaken? Introducing a cross‐validated predictive ability test in
partial least squares path modeling. Decision Sciences, 52(2), 362-392.
}
\seealso{
[assess_cvpat_compare()] for comparing two PLS models
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_nca.R
\name{plot.nca_esse}
\alias{plot.nca_esse}
\title{Plot NCA-ESSE Results}
\usage{
\method{plot}{nca_esse}(x, type = c("sensitivity", "difference"), ...)
}
\arguments{
\item{x}{An \code{nca_esse} object from \code{assess_nca_esse()}.}

\item{type}{One of \code{"sensitivity"} (effect size vs threshold, Fig. 4
in Becker et al.) or \code{"difference"} (incremental empirical minus
benchmark difference, Fig. 6 in Becker et al.).}

\item{...}{Additional arguments passed to the underlying plot function.}
}
\description{
Plot NCA-ESSE Results
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_pcm.R
\name{assess_pcm}
\alias{assess_pcm}
\title{Predictive Contribution of the Mediator (PCM)}
\usage{
assess_pcm(seminr_model, target = NULL, noFolds = 10, reps = 10)
}
\arguments{
\item{seminr_model}{A PLS model estimated by \code{seminr::estimate_pls()}.}

\item{target}{Character string specifying the outcome construct. If
\code{NULL}, auto-detects when a single final endogenous construct exists.}

\item{noFolds}{Integer, number of folds for cross-validation (default 10).}

\item{reps}{Integer, number of cross-validation repetitions (default 10).}
}
\value{
An object of class \code{pcm_analysis} containing:
\describe{
  \item{target}{The outcome construct name.}
  \item{mediation_paths}{List of identified mediation triples
    (antecedent, mediator, target).}
  \item{pcm_results}{List of per-path results, each with RMSE/MAE for DA
    and EA approaches plus PCM values per indicator.}
  \item{noFolds}{Number of cross-validation folds used.}
  \item{reps}{Number of cross-validation repetitions used.}
}
}
\description{
Evaluates the predictive contribution of mediating constructs in a PLS-SEM
model. For each mediation path (antecedent -> mediator -> target), computes
the PCM metric by comparing cross-validated predictions from the Direct
Antecedents (DA) and Earliest Antecedents (EA) approaches on an isolated
partial mediation sub-model.
}
\details{
The PCM metric (Danks, 2021) quantifies the predictive improvement due to
the mediator. It is computed by isolating each mediation path into a partial
mediation sub-model (X -> M -> Y with direct path X -> Y), then comparing
out-of-sample predictions from two approaches:

\itemize{
  \item \strong{DA (Direct Antecedents)}: Predicts Y using its direct
    structural predictors (M and X).
  \item \strong{EA (Earliest Antecedents)}: Predicts Y using only the
    earliest antecedent (X), propagated through the structural model.
}

\deqn{PCM = \frac{METRIC_{EA} - METRIC_{DA}}{METRIC_{EA}}}

Rules of thumb for PCM interpretation:
\itemize{
  \item \strong{0 to 0.05}: Weak predictive contribution
  \item \strong{0.05 to 0.10}: Moderate predictive contribution
  \item \strong{Greater than 0.10}: Strong predictive contribution
  \item \strong{Negative}: Mediator damages predictive accuracy
}
}
\examples{
\donttest{
library(seminr)
data(mobi)

# Simple mediation: Image -> Satisfaction -> Loyalty
mobi_mm <- constructs(
  composite("Image",        multi_items("IMAG", 1:5)),
  composite("Satisfaction", multi_items("CUSA", 1:3)),
  composite("Loyalty",      multi_items("CUSL", 1:3))
)
mobi_sm <- relationships(
  paths(from = "Image", to = "Satisfaction"),
  paths(from = "Satisfaction", to = "Loyalty"),
  paths(from = "Image", to = "Loyalty")
)
model <- estimate_pls(mobi, mobi_mm, mobi_sm)

# Compute PCM
pcm <- assess_pcm(model, target = "Loyalty", noFolds = 10, reps = 10)
pcm
summary(pcm)
}

}
\references{
Danks, N. P. (2021). The Piggy in the Middle: The Role of
Mediators in PLS-SEM Prediction. \emph{The DATA BASE for Advances in
Information Systems}, 52(SI), 24-42.
}
\seealso{
\code{\link[seminr]{predict_pls}} for the underlying prediction
  methods.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_cipma.R
\name{plot.cipma_analysis}
\alias{plot.cipma_analysis}
\title{Plot cIPMA Results}
\usage{
\method{plot}{cipma_analysis}(
  x,
  type = c("cipma", "ipma"),
  importance_metric = c("unstandardized", "standardized"),
  ...
)
}
\arguments{
\item{x}{A \code{cipma_analysis} object from \code{assess_cipma()}.}

\item{type}{One of \code{"cipma"} (default, with NCA overlay) or
\code{"ipma"} (standard IPMA without NCA distinction).}

\item{importance_metric}{One of \code{"unstandardized"} (default) or
\code{"standardized"} to choose which total effect is plotted on the
x-axis.}

\item{...}{Additional arguments passed to \code{plot()}.}
}
\description{
Produces the Importance-Performance Map with optional NCA overlay.
When \code{type = "cipma"} (default), necessary conditions are
highlighted with filled red circles; other constructs use open blue circles.
When \code{type = "ipma"}, all constructs use the same symbol.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_pos.R
\name{assess_pos_compare}
\alias{assess_pos_compare}
\title{Compare PLS-POS Solutions Across K Values}
\usage{
assess_pos_compare(
  seminr_model,
  K_range = 2:5,
  nstart = 10,
  max_iter = 100,
  search_depth = NULL,
  min_segment_size = NULL,
  seed = 123
)
}
\arguments{
\item{seminr_model}{An estimated SEMinR model from \code{estimate_pls()}.}

\item{K_range}{Integer vector of K values to compare (default \code{2:5}).}

\item{nstart}{Number of random starting partitions (default 10). The best
solution (highest objective) is returned.}

\item{max_iter}{Maximum hill-climbing iterations per start (default 100).}

\item{search_depth}{Maximum number of candidates to evaluate per iteration
before accepting the first improvement (default = N, the sample size).
Use a smaller value for faster but potentially less optimal results.}

\item{min_segment_size}{Minimum observations per segment. Default is
\code{max(10, max_predictors + 2)} where max_predictors is the largest
number of predictors for any endogenous construct.}

\item{seed}{Random seed for reproducibility.}
}
\value{
An S3 object of class \code{"pos_comparison"} with components:
  \describe{
    \item{solutions}{Named list of \code{pos_analysis} objects}
    \item{fit_table}{Data frame comparing K values}
    \item{K_range}{The K values compared}
  }
}
\description{
Runs \code{\link{assess_pos}} for each K in \code{K_range} and returns a
comparison table of objective criterion (sum of R-squared) values. This
helps identify the optimal number of segments.
}
\examples{
\donttest{
library(seminr)

mobi_mm <- constructs(
  composite("Image",        multi_items("IMAG", 1:5)),
  composite("Expectation",  multi_items("CUEX", 1:3)),
  composite("Satisfaction", multi_items("CUSA", 1:3)),
  composite("Loyalty",      multi_items("CUSL", 1:3))
)
mobi_sm <- relationships(
  paths(from = "Image",       to = c("Expectation", "Satisfaction", "Loyalty")),
  paths(from = "Expectation", to = "Satisfaction"),
  paths(from = "Satisfaction", to = "Loyalty")
)
pls_model <- estimate_pls(data = mobi, measurement_model = mobi_mm,
                           structural_model = mobi_sm)

pos_comp <- assess_pos_compare(pls_model, K_range = 2:3,
                                nstart = 2, max_iter = 10, seed = 123)
print(pos_comp)
plot(pos_comp)
}

}
\seealso{
[assess_fimix_compare()] for comparing FIMIX solutions across K
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_coa.R
\name{deviance_tree}
\alias{deviance_tree}
\title{Build Deviance Tree}
\usage{
deviance_tree(pd_result, deviance_bounds = c(0.025, 0.975))
}
\arguments{
\item{pd_result}{A `coa_deviance` object from `predictive_deviance()`.}

\item{deviance_bounds}{Two-element numeric vector of quantile bounds
(default `c(0.025, 0.975)`).}
}
\value{
An object of class `coa_dtree` containing the rpart tree,
  deviant groups, group roots, unique deviants, and sorted PD values.
}
\description{
Grows a decision tree on predictive deviance scores to identify groups
of cases with extreme (deviant) PD values.
}
\seealso{
[assess_coa()], [predictive_deviance()], [group_rules()], [competes()]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_fimix.R
\name{plot.fimix_comparison}
\alias{plot.fimix_comparison}
\title{Plot FIMIX-PLS Comparison}
\usage{
\method{plot}{fimix_comparison}(
  x,
  type = c("criteria", "entropy"),
  criteria = c("AIC3", "AIC4", "BIC", "CAIC"),
  ...
)
}
\arguments{
\item{x}{A \code{fimix_comparison} object from \code{assess_fimix_compare()}.}

\item{type}{One of \code{"criteria"} (line plot of IC vs K) or
\code{"entropy"} (EN vs K).}

\item{criteria}{Character vector of criteria to plot when
\code{type = "criteria"} (default: \code{c("AIC3", "AIC4", "BIC", "CAIC")}).}

\item{...}{Additional arguments passed to \code{plot()}.}
}
\description{
Plot FIMIX-PLS Comparison
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_nca.R
\name{assess_nca}
\alias{assess_nca}
\title{Necessary Condition Analysis (NCA) for PLS-SEM Models}
\usage{
assess_nca(
  seminr_model,
  target,
  predictors = NULL,
  ceilings = c("ce_fdh", "cr_fdh"),
  test.rep = 1000,
  steps = 10,
  seed = 123,
  ...
)
}
\arguments{
\item{seminr_model}{An estimated SEMinR model from \code{estimate_pls()}.}

\item{target}{Name of the endogenous (outcome) construct (character).}

\item{predictors}{Optional character vector of predictor construct names.
If \code{NULL} (default), auto-detected from the structural model as
direct predictors of \code{target}.}

\item{ceilings}{Character vector of ceiling techniques to use
(default \code{c("ce_fdh", "cr_fdh")}).}

\item{test.rep}{Number of permutation test repetitions for significance
testing (default 1000). Set to 0 to skip significance testing.}

\item{steps}{Number of steps in the bottleneck table (default 10).}

\item{seed}{Random seed for reproducibility (default 123).}

\item{...}{Additional arguments (currently unused).}
}
\value{
An object of class \code{nca_analysis} containing:
  \item{nca_raw}{Reserved (always NULL)}
  \item{effect_sizes}{Matrix of effect sizes (d) per predictor and ceiling}
  \item{significance}{Matrix of p-values per predictor and ceiling (NULL if test.rep = 0)}
  \item{bottleneck}{List of bottleneck tables, one per ceiling technique}
  \item{pls_model}{The original estimated seminr model}
  \item{target}{Name of the target construct}
  \item{predictors}{Character vector of predictor names used}
  \item{ceilings}{Character vector of ceiling techniques used}
}
\description{
`assess_nca` conducts a Necessary Condition Analysis on construct scores
from an estimated PLS-SEM model. It identifies whether predictors are
necessary conditions for achieving a given level of the target construct.
}
\details{
NCA complements PLS-SEM's sufficiency-based regression by testing whether
a certain level of a predictor is \emph{necessary} (but not sufficient)
for a certain level of the outcome. A predictor is considered a necessary
condition when it has an effect size (d) >= 0.1 and a significant p-value
(p < 0.05).

The function extracts construct scores from the seminr model, auto-detects
direct predictors of the target from the structural model, and computes
ceiling envelopes using the specified techniques.
}
\examples{
library(seminr)
library(seminrExtras)

mobi_mm <- constructs(
  composite("Image",        multi_items("IMAG", 1:5)),
  composite("Value",        multi_items("PERV", 1:2)),
  composite("Satisfaction", multi_items("CUSA", 1:3)),
  composite("Loyalty",      multi_items("CUSL", 1:3))
)

mobi_sm <- relationships(
  paths(from = c("Image", "Value"), to = "Satisfaction"),
  paths(from = "Satisfaction", to = "Loyalty")
)

mobi_pls <- estimate_pls(data = mobi,
                          measurement_model = mobi_mm,
                          structural_model  = mobi_sm)

\donttest{
nca_result <- assess_nca(mobi_pls,
                          target = "Satisfaction",
                          test.rep = 100)
print(nca_result)
summary(nca_result)
plot(nca_result, type = "effects")
}

}
\references{
Dul, J. (2016). Necessary Condition Analysis (NCA): Logic and Methodology
of 'Necessary but Not Sufficient' Causality. Organizational Research
Methods, 19(1), 10-52.

Richter, N. F., Schubring, S., Hauff, S., Ringle, C. M., & Sarstedt, M.
(2020). When predictors of outcomes are necessary: guidelines for the
combined use of PLS-SEM and NCA. Industrial Management & Data Systems,
120(12), 2243-2267.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_cipma.R
\name{assess_cipma}
\alias{assess_cipma}
\title{Combined Importance-Performance Map Analysis (cIPMA) for PLS-SEM}
\usage{
assess_cipma(
  seminr_model,
  target,
  scale_min = 1,
  scale_max = 7,
  nca = TRUE,
  nca_ceilings = c("ce_fdh", "cr_fdh"),
  nca_test.rep = 0,
  nca_steps = 10,
  seed = 123
)
}
\arguments{
\item{seminr_model}{An estimated SEMinR model from \code{estimate_pls()}.}

\item{target}{Name of the target (endogenous) construct for the IPMA.}

\item{scale_min}{Minimum of the measurement scale (default 1).}

\item{scale_max}{Maximum of the measurement scale (default 7).}

\item{nca}{Logical; if \code{TRUE} (default), run NCA to produce a cIPMA.}

\item{nca_ceilings}{Character vector of ceiling techniques for NCA
(default \code{c("ce_fdh", "cr_fdh")}).}

\item{nca_test.rep}{Number of NCA permutation test repetitions (default 0).
Set > 0 to obtain significance for the necessity classification.}

\item{nca_steps}{Number of bottleneck table steps (default 10).}

\item{seed}{Random seed for reproducibility (default 123).}
}
\value{
An object of class \code{cipma_analysis} containing:
  \item{importance_unstd}{Named numeric vector of unstandardized total effects}
  \item{importance_std}{Named numeric vector of standardized total effects}
  \item{performance}{Named numeric vector of construct performances (0--100)}
  \item{nca}{An \code{nca_analysis} object (NULL if \code{nca = FALSE})}
  \item{classification}{Data frame classifying each construct}
  \item{target}{Name of the target construct}
  \item{constructs}{Character vector of included constructs}
  \item{scale_range}{Numeric vector \code{c(scale_min, scale_max)}}
  \item{negative_weight_constructs}{Constructs with negative outer weights (if any)}
  \item{excluded_interactions}{Interaction constructs excluded from IPMA}
  \item{pls_model}{The original estimated seminr model}
}
\description{
\code{assess_cipma} conducts an Importance-Performance Map Analysis (IPMA)
and optionally combines it with Necessary Condition Analysis (NCA) to
produce a combined IPMA (cIPMA). The analysis identifies constructs that
are both important (high total effect) and necessary (without which the
outcome cannot reach high levels), following the 7-step cIPMA procedure
of Sarstedt et al. (2024).
}
\details{
\strong{Importance} is measured by the unstandardized total effect of each
construct on the target. Unstandardized effects are obtained by scaling
standardized path coefficients using the standard deviations of rescaled
(0--100) construct scores.

\strong{Performance} is the weighted average of rescaled indicator means,
where each indicator is rescaled from the original measurement scale
([\code{scale_min}, \code{scale_max}]) to 0--100. Weights are the PLS
outer weights, which must all be positive for valid IPMA rescaling.

Interaction constructs (e.g., \code{"X*W"}) are automatically excluded
from the IPMA because their performance is not meaningful on a 0--100
scale.

When \code{nca = TRUE}, NCA is run on construct scores for each
predecessor--target pair, and constructs are classified into four
categories based on crossing importance (above/below median) with
necessity (NCA d >= 0.1 and p < 0.05).
}
\examples{
library(seminr)
library(seminrExtras)

mobi_mm <- constructs(
  composite("Image",        multi_items("IMAG", 1:5)),
  composite("Expectation",  multi_items("CUEX", 1:3)),
  composite("Value",        multi_items("PERV", 1:2)),
  composite("Satisfaction", multi_items("CUSA", 1:3)),
  composite("Loyalty",      multi_items("CUSL", 1:3))
)

mobi_sm <- relationships(
  paths(from = "Image",       to = c("Expectation", "Satisfaction", "Loyalty")),
  paths(from = "Expectation", to = c("Value", "Satisfaction")),
  paths(from = "Value",       to = "Satisfaction"),
  paths(from = "Satisfaction",to = "Loyalty")
)

mobi_pls <- estimate_pls(data = mobi,
                          measurement_model = mobi_mm,
                          structural_model  = mobi_sm)

\donttest{
cipma_result <- assess_cipma(mobi_pls,
                              target = "Loyalty",
                              scale_min = 1,
                              scale_max = 10)
print(cipma_result)
summary(cipma_result)
plot(cipma_result)
}

}
\references{
Ringle, C. M. & Sarstedt, M. (2016). Gain More Insight from Your PLS-SEM
Results: The Importance-Performance Map Analysis. Industrial Management &
Data Systems, 119(9), 1865-1886.

Sarstedt, M., Richter, N. F., Hauff, S. & Ringle, C. M. (2024). Combined
Importance-Performance Map Analysis (cIPMA): A SmartPLS 4 Tutorial.
Journal of Marketing Analytics, 12, 746-760.

Hauff, S. et al. (2024). Importance and Performance in PLS-SEM and NCA:
Introducing the cIPMA. Journal of Retailing and Consumer Services, 78,
103723.
}
\seealso{
\code{\link{assess_ipma}} for IPMA without NCA,
  \code{\link{assess_nca}} for standalone NCA analysis
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_pcm.R
\name{plot.pcm_analysis}
\alias{plot.pcm_analysis}
\title{Plot PCM Results}
\usage{
\method{plot}{pcm_analysis}(
  x,
  metric = "RMSE",
  legend_pos = "topright",
  bar_col = "steelblue",
  neg_col = "tomato",
  cex.labels = 0.8,
  cex.legend = 0.75,
  ...
)
}
\arguments{
\item{x}{An object of class \code{pcm_analysis} from \code{\link{assess_pcm}}.}

\item{metric}{Character, either \code{"RMSE"} (default) or \code{"MAE"}.}

\item{legend_pos}{Character giving the legend position (e.g. \code{"topright"},
\code{"topleft"}, \code{"bottomright"}). Set to \code{NULL} or \code{FALSE}
to suppress the legend entirely. Default \code{"topright"}.}

\item{bar_col}{Colour for positive-PCM bars (default \code{"steelblue"}).}

\item{neg_col}{Colour for negative-PCM bars (default \code{"tomato"}).}

\item{cex.labels}{Numeric expansion factor for the value labels printed on
each bar. Default 0.8.}

\item{cex.legend}{Numeric expansion factor for the legend text. Default 0.75.}

\item{...}{Additional graphical parameters passed to \code{\link[graphics]{barplot}}.}
}
\description{
Creates a grouped barplot of PCM values for each mediation path, with one bar
per indicator and threshold lines for the weak/moderate/strong classification
boundaries.
}
\examples{
\donttest{
# Customise legend placement and bar colours
# plot(pcm, legend_pos = "topleft", bar_col = "dodgerblue")

# Suppress legend entirely
# plot(pcm, legend_pos = NULL)
}

}
